@@ -6,13 +6,13 @@
 /// @author  Walter Bamberger
 /// @author  Jakob Erdmann
 /// @date    July 2010
-/// @version $Id: AGCity.cpp 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: AGCity.cpp 20482 2016-04-18 20:49:42Z behrisch $
 ///
 // City class that contains all other objects of the city: in particular
 // streets, households, bus lines, work positions and school
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2010-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2010-2016 DLR (http://www.dlr.de/) and contributors
 // activitygen module
 // Copyright 2010 TUM (Technische Universitaet Muenchen, http://www.tum.de/)
 /****************************************************************************/
@@ -61,11 +61,11 @@ AGCity::completeStreets() {
     }
 
     SUMOReal pop = 0, work = 0;
-    std::vector<AGStreet>::iterator it;
+    std::vector<AGStreet*>::iterator it;
 
     for (it = streets.begin(); it != streets.end(); ++it) {
-        pop += it->getPopulation();
-        work += it->getWorkplaceNumber();
+        pop += (*it)->getPopulation();
+        work += (*it)->getWorkplaceNumber();
     }
     statData.factorInhabitants = (SUMOReal)statData.inhabitants / pop;
     //can be improved with other input data
@@ -79,38 +79,38 @@ AGCity::completeStreets() {
     statData.factorWorkPositions = neededWorkPositionsInCity / (SUMOReal) work;
 
     for (it = streets.begin(); it != streets.end(); ++it) {
-        it->setPopulation(it->getPopulation() * statData.factorInhabitants);
-        it->setWorkplaceNumber(it->getWorkplaceNumber() * statData.factorWorkPositions);
+        (*it)->setPopulation((*it)->getPopulation() * statData.factorInhabitants);
+        (*it)->setWorkplaceNumber((*it)->getWorkplaceNumber() * statData.factorWorkPositions);
         //it->print();
     }
 
     //completing streets from edges of the network not handled/present in STAT file (no population no work position)
     std::map<std::string, ROEdge*>::const_iterator itE;
-    std::vector<AGStreet>::iterator itS;
+    std::vector<AGStreet*>::iterator itS;
 
     for (itE = net->getEdgeMap().begin(); itE != net->getEdgeMap().end(); ++itE) {
         for (itS = streets.begin(); itS != streets.end(); ++itS) {
-            if (itS->getName() == itE->second->getID()) {
+            if (*itS == itE->second) {
                 break;
             }
         }
         //if this edge isn't represented by a street
-        if (itS == streets.end() && itE->second->getType() != ROEdge::ET_INTERNAL) {
-            streets.push_back(AGStreet(itE->second));
+        if (itS == streets.end() && itE->second->getFunc() != ROEdge::ET_INTERNAL) {
+            streets.push_back(static_cast<AGStreet*>(itE->second));
         }
     }
 }
 
 void
 AGCity::generateWorkPositions() {
-    std::vector<AGStreet>::iterator it;
+    std::vector<AGStreet*>::iterator it;
     int workPositionCounter = 0;
 
     try {
         for (it = streets.begin(); it != streets.end(); ++it) {
             //std::cout << "number of work positions in street: " << it->getWorkplaceNumber() << std::endl;
-            for (int i = 0; i < it->getWorkplaceNumber(); ++i) {
-                workPositions.push_back(AGWorkPosition(&statData, *it));
+            for (int i = 0; i < (*it)->getWorkplaceNumber(); ++i) {
+                workPositions.push_back(AGWorkPosition(&statData, **it));
                 ++workPositionCounter;
             }
         }
@@ -167,7 +167,7 @@ AGCity::completeBusLines() {
 
 void
 AGCity::generatePopulation() {
-    std::vector<AGStreet>::iterator it;
+    std::vector<AGStreet*>::iterator it;
     SUMOReal people = 0;
     nbrCars = 0;
     unsigned int idHouseholds = 0;
@@ -196,11 +196,11 @@ AGCity::generatePopulation() {
         }
     }
     for (it = streets.begin(); it != streets.end(); ++it) {
-        people += it->getPopulation();
+        people += (*it)->getPopulation();
         while (people > 0 && idHouseholds < (unsigned int)numAdults.size()) {
             size_t i = RandHelper::rand(numAdults.size() - idHouseholds);
             ++idHouseholds;
-            households.push_back(AGHousehold(&*it, this, idHouseholds));
+            households.push_back(AGHousehold(*it, this, idHouseholds));
             households.back().generatePeople(abs(numAdults[i]), numChilds[i], numAdults[i] < 0); //&statData
             //households.back().generateCars(statData.carRate);
             people -= households.back().getPeopleNbr();
@@ -277,11 +277,11 @@ AGCity::schoolAllocation() {
             /*ofstream fichier("test.txt", ios::app);  // ouverture en écriture avec effacement du fichier ouvert
             if(fichier)
             {
-            	fichier << "===> WARNING: Not enough school places in the city for all children..." << std::endl;
-            	fichier.close();
+              fichier << "===> WARNING: Not enough school places in the city for all children..." << std::endl;
+              fichier.close();
             }
             else
-            	cerr << "Impossible d'ouvrir le fichier !" << std::endl;*/
+              cerr << "Impossible d'ouvrir le fichier !" << std::endl;*/
 
             //std::cout << "===> WARNING: Not enough school places in the city for all children..." << std::endl;
         }
@@ -404,23 +404,23 @@ AGCity::getStreet(const std::string& edge) {
         std::cout << "first completed in getStreet() of City: Consolidation of data not needed in ActivityGen any more" << std::endl;
     }
     //rest of the function
-    std::vector<AGStreet>::iterator it = streets.begin();
+    std::vector<AGStreet*>::iterator it = streets.begin();
     while (it != streets.end()) {
-        if (it->getName() == edge) {
-            return *it;
+        if ((*it)->getID() == edge) {
+            return **it;
         }
         ++it;
     }
     std::cout << "===> ERROR: WRONG STREET EDGE (" << edge << ") given and not found in street set." << std::endl;
-    throw(std::runtime_error("Street not found with edge id " + edge));
+    throw (std::runtime_error("Street not found with edge id " + edge));
 }
 
 const AGStreet&
 AGCity::getRandomStreet() {
     if (streets.empty()) {
-        throw(std::runtime_error("No street found in this city"));
+        throw (std::runtime_error("No street found in this city"));
     }
-    return streets[RandHelper::rand(streets.size())];
+    return *streets[RandHelper::rand(streets.size())];
 }
 
 /****************************************************************************/
