@@ -5,12 +5,12 @@
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    July 2010
-/// @version $Id: AGStreet.cpp 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: AGStreet.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // Represents a SUMO edge and contains people and work densities
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2010-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2010-2016 DLR (http://www.dlr.de/) and contributors
 // activitygen module
 // Copyright 2010 TUM (Technische Universitaet Muenchen, http://www.tum.de/)
 /****************************************************************************/
@@ -41,52 +41,45 @@
 // ===========================================================================
 // method definitions
 // ===========================================================================
-AGStreet::AGStreet(const ROEdge* edge, SUMOReal popDensity, SUMOReal workDensity) :
-    edge(edge) {
-    pop = popDensity * edge->getLength();
-    work = workDensity * edge->getLength();
+AGStreet::AGStreet(const std::string& id, RONode* from, RONode* to, unsigned int index, const int priority) :
+    ROEdge(id, from, to, index, priority), myPopulation(0.), myNumWorkplaces(0.) {
 }
 
 
 void
 AGStreet::print() const {
-    std::cout << "- AGStreet: Name=" << edge->getID() << " Length=" << edge->getLength() << " pop=" << pop << " work=" << work << std::endl;
-}
-
-
-SUMOReal
-AGStreet::getLength() const {
-    return edge->getLength();
-}
-
-
-const std::string&
-AGStreet::getName() const {
-    return edge->getID();
+    std::cout << "- AGStreet: Name=" << getID() << " Length=" << getLength() << " pop=" << myPopulation << " work=" << myNumWorkplaces << std::endl;
 }
 
 
 SUMOReal
 AGStreet::getPopulation() const {
-    return pop;
+    return myPopulation;
 }
 
 
 void
 AGStreet::setPopulation(const SUMOReal population) {
-    pop = population;
+    myPopulation = population;
 }
 
 
 SUMOReal
 AGStreet::getWorkplaceNumber() const {
-    return work;
+    return myNumWorkplaces;
 }
 
 
 void
 AGStreet::setWorkplaceNumber(const SUMOReal workPositions) {
-    work = workPositions;
+    myNumWorkplaces = workPositions;
 }
 
+
+bool
+AGStreet::allows(const SUMOVehicleClass vclass) const {
+    return (getPermissions() & vclass) == vclass;
+}
+
+
 /****************************************************************************/
