@@ -5,12 +5,12 @@
 /// @author  Michael Behrisch
 /// @author  Melanie Knocke
 /// @date    Thu, 16.03.2006
-/// @version $Id: RODFDetectorFlow.cpp 18095 2015-03-17 09:39:00Z behrisch $
+/// @version $Id: RODFDetectorFlow.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // Storage for flows within the DFROUTER
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2006-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2006-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -69,8 +69,9 @@ RODFDetectorFlows::addFlow(const std::string& id, SUMOTime t, const FlowDef& fd)
             (*i).firstSet = true;
         }
     }
-    assert((t - myBeginTime) / myStepOffset < (int) myFastAccessFlows[id].size());
-    FlowDef& ofd = myFastAccessFlows[id][(t - myBeginTime) / myStepOffset];
+    const int index = (int)((t - myBeginTime) / myStepOffset);
+    assert(index < (int) myFastAccessFlows[id].size());
+    FlowDef& ofd = myFastAccessFlows[id][index];
     if (ofd.firstSet) {
         ofd = fd;
         ofd.firstSet = false;
