@@ -5,12 +5,12 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Thu, 16.03.2006
-/// @version $Id: RODFNet.cpp 18627 2015-08-13 08:58:59Z namdre $
+/// @version $Id: RODFNet.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // A DFROUTER-network
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -32,6 +32,7 @@
 #include <cassert>
 #include <iostream>
 #include <map>
+#include <queue>
 #include <vector>
 #include <iterator>
 #include "RODFNet.h"
@@ -988,11 +989,11 @@ RODFNet::buildEdgeFlowMap(const RODFDetectorFlows& flows,
                 fd.fLKW += (srcFD.fLKW / (SUMOReal) maxClique->size());
                 fd.isLKW += (srcFD.isLKW / (SUMOReal) maxClique->size());
                 if (!didWarn && srcFD.vPKW > 0 && srcFD.vPKW < 255 && srcFD.vPKW / 3.6 > into->getSpeed()) {
-                    WRITE_MESSAGE("Detected PKW speed higher than allowed speed at '" + (*l) + "' on '" + into->getID() + "'.");
+                    WRITE_MESSAGE("Detected PKW speed higher than allowed speed at '" + (*l) + "' on edge '" + into->getID() + "'.");
                     didWarn = true;
                 }
                 if (!didWarn && srcFD.vLKW > 0 && srcFD.vLKW < 255 && srcFD.vLKW / 3.6 > into->getSpeed()) {
-                    WRITE_MESSAGE("Detected LKW speed higher than allowed speed at '" + (*l) + "' on '" + into->getID() + "'.");
+                    WRITE_MESSAGE("Detected LKW speed higher than allowed speed at '" + (*l) + "' on edge '" + into->getID() + "'.");
                     didWarn = true;
                 }
             }
