@@ -4,12 +4,12 @@
 /// @author  Eric Nicolay
 /// @author  Michael Behrisch
 /// @date    Thu, 16.03.2006
-/// @version $Id: RODFNet.h 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: RODFNet.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // A DFROUTER-network
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -32,13 +32,14 @@
 #include <config.h>
 #endif
 
-#include <router/RONet.h>
 #include <utils/options/OptionsCont.h>
+#include <utils/common/StdDefs.h>
+#include <utils/common/SUMOTime.h>
+#include <router/ROEdge.h>
+#include <router/RONet.h>
 #include "RODFDetector.h"
 #include "RODFRouteDesc.h"
 #include "RODFRouteCont.h"
-#include <utils/common/StdDefs.h>
-#include <utils/common/SUMOTime.h>
 
 
 // ===========================================================================
@@ -104,20 +105,20 @@ protected:
 
     ROEdge* getDetectorEdge(const RODFDetector& det) const;
     bool isSource(const RODFDetector& det, ROEdge* edge,
-                  std::vector<ROEdge*>& seen, const RODFDetectorCon& detectors,
+                  ROEdgeVector& seen, const RODFDetectorCon& detectors,
                   bool strict) const;
     bool isFalseSource(const RODFDetector& det, ROEdge* edge,
-                       std::vector<ROEdge*>& seen, const RODFDetectorCon& detectors) const;
-    bool isDestination(const RODFDetector& det, ROEdge* edge, std::vector<ROEdge*>& seen,
+                       ROEdgeVector& seen, const RODFDetectorCon& detectors) const;
+    bool isDestination(const RODFDetector& det, ROEdge* edge, ROEdgeVector& seen,
                        const RODFDetectorCon& detectors) const;
 
     void computeRoutesFor(ROEdge* edge, RODFRouteDesc& base, int no,
                           bool keepUnfoundEnds,
                           bool keepShortestOnly,
-                          std::vector<ROEdge*>& visited, const RODFDetector& det,
+                          ROEdgeVector& visited, const RODFDetector& det,
                           RODFRouteCont& into, const RODFDetectorCon& detectors,
                           int maxFollowingLength,
-                          std::vector<ROEdge*>& seen) const;
+                          ROEdgeVector& seen) const;
 
     void buildDetectorEdgeDependencies(RODFDetectorCon& dets) const;
 
@@ -158,10 +159,10 @@ private:
     };
 
     /// @brief Map of edge name->list of names of this edge approaching edges
-    std::map<ROEdge*, std::vector<ROEdge*> > myApproachingEdges;
+    std::map<ROEdge*, ROEdgeVector > myApproachingEdges;
 
     /// @brief Map of edge name->list of names of edges approached by this edge
-    std::map<ROEdge*, std::vector<ROEdge*> > myApproachedEdges;
+    std::map<ROEdge*, ROEdgeVector > myApproachedEdges;
 
     mutable std::map<ROEdge*, std::vector<std::string>, idComp> myDetectorsOnEdges;
     mutable std::map<std::string, ROEdge*> myDetectorEdges;
