@@ -5,12 +5,12 @@
 /// @author  Michael Behrisch
 /// @author  Andreas Gaubatz
 /// @date    Sept 2002
-/// @version $Id: GUIApplicationWindow.cpp 18761 2015-09-01 14:15:44Z behrisch $
+/// @version $Id: GUIApplicationWindow.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // The main window of the SUMO-gui.
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -60,10 +60,10 @@
 #include <utils/foxtools/FXLCDLabel.h>
 #include <utils/foxtools/FXRealSpinDial.h>
 #include <utils/foxtools/FXThreadEvent.h>
+#include <utils/foxtools/FXLinkLabel.h>
 
 #include <utils/xml/XMLSubSys.h>
 #include <utils/gui/images/GUITexturesHelper.h>
-#include <utils/gui/windows/GUIAppEnum.h>
 #include <utils/gui/events/GUIEvent_SimulationStep.h>
 #include <utils/gui/events/GUIEvent_Message.h>
 #include <utils/gui/div/GUIMessageWindow.h>
@@ -77,6 +77,10 @@
 #include <utils/gui/globjects/GUIGlObjectStorage.h>
 #include <utils/gui/div/GUIGlobalSelection.h>
 #include <utils/gui/settings/GUISettingsHandler.h>
+#include <utils/gui/windows/GUIAppEnum.h>
+#include <utils/gui/windows/GUISUMOAbstractView.h>
+#include <utils/gui/windows/GUIPerspectiveChanger.h>
+#include <utils/options/OptionsCont.h>
 #include "GUIGlobals.h"
 #include "dialogs/GUIDialog_AboutSUMO.h"
 #include "dialogs/GUIDialog_AppSettings.h"
@@ -86,6 +90,7 @@
 #include <foreign/nvwa/debug_new.h>
 #endif
 
+//#define HAVE_DANGEROUS_SOUNDS
 
 // ===========================================================================
 // FOX-declarations
@@ -103,6 +108,7 @@ FXDEFMAP(GUIApplicationWindow) GUIApplicationWindowMap[] = {
     FXMAPFUNC(SEL_COMMAND,  MID_CLOSE,             GUIApplicationWindow::onCmdClose),
     FXMAPFUNC(SEL_COMMAND,  MID_EDITCHOSEN,        GUIApplicationWindow::onCmdEditChosen),
     FXMAPFUNC(SEL_COMMAND,  MID_EDIT_BREAKPOINTS,  GUIApplicationWindow::onCmdEditBreakpoints),
+    FXMAPFUNC(SEL_COMMAND,  MID_NETEDIT,           GUIApplicationWindow::onCmdNetedit),
 
     FXMAPFUNC(SEL_COMMAND,  MID_APPSETTINGS,        GUIApplicationWindow::onCmdAppSettings),
     FXMAPFUNC(SEL_COMMAND,  MID_GAMING,             GUIApplicationWindow::onCmdGaming),
@@ -135,6 +141,8 @@ FXDEFMAP(GUIApplicationWindow) GUIApplicationWindowMap[] = {
     FXMAPFUNC(SEL_UPDATE,   MID_STEP,              GUIApplicationWindow::onUpdStep),
     FXMAPFUNC(SEL_UPDATE,   MID_EDITCHOSEN,        GUIApplicationWindow::onUpdNeedsSimulation),
     FXMAPFUNC(SEL_UPDATE,   MID_EDIT_BREAKPOINTS,  GUIApplicationWindow::onUpdNeedsSimulation),
+    FXMAPFUNC(SEL_UPDATE,   MID_NETEDIT,           GUIApplicationWindow::onUpdNeedsSimulation),
+    FXMAPFUNC(SEL_COMMAND,  MID_HELP,              GUIApplicationWindow::onCmdHelp),
 
     // forward requests to the active view
     FXMAPFUNC(SEL_COMMAND,  MID_LOCATEJUNCTION, GUIApplicationWindow::onCmdLocate),
@@ -224,11 +232,11 @@ GUIApplicationWindow::dependentBuild() {
         myGeoFrame =
             new FXHorizontalFrame(myStatusbar, LAYOUT_FIX_WIDTH | LAYOUT_FILL_Y | LAYOUT_RIGHT | FRAME_SUNKEN,
                                   0, 0, 20, 0, 0, 0, 0, 0, 0, 0);
-        myGeoCoordinate = new FXLabel(myGeoFrame, "N/A", 0, LAYOUT_CENTER_Y);
+        myGeoCoordinate = new FXLabel(myGeoFrame, "N/A\t\tOriginal coordinate (before coordinate transformation in NETCONVERT)", 0, LAYOUT_CENTER_Y);
         myCartesianFrame =
             new FXHorizontalFrame(myStatusbar, LAYOUT_FIX_WIDTH | LAYOUT_FILL_Y | LAYOUT_RIGHT | FRAME_SUNKEN,
                                   0, 0, 20, 0, 0, 0, 0, 0, 0, 0);
-        myCartesianCoordinate = new FXLabel(myCartesianFrame, "N/A", 0, LAYOUT_CENTER_Y);
+        myCartesianCoordinate = new FXLabel(myCartesianFrame, "N/A\t\tNetwork coordinate", 0, LAYOUT_CENTER_Y);
     }
 
     // make the window a mdi-window
@@ -259,7 +267,7 @@ GUIApplicationWindow::dependentBuild() {
     // set the status bar
     myStatusbar->getStatusLine()->setText("Ready.");
     // set the caption
-    setTitle(MFXUtils::getTitleText(("SUMO " + getBuildName(VERSION_STRING)).c_str()));
+    setTitle(MFXUtils::getTitleText("SUMO " VERSION_STRING));
 
     // start the simulation-thread (it will loop until the application ends deciding by itself whether to perform a step or not)
     myRunThread->start();
@@ -352,20 +360,20 @@ GUIApplicationWindow::fillMenuBar() {
     myFileMenu = new FXMenuPane(this);
     new FXMenuTitle(myMenuBar, "&File", NULL, myFileMenu);
     new FXMenuCommand(myFileMenu,
-                      "&Open Simulation...\tCtl-O\tOpen a simulation (Configuration file).",
+                      "&Open Simulation...\tCtrl+O\tOpen a simulation (Configuration file).",
                       GUIIconSubSys::getIcon(ICON_OPEN_CONFIG), this, MID_OPEN_CONFIG);
     new FXMenuCommand(myFileMenu,
-                      "Open &Network...\tCtl-N\tOpen a network.",
+                      "Open &Network...\tCtrl+N\tOpen a network.",
                       GUIIconSubSys::getIcon(ICON_OPEN_NET), this, MID_OPEN_NETWORK);
     new FXMenuCommand(myFileMenu,
-                      "Open Shapes \tCtl-P\tLoad POIs and Polygons for visualization.",
+                      "Open Shapes \tCtrl+P\tLoad POIs and Polygons for visualization.",
                       GUIIconSubSys::getIcon(ICON_OPEN_SHAPES), this, MID_OPEN_SHAPES);
     new FXMenuCommand(myFileMenu,
-                      "&Reload\tCtl-R\tReloads the simulation / the network.",
+                      "&Reload\tCtrl+R\tReloads the simulation / the network.",
                       GUIIconSubSys::getIcon(ICON_RELOAD), this, MID_RELOAD);
     new FXMenuSeparator(myFileMenu);
     new FXMenuCommand(myFileMenu,
-                      "Close\tCtl-W\tClose the simulation.",
+                      "Close\tCtrl+W\tClose the simulation.",
                       GUIIconSubSys::getIcon(ICON_CLOSE), this, MID_CLOSE);
     // Recent files
     FXMenuSeparator* sep1 = new FXMenuSeparator(myFileMenu);
@@ -402,7 +410,7 @@ GUIApplicationWindow::fillMenuBar() {
     myRecentNets.setSelector(MID_RECENTFILE);
     new FXMenuSeparator(myFileMenu);
     new FXMenuCommand(myFileMenu,
-                      "&Quit\tCtl-Q\tQuit the Application.",
+                      "&Quit\tCtrl+Q\tQuit the Application.",
                       0, this, MID_QUIT, 0);
 
     // build edit menu
@@ -416,15 +424,19 @@ GUIApplicationWindow::fillMenuBar() {
     myEditMenu = new FXMenuPane(this);
     new FXMenuTitle(myMenuBar, "&Edit", NULL, myEditMenu);
     new FXMenuCommand(myEditMenu,
-                      "Edit Selected...\tCtl-E\tOpens a Dialog for editing the List of Selected Items.",
+                      "Edit Selected...\tCtrl+E\tOpens a dialog for editing the list of selected items.",
                       GUIIconSubSys::getIcon(ICON_FLAG), this, MID_EDITCHOSEN);
     new FXMenuCascade(myEditMenu,
                       "Select lanes which allow...\t\tOpens a menu for selecting a vehicle class by which to selected lanes.",
                       GUIIconSubSys::getIcon(ICON_FLAG), mySelectByPermissions);
     new FXMenuSeparator(myEditMenu);
     new FXMenuCommand(myEditMenu,
-                      "Edit Breakpoints...\tCtl-B\tOpens a Dialog for editing breakpoints.",
+                      "Edit Breakpoints...\tCtrl+B\tOpens a dialog for editing breakpoints.",
                       0, this, MID_EDIT_BREAKPOINTS);
+    new FXMenuSeparator(myEditMenu);
+    new FXMenuCommand(myEditMenu,
+                      "Open in Netedit...\tCtrl+T\tOpens the netedit application with the current network.",
+                      0, this, MID_NETEDIT);
 
     // build settings menu
     mySettingsMenu = new FXMenuPane(this);
@@ -433,10 +445,10 @@ GUIApplicationWindow::fillMenuBar() {
                       "Application Settings...\t\tOpen a Dialog for Application Settings editing.",
                       NULL, this, MID_APPSETTINGS);
     new FXMenuCheck(mySettingsMenu,
-                    "Gaming Mode\tCtl-G\tToggle gaming mode on/off.",
+                    "Gaming Mode\tCtrl+G\tToggle gaming mode on/off.",
                     this, MID_GAMING);
     new FXMenuCheck(mySettingsMenu,
-                    "Full Screen Mode\tCtl-F\tToggle full screen mode on/off.",
+                    "Full Screen Mode\tCtrl+F\tToggle full screen mode on/off.",
                     this, MID_FULLSCREEN);
     // build Locate menu
     myLocatorMenu = new FXMenuPane(this);
@@ -475,13 +487,13 @@ GUIApplicationWindow::fillMenuBar() {
     myControlMenu = new FXMenuPane(this);
     new FXMenuTitle(myMenuBar, "Simulation", NULL, myControlMenu);
     new FXMenuCommand(myControlMenu,
-                      "Run\tCtl-A\tStart running the simulation.",
+                      "Run\tCtrl+A\tStart running the simulation.",
                       NULL, this, MID_START);
     new FXMenuCommand(myControlMenu,
-                      "Stop\tCtl-S\tStop running the simulation.",
+                      "Stop\tCtrl+S\tStop running the simulation.",
                       NULL, this, MID_STOP);
     new FXMenuCommand(myControlMenu,
-                      "Step\tCtl-D\tPerform one simulation step.",
+                      "Step\tCtrl+D\tPerform one simulation step.",
                       NULL, this, MID_STEP);
 
     // build windows menu
@@ -528,6 +540,7 @@ GUIApplicationWindow::fillMenuBar() {
     // build help menu
     myHelpMenu = new FXMenuPane(this);
     new FXMenuTitle(myMenuBar, "&Help", NULL, myHelpMenu);
+    new FXMenuCommand(myHelpMenu, "&Online Documentation", 0, this, MID_HELP);
     new FXMenuCommand(myHelpMenu, "&About", GUIIconSubSys::getIcon(ICON_APP),
                       this, MID_ABOUT);
 }
@@ -678,8 +691,8 @@ GUIApplicationWindow::onCmdEditChosen(FXObject* menu, FXSelector, void*) {
     } else {
         if (!myAmLoading && myRunThread->simulationAvailable()) {
             const SUMOVehicleClass svc = SumoVehicleClassStrings.get(mc->getText().text());
-            for (size_t i = 0; i < MSEdge::dictSize(); ++i) {
-                const std::vector<MSLane*>& lanes = MSEdge::dictionary(i)->getLanes();
+            for (MSEdgeVector::const_iterator i = MSEdge::getAllEdges().begin(); i != MSEdge::getAllEdges().end(); ++i) {
+                const std::vector<MSLane*>& lanes = (*i)->getLanes();
                 for (std::vector<MSLane*>::const_iterator it = lanes.begin(); it != lanes.end(); ++it) {
                     GUILane* lane = dynamic_cast<GUILane*>(*it);
                     assert(lane != 0);
@@ -711,6 +724,48 @@ GUIApplicationWindow::onCmdEditBreakpoints(FXObject*, FXSelector, void*) {
 }
 
 
+long
+GUIApplicationWindow::onCmdHelp(FXObject*, FXSelector, void*) {
+    FXLinkLabel::fxexecute("http://sumo.dlr.de/wiki/SUMO-GUI");
+    return 1;
+}
+
+
+long
+GUIApplicationWindow::onCmdNetedit(FXObject*, FXSelector, void*) {
+    if (mySubWindows.empty()) {
+        return 1;
+    }
+    FXRegistry reg("Netedit", "DLR");
+    reg.read();
+    const GUISUMOAbstractView* const v = static_cast<GUIGlChildWindow*>(mySubWindows[0])->getView();
+    reg.writeIntEntry("viewport", "x", v->getChanger().getXPos());
+    reg.writeIntEntry("viewport", "y", v->getChanger().getYPos());
+    reg.writeIntEntry("viewport", "z", v->getChanger().getZoom());
+    reg.write();
+    std::string netedit = "netedit";
+    const char* sumoPath = getenv("SUMO_HOME");
+    if (sumoPath != 0) {
+        std::string newPath = std::string(sumoPath) + "/bin/netedit";
+        if (FileHelpers::isReadable(newPath) || FileHelpers::isReadable(newPath + ".exe")) {
+            netedit = "\"" + newPath + "\"";
+        }
+    }
+    std::string cmd = netedit + " --registry-viewport -s "  + OptionsCont::getOptions().getString("net-file");
+    // start in background
+#ifndef WIN32
+    cmd = cmd + " &";
+#else
+    // see "help start" for the parameters
+    cmd = "start /B \"\" " + cmd;
+#endif
+    WRITE_MESSAGE("Running " + cmd + ".");
+    // yay! fun with dangerous commands... Never use this over the internet
+    SysUtils::runHiddenCommand(cmd);
+    return 1;
+}
+
+
 long
 GUIApplicationWindow::onCmdOpenConfiguration(FXObject*, FXSelector, void*) {
     // get the new file name
@@ -1217,8 +1272,7 @@ GUIApplicationWindow::handleEvent_SimulationLoaded(GUIEvent* e) {
                 setTitle("SUMO Interactive Traffic Light");
             } else {
                 // set simulation name on the caption
-                std::string caption = "SUMO " + getBuildName(VERSION_STRING);
-                setTitle(MFXUtils::getTitleText(caption.c_str(), ec->myFile.c_str()));
+                setTitle(MFXUtils::getTitleText("SUMO " VERSION_STRING, ec->myFile.c_str()));
             }
             // set simulation step begin information
             myLCDLabel->setText("-------------");
@@ -1293,7 +1347,7 @@ GUIApplicationWindow::checkGamingEvents() {
     MSVehicleControl& vc = MSNet::getInstance()->getVehicleControl();
     MSVehicleControl::constVehIt it = vc.loadedVehBegin();
     MSVehicleControl::constVehIt end = vc.loadedVehEnd();
-#ifdef HAVE_INTERNAL
+#ifdef HAVE_DANGEROUS_SOUNDS // disable user-configurable command execution for public build
     if (myJamSounds.getOverallProb() > 0) {
         // play honking sound if some vehicle is waiting too long
         for (; it != end; ++it) {
@@ -1388,7 +1442,7 @@ GUIApplicationWindow::closeAllWindows() {
     // delete the simulation
     myRunThread->deleteSim();
     // reset the caption
-    setTitle(MFXUtils::getTitleText(("SUMO " + getBuildName(VERSION_STRING)).c_str()));
+    setTitle(MFXUtils::getTitleText("SUMO " VERSION_STRING));
     // delete other children
     while (myTrackerWindows.size() != 0) {
         delete myTrackerWindows[0];
@@ -1439,6 +1493,10 @@ GUIApplicationWindow::setStatusBarText(const std::string& text) {
 void
 GUIApplicationWindow::updateTimeLCD(SUMOTime time) {
     time -= DELTA_T; // synchronize displayed time with netstate output
+    if (time < 0) {
+        myLCDLabel->setText("-------------");
+        return;
+    }
     if (myAmGaming) {
         // show time counting backwards
         time = myRunThread->getSimEndTime() - time;
