@@ -5,12 +5,12 @@
 /// @author  Christian Roessel
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: GUIApplicationWindow.h 18761 2015-09-01 14:15:44Z behrisch $
+/// @version $Id: GUIApplicationWindow.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // The main window of the SUMO-gui.
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -158,6 +158,12 @@ public:
     /// @brief Called on menu Edit->Edit Breakpoints
     long onCmdEditBreakpoints(FXObject*, FXSelector, void*);
 
+    /// @brief called if the user selects help->Documentation
+    long onCmdHelp(FXObject* sender, FXSelector sel, void* ptr);
+
+    /// @brief Called on menu Edit->Netedit
+    long onCmdNetedit(FXObject*, FXSelector, void*);
+
     /// @brief Opens the application settings menu (Settings->Application Settings...)
     long onCmdAppSettings(FXObject*, FXSelector, void*);
 
@@ -248,6 +254,12 @@ public:
         return mySimDelayTarget->getValue();
     }
 
+    /** @brief Sets the delay of the parent application
+     */
+    virtual void setDelay(SUMOReal delay) {
+        mySimDelayTarget->setValue(delay);
+    }
+
     /** @brief Sends an event from the application thread to the GUI and waits until it is handled
      * @param event the event to send
      */
