@@ -4,12 +4,12 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: GUILoadThread.cpp 18498 2015-06-22 05:59:25Z behrisch $
+/// @version $Id: GUILoadThread.cpp 20482 2016-04-18 20:49:42Z behrisch $
 ///
 // Class describing the thread that performs the loading of a simulation
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -63,9 +63,7 @@
 #include "GUIGlobals.h"
 #include "GUIEvent_SimulationLoaded.h"
 
-#ifdef HAVE_INTERNAL
 #include <mesogui/GUIMEVehicleControl.h>
-#endif
 
 #ifndef NO_TRACI
 #include <traci-server/TraCIServer.h>
@@ -126,6 +124,7 @@ GUILoadThread::run() {
                 myFile = oc.getString("configuration-file");
             } else if (oc.isSet("net-file")) {
                 myFile = oc.getString("net-file");
+                myLoadNet = true;
             }
         }
         myTitle = myFile;
@@ -134,6 +133,11 @@ GUILoadThread::run() {
         MsgHandler::getWarningInstance()->removeRetriever(&OutputDevice::getDevice("stderr"));
         MsgHandler::getErrorInstance()->removeRetriever(&OutputDevice::getDevice("stderr"));
         // do this once again to get parsed options
+        if (oc.getBool("duration-log.statistics") && oc.isDefault("verbose")) {
+            // must be done before calling initOutputOptions (which checks option "verbose")
+            // but initOutputOptions must come before checkOptions (so that warnings are printed)
+            oc.set("verbose", "true");
+        }
         MsgHandler::initOutputOptions();
         XMLSubSys::setValidation(oc.getString("xml-validation"), oc.getString("xml-validation.net"));
         GUIGlobals::gRunAfterLoad = oc.getBool("start");
@@ -161,13 +165,12 @@ GUILoadThread::run() {
         myParent->onCmdGaming(0, 0, 0);
     }
     MSVehicleControl* vehControl = 0;
-#ifdef HAVE_INTERNAL
     GUIVisualizationSettings::UseMesoSim = MSGlobals::gUseMesoSim;
     if (MSGlobals::gUseMesoSim) {
         vehControl = new GUIMEVehicleControl();
-    } else
-#endif
+    } else {
         vehControl = new GUIVehicleControl();
+    }
 
     GUINet* net = 0;
     int simStartTime = 0;
@@ -206,7 +209,7 @@ GUILoadThread::run() {
             simStartTime = string2time(oc.getString("begin"));
             simEndTime = string2time(oc.getString("end"));
             guiSettingsFiles = oc.getStringVector("gui-settings-file");
-#ifdef HAVE_INTERNAL
+#ifdef HAVE_OSG
             osgView = oc.getBool("osg-view");
 #endif
         }
