@@ -3,12 +3,12 @@
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    Mon, 08.03.2004
-/// @version $Id: GUIDialog_AppSettings.h 18095 2015-03-17 09:39:00Z behrisch $
+/// @version $Id: GUIDialog_AppSettings.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // The application-settings dialog
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -71,6 +71,9 @@ public:
 
     /// @brief Called on "Allow textures"-button change
     long onCmdAllowTextures(FXObject*, FXSelector, void*);
+
+    /// @brief Called on "Locate links in messages"-button change
+    long onCmdLocateLinks(FXObject*, FXSelector, void*);
     /// @}
 
 
@@ -81,6 +84,9 @@ private:
     /// @brief Information whether textures may be used
     bool myAllowTextures;
 
+    /// @brief Information whether locate links appear in messages
+    bool myLocateLinks;
+
 
 protected:
     /// default constructor (needed by FOX)
