@@ -4,12 +4,12 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Thu, 17 Jun 2004
-/// @version $Id: GUIDialog_Breakpoints.cpp 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: GUIDialog_Breakpoints.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // Editor for simulation breakpoints
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -143,7 +143,7 @@ GUIDialog_Breakpoints::rebuildList() {
     }
     // insert into table
     FXint row = 0;
-    std::vector<int>::iterator j;
+    std::vector<SUMOTime>::iterator j;
     for (j = myBreakpoints->begin(); j != myBreakpoints->end(); ++j) {
         myTable->setItemText(row, 0, time2string(*j).c_str());
         row++;
@@ -200,7 +200,7 @@ GUIDialog_Breakpoints::encode2TXT() {
     myBreakpointLock->lock();
     std::ostringstream strm;
     std::sort(myBreakpoints->begin(), myBreakpoints->end());
-    for (std::vector<int>::iterator j = myBreakpoints->begin(); j != myBreakpoints->end(); ++j) {
+    for (std::vector<SUMOTime>::iterator j = myBreakpoints->begin(); j != myBreakpoints->end(); ++j) {
         if ((*j) != INVALID_VALUE) {
             strm << time2string(*j) << std::endl;
         }
