@@ -4,12 +4,12 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Mon, 26.04.2004
-/// @version $Id: GUITriggerBuilder.cpp 19535 2015-12-05 13:47:18Z behrisch $
+/// @version $Id: GUITriggerBuilder.cpp 20482 2016-04-18 20:49:42Z behrisch $
 ///
 // Builds trigger objects for guisim
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -75,31 +75,28 @@ GUITriggerBuilder::buildRerouter(MSNet& net, const std::string& id,
     return rr;
 }
 
-void
-GUITriggerBuilder::buildBusStop(MSNet& net, const std::string& id,
-                                const std::vector<std::string>& lines,
-                                MSLane* lane,
-                                SUMOReal frompos, SUMOReal topos) {
-
-    GUIBusStop* stop = new GUIBusStop(id, lines, *lane, frompos, topos);
-    if (!net.addBusStop(stop)) {
-        delete stop;
-        throw InvalidArgument("Could not build bus stop '" + id + "'; probably declared twice.");
-    }
-    static_cast<GUINet&>(net).getVisualisationSpeedUp().addAdditionalGLObject(stop);
-}
 
 void
-GUITriggerBuilder::buildContainerStop(MSNet& net, const std::string& id,
-                                      const std::vector<std::string>& lines,
-                                      MSLane* lane,
-                                      SUMOReal frompos, SUMOReal topos) {
-    GUIContainerStop* stop = new GUIContainerStop(id, lines, *lane, frompos, topos);
-    if (!net.addContainerStop(stop)) {
-        delete stop;
-        throw InvalidArgument("Could not build container stop '" + id + "'; probably declared twice.");
+GUITriggerBuilder::buildStoppingPlace(MSNet& net, const std::string& id, const std::vector<std::string>& lines,
+                                      MSLane* lane, SUMOReal frompos, SUMOReal topos, const SumoXMLTag element) {
+    bool success = false;
+    GUIGlObject* o = 0;
+    if (element == SUMO_TAG_CONTAINER_STOP) {
+        GUIContainerStop* stop = new GUIContainerStop(id, lines, *lane, frompos, topos);
+        success = net.addContainerStop(stop);
+        o = stop;
+        myCurrentStop = stop;
+    } else {
+        GUIBusStop* stop = new GUIBusStop(id, lines, *lane, frompos, topos);
+        success = net.addBusStop(stop);
+        o = stop;
+        myCurrentStop = stop;
     }
-    static_cast<GUINet&>(net).getVisualisationSpeedUp().addAdditionalGLObject(stop);
+    if (!success) {
+        delete o;
+        throw InvalidArgument("Could not build " + toString(element) + " stop '" + id + "'; probably declared twice.");
+    }
+    static_cast<GUINet&>(net).getVisualisationSpeedUp().addAdditionalGLObject(o);
 }
 
 
