@@ -4,12 +4,12 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Wed, 07.12.2005
-/// @version $Id: GUIBusStop.h 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: GUIBusStop.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // A lane area vehicles can halt at (gui-version)
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -37,7 +37,7 @@
 #include <utils/common/Command.h>
 #include <utils/common/VectorHelper.h>
 #include <utils/geom/PositionVector.h>
-#include <microsim/trigger/MSBusStop.h>
+#include <microsim/MSStoppingPlace.h>
 #include <utils/gui/globjects/GUIGlObject.h>
 #include <utils/gui/globjects/GUIGlObject_AbstractAdd.h>
 #include <utils/gui/globjects/GUIGLObjectPopupMenu.h>
@@ -61,14 +61,14 @@ class GUIManipulator;
  * @class GUIBusStop
  * @brief A lane area vehicles can halt at (gui-version)
  *
- * This gui-version of a bus-stop extends MSBusStop by methods for displaying
+ * This gui-version of a bus-stop extends MSStoppingPlace by methods for displaying
  *  and interaction.
  *
- * @see MSBusStop
+ * @see MSStoppingPlace
  * @see GUIGlObject_AbstractAdd
  * @see GUIGlObject
  */
-class GUIBusStop : public MSBusStop, public GUIGlObject_AbstractAdd {
+class GUIBusStop : public MSStoppingPlace, public GUIGlObject_AbstractAdd {
 public:
     /** @brief Constructor
      * @param[in] idStorage The gl-id storage for giving this object an gl-id
@@ -87,6 +87,8 @@ public:
     ~GUIBusStop();
 
 
+    /// @brief adds an access point to this stop
+    void addAccess(MSLane* lane, const SUMOReal pos);
 
     /// @name inherited from GUIGlObject
     //@{
@@ -147,6 +149,9 @@ private:
     /// @brief The rotation of the sign
     SUMOReal myFGSignRot;
 
+    /// @brief The coordinates of access points
+    PositionVector myAccessCoords;
+
 
 };
 
