@@ -4,12 +4,12 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: GUILane.cpp 18639 2015-08-17 13:59:19Z namdre $
+/// @version $Id: GUILane.cpp 20482 2016-04-18 20:49:42Z behrisch $
 ///
 // Representation of a lane in the micro simulation (gui-version)
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -44,12 +44,14 @@
 #include <utils/gui/windows/GUISUMOAbstractView.h>
 #include <utils/gui/div/GUIParameterTableWindow.h>
 #include <utils/gui/div/GUIGlobalSelection.h>
+#include <utils/gui/windows/GUIAppEnum.h>
 #include <microsim/MSGlobals.h>
 #include <microsim/MSLane.h>
 #include <microsim/MSVehicleControl.h>
 #include <microsim/MSVehicleTransfer.h>
 #include <microsim/MSNet.h>
 #include <microsim/MSEdgeWeightsStorage.h>
+#include <microsim/devices/MSDevice_Routing.h>
 #include "GUILane.h"
 #include "GUIEdge.h"
 #include "GUIVehicle.h"
@@ -72,8 +74,9 @@ GUILane::GUILane(const std::string& id, SUMOReal maxSpeed, SUMOReal length,
                  MSEdge* const edge, unsigned int numericalID,
                  const PositionVector& shape, SUMOReal width,
                  SVCPermissions permissions, unsigned int index) :
-    MSLane(id, maxSpeed, length, edge, numericalID, shape, width, permissions),
-    GUIGlObject(GLO_LANE, id) {
+    MSLane(id, maxSpeed, length, edge, numericalID, shape, width, permissions, index),
+    GUIGlObject(GLO_LANE, id),
+    myAmClosed(false) {
     myShapeRotations.reserve(myShape.size() - 1);
     myShapeLengths.reserve(myShape.size() - 1);
     myShapeColors.reserve(myShape.size() - 1);
@@ -87,7 +90,6 @@ GUILane::GUILane(const std::string& id, SUMOReal maxSpeed, SUMOReal length,
     //
     myHalfLaneWidth = (SUMOReal)(myWidth / 2.);
     myQuarterLaneWidth = (SUMOReal)(myWidth / 4.);
-    myIndex = index;
 }
 
 
@@ -167,7 +169,7 @@ GUILane::detectCollisions(SUMOTime timestep, const std::string& stage) {
 
 // ------ Drawing methods ------
 void
-GUILane::drawLinkNo() const {
+GUILane::drawLinkNo(const GUIVisualizationSettings& s) const {
     unsigned int noLinks = (unsigned int)myLinks.size();
     if (noLinks == 0) {
         return;
@@ -178,23 +180,24 @@ GUILane::drawLinkNo() const {
         MSLink* link = MSLinkContHelper::getConnectingLink(*getLogicalPredecessorLane(), *this);
         PositionVector shape = getShape();
         shape.extrapolate(0.5); // draw on top of the walking area
-        drawTextAtEnd(toString(link->getIndex()), shape, 0);
-        drawTextAtEnd(toString(link->getIndex()), shape.reverse(), 0);
+        drawTextAtEnd(toString(link->getIndex()), shape, 0, s.drawLinkJunctionIndex);
+        drawTextAtEnd(toString(link->getIndex()), shape.reverse(), 0, s.drawLinkJunctionIndex);
         return;
     }
     // draw all links
     SUMOReal w = myWidth / (SUMOReal) noLinks;
     SUMOReal x1 = myHalfLaneWidth;
+    const bool lefthand = MSNet::getInstance()->lefthand();
     for (int i = noLinks; --i >= 0;) {
         SUMOReal x2 = x1 - (SUMOReal)(w / 2.);
-        drawTextAtEnd(toString(myLinks[i]->getIndex()), getShape(), x2);
+        drawTextAtEnd(toString(myLinks[lefthand ? noLinks - 1 - i : i]->getIndex()), getShape(), x2, s.drawLinkJunctionIndex);
         x1 -= w;
     }
 }
 
 
 void
-GUILane::drawTLSLinkNo(const GUINet& net) const {
+GUILane::drawTLSLinkNo(const GUIVisualizationSettings& s, const GUINet& net) const {
     unsigned int noLinks = (unsigned int)myLinks.size();
     if (noLinks == 0) {
         return;
@@ -206,35 +209,36 @@ GUILane::drawTLSLinkNo(const GUINet& net) const {
         if (linkNo >= 0) {
             PositionVector shape = getShape();
             shape.extrapolate(0.5); // draw on top of the walking area
-            drawTextAtEnd(toString(linkNo), shape, 0);
-            drawTextAtEnd(toString(linkNo), shape.reverse(), 0);
+            drawTextAtEnd(toString(linkNo), shape, 0, s.drawLinkTLIndex);
+            drawTextAtEnd(toString(linkNo), shape.reverse(), 0, s.drawLinkTLIndex);
         }
         return;
     }
     // draw all links
     SUMOReal w = myWidth / (SUMOReal) noLinks;
     SUMOReal x1 = myHalfLaneWidth;
+    const bool lefthand = MSNet::getInstance()->lefthand();
     for (int i = noLinks; --i >= 0;) {
         SUMOReal x2 = x1 - (SUMOReal)(w / 2.);
-        int linkNo = net.getLinkTLIndex(myLinks[i]);
+        int linkNo = net.getLinkTLIndex(myLinks[lefthand ? noLinks - 1 - i : i]);
         if (linkNo < 0) {
             continue;
         }
-        drawTextAtEnd(toString(linkNo), getShape(), x2);
+        drawTextAtEnd(toString(linkNo), getShape(), x2, s.drawLinkTLIndex);
         x1 -= w;
     }
 }
 
 
 void
-GUILane::drawTextAtEnd(const std::string& text, const PositionVector& shape, SUMOReal x) const {
+GUILane::drawTextAtEnd(const std::string& text, const PositionVector& shape, SUMOReal x, const GUIVisualizationTextSettings& settings) const {
     glPushMatrix();
     const Position& end = shape.back();
     const Position& f = shape[-2];
     const SUMOReal rot = RAD2DEG(atan2((end.x() - f.x()), (f.y() - end.y())));
     glTranslated(end.x(), end.y(), 0);
     glRotated(rot, 0, 0, 1);
-    GLHelper::drawText(text, Position(x, 0.26), 0, .6, RGBColor(128, 128, 255, 255), 180);
+    GLHelper::drawText(text, Position(x, 0.26), 0, .6 * settings.size / 50, settings.color, 180);
     glPopMatrix();
 }
 
@@ -258,9 +262,10 @@ GUILane::drawLinkRules(const GUIVisualizationSettings& s, const GUINet& net) con
     // draw all links
     SUMOReal w = myWidth / (SUMOReal) noLinks;
     SUMOReal x1 = 0;
+    const bool lefthand = MSNet::getInstance()->lefthand();
     for (unsigned int i = 0; i < noLinks; ++i) {
         SUMOReal x2 = x1 + w;
-        drawLinkRule(s, net, myLinks[i], getShape(), x1, x2);
+        drawLinkRule(s, net, myLinks[lefthand ? noLinks - 1 - i : i], getShape(), x1, x2);
         x1 = x2;
     }
 }
@@ -347,39 +352,39 @@ GUILane::drawArrows() const {
         switch (dir) {
             case LINKDIR_STRAIGHT:
                 GLHelper::drawBoxLine(Position(0, 4), 0, 2, .05);
-                GLHelper::drawTriangleAtEnd(Line(Position(0, 4), Position(0, 1)), (SUMOReal) 1, (SUMOReal) .25);
+                GLHelper::drawTriangleAtEnd(Position(0, 4), Position(0, 1), (SUMOReal) 1, (SUMOReal) .25);
                 break;
             case LINKDIR_TURN:
                 GLHelper::drawBoxLine(Position(0, 4), 0, 1.5, .05);
                 GLHelper::drawBoxLine(Position(0, 2.5), 90, .5, .05);
                 GLHelper::drawBoxLine(Position(0.5, 2.5), 180, 1, .05);
-                GLHelper::drawTriangleAtEnd(Line(Position(0.5, 2.5), Position(0.5, 4)), (SUMOReal) 1, (SUMOReal) .25);
+                GLHelper::drawTriangleAtEnd(Position(0.5, 2.5), Position(0.5, 4), (SUMOReal) 1, (SUMOReal) .25);
                 break;
             case LINKDIR_TURN_LEFTHAND:
                 GLHelper::drawBoxLine(Position(0, 4), 0, 1.5, .05);
                 GLHelper::drawBoxLine(Position(0, 2.5), -90, 1, .05);
                 GLHelper::drawBoxLine(Position(-0.5, 2.5), -180, 1, .05);
-                GLHelper::drawTriangleAtEnd(Line(Position(-0.5, 2.5), Position(-0.5, 4)), (SUMOReal) 1, (SUMOReal) .25);
+                GLHelper::drawTriangleAtEnd(Position(-0.5, 2.5), Position(-0.5, 4), (SUMOReal) 1, (SUMOReal) .25);
                 break;
             case LINKDIR_LEFT:
                 GLHelper::drawBoxLine(Position(0, 4), 0, 1.5, .05);
                 GLHelper::drawBoxLine(Position(0, 2.5), 90, 1, .05);
-                GLHelper::drawTriangleAtEnd(Line(Position(0, 2.5), Position(1.5, 2.5)), (SUMOReal) 1, (SUMOReal) .25);
+                GLHelper::drawTriangleAtEnd(Position(0, 2.5), Position(1.5, 2.5), (SUMOReal) 1, (SUMOReal) .25);
                 break;
             case LINKDIR_RIGHT:
                 GLHelper::drawBoxLine(Position(0, 4), 0, 1.5, .05);
                 GLHelper::drawBoxLine(Position(0, 2.5), -90, 1, .05);
-                GLHelper::drawTriangleAtEnd(Line(Position(0, 2.5), Position(-1.5, 2.5)), (SUMOReal) 1, (SUMOReal) .25);
+                GLHelper::drawTriangleAtEnd(Position(0, 2.5), Position(-1.5, 2.5), (SUMOReal) 1, (SUMOReal) .25);
                 break;
             case LINKDIR_PARTLEFT:
                 GLHelper::drawBoxLine(Position(0, 4), 0, 1.5, .05);
                 GLHelper::drawBoxLine(Position(0, 2.5), 45, .7, .05);
-                GLHelper::drawTriangleAtEnd(Line(Position(0, 2.5), Position(1.2, 1.3)), (SUMOReal) 1, (SUMOReal) .25);
+                GLHelper::drawTriangleAtEnd(Position(0, 2.5), Position(1.2, 1.3), (SUMOReal) 1, (SUMOReal) .25);
                 break;
             case LINKDIR_PARTRIGHT:
                 GLHelper::drawBoxLine(Position(0, 4), 0, 1.5, .05);
                 GLHelper::drawBoxLine(Position(0, 2.5), -45, .7, .05);
-                GLHelper::drawTriangleAtEnd(Line(Position(0, 2.5), Position(-1.2, 1.3)), (SUMOReal) 1, (SUMOReal) .25);
+                GLHelper::drawTriangleAtEnd(Position(0, 2.5), Position(-1.2, 1.3), (SUMOReal) 1, (SUMOReal) .25);
                 break;
             default:
                 break;
@@ -403,7 +408,7 @@ GUILane::drawLane2LaneConnections() const {
         glVertex2f(p1.x(), p1.y());
         glVertex2f(p2.x(), p2.y());
         glEnd();
-        GLHelper::drawTriangleAtEnd(Line(p1, p2), (SUMOReal) .4, (SUMOReal) .2);
+        GLHelper::drawTriangleAtEnd(p1, p2, (SUMOReal) .4, (SUMOReal) .2);
     }
 }
 
@@ -416,13 +421,13 @@ GUILane::drawGL(const GUIVisualizationSettings& s) const {
     const bool isWalkingArea = myEdge->getPurpose() == MSEdge::EDGEFUNCTION_WALKINGAREA;
     const bool isInternal = isCrossing || isWalkingArea || myEdge->getPurpose() == MSEdge::EDGEFUNCTION_INTERNAL;
     bool mustDrawMarkings = false;
-    SUMOReal exaggeration = s.laneWidthExaggeration * s.laneScaler.getScheme().getColor(getScaleValue(s.laneScaler.getActive()));
-#ifdef HAVE_INTERNAL
+    SUMOReal exaggeration = s.laneWidthExaggeration;
     if (MSGlobals::gUseMesoSim) {
         GUIEdge* myGUIEdge = dynamic_cast<GUIEdge*>(myEdge);
         exaggeration *= s.edgeScaler.getScheme().getColor(myGUIEdge->getScaleValue(s.edgeScaler.getActive()));
+    } else {
+        exaggeration *= s.laneScaler.getScheme().getColor(getScaleValue(s.laneScaler.getActive()));
     }
-#endif
     const bool drawDetails =  s.scale * exaggeration > 5;
     if (isCrossing || isWalkingArea) {
         // draw internal lanes on top of junctions
@@ -468,27 +473,31 @@ GUILane::drawGL(const GUIVisualizationSettings& s) const {
                 }
                 drawCrossties(0.3 * exaggeration, 1 * exaggeration, 1 * exaggeration);
             } else if (isCrossing) {
-                // determine priority to decide color
-                MSLink* link = MSLinkContHelper::getConnectingLink(*getLogicalPredecessorLane(), *this);
-                if (link->havePriority() || net->getLinkTLIndex(link) > 0) {
-                    glColor3d(0.9, 0.9, 0.9);
-                } else {
-                    glColor3d(0.1, 0.1, 0.1);
+                if (s.drawCrossingsAndWalkingareas) {
+                    // determine priority to decide color
+                    MSLink* link = MSLinkContHelper::getConnectingLink(*getLogicalPredecessorLane(), *this);
+                    if (link->havePriority() || net->getLinkTLIndex(link) > 0) {
+                        glColor3d(0.9, 0.9, 0.9);
+                    } else {
+                        glColor3d(0.1, 0.1, 0.1);
+                    }
+                    glTranslated(0, 0, .2);
+                    drawCrossties(0.5, 1.0, getWidth() * 0.5);
+                    glTranslated(0, 0, -.2);
                 }
-                glTranslated(0, 0, .2);
-                drawCrossties(0.5, 1.0, getWidth() * 0.5);
-                glTranslated(0, 0, -.2);
             } else if (isWalkingArea) {
-                glTranslated(0, 0, .2);
-                if (s.scale * exaggeration < 20.) {
-                    GLHelper::drawFilledPoly(myShape, true);
-                } else {
-                    GLHelper::drawFilledPolyTesselated(myShape, true);
-                }
-                glTranslated(0, 0, -.2);
+                if (s.drawCrossingsAndWalkingareas) {
+                    glTranslated(0, 0, .2);
+                    if (s.scale * exaggeration < 20.) {
+                        GLHelper::drawFilledPoly(myShape, true);
+                    } else {
+                        GLHelper::drawFilledPolyTesselated(myShape, true);
+                    }
+                    glTranslated(0, 0, -.2);
 #ifdef GUILane_DEBUG_DRAW_WALKING_AREA_VERTICES
-                GLHelper::debugVertices(myShape, 80 / s.scale);
+                    GLHelper::debugVertices(myShape, 80 / s.scale);
 #endif
+                }
             } else {
                 const SUMOReal halfWidth = isInternal ? myQuarterLaneWidth : myHalfLaneWidth;
                 mustDrawMarkings = !isInternal && myPermissions != 0 && myPermissions != SVC_PEDESTRIAN && exaggeration == 1.0 && !isWaterway(myPermissions);
@@ -516,11 +525,11 @@ GUILane::drawGL(const GUIVisualizationSettings& s) const {
                     drawLane2LaneConnections();
                 }
                 glTranslated(0, 0, .1);
-                if (s.drawLinkJunctionIndex) {
-                    drawLinkNo();
+                if (s.drawLinkJunctionIndex.show) {
+                    drawLinkNo(s);
                 }
-                if (s.drawLinkTLIndex) {
-                    drawTLSLinkNo(*net);
+                if (s.drawLinkTLIndex.show) {
+                    drawTLSLinkNo(s, *net);
                 }
                 glPopMatrix();
             }
@@ -559,13 +568,12 @@ void
 GUILane::drawMarkings(const GUIVisualizationSettings& s, SUMOReal scale) const {
     glPushMatrix();
     glTranslated(0, 0, GLO_EDGE);
-#ifdef HAVE_INTERNAL
-    if (!MSGlobals::gUseMesoSim)
-#endif
+    if (!MSGlobals::gUseMesoSim) {
         setColor(s);
+    }
     // optionally draw inverse markings
     if (myIndex > 0 && (myEdge->getLanes()[myIndex - 1]->getPermissions() & myPermissions) != 0) {
-        SUMOReal mw = (myHalfLaneWidth + SUMO_const_laneOffset + .01) * scale;
+        SUMOReal mw = (myHalfLaneWidth + SUMO_const_laneOffset + .01) * scale * (MSNet::getInstance()->lefthand() ? -1 : 1);
         int e = (int) getShape().size() - 1;
         for (int i = 0; i < e; ++i) {
             glPushMatrix();
@@ -650,6 +658,7 @@ GUILane::getPopUpMenu(GUIMainWindow& app,
     buildPopupHeader(ret, app);
     buildCenterPopupEntry(ret);
     //
+    new FXMenuCommand(ret, "Copy edge name to clipboard", 0, ret, MID_COPY_EDGE_NAME);
     buildNameCopyPopupEntry(ret);
     buildSelectionPopupEntry(ret);
     //
@@ -659,6 +668,15 @@ GUILane::getPopUpMenu(GUIMainWindow& app,
     new FXMenuCommand(ret, ("pos: " + toString(pos) + " height: " + toString(height)).c_str(), 0, 0, 0);
     new FXMenuSeparator(ret);
     buildPositionCopyEntry(ret, false);
+    new FXMenuSeparator(ret);
+    if (myAmClosed) {
+        new FXMenuCommand(ret, "Reopen lane", 0, &parent, MID_CLOSE_LANE);
+        new FXMenuCommand(ret, "Reopen edge", 0, &parent, MID_CLOSE_EDGE);
+    } else {
+        new FXMenuCommand(ret, "Close lane", 0, &parent, MID_CLOSE_LANE);
+        new FXMenuCommand(ret, "Close edge", 0, &parent, MID_CLOSE_EDGE);
+    }
+    new FXMenuCommand(ret, "Add rerouter", 0, &parent, MID_ADD_REROUTER);
     return ret;
 }
 
@@ -691,7 +709,9 @@ GUILane::getCenteringBoundary() const {
     Boundary b;
     b.add(myShape[0]);
     b.add(myShape[-1]);
-    b.grow(20);
+    b.grow(10);
+    // ensure that vehicles and persons on the side are drawn even if the edge
+    // is outside the view
     return b;
 }
 
@@ -770,7 +790,7 @@ bool
 GUILane::setFunctionalColor(size_t activeScheme) const {
     switch (activeScheme) {
         case 18: {
-            SUMOReal hue = RAD2DEG(myShape.beginEndAngle()) + 180; // [0-360]
+            SUMOReal hue = GeomHelper::naviDegree(myShape.beginEndAngle()); // [0-360]
             GLHelper::setColor(RGBColor::fromHSV(hue, 1., 1.));
             return true;
         }
@@ -815,6 +835,8 @@ GUILane::getColorValue(size_t activeScheme) const {
                     return 3;
                 case SVC_SHIP:
                     return 4;
+                case SVC_AUTHORITY:
+                    return 6;
                 default:
                     break;
             }
@@ -890,6 +912,12 @@ GUILane::getColorValue(size_t activeScheme) const {
             // color by average relative speed
             return getMeanSpeed() / myMaxSpeed;
         }
+        case 27: {
+            // color by routing device assumed speed
+            return MSDevice_Routing::getAssumedSpeed(&getEdge());
+        }
+        case 28:
+            return getElectricityConsumption() / myLength;
     }
     return 0;
 }
@@ -957,6 +985,8 @@ GUILane::getScaleValue(size_t activeScheme) const {
             // scale by average relative speed
             return getMeanSpeed() / myMaxSpeed;
         }
+        case 21:
+            return getElectricityConsumption() / myLength;
     }
     return 0;
 }
@@ -985,6 +1015,20 @@ GUILane::updateColor(const GUIVisualizationSettings& s) {
 #endif
 
 
+void
+GUILane::closeTraffic(bool rebuildAllowed) {
+    MSGlobals::gCheckRoutes = false;
+    if (myAmClosed) {
+        myPermissions = myOriginalPermissions;
+    } else {
+        myOriginalPermissions = myPermissions;
+        myPermissions = SVC_AUTHORITY;
+    }
+    myAmClosed = !myAmClosed;
+    if (rebuildAllowed) {
+        getEdge().rebuildAllowedLanes();
+    }
+}
 
 /****************************************************************************/
 
