@@ -4,12 +4,12 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: GUINet.h 18095 2015-03-17 09:39:00Z behrisch $
+/// @version $Id: GUINet.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // A MSNet extended by some values for usage within the gui
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -35,6 +35,7 @@
 #include <string>
 #include <utility>
 #include <microsim/MSNet.h>
+#include <microsim/devices/MSDevice_Tripinfo.h>
 #include <utils/geom/Boundary.h>
 #include <utils/geom/Position.h>
 #include <foreign/rtree/SUMORTree.h>
@@ -64,9 +65,7 @@ class GUIVehicle;
 class GUIVehicleControl;
 class MSVehicleControl;
 class MFXMutex;
-#ifdef HAVE_INTERNAL
 class GUIMEVehicleControl;
-#endif
 
 
 // ===========================================================================
@@ -208,6 +207,21 @@ public:
     void setIdleDuration(int val);
     //}
 
+    SUMOReal getAvgRouteLength() const {
+        return MSDevice_Tripinfo::getAvgRouteLength();
+    }
+    SUMOReal getAvgDuration() const {
+        return MSDevice_Tripinfo::getAvgDuration();
+    }
+    SUMOReal getAvgWaitingTime() const {
+        return MSDevice_Tripinfo::getAvgWaitingTime();
+    }
+    SUMOReal getAvgTimeLoss() const {
+        return MSDevice_Tripinfo::getAvgTimeLoss();
+    }
+    SUMOReal getAvgDepartDelay() const {
+        return MSDevice_Tripinfo::getAvgDepartDelay();
+    }
 
     /** @brief Returns the person control
      *
@@ -279,14 +293,12 @@ public:
      */
     GUIVehicleControl* getGUIVehicleControl();
 
-#ifdef HAVE_INTERNAL
     /** @brief Returns the vehicle control
      * @return The vehicle control
      * @see MSVehicleControl
      * @see myVehicleControl
      */
     GUIMEVehicleControl* getGUIMEVehicleControl();
-#endif
 
 #ifdef HAVE_OSG
     void updateColor(const GUIVisualizationSettings& s);
