@@ -4,12 +4,12 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: GUIPerson.cpp 18274 2015-04-22 15:00:44Z behrisch $
+/// @version $Id: GUIPerson.cpp 20482 2016-04-18 20:49:42Z behrisch $
 ///
 // A MSPerson extended by some values for usage within the gui
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -39,11 +39,11 @@
 #include <microsim/logging/CastingFunctionBinding.h>
 #include <microsim/logging/FunctionBinding.h>
 #include <microsim/MSVehicleControl.h>
-//#include <microsim/MSAbstractLaneChangeModel.h>
 #include <microsim/devices/MSDevice_Vehroutes.h>
 #include <utils/common/StringUtils.h>
 #include <utils/vehicle/SUMOVehicleParameter.h>
 #include <utils/common/AbstractMutex.h>
+#include <utils/geom/GeomHelper.h>
 #include <utils/gui/images/GUITexturesHelper.h>
 #include <utils/gui/windows/GUISUMOAbstractView.h>
 #include <utils/gui/windows/GUIAppEnum.h>
@@ -51,12 +51,13 @@
 #include <utils/gui/div/GUIGlobalSelection.h>
 #include <utils/gui/div/GLHelper.h>
 #include <utils/gui/div/GLObjectValuePassConnector.h>
+#include <utils/gui/globjects/GLIncludes.h>
 #include <gui/GUIApplicationWindow.h>
 #include <gui/GUIGlobals.h>
-#include "GUIPerson.h"
+#include "GUILane.h"
 #include "GUINet.h"
 #include "GUIEdge.h"
-#include <utils/gui/globjects/GLIncludes.h>
+#include "GUIPerson.h"
 
 #ifdef CHECK_MEMORY_LEAKS
 #include <foreign/nvwa/debug_new.h>
@@ -68,8 +69,12 @@
 // FOX callback mapping
 // ===========================================================================
 FXDEFMAP(GUIPerson::GUIPersonPopupMenu) GUIPersonPopupMenuMap[] = {
+    FXMAPFUNC(SEL_COMMAND, MID_SHOW_CURRENTROUTE,     GUIPerson::GUIPersonPopupMenu::onCmdShowCurrentRoute),
+    FXMAPFUNC(SEL_COMMAND, MID_HIDE_CURRENTROUTE,     GUIPerson::GUIPersonPopupMenu::onCmdHideCurrentRoute),
     FXMAPFUNC(SEL_COMMAND, MID_SHOW_WALKINGAREA_PATH, GUIPerson::GUIPersonPopupMenu::onCmdShowWalkingareaPath),
     FXMAPFUNC(SEL_COMMAND, MID_HIDE_WALKINGAREA_PATH, GUIPerson::GUIPersonPopupMenu::onCmdHideWalkingareaPath),
+    FXMAPFUNC(SEL_COMMAND, MID_START_TRACK,           GUIPerson::GUIPersonPopupMenu::onCmdStartTrack),
+    FXMAPFUNC(SEL_COMMAND, MID_STOP_TRACK,            GUIPerson::GUIPersonPopupMenu::onCmdStopTrack),
 };
 
 // Object implementation
@@ -85,13 +90,32 @@ FXIMPLEMENT(GUIPerson::GUIPersonPopupMenu, GUIGLObjectPopupMenu, GUIPersonPopupM
  * ----------------------------------------------------------------------- */
 GUIPerson::GUIPersonPopupMenu::GUIPersonPopupMenu(
     GUIMainWindow& app, GUISUMOAbstractView& parent,
-    GUIGlObject& o, std::map<GUISUMOAbstractView*, int>& additionalVisualizations)
-    : GUIGLObjectPopupMenu(app, parent, o), myVehiclesAdditionalVisualizations(additionalVisualizations) {
+    GUIGlObject& o, std::map<GUISUMOAbstractView*, int>& additionalVisualizations) :
+    GUIGLObjectPopupMenu(app, parent, o),
+    myVehiclesAdditionalVisualizations(additionalVisualizations) {
 }
 
 
 GUIPerson::GUIPersonPopupMenu::~GUIPersonPopupMenu() {}
 
+long
+GUIPerson::GUIPersonPopupMenu::onCmdShowCurrentRoute(FXObject*, FXSelector, void*) {
+    assert(myObject->getType() == GLO_PERSON);
+    if (!static_cast<GUIPerson*>(myObject)->hasActiveAddVisualisation(myParent, VO_SHOW_ROUTE)) {
+        static_cast<GUIPerson*>(myObject)->addActiveAddVisualisation(myParent, VO_SHOW_ROUTE);
+    }
+    return 1;
+}
+
+long
+GUIPerson::GUIPersonPopupMenu::onCmdHideCurrentRoute(FXObject*, FXSelector, void*) {
+    assert(myObject->getType() == GLO_PERSON);
+    static_cast<GUIPerson*>(myObject)->removeActiveAddVisualisation(myParent, VO_SHOW_ROUTE);
+    return 1;
+}
+
+
+
 long
 GUIPerson::GUIPersonPopupMenu::onCmdShowWalkingareaPath(FXObject*, FXSelector, void*) {
     assert(myObject->getType() == GLO_PERSON);
@@ -109,6 +133,25 @@ GUIPerson::GUIPersonPopupMenu::onCmdHideWalkingareaPath(FXObject*, FXSelector, v
 }
 
 
+long
+GUIPerson::GUIPersonPopupMenu::onCmdStartTrack(FXObject*, FXSelector, void*) {
+    assert(myObject->getType() == GLO_PERSON);
+    if (!static_cast<GUIPerson*>(myObject)->hasActiveAddVisualisation(myParent, VO_TRACKED)) {
+        myParent->startTrack(static_cast<GUIPerson*>(myObject)->getGlID());
+        static_cast<GUIPerson*>(myObject)->addActiveAddVisualisation(myParent, VO_TRACKED);
+    }
+    return 1;
+}
+
+long
+GUIPerson::GUIPersonPopupMenu::onCmdStopTrack(FXObject*, FXSelector, void*) {
+    assert(myObject->getType() == GLO_PERSON);
+    static_cast<GUIPerson*>(myObject)->removeActiveAddVisualisation(myParent, VO_TRACKED);
+    myParent->stopTrack();
+    return 1;
+}
+
+
 
 
 /* -------------------------------------------------------------------------
@@ -116,8 +159,9 @@ GUIPerson::GUIPersonPopupMenu::onCmdHideWalkingareaPath(FXObject*, FXSelector, v
  * ----------------------------------------------------------------------- */
 GUIPerson::GUIPerson(const SUMOVehicleParameter* pars, const MSVehicleType* vtype, MSTransportable::MSTransportablePlan* plan) :
     MSPerson(pars, vtype, plan),
-    GUIGlObject(GLO_PERSON, pars->id)
-{ }
+    GUIGlObject(GLO_PERSON, pars->id),
+    myPositionInVehicle(Position::INVALID) {
+}
 
 
 GUIPerson::~GUIPerson() {
@@ -132,12 +176,24 @@ GUIPerson::getPopUpMenu(GUIMainWindow& app,
     buildCenterPopupEntry(ret);
     buildNameCopyPopupEntry(ret);
     buildSelectionPopupEntry(ret);
-
+    if (hasActiveAddVisualisation(&parent, VO_SHOW_ROUTE)) {
+        new FXMenuCommand(ret, "Hide Current Route", 0, ret, MID_HIDE_CURRENTROUTE);
+    } else {
+        new FXMenuCommand(ret, "Show Current Route", 0, ret, MID_SHOW_CURRENTROUTE);
+    }
     if (hasActiveAddVisualisation(&parent, VO_SHOW_WALKINGAREA_PATH)) {
         new FXMenuCommand(ret, "Hide Walkingarea Path", 0, ret, MID_HIDE_WALKINGAREA_PATH);
     } else {
         new FXMenuCommand(ret, "Show Walkingarea Path", 0, ret, MID_SHOW_WALKINGAREA_PATH);
     }
+    new FXMenuSeparator(ret);
+    int trackedID = parent.getTrackedID();
+    if (trackedID < 0 || (size_t)trackedID != getGlID()) {
+        new FXMenuCommand(ret, "Start Tracking", 0, ret, MID_START_TRACK);
+    } else {
+        new FXMenuCommand(ret, "Stop Tracking", 0, ret, MID_STOP_TRACK);
+    }
+    new FXMenuSeparator(ret);
     //
     buildShowParamsPopupEntry(ret);
     buildPositionCopyEntry(ret, false);
@@ -149,7 +205,7 @@ GUIParameterTableWindow*
 GUIPerson::getParameterWindow(GUIMainWindow& app,
                               GUISUMOAbstractView&) {
     GUIParameterTableWindow* ret =
-        new GUIParameterTableWindow(app, *this, 8);
+        new GUIParameterTableWindow(app, *this, 16);
     // add items
     //ret->mkItem("type [NAME]", false, myType->getID());
     ret->mkItem("stage", false, getCurrentStageDescription());
@@ -158,8 +214,18 @@ GUIPerson::getParameterWindow(GUIMainWindow& app,
     ret->mkItem("edge [id]", false, getEdge()->getID());
     ret->mkItem("position [m]", true, new FunctionBinding<GUIPerson, SUMOReal>(this, &GUIPerson::getEdgePos));
     ret->mkItem("speed [m/s]", true, new FunctionBinding<GUIPerson, SUMOReal>(this, &GUIPerson::getSpeed));
-    ret->mkItem("angle [degree]", true, new FunctionBinding<GUIPerson, SUMOReal>(this, &GUIPerson::getAngle));
+    ret->mkItem("angle [degree]", true, new FunctionBinding<GUIPerson, SUMOReal>(this, &GUIPerson::getNaviDegree));
     ret->mkItem("waiting time [s]", true, new FunctionBinding<GUIPerson, SUMOReal>(this, &GUIPerson::getWaitingSeconds));
+
+    ret->mkItem("parameters [key:val]", false, toString(getParameter().getMap()));
+    ret->mkItem("", false, "");
+    ret->mkItem("Type Information:", false, "");
+    ret->mkItem("type [id]", false, myVType->getID());
+    ret->mkItem("length", false, myVType->getLength());
+    ret->mkItem("minGap", false, myVType->getMinGap());
+    ret->mkItem("maximum speed [m/s]", false, myVType->getMaxSpeed());
+
+    ret->mkItem("type parameters [key:val]", false, toString(myVType->getParameter().getMap()));
     // close building
     ret->closeBuilding();
     return ret;
@@ -169,8 +235,9 @@ GUIPerson::getParameterWindow(GUIMainWindow& app,
 Boundary
 GUIPerson::getCenteringBoundary() const {
     Boundary b;
+    // ensure that the vehicle is drawn, otherwise myPositionInVehicle will not be updated
     b.add(getPosition());
-    b.grow(20);
+    b.grow(MAX2(getVehicleType().getWidth(), getVehicleType().getLength()));
     return b;
 }
 
@@ -180,9 +247,11 @@ GUIPerson::drawGL(const GUIVisualizationSettings& s) const {
     glPushName(getGlID());
     glPushMatrix();
     Position p1 = getPosition();
+    if (getCurrentStageType() == DRIVING && !isWaiting4Vehicle()) {
+        p1 = myPositionInVehicle;
+    }
     glTranslated(p1.x(), p1.y(), getType());
     glRotated(90, 0, 0, 1);
-    // XXX use person specific gui settings
     // set person color
     setColor(s);
     // scale
@@ -235,6 +304,22 @@ GUIPerson::drawGLAdditional(GUISUMOAbstractView* const parent, const GUIVisualiz
     if (hasActiveAddVisualisation(parent, VO_SHOW_WALKINGAREA_PATH)) {
         drawAction_drawWalkingareaPath(s);
     }
+    if (hasActiveAddVisualisation(parent, VO_SHOW_ROUTE)) {
+        if (getCurrentStageType() == MOVING_WITHOUT_VEHICLE) {
+            setColor(s);
+            RGBColor current = GLHelper::getColor();
+            RGBColor darker = current.changedBrightness(-51);
+            GLHelper::setColor(darker);
+            MSPersonStage_Walking* stage = dynamic_cast<MSPersonStage_Walking*>(getCurrentStage());
+            assert(stage != 0);
+            const SUMOReal exaggeration = s.personSize.getExaggeration(s);
+            const ConstMSEdgeVector& edges = stage->getRoute();
+            for (ConstMSEdgeVector::const_iterator it = edges.begin(); it != edges.end(); ++it) {
+                GUILane* lane = static_cast<GUILane*>((*it)->getLanes()[0]);
+                GLHelper::drawBoxLines(lane->getShape(), lane->getShapeRotations(), lane->getShapeLengths(), exaggeration);
+            }
+        }
+    }
     glPopMatrix();
     glPopName();
 }
@@ -280,7 +365,7 @@ GUIPerson::setFunctionalColor(size_t activeScheme) const {
             return false;
         }
         case 8: { // color by angle
-            SUMOReal hue = getAngle() + 180; // [0-360]
+            SUMOReal hue = GeomHelper::naviDegree(getAngle());
             GLHelper::setColor(RGBColor::fromHSV(hue, 1., 1.));
             return true;
         }
@@ -326,17 +411,14 @@ GUIPerson::getEdgePos() const {
 Position
 GUIPerson::getPosition() const {
     AbstractMutex::ScopedLocker locker(myLock);
-    if (getCurrentStageType() == DRIVING && !isWaiting4Vehicle()) {
-        return myPositionInVehicle;
-    }
     return MSPerson::getPosition();
 }
 
 
 SUMOReal
-GUIPerson::getAngle() const {
+GUIPerson::getNaviDegree() const {
     AbstractMutex::ScopedLocker locker(myLock);
-    return MSPerson::getAngle();
+    return GeomHelper::naviDegree(MSPerson::getAngle());
 }
 
 
@@ -357,7 +439,7 @@ GUIPerson::getSpeed() const {
 void
 GUIPerson::drawAction_drawAsTriangle(const GUIVisualizationSettings& /* s */) const {
     // draw triangle pointing forward
-    glRotated(getAngle(), 0, 0, 1);
+    glRotated(RAD2DEG(getAngle() + PI / 2.), 0, 0, 1);
     glScaled(getVehicleType().getLength(), getVehicleType().getWidth(), 1);
     glBegin(GL_TRIANGLES);
     glVertex2d(0., 0.);
@@ -379,7 +461,7 @@ GUIPerson::drawAction_drawAsTriangle(const GUIVisualizationSettings& /* s */) co
 void
 GUIPerson::drawAction_drawAsPoly(const GUIVisualizationSettings& /* s */) const {
     // draw pedestrian shape
-    glRotated(getAngle(), 0, 0, 1);
+    glRotated(GeomHelper::naviDegree(getAngle()) - 180, 0, 0, -1);
     glScaled(getVehicleType().getLength(), getVehicleType().getWidth(), 1);
     RGBColor lighter = GLHelper::getColor().changedBrightness(51);
     glTranslated(0, 0, .045);
@@ -407,7 +489,7 @@ GUIPerson::drawAction_drawAsImage(const GUIVisualizationSettings& s) const {
     const std::string& file = getVehicleType().getImgFile();
     if (file != "") {
         if (getVehicleType().getGuiShape() == SVS_PEDESTRIAN) {
-            glRotated(getAngle(), 0, 0, 1);
+            glRotated(RAD2DEG(getAngle() + PI / 2.), 0, 0, 1);
         }
         int textureID = GUITexturesHelper::getTextureID(file);
         if (textureID > 0) {
