@@ -5,12 +5,12 @@
 /// @author  Sascha Krieg
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: GUIPerson.h 18274 2015-04-22 15:00:44Z behrisch $
+/// @version $Id: GUIPerson.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // A MSVehicle extended by some values for usage within the gui
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -107,7 +107,6 @@ public:
     void drawGL(const GUIVisualizationSettings& s) const;
 
 
-
     /** @brief Draws additionally triggered visualisations
      * @param[in] parent The view
      * @param[in] s The settings for the current view (may influence drawing)
@@ -132,7 +131,7 @@ public:
     Position getPosition() const;
 
     /// @brief return the current angle of the person
-    SUMOReal getAngle() const;
+    SUMOReal getNaviDegree() const;
 
     /// @brief the time this person spent waiting in seconds
     SUMOReal getWaitingSeconds() const;
@@ -164,10 +163,18 @@ public:
         /// @brief Destructor
         ~GUIPersonPopupMenu();
 
+        /// @brief Called if the current route of the person shall be shown
+        long onCmdShowCurrentRoute(FXObject*, FXSelector, void*);
+        /// @brief Called if the current route of the person shall be hidden
+        long onCmdHideCurrentRoute(FXObject*, FXSelector, void*);
         /// @brief Called if the walkingarea path of the person shall be shown
         long onCmdShowWalkingareaPath(FXObject*, FXSelector, void*);
         /// @brief Called if the walkingarea path of the person shall be hidden
         long onCmdHideWalkingareaPath(FXObject*, FXSelector, void*);
+        /// @brief Called if the person shall be tracked
+        long onCmdStartTrack(FXObject*, FXSelector, void*);
+        /// @brief Called if the person shall not be tracked any longer
+        long onCmdStopTrack(FXObject*, FXSelector, void*);
 
 
     protected:
