@@ -6,12 +6,12 @@
 /// @author  Laura Bieker
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: ROMAEdge.cpp 18095 2015-03-17 09:39:00Z behrisch $
+/// @version $Id: ROMAEdge.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // A basic edge for routing applications
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -43,7 +43,7 @@
 // method definitions
 // ===========================================================================
 ROMAEdge::ROMAEdge(const std::string& id, RONode* from, RONode* to, unsigned int index, const int priority)
-    : ROEdge(id, from, to, index, priority), myHelpFlow(0.) {
+    : ROEdge(id, from, to, index, priority) {
 }
 
 
