@@ -5,12 +5,12 @@
 /// @author  Laura Bieker
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: ROMAFrame.cpp 18565 2015-07-14 10:06:34Z behrisch $
+/// @version $Id: ROMAFrame.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // Sets and checks options for ma-routing
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -81,10 +81,7 @@ ROMAFrame::addImportOptions() {
     // register import options
     oc.doRegister("output-file", 'o', new Option_FileName());
     oc.addSynonyme("output-file", "output");
-    oc.addDescription("output-file", "Output", "Write route distributions to FILE");
-
-    oc.doRegister("flow-output", new Option_FileName());
-    oc.addDescription("flow-output", "Output", "Writes flow definitions into FILE");
+    oc.addDescription("output-file", "Output", "Write flow definitions with route distributions to FILE");
 
     oc.doRegister("ignore-vehicle-type", new Option_Bool(false));
     oc.addSynonyme("ignore-vehicle-type", "no-vtype", true);
@@ -132,6 +129,9 @@ ROMAFrame::addImportOptions() {
     oc.addSynonyme("weight-attribute", "measure", true);
     oc.addDescription("weight-attribute", "Input", "Name of the xml attribute which gives the edge weight");
 
+    oc.doRegister("weight-adaption", new Option_Float(0.));
+    oc.addDescription("weight-adaption", "Input", "The travel time influence of prior intervals");
+
     // register the time settings
     oc.doRegister("begin", 'b', new Option_String("0", "TIME"));
     oc.addDescription("begin", "Time", "Defines the begin time; Previous trips will be discarded");
@@ -162,6 +162,12 @@ ROMAFrame::addImportOptions() {
     oc.doRegister("routing-algorithm", new Option_String("dijkstra"));
     oc.addDescription("routing-algorithm", "Processing", "Select among routing algorithms ['dijkstra', 'astar', 'CH', 'CHWrapper']");
 
+    oc.doRegister("bulk-routing.vtypes", new Option_Bool(false));
+    oc.addDescription("bulk-routing.vtypes", "Processing", "Aggregate routing queries with the same origin for different vehicle types");
+
+    oc.doRegister("routing-threads", new Option_Integer(0));
+    oc.addDescription("routing-threads", "Processing", "The number of parallel execution threads used for routing");
+
     oc.doRegister("weight-period", new Option_String("3600", "TIME"));
     oc.addDescription("weight-period", "Processing", "Aggregation period for the given weight files; triggers rebuilding of Contraction Hierarchy");
 
@@ -212,8 +218,8 @@ ROMAFrame::addAssignmentOptions() {
     oc.doRegister("timeline.day-in-hours", new Option_Bool(false));
     oc.addDescription("timeline.day-in-hours", "Processing", "Uses STR as a 24h-timeline definition");
 
-    oc.doRegister("timesplit", new Option_Bool(false));
-    oc.addDescription("timesplit", "Processing", "Do separate assignments for each time slot");
+    oc.doRegister("additive-traffic", new Option_Bool(false));
+    oc.addDescription("additive-traffic", "Processing", "Keep traffic flows of all time slots in the net");
 
     // register macroscopic SUE-settings
     oc.doRegister("assignment-method", new Option_String("incremental"));
@@ -287,7 +293,7 @@ ROMAFrame::checkOptions() {
         WRITE_ERROR("invalid assignment method");
         return false;
     }
-    if (oc.getString("route-choice-method") != "gawron" && oc.getString("route-choice-method") != "logit" && oc.getString("route-choice-methods") != "lohse") {
+    if (oc.getString("route-choice-method") != "gawron" && oc.getString("route-choice-method") != "logit" && oc.getString("route-choice-method") != "lohse") {
         WRITE_ERROR("invalid route choice method");
         return false;
     }
