@@ -3,12 +3,12 @@
 /// @author  Melanie Weber
 /// @author  Andreas Kendziorra
 /// @date    Thu, 12 Jun 2014
-/// @version $Id: MSContainer.cpp 18274 2015-04-22 15:00:44Z behrisch $
+/// @version $Id: MSContainer.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // The class for modelling container-movements
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -34,6 +34,7 @@
 #include <utils/iodevices/OutputDevice.h>
 #include <utils/options/OptionsCont.h>
 #include <utils/common/ToString.h>
+#include <utils/geom/GeomHelper.h>
 #include "MSNet.h"
 #include "MSEdge.h"
 #include "MSLane.h"
@@ -60,30 +61,34 @@ const SUMOReal MSContainer::ROADSIDE_OFFSET(3);
  * MSContainer::MSContainerStage_Driving - methods
  * ----------------------------------------------------------------------- */
 MSContainer::MSContainerStage_Driving::MSContainerStage_Driving(const MSEdge& destination,
-        MSStoppingPlace* toCS, const std::vector<std::string>& lines)
-    : MSTransportable::Stage(destination, DRIVING), myLines(lines.begin(), lines.end()),
-      myVehicle(0), myDestinationContainerStop(toCS) {}
+        MSStoppingPlace* toStop, const SUMOReal arrivalPos, const std::vector<std::string>& lines)
+    : MSTransportable::Stage(destination, toStop, arrivalPos, DRIVING), myLines(lines.begin(), lines.end()),
+      myVehicle(0) {}
 
 
 MSContainer::MSContainerStage_Driving::~MSContainerStage_Driving() {}
 
 void
-MSContainer::MSContainerStage_Driving::proceed(MSNet* net, MSTransportable* container, SUMOTime now,
-        MSEdge* previousEdge, const SUMOReal at) {
-    myWaitingEdge = previousEdge;
-    myWaitingPos = at;
+MSContainer::MSContainerStage_Driving::proceed(MSNet* net, MSTransportable* container, SUMOTime now, Stage* previous) {
+    if (previous->getDestinationStop() != 0) {
+        // the arrival stop may have an access point
+        myWaitingEdge = &previous->getDestinationStop()->getLane().getEdge();
+    } else {
+        myWaitingEdge = previous->getEdge();
+    }
+    myWaitingPos = previous->getEdgePos(now);
     myWaitingSince = now;
-    SUMOVehicle* availableVehicle = net->getVehicleControl().getWaitingVehicle(previousEdge, myLines, myWaitingPos, container->getID());
+    SUMOVehicle* availableVehicle = net->getVehicleControl().getWaitingVehicle(myWaitingEdge, myLines, myWaitingPos, container->getID());
     if (availableVehicle != 0 && availableVehicle->getParameter().departProcedure == DEPART_CONTAINER_TRIGGERED) {
         myVehicle = availableVehicle;
-        previousEdge->removeContainer(container);
+        myWaitingEdge->removeContainer(container);
         myVehicle->addContainer(container);
         net->getInsertionControl().add(myVehicle);
-        net->getVehicleControl().removeWaiting(previousEdge, myVehicle);
+        net->getVehicleControl().removeWaiting(myWaitingEdge, myVehicle);
         net->getVehicleControl().unregisterOneWaitingForContainer();
     } else {
-        net->getContainerControl().addWaiting(previousEdge, container);
-        previousEdge->addContainer(container);
+        net->getContainerControl().addWaiting(myWaitingEdge, container);
+        myWaitingEdge->addContainer(container);
     }
 }
 
@@ -114,8 +119,7 @@ MSContainer::MSContainerStage_Driving::getEdgePos(SUMOTime /* now */) const {
 Position
 MSContainer::MSContainerStage_Driving::getPosition(SUMOTime /* now */) const {
     if (myVehicle != 0) {
-        /// @bug this fails while vehicle is driving across a junction
-        return myVehicle->getEdge()->getLanes()[0]->getShape().positionAtOffset(myVehicle->getPositionOnLane());
+        return myVehicle->getPosition();
     }
     return getEdgePosition(myWaitingEdge, myWaitingPos, ROADSIDE_OFFSET);
 }
@@ -130,7 +134,7 @@ MSContainer::MSContainerStage_Driving::getAngle(SUMOTime /* now */) const {
             return 0;
         }
     }
-    return getEdgeAngle(myWaitingEdge, myWaitingPos) + 90;
+    return getEdgeAngle(myWaitingEdge, myWaitingPos) + M_PI / 2.;
 }
 
 bool
@@ -191,13 +195,11 @@ MSContainer::MSContainerStage_Driving::endEventOutput(const MSTransportable& con
  * ----------------------------------------------------------------------- */
 MSContainer::MSContainerStage_Waiting::MSContainerStage_Waiting(const MSEdge& destination,
         SUMOTime duration, SUMOTime until, SUMOReal pos, const std::string& actType) :
-    MSTransportable::Stage(destination, WAITING),
+    MSTransportable::Stage(destination, 0, SUMOVehicleParameter::interpretEdgePos(
+                               pos, destination.getLength(), SUMO_ATTR_DEPARTPOS, "container stopping at " + destination.getID()), WAITING),
     myWaitingDuration(duration),
     myWaitingUntil(until),
-    myActType(actType),
-    myStartPos(pos) {
-    myStartPos = SUMOVehicleParameter::interpretEdgePos(
-                     myStartPos, myDestination.getLength(), SUMO_ATTR_DEPARTPOS, "container stopping at " + myDestination.getID());
+    myActType(actType) {
 }
 
 MSContainer::MSContainerStage_Waiting::~MSContainerStage_Waiting() {}
@@ -214,7 +216,7 @@ MSContainer::MSContainerStage_Waiting::getFromEdge() const {
 
 SUMOReal
 MSContainer::MSContainerStage_Waiting::getEdgePos(SUMOTime /* now */) const {
-    return myStartPos;
+    return myArrivalPos;
 }
 
 SUMOTime
@@ -224,12 +226,12 @@ MSContainer::MSContainerStage_Waiting::getUntil() const {
 
 Position
 MSContainer::MSContainerStage_Waiting::getPosition(SUMOTime /* now */) const {
-    return getEdgePosition(&myDestination, myStartPos, ROADSIDE_OFFSET);
+    return getEdgePosition(&myDestination, myArrivalPos, ROADSIDE_OFFSET);
 }
 
 SUMOReal
 MSContainer::MSContainerStage_Waiting::getAngle(SUMOTime /* now */) const {
-    return getEdgeAngle(&myDestination, myStartPos) - 180;
+    return getEdgeAngle(&myDestination, myArrivalPos) + M_PI;
 }
 
 SUMOTime
@@ -248,9 +250,8 @@ MSContainer::MSContainerStage_Waiting::getDepartContainerStop() const {
 }
 
 void
-MSContainer::MSContainerStage_Waiting::proceed(MSNet* net, MSTransportable* container, SUMOTime now,
-        MSEdge* previousEdge, const SUMOReal /* at */) {
-    previousEdge->addContainer(container);
+MSContainer::MSContainerStage_Waiting::proceed(MSNet* net, MSTransportable* container, SUMOTime now, Stage* previous) {
+    previous->getEdge()->addContainer(container);
     myWaitingStart = now;
     const SUMOTime until = MAX3(now, now + myWaitingDuration, myWaitingUntil);
     net->getContainerControl().setWaitEnd(until, container);
@@ -289,29 +290,25 @@ MSContainer::MSContainerStage_Waiting::endEventOutput(const MSTransportable& con
  * MSContainer::MSContainerStage_Tranship - methods
  * ----------------------------------------------------------------------- */
 MSContainer::MSContainerStage_Tranship::MSContainerStage_Tranship(const std::vector<const MSEdge*>& route,
-        MSStoppingPlace* toCS,
+        MSStoppingPlace* toStop,
         SUMOReal speed,
         SUMOReal departPos, SUMOReal arrivalPos) :
-    MSTransportable::Stage(*route.back(), MOVING_WITHOUT_VEHICLE), myRoute(route),
-    myDestinationContainerStop(toCS),
+    MSTransportable::Stage(*route.back(), toStop, SUMOVehicleParameter::interpretEdgePos(
+                               arrivalPos, route.back()->getLength(), SUMO_ATTR_ARRIVALPOS, "container getting transhipped to " + route.back()->getID()), MOVING_WITHOUT_VEHICLE), myRoute(route),
     mySpeed(speed), myContainerState(0), myCurrentInternalEdge(0) {
     myDepartPos = SUMOVehicleParameter::interpretEdgePos(
                       departPos, myRoute.front()->getLength(), SUMO_ATTR_DEPARTPOS, "container getting transhipped from " + myRoute.front()->getID());
-    myArrivalPos = SUMOVehicleParameter::interpretEdgePos(
-                       arrivalPos, myRoute.back()->getLength(), SUMO_ATTR_ARRIVALPOS, "container getting transhipped to " + myRoute.back()->getID());
 }
 
 MSContainer::MSContainerStage_Tranship::~MSContainerStage_Tranship() {
 }
 
 void
-MSContainer::MSContainerStage_Tranship::proceed(MSNet* /* net */, MSTransportable* container, SUMOTime now, MSEdge* previousEdge, const SUMOReal at) {
-    previousEdge->removeContainer(container);
+MSContainer::MSContainerStage_Tranship::proceed(MSNet* /* net */, MSTransportable* container, SUMOTime now, Stage* previous) {
+    previous->getEdge()->removeContainer(container);
     myRouteStep = myRoute.end() - 1;   //define that the container is already on its destination edge
     MSNet::getInstance()->getContainerControl().setTranship(container);
-    if (at >= 0) {
-        myDepartPos = at;
-    }
+    myDepartPos = previous->getEdgePos(now);
     myContainerState = MSCModel_NonInteracting::getModel()->add(container, this, now);
     (*myRouteStep)->addContainer(container);
 }
@@ -397,8 +394,8 @@ MSContainer::MSContainerStage_Tranship::moveToNextEdge(MSTransportable* containe
     ((MSEdge*)getEdge())->removeContainer(container);
     if (myRouteStep == myRoute.end() - 1) {
         MSNet::getInstance()->getContainerControl().unsetTranship(container);
-        if (myDestinationContainerStop != 0) {
-            myDestinationContainerStop->addTransportable(container);    //jakob
+        if (myDestinationStop != 0) {
+            myDestinationStop->addTransportable(container);    //jakob
         }
         if (!container->proceed(MSNet::getInstance(), currentTime)) {
             MSNet::getInstance()->getContainerControl().erase(container);
@@ -428,15 +425,14 @@ MSContainer::~MSContainer() {
 
 bool
 MSContainer::proceed(MSNet* net, SUMOTime time) {
-    MSEdge* arrivedAt = (MSEdge*)(*myStep)->getEdge();
-    SUMOReal atPos = (*myStep)->getEdgePos(time);
-    (*myStep)->setArrived(time);
+    Stage* prior = *myStep;
+    prior->setArrived(time);
     myStep++;
     if (myStep != myPlan->end()) {
-        (*myStep)->proceed(net, this, time, arrivedAt, atPos);
+        (*myStep)->proceed(net, this, time, prior);
         return true;
     } else {
-        arrivedAt->removeContainer(this);
+        prior->getEdge()->removeContainer(this);
         return false;
     }
 }
