@@ -4,12 +4,12 @@
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    late summer 2003
-/// @version $Id: MSGlobals.cpp 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: MSGlobals.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // Some static variables for faster access
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2003-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2003-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -55,14 +55,13 @@ bool MSGlobals::gCheckRoutes;
 SUMOTime MSGlobals::gLaneChangeDuration;
 
 bool MSGlobals::gStateLoaded;
-#ifdef HAVE_INTERNAL
 bool MSGlobals::gUseMesoSim;
 bool MSGlobals::gMesoLimitedJunctionControl;
 bool MSGlobals::gMesoOvertaking;
+SUMOReal MSGlobals::gMesoTLSPenalty;
 MELoop* MSGlobals::gMesoNet;
-#else
-const bool MSGlobals::gUseMesoSim = false;
-#endif
 
+
+SUMOTime MSGlobals::gWaitingTimeMemory;
 /****************************************************************************/
 
