@@ -5,12 +5,12 @@
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    late summer 2003
-/// @version $Id: MSGlobals.h 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: MSGlobals.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // Some static variables for faster access
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2003-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2003-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -40,9 +40,7 @@
 // ===========================================================================
 // class declarations
 // ===========================================================================
-#ifdef HAVE_INTERNAL
 class MELoop;
-#endif
 
 
 // ===========================================================================
@@ -85,7 +83,6 @@ public:
     /// Information whether a state has been loaded
     static bool gStateLoaded;
 
-#ifdef HAVE_INTERNAL
     /** Information whether mesosim shall be used */
     static bool gUseMesoSim;
 
@@ -95,13 +92,14 @@ public:
     /** Information whether overtaking is enabled in the mesoscopic simulation */
     static bool gMesoOvertaking;
 
+    /** scaling factor for macroscopic time penalty when passing tls controlled intersection */
+    static SUMOReal gMesoTLSPenalty;
+
     /// mesoscopic simulation infrastructure
     static MELoop* gMesoNet;
-#else
-    /** Information whether mesosim shall be used = constant false */
-    const static bool gUseMesoSim;
 
-#endif
+    /// length of memory for waiting times (in millisecs)
+    static SUMOTime gWaitingTimeMemory;
 
 };
 
