@@ -6,12 +6,12 @@
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    Mon, 12 Mar 2001
-/// @version $Id: MSInsertionControl.cpp 18473 2015-06-03 21:35:22Z namdre $
+/// @version $Id: MSInsertionControl.cpp 20482 2016-04-18 20:49:42Z behrisch $
 ///
 // Inserts vehicles into the network when their departure time is reached
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -54,9 +54,13 @@
 // ===========================================================================
 MSInsertionControl::MSInsertionControl(MSVehicleControl& vc,
                                        SUMOTime maxDepartDelay,
-                                       bool checkEdgesOnce)
-    : myVehicleControl(vc), myMaxDepartDelay(maxDepartDelay),
-      myCheckEdgesOnce(checkEdgesOnce) {}
+                                       bool checkEdgesOnce,
+                                       int maxVehicleNumber) :
+    myVehicleControl(vc),
+    myMaxDepartDelay(maxDepartDelay),
+    myCheckEdgesOnce(checkEdgesOnce),
+    myMaxVehicleNumber(maxVehicleNumber) {
+}
 
 
 MSInsertionControl::~MSInsertionControl() {
@@ -142,14 +146,13 @@ MSInsertionControl::tryInsert(SUMOTime time, SUMOVehicle* veh,
     assert(veh->getParameter().depart < time + DELTA_T);
     const MSEdge& edge = *veh->getEdge();
     if (veh->isOnRoad()) {
-        // may have been inserted forcefully already
-        veh->onDepart();
         return 1;
     }
-    if ((!myCheckEdgesOnce || edge.getLastFailedInsertionTime() != time) && edge.insertVehicle(*veh, time)) {
+    if ((myMaxVehicleNumber < 0 || (int)MSNet::getInstance()->getVehicleControl().getRunningVehicleNo() < myMaxVehicleNumber)
+            && (!myCheckEdgesOnce || edge.getLastFailedInsertionTime() != time)
+            && edge.insertVehicle(*veh, time)) {
         // Successful insertion
         checkFlowWait(veh);
-        veh->onDepart();
         return 1;
     }
     if (myMaxDepartDelay >= 0 && time - veh->getParameter().depart > myMaxDepartDelay) {
@@ -261,6 +264,7 @@ MSInsertionControl::determineCandidates(SUMOTime time) {
             } else {
                 // strange: another vehicle with the same id already exists
                 if (MSGlobals::gStateLoaded) {
+                    vehControl.discountStateLoaded();
                     break;
                 }
                 throw ProcessError("Another vehicle with the id '" + newPars->id + "' exists.");
