@@ -5,12 +5,12 @@
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    Mon, 12 Mar 2001
-/// @version $Id: MSInsertionControl.h 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: MSInsertionControl.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // Inserts vehicles into the network when their departure time is reached
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -71,8 +71,9 @@ public:
      * @param[in] vc The assigned vehicle control (needed for vehicle re-insertion and deletion)
      * @param[in] maxDepartDelay Vehicles waiting for insertion longer than this time are deleted (-1: no deletion)
      * @param[in] checkEdgesOnce Whether an edge on which a vehicle could not depart should be ignored in the same step
+     * @param[in] maxVehicleNumber The maximum number of vehicles that should not be exceeded
      */
-    MSInsertionControl(MSVehicleControl& vc, SUMOTime maxDepartDelay, bool checkEdgesOnce);
+    MSInsertionControl(MSVehicleControl& vc, SUMOTime maxDepartDelay, bool checkEdgesOnce, int maxVehicleNumber);
 
 
     /// @brief Destructor.
@@ -110,8 +111,9 @@ public:
     /** @brief Adds parameter for a vehicle flow for departure
      *
      * @param[in] flow The flow to add for later insertion
+     * @return whether it could be added (no other flow with the same id was present)
      */
-    void add(SUMOVehicleParameter* pars);
+    bool add(SUMOVehicleParameter* const pars);
 
 
     /** @brief Returns the number of waiting vehicles
@@ -213,12 +215,18 @@ private:
     /// @brief Container for periodical vehicle parameters
     std::vector<Flow> myFlows;
 
+    /// @brief Cache for periodical vehicle ids for quicker checking
+    std::set<std::string> myFlowIDs;
+
     /// @brief The maximum waiting time; vehicles waiting longer are deleted (-1: no deletion)
     SUMOTime myMaxDepartDelay;
 
     /// @brief Whether an edge on which a vehicle could not depart should be ignored in the same step
     bool myCheckEdgesOnce;
 
+    /// @brief Storage for maximum vehicle number
+    int myMaxVehicleNumber;
+
 
 private:
     /// @brief Invalidated copy constructor.
