@@ -5,12 +5,12 @@
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    Wed, 12 Dez 2001
-/// @version $Id: MSInternalJunction.cpp 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: MSInternalJunction.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // junction.
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -32,7 +32,9 @@
 #endif
 
 #include "MSInternalJunction.h"
+#include "MSRightOfWayJunction.h"
 #include "MSLane.h"
+#include "MSEdge.h"
 #include "MSJunctionLogic.h"
 #include <algorithm>
 #include <cassert>
@@ -67,14 +69,29 @@ MSInternalJunction::postloadInit() {
     }
     // the first lane in the list of incoming lanes is special. It defines the
     // link that needs to do all the checking for this internal junction
-    assert(myIncomingLanes[0]->getLinkCont().size() == 1);
-    MSLink* thisLink = myIncomingLanes[0]->getLinkCont()[0];
+    const MSLane* specialLane = myIncomingLanes[0];
+    assert(specialLane->getLinkCont().size() == 1);
+    MSLink* thisLink = specialLane->getLinkCont()[0];
+    const MSRightOfWayJunction* parent = dynamic_cast<const MSRightOfWayJunction*>(specialLane->getEdge().getToJunction());
+    if (parent == 0) {
+        // parent has type traffic_light_unregulated
+        return;
+    }
+    const int ownLinkIndex = specialLane->getIncomingLanes()[0].viaLink->getIndex();
+    const MSLogicJunction::LinkBits& response = parent->getLogic()->getResponseFor(ownLinkIndex);
     // inform links where they have to report approaching vehicles to
     unsigned int requestPos = 0;
     for (std::vector<MSLane*>::iterator i = myInternalLanes.begin(); i != myInternalLanes.end(); ++i) {
         const MSLinkCont& lc = (*i)->getLinkCont();
         for (MSLinkCont::const_iterator q = lc.begin(); q != lc.end(); ++q) {
             if ((*q)->getViaLane() != 0) {
+                const int foeIndex = (*i)->getIncomingLanes()[0].viaLink->getIndex();
+                if (response.test(foeIndex)) {
+                    // only respect vehicles before internal junctions if they
+                    // have priority (see the analogous foeLinks.test() when
+                    // initializing myLinkFoeInternalLanes in MSRightOfWayJunction
+                    myInternalLaneFoes.push_back(*i);
+                }
                 myInternalLaneFoes.push_back((*q)->getViaLane());
             } else {
                 myInternalLaneFoes.push_back(*i);
