@@ -9,12 +9,12 @@
 /// @author  Christoph Sommer
 /// @author  Mario Krumnow
 /// @date    Mon, 05 Mar 2001
-/// @version $Id: MSLane.cpp 18756 2015-08-31 19:16:33Z behrisch $
+/// @version $Id: MSLane.cpp 20482 2016-04-18 20:49:42Z behrisch $
 ///
 // Representation of a lane in the micro simulation
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -67,7 +67,6 @@
 #include <utils/common/ToString.h>
 #include <utils/options/OptionsCont.h>
 #include <utils/emissions/HelpersHarmonoise.h>
-#include <utils/geom/Line.h>
 #include <utils/geom/GeomHelper.h>
 
 #ifdef CHECK_MEMORY_LEAKS
@@ -86,14 +85,14 @@ MSLane::DictType MSLane::myDict;
 // ===========================================================================
 MSLane::MSLane(const std::string& id, SUMOReal maxSpeed, SUMOReal length, MSEdge* const edge,
                unsigned int numericalID, const PositionVector& shape, SUMOReal width,
-               SVCPermissions permissions) :
+               SVCPermissions permissions, int index) :
     Named(id),
-    myShape(shape), myNumericalID(numericalID),
+    myNumericalID(numericalID), myShape(shape), myIndex(index),
     myVehicles(), myLength(length), myWidth(width), myEdge(edge), myMaxSpeed(maxSpeed),
     myPermissions(permissions),
     myLogicalPredecessorLane(0),
     myBruttoVehicleLengthSum(0), myNettoVehicleLengthSum(0), myInlappingVehicleEnd(10000), myInlappingVehicle(0),
-    myLengthGeometryFactor(myShape.length() / myLength) {
+    myLengthGeometryFactor(MAX2(POSITION_EPS, myShape.length()) / myLength) { // factor should not be 0
     myRestrictions = MSNet::getInstance()->getRestrictions(edge->getEdgeType());
 }
 
@@ -144,130 +143,17 @@ MSLane::incorporateVehicle(MSVehicle* veh, SUMOReal pos, SUMOReal speed, const M
 
 
 bool
-MSLane::pWagGenericInsertion(MSVehicle& veh, SUMOReal mspeed, SUMOReal maxPos, SUMOReal minPos) {
-    veh.setTentativeLaneAndPosition(this, maxPos);
-    veh.updateBestLanes(false, this);
-    SUMOReal xIn = maxPos;
-    SUMOReal vIn = mspeed;
-    SUMOReal leaderDecel;
-    if (myVehicles.size() != 0) {
-        MSVehicle* leader = myVehicles.front();
-        xIn = leader->getPositionOnLane() - leader->getVehicleType().getLength() - veh.getVehicleType().getMinGap();
-        vIn = leader->getSpeed();
-        leaderDecel = leader->getCarFollowModel().getMaxDecel();
-    } else {
-        SUMOReal brakeGap = veh.getCarFollowModel().brakeGap(mspeed);
-        std::pair<MSVehicle* const, SUMOReal> leader = getLeaderOnConsecutive(brakeGap, 0, mspeed, veh, veh.getBestLanesContinuation(this));
-        if (leader.first != 0) {
-            xIn = getLength() + leader.second;
-            vIn = leader.first->getSpeed();
-            leaderDecel = leader.first->getCarFollowModel().getMaxDecel();
-        } else {
-            incorporateVehicle(&veh, maxPos, mspeed, myVehicles.end());
-            return true;
-        }
-    }
-    const SUMOReal vHlp = 0.5 * (vIn + mspeed);
-    SUMOReal x2 = xIn;// have seen leader length already - skCar::lCar;
-    SUMOReal x1 = x2 - 100.0;
-    SUMOReal x = 0;
-    for (int i = 0; i <= 10; i++) {
-        x = 0.5 * (x1 + x2);
-        veh.setTentativeLaneAndPosition(this, x);
-        SUMOReal vSafe = veh.getCarFollowModel().insertionFollowSpeed(&veh, vHlp, xIn - x, vIn, leaderDecel);
-        if (vSafe < vHlp) {
-            x2 = x;
-        } else {
-            x1 = x;
-        }
-    }
-    if (x < minPos) {
-        return false;
-    } else if (x > maxPos) {
-        x = maxPos;
-    }
-    incorporateVehicle(&veh, x, vHlp, myVehicles.begin());
-    return true;
-}
-
-
-bool
-MSLane::pWagSimpleInsertion(MSVehicle& veh, SUMOReal mspeed, SUMOReal maxPos, SUMOReal minPos) {
-    veh.setTentativeLaneAndPosition(this, maxPos);
-    veh.updateBestLanes(false, this);
-    SUMOReal xIn = maxPos;
-    SUMOReal vIn = mspeed;
-    if (myVehicles.size() != 0) {
-        MSVehicle* leader = myVehicles.front();
-        xIn = leader->getPositionOnLane() - leader->getVehicleType().getLength() - veh.getVehicleType().getMinGap();
-        vIn = leader->getSpeed();
-    } else {
-        SUMOReal brakeGap = veh.getCarFollowModel().brakeGap(mspeed);
-        std::pair<MSVehicle* const, SUMOReal> leader = getLeaderOnConsecutive(brakeGap, 0, mspeed, veh, veh.getBestLanesContinuation(this));
-        if (leader.first != 0) {
-            xIn = getLength() + leader.second;
-            vIn = leader.first->getSpeed();
-        } else {
-            incorporateVehicle(&veh, maxPos, mspeed, myVehicles.end());
-            return true;
-        }
-    }
-    const SUMOReal vHlp = 0.5 * (mspeed + vIn);
-    xIn = xIn - vHlp * veh.getCarFollowModel().getHeadwayTime() - veh.getVehicleType().getMinGap();
-    if (xIn < minPos) {
-        return false;
-    } else if (xIn > maxPos) {
-        xIn = maxPos;
-    }
-    incorporateVehicle(&veh, xIn, vHlp, myVehicles.begin());
-    return true;
-}
-
-
-bool
-MSLane::maxSpeedGapInsertion(MSVehicle& veh, SUMOReal mspeed) {
+MSLane::lastInsertion(MSVehicle& veh, SUMOReal mspeed) {
+    const bool adaptSpeed = false;
     if (myVehicles.size() == 0) {
-        return isInsertionSuccess(&veh, mspeed, myLength / 2, true, MSMoveReminder::NOTIFICATION_DEPARTED);
+        return isInsertionSuccess(&veh, mspeed, basePos(veh), adaptSpeed, MSMoveReminder::NOTIFICATION_DEPARTED);
     }
-    // go through the lane, look for free positions (starting after the last vehicle)
-    MSLane::VehCont::iterator predIt = myVehicles.begin();
-    SUMOReal maxSpeed = 0;
-    SUMOReal maxPos = 0;
-    MSLane::VehCont::iterator maxIt = myVehicles.begin();
-    while (predIt != myVehicles.end()) {
-        // get leader (may be zero) and follower
-        const MSVehicle* leader = predIt != myVehicles.end() - 1 ? *(predIt + 1) : getPartialOccupator();
-        const MSVehicle* follower = *predIt;
-        SUMOReal leaderRearPos = getLength();
-        SUMOReal leaderSpeed = mspeed;
-        if (leader != 0) {
-            leaderRearPos = leader->getPositionOnLane() - leader->getVehicleType().getLength();
-            if (leader == getPartialOccupator()) {
-                leaderRearPos = getPartialOccupatorEnd();
-            }
-            leaderSpeed = leader->getSpeed();
-        }
-        const SUMOReal nettoGap = leaderRearPos - follower->getPositionOnLane() - veh.getVehicleType().getLengthWithGap();
-        if (nettoGap > 0) {
-            const SUMOReal tau = veh.getCarFollowModel().getHeadwayTime();
-            const SUMOReal tauDecel = tau * veh.getCarFollowModel().getMaxDecel();
-            const SUMOReal fSpeed = follower->getSpeed();
-            const SUMOReal lhs = nettoGap / tau + tauDecel - fSpeed - fSpeed * fSpeed / (2 * tauDecel) + leaderSpeed * leaderSpeed / (2 * tauDecel);
-            if (lhs >= sqrt(tauDecel * tauDecel + leaderSpeed * leaderSpeed)) {
-                const SUMOReal frontGap = (lhs * lhs - tauDecel * tauDecel - leaderSpeed * leaderSpeed) / (2 * veh.getCarFollowModel().getMaxDecel());
-                const SUMOReal currentMaxSpeed = lhs - tauDecel;
-                if (MIN2(currentMaxSpeed, mspeed) > maxSpeed) {
-                    maxSpeed = currentMaxSpeed;
-                    maxPos = MIN2(leaderRearPos + frontGap, myLength);
-                    maxIt = predIt + 1;
-                }
-            }
-        }
-        ++predIt;
-    }
-    if (maxSpeed > 0) {
-        incorporateVehicle(&veh, maxPos, maxSpeed, maxIt);
-        return true;
+    // try to insert behind the last vehicle
+    const MSVehicle* leader = myVehicles.front();
+    const SUMOReal leaderBack = leader->getPositionOnLane() - leader->getVehicleType().getLength();
+    const SUMOReal frontGapNeeded = veh.getCarFollowModel().getSecureGap(mspeed, leader->getSpeed(), leader->getCarFollowModel().getMaxDecel()) + veh.getVehicleType().getMinGap() + POSITION_EPS;
+    if (leaderBack >= frontGapNeeded) {
+        return isInsertionSuccess(&veh, mspeed, leaderBack - frontGapNeeded, adaptSpeed, MSMoveReminder::NOTIFICATION_DEPARTED);
     }
     return false;
 }
@@ -415,16 +301,12 @@ MSLane::insertVehicle(MSVehicle& veh) {
         break;
         case DEPART_POS_FREE:
             return freeInsertion(veh, speed);
-        case DEPART_POS_PWAG_SIMPLE:
-            return pWagSimpleInsertion(veh, speed, getLength(), 0.0);
-        case DEPART_POS_PWAG_GENERIC:
-            return pWagGenericInsertion(veh, speed, getLength(), 0.0);
-        case DEPART_POS_MAX_SPEED_GAP:
-            return maxSpeedGapInsertion(veh, speed);
+        case DEPART_POS_LAST:
+            return lastInsertion(veh, speed);
         case DEPART_POS_BASE:
         case DEPART_POS_DEFAULT:
         default:
-            pos = MIN2(static_cast<SUMOReal>(veh.getVehicleType().getLength() + POSITION_EPS), myLength);
+            pos = basePos(veh);
             break;
     }
     // try to insert
@@ -432,6 +314,11 @@ MSLane::insertVehicle(MSVehicle& veh) {
 }
 
 
+SUMOReal
+MSLane::basePos(const MSVehicle& veh) const {
+    return MIN2(veh.getVehicleType().getLength() + POSITION_EPS, myLength);
+}
+
 bool
 MSLane::checkFailure(MSVehicle* aVehicle, SUMOReal& speed, SUMOReal& dist, const SUMOReal nspeed, const bool patchSpeed, const std::string errorMsg) const {
     if (nspeed < speed) {
@@ -460,11 +347,6 @@ MSLane::isInsertionSuccess(MSVehicle* aVehicle,
                       aVehicle->getID() + "'. Inserting at lane end instead.");
         pos = myLength;
     }
-    std::string msg;
-    if (!MSGlobals::gCheckRoutes && aVehicle->getParameter().wasSet(VEHPARS_FORCE_REROUTE) && !aVehicle->hasValidRoute(msg)) {
-        MSNet::getInstance()->getInsertionControl().descheduleDeparture(aVehicle);
-        return false;
-    }
     aVehicle->setTentativeLaneAndPosition(this, pos);
     aVehicle->updateBestLanes(false, this);
     const MSCFModel& cfModel = aVehicle->getCarFollowModel();
@@ -651,7 +533,7 @@ MSLane::setPartialOccupation(MSVehicle* v, SUMOReal leftVehicleLength) {
     myInlappingVehicleEnd = myLength - leftVehicleLength;
     if (v->getLaneChangeModel().isChangingLanes()) {
         MSLane* shadowLane = v->getLaneChangeModel().getShadowLane(this);
-        if (shadowLane != 0) {
+        if (shadowLane != 0 && v->getLaneChangeModel().hasShadowVehicle()) {
             v->getLaneChangeModel().setShadowPartialOccupator(shadowLane);
             shadowLane->myInlappingVehicle = v;
             shadowLane->myInlappingVehicleEnd = myLength - leftVehicleLength;
@@ -757,7 +639,7 @@ MSLane::handleCollision(SUMOTime timestep, const std::string& stage, MSVehicle*
                 // synchroneous lane change maneuver
                 return false;
             }
-            WRITE_WARNING("Teleporting vehicle '" + collider->getID() + "'; collision with '"
+            WRITE_WARNING("Teleporting vehicle '" + collider->getID() + "'; collision with vehicle '"
                           + victim->getID() + "', lane='" + getID() + "', gap=" + toString(gap)
                           + ", time=" + time2string(MSNet::getInstance()->getCurrentTimeStep()) + " stage=" + stage + ".");
             MSNet::getInstance()->getVehicleControl().registerCollision();
@@ -827,7 +709,7 @@ MSLane::executeMovements(SUMOTime t, std::vector<MSLane*>& into) {
             // for any reasons the vehicle is beyond its lane...
             // this should never happen because it is handled in MSVehicle::executeMove
             assert(false);
-            WRITE_WARNING("Teleporting vehicle '" + veh->getID() + "'; beyond end of lane, targetLane='" + getID() + "', time=" +
+            WRITE_WARNING("Teleporting vehicle '" + veh->getID() + "'; beyond end of lane, target lane='" + getID() + "', time=" +
                           time2string(MSNet::getInstance()->getCurrentTimeStep()) + ".");
             MSNet::getInstance()->getVehicleControl().registerCollision();
             MSVehicleTransfer::getInstance()->add(t, veh);
@@ -1053,7 +935,7 @@ MSLane::succLinkSec(const SUMOVehicle& veh, unsigned int nRouteSuccs,
     }
     // the only case where this should happen is for a disconnected route (deliberately ignored)
 #ifdef _DEBUG
-    WRITE_WARNING("Could not find connection between '" + succLinkSource.getID() + "' and '" + conts[nRouteSuccs]->getID() +
+    WRITE_WARNING("Could not find connection between lane '" + succLinkSource.getID() + "' and lane '" + conts[nRouteSuccs]->getID() +
                   "' for vehicle '" + veh.getID() + "' time=" + time2string(MSNet::getInstance()->getCurrentTimeStep()) + ".");
 #endif
     return succLinkSource.myLinks.end();
@@ -1122,11 +1004,11 @@ MSLane::addIncomingLane(MSLane* lane, MSLink* viaLink) {
 
 
 void
-MSLane::addApproachingLane(MSLane* lane) {
+MSLane::addApproachingLane(MSLane* lane, bool warnMultiCon) {
     MSEdge* approachingEdge = &lane->getEdge();
     if (myApproachingLanes.find(approachingEdge) == myApproachingLanes.end()) {
         myApproachingLanes[approachingEdge] = std::vector<MSLane*>();
-    } else if (approachingEdge->getPurpose() != MSEdge::EDGEFUNCTION_INTERNAL) {
+    } else if (approachingEdge->getPurpose() != MSEdge::EDGEFUNCTION_INTERNAL && warnMultiCon) {
         // whenever a normal edge connects twice, there is a corresponding
         // internal edge wich connects twice, one warning is sufficient
         WRITE_WARNING("Lane '" + getID() + "' is approached multiple times from edge '" + approachingEdge->getID() + "'. This may cause collisions.");
@@ -1154,7 +1036,7 @@ MSLane::isApproachedFrom(MSEdge* const edge, MSLane* const lane) {
 
 class by_second_sorter {
 public:
-    inline int operator()(const std::pair<const MSVehicle* , SUMOReal>& p1, const std::pair<const MSVehicle* , SUMOReal>& p2) const {
+    inline int operator()(const std::pair<const MSVehicle*, SUMOReal>& p1, const std::pair<const MSVehicle*, SUMOReal>& p2) const {
         return p1.second < p2.second;
     }
 };
@@ -1646,6 +1528,18 @@ MSLane::getFuelConsumption() const {
 }
 
 
+SUMOReal
+MSLane::getElectricityConsumption() const {
+    SUMOReal ret = 0;
+    const MSLane::VehCont& vehs = getVehiclesSecure();
+    for (MSLane::VehCont::const_iterator i = vehs.begin(); i != vehs.end(); ++i) {
+        ret += (*i)->getElectricityConsumption();
+    }
+    releaseVehicles();
+    return ret;
+}
+
+
 SUMOReal
 MSLane::getHarmonoise_NoiseEmissions() const {
     SUMOReal ret = 0;
@@ -1676,8 +1570,8 @@ MSLane::vehicle_position_sorter::operator()(MSVehicle* v1, MSVehicle* v2) const
 
 MSLane::by_connections_to_sorter::by_connections_to_sorter(const MSEdge* const e) :
     myEdge(e),
-    myLaneDir(e->getLanes()[0]->getShape().getBegLine().atan2PositiveAngle())
-{ }
+    myLaneDir(e->getLanes()[0]->getShape().angleAt2D(0)) {
+}
 
 
 int
@@ -1686,11 +1580,11 @@ MSLane::by_connections_to_sorter::operator()(const MSEdge* const e1, const MSEdg
     const std::vector<MSLane*>* ae2 = e2->allowedLanes(*myEdge);
     SUMOReal s1 = 0;
     if (ae1 != 0 && ae1->size() != 0) {
-        s1 = (SUMOReal) ae1->size() + GeomHelper::getMinAngleDiff((*ae1)[0]->getShape().getBegLine().atan2PositiveAngle(), myLaneDir) / M_PI / 2.;
+        s1 = (SUMOReal) ae1->size() + fabs(GeomHelper::angleDiff((*ae1)[0]->getShape().angleAt2D(0), myLaneDir)) / M_PI / 2.;
     }
     SUMOReal s2 = 0;
     if (ae2 != 0 && ae2->size() != 0) {
-        s2 = (SUMOReal) ae2->size() + GeomHelper::getMinAngleDiff((*ae2)[0]->getShape().getBegLine().atan2PositiveAngle(), myLaneDir) / M_PI / 2.;
+        s2 = (SUMOReal) ae2->size() + fabs(GeomHelper::angleDiff((*ae2)[0]->getShape().angleAt2D(0), myLaneDir)) / M_PI / 2.;
     }
     return s1 < s2;
 }
@@ -1699,6 +1593,7 @@ MSLane::by_connections_to_sorter::operator()(const MSEdge* const e1, const MSEdg
 void
 MSLane::saveState(OutputDevice& out) {
     out.openTag(SUMO_TAG_LANE);
+    out.writeAttr("id", getID()); // using "id" instead of SUMO_ATTR_ID makes the value only show up in xml state
     out.openTag(SUMO_TAG_VIEWSETTINGS_VEHICLES);
     out.writeAttr(SUMO_ATTR_VALUE, myVehicles);
     out.closeTag();
