@@ -4,12 +4,12 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: MSLink.h 18319 2015-04-29 06:49:01Z namdre $
+/// @version $Id: MSLink.h 20482 2016-04-18 20:49:42Z behrisch $
 ///
 // A connnection between lanes
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -76,6 +76,9 @@ class OutputDevice;
 class MSLink {
 public:
 
+    // distance to link in m below which adaptation for zipper-merging should take place
+    static const SUMOReal ZIPPER_ADAPT_DIST;
+
     struct LinkLeader {
         LinkLeader(MSVehicle* _veh, SUMOReal _gap, SUMOReal _distToCrossing) :
             vehAndGap(std::make_pair(_veh, _gap)),
@@ -101,14 +104,17 @@ public:
                                       const bool _willPass,
                                       const SUMOTime _arrivalTimeBraking,
                                       const SUMOReal _arrivalSpeedBraking,
-                                      const SUMOTime _waitingTime
+                                      const SUMOTime _waitingTime,
+                                      const SUMOReal _dist
                                      ) :
             arrivalTime(_arrivalTime), leavingTime(_leavingTime),
             arrivalSpeed(_arrivalSpeed), leaveSpeed(_leaveSpeed),
             willPass(_willPass),
             arrivalTimeBraking(_arrivalTimeBraking),
             arrivalSpeedBraking(_arrivalSpeedBraking),
-            waitingTime(_waitingTime) {}
+            waitingTime(_waitingTime),
+            dist(_dist) {
+        }
 
         /// @brief The time the vehicle's front arrives at the link
         const SUMOTime arrivalTime;
@@ -126,6 +132,8 @@ public:
         const SUMOReal arrivalSpeedBraking;
         /// @brief The waiting duration at the current link
         const SUMOTime waitingTime;
+        /// @brief The distance up to the current link
+        const SUMOReal dist;
 
     private:
         /// invalidated assignment operator
@@ -180,7 +188,7 @@ public:
     void setApproaching(const SUMOVehicle* approaching, const SUMOTime arrivalTime,
                         const SUMOReal arrivalSpeed, const SUMOReal leaveSpeed, const bool setRequest,
                         const SUMOTime arrivalTimeBraking, const SUMOReal arrivalSpeedBraking,
-                        const SUMOTime waitingTime);
+                        const SUMOTime waitingTime, SUMOReal dist);
 
     /// @brief removes the vehicle from myApproachingVehicles
     void removeApproaching(const SUMOVehicle* veh);
@@ -192,6 +200,11 @@ public:
      * @note used for visualisation of link items */
     ApproachingVehicleInformation getApproaching(const SUMOVehicle* veh) const;
 
+    /// @brief return all approaching vehicles
+    const std::map<const SUMOVehicle*, ApproachingVehicleInformation>& getApproaching() const {
+        return myApproachingVehicles;
+    }
+
     /** @brief Returns the information whether the link may be passed
      *
      * Valid after the junctions have set their reponds
@@ -309,6 +322,9 @@ public:
         return myState == LINKSTATE_TL_RED || myState == LINKSTATE_TL_REDYELLOW;
     }
 
+    inline bool isTLSControlled() const {
+        return myLastStateChange != SUMOTime_MIN;
+    }
 
     /** @brief Returns the length of this link
      *
@@ -359,9 +375,16 @@ public:
     LinkLeaders getLeaderInfo(SUMOReal dist, SUMOReal minGap, std::vector<const MSPerson*>* collectBlockers = 0) const;
 #endif
 
+    /// @brief return the speed at which ego vehicle must approach the zipper link
+    SUMOReal getZipperSpeed(const MSVehicle* ego, const SUMOReal dist, SUMOReal vSafe,
+                            SUMOTime arrivalTime,
+                            std::vector<const SUMOVehicle*>* collectFoes) const;
+
     /// @brief return the via lane if it exists and the lane otherwise
     MSLane* getViaLaneOrLane() const;
 
+    /// @brief return myInternalLaneBefore (always 0 when compiled without internal lanes)
+    const MSLane* getInternalLaneBefore() const;
 
     /// @brief return the expected time at which the given vehicle will clear the link
     SUMOTime getLeaveTime(const SUMOTime arrivalTime, const SUMOReal arrivalSpeed, const SUMOReal leaveSpeed, const SUMOReal vehicleLength) const;
@@ -369,9 +392,29 @@ public:
     /// @brief write information about all approaching vehicles to the given output device
     void writeApproaching(OutputDevice& od, const std::string fromLaneID) const;
 
-    /// @brief return the junction to which this link belongs
-    const MSJunction* getJunction() const {
-        return myJunction;
+    /// @brief erase vehicle from myLinkLeaders of this links junction
+    void passedJunction(const MSVehicle* vehicle);
+
+    //// @brief @return whether the foe vehicle is a leader for ego
+    bool isLeader(const MSVehicle* ego, const MSVehicle* foe);
+
+    /// @brief return whether the fromLane of this link is an internal lane
+    bool fromInternalLane() const;
+
+    /// @brief return whether the fromLane of this link is an internal lane and toLane is a normal lane
+    bool isExitLink() const;
+
+    /// @brief return whether the fromLane and the toLane of this link are internal lanes
+    bool isInternalJunctionLink() const;
+
+    /** @brief Returns the time penalty for passing a tls-controlled link (meso) */
+    SUMOTime getMesoTLSPenalty() const {
+        return myMesoTLSPenalty;
+    }
+
+    /** @brief Sets the time penalty for passing a tls-controlled link (meso) */
+    void setMesoTLSPenalty(const SUMOTime penalty) {
+        myMesoTLSPenalty = penalty;
     }
 
 private:
@@ -384,6 +427,9 @@ private:
     /// @brief returns whether the given lane may still be occupied by a vehicle currently on it
     static bool maybeOccupied(MSLane* lane);
 
+    /// @brief whether fllower could stay behind leader (possibly by braking)
+    static bool couldBrakeForLeader(SUMOReal followDist, SUMOReal leaderDist, const MSVehicle* follow, const MSVehicle* leader);
+
 private:
     /// @brief The lane (but the internal one) approached by this link
     MSLane* myLane;
@@ -413,6 +459,9 @@ private:
 
     bool myKeepClear;
 
+    /// @brief penalty time for mesoscopic simulation
+    SUMOTime myMesoTLSPenalty;
+
 #ifdef HAVE_INTERNAL_LANES
     /// @brief The following junction-internal lane if used
     MSLane* const myJunctionInlane;
@@ -429,11 +478,13 @@ private:
 #endif
 
     /// @brief the junction to which this link belongs
-    const MSJunction* myJunction;
+    MSJunction* myJunction;
 
     std::vector<MSLink*> myFoeLinks;
     std::vector<const MSLane*> myFoeLanes;
-    static SUMOTime myLookaheadTime;
+
+    static const SUMOTime myLookaheadTime;
+    static const SUMOTime myLookaheadTimeZipper;
 
 
 private:
