@@ -10,12 +10,12 @@
 /// @author  Mario Krumnow
 /// @author  Christoph Sommer
 /// @date    Tue, 06 Mar 2001
-/// @version $Id: MSNet.cpp 18756 2015-08-31 19:16:33Z behrisch $
+/// @version $Id: MSNet.cpp 20482 2016-04-18 20:49:42Z behrisch $
 ///
 // The simulated network and simulation perfomer
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -70,6 +70,7 @@
 #include <microsim/devices/MSDevice_Routing.h>
 #include <microsim/devices/MSDevice_Vehroutes.h>
 #include <microsim/devices/MSDevice_Tripinfo.h>
+#include <microsim/devices/MSDevice_BTsender.h>
 #include "traffic_lights/MSTrafficLightLogic.h"
 #include <utils/shapes/Polygon.h>
 #include <utils/shapes/ShapeContainer.h>
@@ -99,10 +100,7 @@
 #include "MSEdgeWeightsStorage.h"
 #include "MSStateHandler.h"
 
-#ifdef HAVE_INTERNAL
 #include <mesosim/MELoop.h>
-#include <utils/iodevices/BinaryInputDevice.h>
-#endif
 
 #ifndef NO_TRACI
 #include <traci-server/TraCIServer.h>
@@ -186,8 +184,7 @@ MSNet::MSNet(MSVehicleControl* vc, MSEventControl* beginOfTimestepEvents,
     myStep = string2time(oc.getString("begin"));
     myLogExecutionTime = !oc.getBool("no-duration-log");
     myLogStepNumber = !oc.getBool("no-step-log");
-    myTooManyVehicles = oc.getInt("max-num-vehicles");
-    myInserter = new MSInsertionControl(*vc, string2time(oc.getString("max-depart-delay")), !oc.getBool("eager-insert"));
+    myInserter = new MSInsertionControl(*vc, string2time(oc.getString("max-depart-delay")), !oc.getBool("eager-insert"), oc.getInt("max-num-vehicles"));
     myVehicleControl = vc;
     myDetectorControl = new MSDetectorControl();
     myEdges = 0;
@@ -204,11 +201,9 @@ MSNet::MSNet(MSVehicleControl* vc, MSEventControl* beginOfTimestepEvents,
     myInsertionEvents = insertionEvents;
     myLanesRTree.first = false;
 
-#ifdef HAVE_INTERNAL
     if (MSGlobals::gUseMesoSim) {
         MSGlobals::gMesoNet = new MELoop(string2time(oc.getString("meso-recheck")));
     }
-#endif
     myInstance = this;
 }
 
@@ -221,7 +216,9 @@ MSNet::closeBuilding(MSEdgeControl* edges, MSJunctionControl* junctions,
                      MSTLLogicControl* tlc,
                      std::vector<SUMOTime> stateDumpTimes,
                      std::vector<std::string> stateDumpFiles,
-                     bool hasInternalLinks) {
+                     bool hasInternalLinks,
+                     bool lefthand,
+                     SUMOReal version) {
     myEdges = edges;
     myJunctions = junctions;
     myRouteLoaders = routeLoaders;
@@ -239,6 +236,8 @@ MSNet::closeBuilding(MSEdgeControl* edges, MSJunctionControl* junctions,
     }
     myHasInternalLinks = hasInternalLinks;
     myHasElevation = checkElevation();
+    myLefthand = lefthand;
+    myVersion = version;
 }
 
 
@@ -270,13 +269,15 @@ MSNet::~MSNet() {
     delete myRouterTTDijkstra;
     delete myRouterTTAStar;
     delete myRouterEffort;
+    if (myPedestrianRouter != 0) {
+        delete myPedestrianRouter;
+    }
     myLanesRTree.second.RemoveAll();
     clearAll();
-#ifdef HAVE_INTERNAL
     if (MSGlobals::gUseMesoSim) {
         delete MSGlobals::gMesoNet;
     }
-#endif
+//    delete myPedestrianRouter;
     myInstance = 0;
 }
 
@@ -389,6 +390,17 @@ MSNet::closeSimulation(SUMOTime start) {
         if (myVehicleControl->getEmergencyStops() > 0) {
             msg << "Emergency Stops: " << myVehicleControl->getEmergencyStops() << "\n";
         }
+        if (myPersonControl != 0 && myPersonControl->getLoadedPersonNumber() > 0) {
+            msg << "Persons: " << "\n"
+                << " Inserted: " << myPersonControl->getLoadedPersonNumber() << "\n"
+                << " Running: " << myPersonControl->getRunningPersonNumber() << "\n";
+            if (myPersonControl->getJammedPersonNumber() > 0) {
+                msg << " Jammed: " << myPersonControl->getJammedPersonNumber() << "\n";
+            }
+        }
+        if (OptionsCont::getOptions().getBool("duration-log.statistics")) {
+            msg << MSDevice_Tripinfo::printStatistics();
+        }
         WRITE_MESSAGE(msg.str());
     }
     myDetectorControl->close(myStep);
@@ -433,12 +445,9 @@ MSNet::simulationStep() {
     // check whether the tls programs need to be switched
     myLogics->check2Switch(myStep);
 
-#ifdef HAVE_INTERNAL
     if (MSGlobals::gUseMesoSim) {
         MSGlobals::gMesoNet->simulate(myStep);
     } else {
-#endif
-
         // assure all lanes with vehicles are 'active'
         myEdges->patchActiveLanes();
 
@@ -458,9 +467,7 @@ MSNet::simulationStep() {
         if (MSGlobals::gCheck4Accidents) {
             myEdges->detectCollisions(myStep, STAGE_LANECHANGE);
         }
-#ifdef HAVE_INTERNAL
     }
-#endif
     loadRoutes();
 
     // persons
@@ -504,9 +511,6 @@ MSNet::simulationStep() {
 
 MSNet::SimulationState
 MSNet::simulationState(SUMOTime stopTime) const {
-    if (myTooManyVehicles > 0 && (int) myVehicleControl->getRunningVehicleNo() > myTooManyVehicles) {
-        return SIMSTATE_TOO_MANY_VEHICLES;
-    }
 #ifndef NO_TRACI
     if (TraCIServer::wasClosed()) {
         return SIMSTATE_CONNECTION_CLOSED;
@@ -570,7 +574,7 @@ MSNet::clearAll() {
     MSCalibrator::cleanup();
     MSPModel::cleanup();
     MSCModel_NonInteracting::cleanup();
-    PedestrianEdge<MSEdge, MSLane, MSJunction>::cleanup();
+    MSDevice_BTsender::cleanup();
 }
 
 
@@ -593,7 +597,8 @@ MSNet::writeOutput() {
 
     // check emission dumps
     if (OptionsCont::getOptions().isSet("emission-output")) {
-        MSEmissionExport::write(OutputDevice::getDeviceByOption("emission-output"), myStep);
+        MSEmissionExport::write(OutputDevice::getDeviceByOption("emission-output"), myStep,
+                                oc.getInt("emission-output.precision"));
     }
 
     // battery dumps
@@ -814,24 +819,24 @@ MSNet::getContainerStopID(const MSLane* lane, const SUMOReal pos) const {
 
 
 bool
-MSNet::addChrgStn(MSChrgStn* chrgStn) {
-    return myChrgStnDict.add(chrgStn->getID(), chrgStn);
+MSNet::addChargingStation(MSChargingStation* chargingStation) {
+    return myChargingStationDict.add(chargingStation->getID(), chargingStation);
 }
 
 
-MSChrgStn*
-MSNet::getChrgStn(const std::string& id) const {
-    return myChrgStnDict.get(id);
+MSChargingStation*
+MSNet::getChargingStation(const std::string& id) const {
+    return myChargingStationDict.get(id);
 }
 
 
 std::string
-MSNet::getChrgStnID(const MSLane* lane, const SUMOReal pos) const {
-    const std::map<std::string, MSChrgStn*>& vals = myChrgStnDict.getMyMap();
-    for (std::map<std::string, MSChrgStn*>::const_iterator it = vals.begin(); it != vals.end(); ++it) {
-        MSChrgStn* chrgStn = it->second;
-        if (&chrgStn->getLane() == lane && chrgStn->getBeginLanePosition() <= pos && chrgStn->getEndLanePosition() >= pos) {
-            return chrgStn->getID();
+MSNet::getChargingStationID(const MSLane* lane, const SUMOReal pos) const {
+    const std::map<std::string, MSChargingStation*>& vals = myChargingStationDict.getMyMap();
+    for (std::map<std::string, MSChargingStation*>::const_iterator it = vals.begin(); it != vals.end(); ++it) {
+        MSChargingStation* chargingStation = it->second;
+        if (&chargingStation->getLane() == lane && chargingStation->getBeginLanePosition() <= pos && chargingStation->getEndLanePosition() >= pos) {
+            return chargingStation->getID();
         }
     }
     return "";
@@ -845,13 +850,13 @@ MSNet::getRouterTT(const MSEdgeVector& prohibited) const {
         const std::string routingAlgorithm = OptionsCont::getOptions().getString("routing-algorithm");
         if (routingAlgorithm == "dijkstra") {
             myRouterTTDijkstra = new DijkstraRouterTT<MSEdge, SUMOVehicle, prohibited_withPermissions<MSEdge, SUMOVehicle> >(
-                MSEdge::numericalDictSize(), true, &MSNet::getTravelTime);
+                MSEdge::getAllEdges(), true, &MSNet::getTravelTime);
         } else {
             if (routingAlgorithm != "astar") {
                 WRITE_WARNING("TraCI and Triggers cannot use routing algorithm '" + routingAlgorithm + "'. using 'astar' instead.");
             }
             myRouterTTAStar = new AStarRouter<MSEdge, SUMOVehicle, prohibited_withPermissions<MSEdge, SUMOVehicle> >(
-                MSEdge::numericalDictSize(), true, &MSNet::getTravelTime);
+                MSEdge::getAllEdges(), true, &MSNet::getTravelTime);
         }
     }
     if (myRouterTTDijkstra != 0) {
@@ -869,7 +874,7 @@ SUMOAbstractRouter<MSEdge, SUMOVehicle>&
 MSNet::getRouterEffort(const MSEdgeVector& prohibited) const {
     if (myRouterEffort == 0) {
         myRouterEffort = new DijkstraRouterEffort<MSEdge, SUMOVehicle, prohibited_withPermissions<MSEdge, SUMOVehicle> >(
-            MSEdge::numericalDictSize(), true, &MSNet::getEffort, &MSNet::getTravelTime);
+            MSEdge::getAllEdges(), true, &MSNet::getEffort, &MSNet::getTravelTime);
     }
     myRouterEffort->prohibit(prohibited);
     return *myRouterEffort;
