@@ -5,12 +5,12 @@
 /// @author  Sascha Krieg
 /// @author  Michael Behrisch
 /// @date    Mon, 9 Jul 2001
-/// @version $Id: MSPersonControl.cpp 18756 2015-08-31 19:16:33Z behrisch $
+/// @version $Id: MSPersonControl.cpp 20482 2016-04-18 20:49:42Z behrisch $
 ///
 // Stores all persons in the net and handles their waiting for cars.
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -49,7 +49,11 @@
 // ===========================================================================
 // method definitions
 // ===========================================================================
-MSPersonControl::MSPersonControl() {}
+MSPersonControl::MSPersonControl():
+    myLoadedPersonNumber(0),
+    myRunningPersonNumber(0),
+    myJammedPersonNumber(0) {
+}
 
 
 MSPersonControl::~MSPersonControl() {
@@ -65,6 +69,8 @@ bool
 MSPersonControl::add(const std::string& id, MSPerson* person) {
     if (myPersons.find(id) == myPersons.end()) {
         myPersons[id] = person;
+        myLoadedPersonNumber++;
+        myRunningPersonNumber++;
         return true;
     }
     return false;
@@ -99,6 +105,7 @@ MSPersonControl::erase(MSTransportable* person) {
     }
     const std::map<std::string, MSTransportable*>::iterator i = myPersons.find(id);
     if (i != myPersons.end()) {
+        myRunningPersonNumber--;
         delete i->second;
         myPersons.erase(i);
     }
@@ -238,7 +245,7 @@ MSPersonControl::abortWaiting() {
         for (PersonVector::const_iterator j = pv.begin(); j != pv.end(); ++j) {
             MSTransportable* p = (*j);
             edge->removePerson(p);
-            WRITE_WARNING("Person " + p->getID() + " aborted waiting for a ride that will never come.");
+            WRITE_WARNING("Person '" + p->getID() + "' aborted waiting for a ride that will never come.");
             erase(p);
         }
     }
