@@ -6,12 +6,12 @@
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    Sept 2002
-/// @version $Id: MSRoute.h 17262 2014-11-04 20:41:44Z behrisch $
+/// @version $Id: MSRoute.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // A vehicle route
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2002-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2002-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -43,6 +43,10 @@
 #include <utils/common/RGBColor.h>
 #include <utils/vehicle/SUMOVehicleParameter.h>
 #include <utils/common/Parameterised.h>
+#ifdef HAVE_FOX
+#include <fx.h>
+#include <FXThread.h>
+#endif
 
 
 // ===========================================================================
@@ -56,8 +60,9 @@ class OutputDevice;
 // ===========================================================================
 // types definitions
 // ===========================================================================
-typedef std::vector<const MSEdge*> MSEdgeVector;
-typedef MSEdgeVector::const_iterator MSRouteIterator;
+typedef std::vector<const MSEdge*> ConstMSEdgeVector;
+typedef std::vector<MSEdge*> MSEdgeVector;
+typedef ConstMSEdgeVector::const_iterator MSRouteIterator;
 
 
 // ===========================================================================
@@ -69,7 +74,7 @@ typedef MSEdgeVector::const_iterator MSRouteIterator;
 class MSRoute : public Named, public Parameterised {
 public:
     /// Constructor
-    MSRoute(const std::string& id, const MSEdgeVector& edges,
+    MSRoute(const std::string& id, const ConstMSEdgeVector& edges,
             const bool isPermanent, const RGBColor* const c,
             const std::vector<SUMOVehicleParameter::Stop>& stops);
 
@@ -106,7 +111,7 @@ public:
         return std::find(myEdges.begin(), myEdges.end(), edge) != myEdges.end();
     }
 
-    bool containsAnyOf(const std::vector<MSEdge*>& edgelist) const;
+    bool containsAnyOf(const MSEdgeVector& edgelist) const;
 
     const MSEdge* operator[](unsigned index) const;
 
@@ -120,16 +125,17 @@ public:
     static void dict_saveState(OutputDevice& out);
     /// @}
 
-    const MSEdgeVector& getEdges() const {
+    const ConstMSEdgeVector& getEdges() const {
         return myEdges;
     }
 
-    SUMOReal getLength() const;
-
     /** @brief Compute the distance between 2 given edges on this route, including the length of internal lanes.
+     * Note, that for edges which contain loops:
+     * - the first occurance of fromEdge will be used
+     * - the first occurance of toEdge after the first occurance of fromEdge will be used
      *
      * @param[in] fromPos  position on the first edge, at wich the computed distance begins
-     * @param[in] toPos	   position on the last edge, at which the coumputed distance endsance
+     * @param[in] toPos    position on the last edge, at which the coumputed distance endsance
      * @param[in] fromEdge edge at wich computation begins
      * @param[in] toEdge   edge at which distance computation shall stop
      * @param[in] includeInternal Whether the lengths of internal edges shall be counted
@@ -137,6 +143,19 @@ public:
      */
     SUMOReal getDistanceBetween(SUMOReal fromPos, SUMOReal toPos, const MSEdge* fromEdge, const MSEdge* toEdge, bool includeInternal = true) const;
 
+    /** @brief Compute the distance between 2 given edges on this route, including the length of internal lanes.
+     * This has the same semantics as above but uses iterators instead of edge
+     * points so looping routes are not an issue.
+     *
+     * @param[in] fromPos  position on the first edge, at wich the computed distance begins
+     * @param[in] toPos    position on the last edge, at which the coumputed distance endsance
+     * @param[in] fromEdge edge at wich computation begins
+     * @param[in] toEdge   edge at which distance computation shall stop
+     * @param[in] includeInternal Whether the lengths of internal edges shall be counted
+     * @return             distance between the position fromPos on fromEdge and toPos on toEdge
+     */
+    SUMOReal getDistanceBetween(SUMOReal fromPos, SUMOReal toPos, const MSRouteIterator& fromEdge, const MSRouteIterator& toEdge, bool includeInternal = true) const;
+
     /// Returns the color
     const RGBColor& getColor() const;
 
@@ -212,7 +231,7 @@ public:
 
 private:
     /// The list of edges to pass
-    MSEdgeVector myEdges;
+    ConstMSEdgeVector myEdges;
 
     /// whether the route may be deleted after the last vehicle abandoned it
     const bool myAmPermanent;
@@ -242,6 +261,10 @@ private:
     /// The dictionary container
     static RouteDistDict myDistDict;
 
+#ifdef HAVE_FOX
+    /// @brief the mutex for the route dictionaries
+    static FXMutex myDictMutex;
+#endif
 private:
     /** invalid assignment operator */
     MSRoute& operator=(const MSRoute& s);
