@@ -4,7 +4,7 @@
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    Thu, 13 Dec 2012
-/// @version $Id: MSStateHandler.cpp 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: MSStateHandler.cpp 20482 2016-04-18 20:49:42Z behrisch $
 ///
 // Parser and output filter for routes and vehicles state saving and loading
 /****************************************************************************/
@@ -35,9 +35,11 @@
 #endif
 
 #include <sstream>
+#include <utils/common/TplConvert.h>
 #include <utils/iodevices/OutputDevice.h>
 #include <utils/xml/SUMOXMLDefinitions.h>
 #include <utils/xml/SUMOVehicleParserHelper.h>
+#include <microsim/devices/MSDevice_Routing.h>
 #include <microsim/MSEdge.h>
 #include <microsim/MSLane.h>
 #include <microsim/MSGlobals.h>
@@ -46,10 +48,8 @@
 #include <microsim/MSRoute.h>
 #include "MSStateHandler.h"
 
-#ifdef HAVE_INTERNAL
 #include <mesosim/MESegment.h>
 #include <mesosim/MELoop.h>
-#endif
 
 #ifdef CHECK_MEMORY_LEAKS
 #include <foreign/nvwa/debug_new.h>
@@ -61,15 +61,14 @@
 // ===========================================================================
 MSStateHandler::MSStateHandler(const std::string& file, const SUMOTime offset) :
     SUMOSAXHandler(file), myOffset(offset),
-#ifdef HAVE_INTERNAL
     mySegment(0),
-#endif
     myEdgeAndLane(0, -1),
     myCurrentVType(0) {
 }
 
 
 MSStateHandler::~MSStateHandler() {
+    delete myCurrentVType;
 }
 
 
@@ -77,20 +76,18 @@ void
 MSStateHandler::saveState(const std::string& file, SUMOTime step) {
     OutputDevice& out = OutputDevice::getDevice(file);
     out.writeHeader<MSEdge>(SUMO_TAG_SNAPSHOT);
-    out.writeAttr(SUMO_ATTR_VERSION, VERSION_STRING).writeAttr(SUMO_ATTR_TIME, step);
+    out.writeAttr(SUMO_ATTR_VERSION, VERSION_STRING).writeAttr(SUMO_ATTR_TIME, time2string(step));
     MSRoute::dict_saveState(out);
     MSNet::getInstance()->getVehicleControl().saveState(out);
     if (MSGlobals::gUseMesoSim) {
-#ifdef HAVE_INTERNAL
         for (size_t i = 0; i < MSEdge::dictSize(); i++) {
-            for (MESegment* s = MSGlobals::gMesoNet->getSegmentForEdge(*MSEdge::dictionary(i)); s != 0; s = s->getNextSegment()) {
+            for (MESegment* s = MSGlobals::gMesoNet->getSegmentForEdge(*MSEdge::getAllEdges()[i]); s != 0; s = s->getNextSegment()) {
                 s->saveState(out);
             }
         }
-#endif
     } else {
         for (size_t i = 0; i < MSEdge::dictSize(); i++) {
-            const std::vector<MSLane*>& lanes = MSEdge::dictionary(i)->getLanes();
+            const std::vector<MSLane*>& lanes = MSEdge::getAllEdges()[i]->getLanes();
             for (std::vector<MSLane*>::const_iterator it = lanes.begin(); it != lanes.end(); ++it) {
                 (*it)->saveState(out);
             }
@@ -104,7 +101,7 @@ MSStateHandler::myStartElement(int element, const SUMOSAXAttributes& attrs) {
     MSVehicleControl& vc = MSNet::getInstance()->getVehicleControl();
     switch (element) {
         case SUMO_TAG_SNAPSHOT: {
-            myTime = attrs.getInt(SUMO_ATTR_TIME);
+            myTime = string2time(attrs.getString(SUMO_ATTR_TIME));
             const std::string& version = attrs.getString(SUMO_ATTR_VERSION);
             if (version != VERSION_STRING) {
                 WRITE_WARNING("State was written with sumo version " + version + " (present: " + VERSION_STRING + ")!");
@@ -112,14 +109,17 @@ MSStateHandler::myStartElement(int element, const SUMOSAXAttributes& attrs) {
             break;
         }
         case SUMO_TAG_DELAY: {
-            vc.setState(attrs.getInt(SUMO_ATTR_NUMBER), attrs.getInt(SUMO_ATTR_END),
-                        attrs.getFloat(SUMO_ATTR_DEPART), attrs.getFloat(SUMO_ATTR_TIME));
+            vc.setState(attrs.getInt(SUMO_ATTR_NUMBER),
+                        attrs.getInt(SUMO_ATTR_BEGIN),
+                        attrs.getInt(SUMO_ATTR_END),
+                        attrs.getFloat(SUMO_ATTR_DEPART),
+                        attrs.getFloat(SUMO_ATTR_TIME));
             break;
         }
         case SUMO_TAG_ROUTE: {
             const std::string id = attrs.getString(SUMO_ATTR_ID);
             if (MSRoute::dictionary(id) == 0) {
-                MSEdgeVector edges;
+                ConstMSEdgeVector edges;
                 MSEdge::parseEdgesList(attrs.getString(SUMO_ATTR_EDGES), edges, id);
                 MSRoute* r = new MSRoute(id, edges, attrs.getBool(SUMO_ATTR_STATE),
                                          0, std::vector<SUMOVehicleParameter::Stop>());
@@ -171,7 +171,7 @@ MSStateHandler::myStartElement(int element, const SUMOSAXAttributes& attrs) {
         case SUMO_TAG_VEHICLE: {
             SUMOVehicleParameter* p = new SUMOVehicleParameter();
             p->id = attrs.getString(SUMO_ATTR_ID);
-            p->depart = attrs.getInt(SUMO_ATTR_DEPART) - myOffset;
+            p->depart = string2time(attrs.getString(SUMO_ATTR_DEPART)) - myOffset;
             p->routeid = attrs.getString(SUMO_ATTR_ROUTE);
             p->vtypeid = attrs.getString(SUMO_ATTR_TYPE);
             const MSRoute* route = MSRoute::dictionary(p->routeid);
@@ -181,6 +181,7 @@ MSStateHandler::myStartElement(int element, const SUMOSAXAttributes& attrs) {
             assert(vc.getVehicle(p->id) == 0);
 
             SUMOVehicle* v = vc.buildVehicle(p, route, type, true);
+            vc.discountStateLoaded(); // already included (see SUMO_TAG_DELAY)
             v->loadState(attrs, myOffset);
             if (!vc.addVehicle(p->id, v)) {
                 throw ProcessError("Error: Could not build vehicle " + p->id + "!");
@@ -188,25 +189,29 @@ MSStateHandler::myStartElement(int element, const SUMOSAXAttributes& attrs) {
             if (!v->hasDeparted()) {
                 // !!! the save did not keep the order in which the vehicles are checked for insertion
                 MSNet::getInstance()->getInsertionControl().add(v);
+            } else {
+                // vehicle already departed: disable pre-insertion rerouting and enable regular routing behavior
+                MSDevice_Routing* routingDevice = static_cast<MSDevice_Routing*>(v->getDevice(typeid(MSDevice_Routing)));
+                if (routingDevice != 0) {
+                    routingDevice->notifyEnter(*v, MSMoveReminder::NOTIFICATION_DEPARTED);
+                }
             }
             break;
         }
-#ifdef HAVE_INTERNAL
         case SUMO_TAG_SEGMENT: {
             if (mySegment == 0) {
-                mySegment = MSGlobals::gMesoNet->getSegmentForEdge(*MSEdge::dictionary(0));
+                mySegment = MSGlobals::gMesoNet->getSegmentForEdge(*MSEdge::getAllEdges()[0]);
             } else if (mySegment->getNextSegment() == 0) {
-                mySegment = MSGlobals::gMesoNet->getSegmentForEdge(*MSEdge::dictionary(mySegment->getEdge().getNumericalID() + 1));
+                mySegment = MSGlobals::gMesoNet->getSegmentForEdge(*MSEdge::getAllEdges()[mySegment->getEdge().getNumericalID() + 1]);
             } else {
                 mySegment = mySegment->getNextSegment();
             }
             myQueIndex = 0;
             break;
         }
-#endif
         case SUMO_TAG_LANE: {
             myEdgeAndLane.second++;
-            if (myEdgeAndLane.second == (int)MSEdge::dictionary(myEdgeAndLane.first)->getLanes().size()) {
+            if (myEdgeAndLane.second == (int)MSEdge::getAllEdges()[myEdgeAndLane.first]->getLanes().size()) {
                 myEdgeAndLane.first++;
                 myEdgeAndLane.second = 0;
             }
@@ -216,11 +221,9 @@ MSStateHandler::myStartElement(int element, const SUMOSAXAttributes& attrs) {
             std::vector<std::string> vehIDs;
             SUMOSAXAttributes::parseStringVector(attrs.getString(SUMO_ATTR_VALUE), vehIDs);
             if (MSGlobals::gUseMesoSim) {
-#ifdef HAVE_INTERNAL
-                mySegment->loadState(vehIDs, MSNet::getInstance()->getVehicleControl(), attrs.getInt(SUMO_ATTR_TIME) - myOffset, myQueIndex++);
-#endif
+                mySegment->loadState(vehIDs, MSNet::getInstance()->getVehicleControl(), TplConvert::_2long(attrs.getString(SUMO_ATTR_TIME).c_str()) - myOffset, myQueIndex++);
             } else {
-                MSEdge::dictionary(myEdgeAndLane.first)->getLanes()[myEdgeAndLane.second]->loadState(
+                MSEdge::getAllEdges()[myEdgeAndLane.first]->getLanes()[myEdgeAndLane.second]->loadState(
                     vehIDs, MSNet::getInstance()->getVehicleControl());
             }
             break;
