@@ -5,12 +5,12 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Mon, 12 Mar 2001
-/// @version $Id: MSVehicleType.h 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: MSVehicleType.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // The car-following model and parameter
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -271,6 +271,36 @@ public:
         return myParameter.imgFile;
     }
 
+
+    /** @brief Get this vehicle type's person capacity
+     * @return The person capacity of this vehicle type
+     */
+    unsigned int getPersonCapacity() const {
+        return myParameter.personCapacity;
+    }
+
+
+    /** @brief Get this vehicle type's container capacity
+     * @return The container capacity of this vehicle type
+     */
+    unsigned int getContainerCapacity() const {
+        return myParameter.containerCapacity;
+    }
+
+    /** @brief Get this vehicle type's boarding duration
+     * @return The time a person needs to board a vehicle of this type
+     */
+    SUMOTime getBoardingDuration() const {
+        return myParameter.boardingDuration;
+    }
+
+    /** @brief Get this vehicle type's loading duration
+     * @return The time a container needs to get laoded on a vehicle of this type
+     */
+    SUMOTime getLoadingDuration() const {
+        return myParameter.loadingDuration;
+    }
+
     /// @}
 
 
@@ -419,7 +449,7 @@ private:
     /// @brief the running index
     const int myIndex;
 
-    /// @brief ID of the car following model.
+    /// @brief instance of the car following model.
     MSCFModel* myCarFollowModel;
 
     /// @brief The original type
