@@ -3,12 +3,12 @@
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    14.08.2013
-/// @version $Id: MSDevice_BTsender.h 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: MSDevice_BTsender.h 20447 2016-04-14 13:02:24Z luecken $
 ///
 // A BT sender
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2013-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2013-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -35,7 +35,7 @@
 #include <string>
 #include "MSDevice.h"
 #include <utils/common/SUMOTime.h>
-#include <utils/geom/Position.h>
+#include <utils/geom/Boundary.h>
 
 
 // ===========================================================================
@@ -74,6 +74,11 @@ public:
     static void buildVehicleDevices(SUMOVehicle& v, std::vector<MSDevice*>& into);
 
 
+    /** @brief removes remaining vehicleInformation in sVehicles
+     */
+    static void cleanup();
+
+
     /// for accessing the maps of running/arrived vehicles
     friend class MSDevice_BTreceiver;
 
@@ -141,8 +146,9 @@ public:
          * @param[in] _laneID The id of the lane the vehicle is located at
          * @param[in] _lanePos The position of the vehicle along the lane
          */
-        VehicleState(SUMOReal _speed, const Position& _position, const std::string& _laneID, SUMOReal _lanePos)
-            : speed(_speed), position(_position), laneID(_laneID), lanePos(_lanePos) {}
+        VehicleState(const SUMOReal _speed, const Position& _position, const std::string& _laneID,
+                     const SUMOReal _lanePos, const unsigned int _routePos)
+            : speed(_speed), position(_position), laneID(_laneID), lanePos(_lanePos), routePos(_routePos) {}
 
         /// @brief Destructor
         ~VehicleState() {}
@@ -155,6 +161,8 @@ public:
         std::string laneID;
         /// @brief The position at the lane of the vehicle
         SUMOReal lanePos;
+        /// @brief The position in the route of the vehicle
+        unsigned int routePos;
 
     };
 
@@ -171,7 +179,7 @@ public:
         VehicleInformation(const std::string& id) : Named(id), amOnNet(true), haveArrived(false)  {}
 
         /// @brief Destructor
-        ~VehicleInformation() {}
+        virtual ~VehicleInformation() {}
 
         /** @brief Returns the boundary of passed positions
          * @return The positions boundary
@@ -193,6 +201,9 @@ public:
         /// @brief Whether the vehicle was removed from the simulation
         bool haveArrived;
 
+        /// @brief List of edges travelled
+        ConstMSEdgeVector route;
+
     };
 
 
