@@ -4,12 +4,12 @@
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    Fri, 30.01.2009
-/// @version $Id: MSDevice_Tripinfo.h 18095 2015-03-17 09:39:00Z behrisch $
+/// @version $Id: MSDevice_Tripinfo.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // A device which collects info on the vehicle trip
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2009-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2009-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -65,9 +65,22 @@ public:
      */
     static void buildVehicleDevices(SUMOVehicle& v, std::vector<MSDevice*>& into);
 
+    /// @brief update tripinfo statistics
+    void updateStatistics() const;
+
     /// @brief generate output for vehicles which are still in the network
     static void generateOutputForUnfinished();
 
+    /// @brief get statistics for printing to stdout
+    static std::string printStatistics();
+
+    /// @brief accessors for GUINet-Parameters
+    static SUMOReal getAvgRouteLength();
+    static SUMOReal getAvgDuration();
+    static SUMOReal getAvgWaitingTime();
+    static SUMOReal getAvgTimeLoss();
+    static SUMOReal getAvgDepartDelay();
+
 public:
     /// @brief Destructor.
     ~MSDevice_Tripinfo();
@@ -135,13 +148,24 @@ private:
     MSDevice_Tripinfo();
 
 
+    /* @brief compute trip length and duration (depending on whether the
+       vehicle arrived or not */
+    void computeLengthAndDuration(SUMOReal& routeLength, SUMOTime& duration) const;
+
+protected:
+    /** @brief Internal notification about the vehicle moves
+     *
+     * @param[in] veh Vehicle that asks this reminder.
+     * @param[in] timeOnLane time the vehicle spent on the lane.
+     * @param[in] speed Moving speed.
+     */
+    void notifyMoveInternal(SUMOVehicle& veh, SUMOReal timeOnLane,
+                            SUMOReal speed);
+
 private:
     /// @brief The lane the vehicle departed at
     std::string myDepartLane;
 
-    /// @brief The position on the lane the vehicle departed at
-    SUMOReal myDepartPos;
-
     /// @brief The speed on departure
     SUMOReal myDepartSpeed;
 
@@ -165,8 +189,17 @@ private:
 
     /// @brief devices which may still need to produce output
     typedef std::set<const MSDevice_Tripinfo*, Named::NamedLikeComparatorIdLess<MSDevice_Tripinfo> > DeviceSet;
+
     static DeviceSet myPendingOutput;
 
+    /// @brief global tripinfo statistics
+    static SUMOReal myVehicleCount;
+    static SUMOReal myTotalRouteLength;
+    static SUMOTime myTotalDuration;
+    static SUMOTime myTotalWaitingTime;
+    static SUMOTime myTotalTimeLoss;
+    static SUMOTime myTotalDepartDelay;
+
 private:
     /// @brief Invalidated copy constructor.
     MSDevice_Tripinfo(const MSDevice_Tripinfo&);
