@@ -4,12 +4,12 @@
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    Fri, 30.01.2009
-/// @version $Id: MSDevice_Vehroutes.h 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: MSDevice_Vehroutes.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // A device which collects info on the vehicle trip
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2009-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2009-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -172,8 +172,8 @@ private:
     /// @brief A shortcut for the Option "vehroute-output.intended-depart"
     static bool myIntendedDepart;
 
-    /// @brief A shortcut for the Option "device.routing.with-taz"
-    static bool myWithTaz;
+    /// @brief A shortcut for the Option "vehroute-output.route-length"
+    static bool myRouteLength;
 
 
     /** @class StateListener
