@@ -6,13 +6,14 @@
 /// @author  Sascha Krieg
 /// @author  Michael Behrisch
 /// @author  Robbin Blokpoel
+/// @author  Jakob Erdmann
 /// @date    Mon Feb 03 2014 10:13 CET
-/// @version $Id: MSE2Collector.cpp 15727 2014-02-25 08:09:54Z behrisch $
+/// @version $Id: MSE2Collector.cpp 20284 2016-03-22 21:07:03Z namdre $
 ///
 // An areal (along a single lane) detector
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo-sim.org/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
+// Copyright (C) 2014 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -63,7 +64,7 @@ MSE2Collector::MSE2Collector(const std::string& id, DetectorUsage usage,
     myCurrentOccupancy(0), myCurrentMeanSpeed(-1), myCurrentJamNo(0),
     myCurrentMaxJamLengthInMeters(0), myCurrentMaxJamLengthInVehicles(0),
     myCurrentJamLengthInMeters(0), myCurrentJamLengthInVehicles(0), myCurrentStartedHalts(0),
-    myCurrentHaltingsNumber(0)
+    myCurrentHaltingsNumber(0), myPassedVeh(0)
 
 {
     assert(myLane != 0);
@@ -81,13 +82,26 @@ MSE2Collector::~MSE2Collector() {
 bool
 MSE2Collector::notifyMove(SUMOVehicle& veh, SUMOReal oldPos,
                           SUMOReal newPos, SUMOReal) {
-    if (newPos < myStartPos) {
+    if (newPos <= myStartPos) {
         // detector not yet reached
         return true;
     }
-    if (newPos >= myStartPos && oldPos < myStartPos) {
+    if (newPos > myStartPos && oldPos <= myStartPos) {
         if (find(myKnownVehicles.begin(), myKnownVehicles.end(), &veh) == myKnownVehicles.end()) {
-            myKnownVehicles.push_back(&veh);
+            std::string type = veh.getVehicleType().getID(); // get vehicle's type
+            if (type.find("COLOMBO_undetectable") == std::string::npos) {
+                myKnownVehicles.push_back(&veh);
+                //Detection entering the sensor
+                myPassedVeh++;
+                DBG(
+                    std::ostringstream str;
+                    str << time2string(MSNet::getInstance()->getCurrentTimeStep())
+                    << " MSE2Collector::notifyMove::"
+                    << " lane " << myLane->getID()
+                    << " passedVeh " << myPassedVeh ;
+                    WRITE_MESSAGE(str.str());
+                )
+            }
         }
     }
     if (newPos - veh.getVehicleType().getLength() > myEndPos) {
@@ -103,7 +117,7 @@ MSE2Collector::notifyMove(SUMOVehicle& veh, SUMOReal oldPos,
 
 bool
 MSE2Collector::notifyLeave(SUMOVehicle& veh, SUMOReal lastPos, MSMoveReminder::Notification reason) {
-    if (reason != MSMoveReminder::NOTIFICATION_JUNCTION || (lastPos >= myStartPos && lastPos - veh.getVehicleType().getLength() < myEndPos)) {
+    if (reason != MSMoveReminder::NOTIFICATION_JUNCTION || (lastPos > myStartPos && lastPos - veh.getVehicleType().getLength() < myEndPos)) {
         std::list<SUMOVehicle*>::iterator i = find(myKnownVehicles.begin(), myKnownVehicles.end(), &veh);
         if (i != myKnownVehicles.end()) {
             myKnownVehicles.erase(i);
@@ -115,17 +129,39 @@ MSE2Collector::notifyLeave(SUMOVehicle& veh, SUMOReal lastPos, MSMoveReminder::N
 
 
 bool
-MSE2Collector::notifyEnter(SUMOVehicle& veh, MSMoveReminder::Notification) {
-    if (veh.getPositionOnLane() >= myStartPos && veh.getPositionOnLane() - veh.getVehicleType().getLength() < myEndPos) {
-        // vehicle is on detector
-        myKnownVehicles.push_back(&veh);
-        return true;
+MSE2Collector::notifyEnter(SUMOVehicle& veh, MSMoveReminder::Notification reason) {
+    if (!veh.isOnRoad()) {
+        // vehicle is teleporting over the edge
+        return false;
+    }
+    if (veh.getPositionOnLane() - veh.getVehicleType().getLength() >= myEndPos) {
+        // vehicle is beyond the detector
+        return false;
+    }
+    if (reason != MSMoveReminder::NOTIFICATION_JUNCTION && veh.getPositionOnLane() > myStartPos) {
+        // the junction case is handled in the notifyMove
+        // vehicle is on the detector, being already beyond was checked before
+        std::string type = veh.getVehicleType().getID(); // get vehicle's type
+        if (type.find("COLOMBO_undetectable") == std::string::npos) {
+            myKnownVehicles.push_back(&veh);
+            //Detection entering the sensor
+            myPassedVeh++;
+            DBG(
+                std::ostringstream str;
+                str << time2string(MSNet::getInstance()->getCurrentTimeStep())
+                << " MSE2Collector::notifyEnter::"
+                << " lane " << myLane->getID()
+                << " passedVeh " << myPassedVeh ;
+                WRITE_MESSAGE(str.str());
+            )
+            return true;
+        }
     }
     if (veh.getPositionOnLane() - veh.getVehicleType().getLength() > myEndPos) {
         // vehicle is beyond detector
         return false;
     }
-    // vehicle is in front of detector
+    // vehicle is in front of the detector
     return true;
 }
 
@@ -146,11 +182,12 @@ MSE2Collector::reset() {
     myTimeSamples = 0;
     myMeanVehicleNumber = 0;
     myMaxVehicleNumber = 0;
-    for (std::map<SUMOVehicle*, SUMOTime>::iterator i = myIntervalHaltingVehicleDurations.begin(); i != myIntervalHaltingVehicleDurations.end(); ++i) {
+    for (std::map<const SUMOVehicle*, SUMOTime>::iterator i = myIntervalHaltingVehicleDurations.begin(); i != myIntervalHaltingVehicleDurations.end(); ++i) {
         (*i).second = 0;
     }
     myPastStandingDurations.clear();
     myPastIntervalStandingDurations.clear();
+    myPassedVeh = 0;
 }
 
 
@@ -158,8 +195,8 @@ MSE2Collector::reset() {
 void
 MSE2Collector::detectorUpdate(const SUMOTime /* step */) {
     JamInfo* currentJam = 0;
-    std::map<SUMOVehicle*, SUMOTime> haltingVehicles;
-    std::map<SUMOVehicle*, SUMOTime> intervalHaltingVehicles;
+    std::map<const SUMOVehicle*, SUMOTime> haltingVehicles;
+    std::map<const SUMOVehicle*, SUMOTime> intervalHaltingVehicles;
     std::vector<JamInfo*> jams;
 
     SUMOReal lengthSum = 0;
@@ -172,7 +209,7 @@ MSE2Collector::detectorUpdate(const SUMOTime /* step */) {
     //  sum up values and prepare the list of jams
     myKnownVehicles.sort(by_vehicle_position_sorter(getLane()));
     for (std::list<SUMOVehicle*>::const_iterator i = myKnownVehicles.begin(); i != myKnownVehicles.end(); ++i) {
-        MSVehicle* veh = static_cast<MSVehicle*>(*i);
+        const MSVehicle* const veh = static_cast<MSVehicle*>(*i);
 
         SUMOReal length = veh->getVehicleType().getLength();
         if (veh->getLane() == getLane()) {
@@ -221,7 +258,7 @@ MSE2Collector::detectorUpdate(const SUMOTime /* step */) {
             }
         } else {
             // is not standing anymore; keep duration information
-            std::map<SUMOVehicle*, SUMOTime>::iterator v = myHaltingVehicleDurations.find(veh);
+            std::map<const SUMOVehicle*, SUMOTime>::iterator v = myHaltingVehicleDurations.find(veh);
             if (v != myHaltingVehicleDurations.end()) {
                 myPastStandingDurations.push_back((*v).second);
                 myHaltingVehicleDurations.erase(v);
@@ -239,7 +276,7 @@ MSE2Collector::detectorUpdate(const SUMOTime /* step */) {
             //  it may be a new one or already an existing one
             if (currentJam == 0) {
                 // the vehicle is the first vehicle in a jam
-                currentJam = new JamInfo;
+                currentJam = new JamInfo();
                 currentJam->firstStandingVehicle = i;
             } else {
                 // ok, we have a jam already. But - maybe it is too far away
@@ -296,8 +333,8 @@ MSE2Collector::detectorUpdate(const SUMOTime /* step */) {
     }
     myCurrentJamNo = (unsigned) jams.size();
 
-    unsigned noVehicles = (unsigned) myKnownVehicles.size();
-    myVehicleSamples += noVehicles;
+    const unsigned numVehicles = (unsigned) myKnownVehicles.size();
+    myVehicleSamples += numVehicles;
     myTimeSamples += 1;
     // compute occupancy values
     SUMOReal currentOccupancy = lengthSum / (myEndPos - myStartPos) * (SUMOReal) 100.;
@@ -313,11 +350,11 @@ MSE2Collector::detectorUpdate(const SUMOTime /* step */) {
     myHaltingVehicleDurations = haltingVehicles;
     myIntervalHaltingVehicleDurations = intervalHaltingVehicles;
     // compute information about vehicle numbers
-    myMeanVehicleNumber += (unsigned) myKnownVehicles.size();
-    myMaxVehicleNumber = MAX2((unsigned) myKnownVehicles.size(), myMaxVehicleNumber);
+    myMeanVehicleNumber += numVehicles;
+    myMaxVehicleNumber = MAX2(numVehicles, myMaxVehicleNumber);
     // norm current values
-    myCurrentMeanSpeed = noVehicles != 0 ? myCurrentMeanSpeed / (SUMOReal) noVehicles : -1;
-    myCurrentMeanLength = noVehicles != 0 ? myCurrentMeanLength / (SUMOReal) noVehicles : -1;
+    myCurrentMeanSpeed = numVehicles != 0 ? myCurrentMeanSpeed / (SUMOReal) numVehicles : -1;
+    myCurrentMeanLength = numVehicles != 0 ? myCurrentMeanLength / (SUMOReal) numVehicles : -1;
 
     // clean up
     for (std::vector<JamInfo*>::iterator i = jams.begin(); i != jams.end(); ++i) {
@@ -346,7 +383,7 @@ MSE2Collector::writeXMLOutput(OutputDevice& dev, SUMOTime startTime, SUMOTime st
         maxHaltingDuration = MAX2(maxHaltingDuration, (*i));
         haltingNo++;
     }
-    for (std::map<SUMOVehicle*, SUMOTime> ::iterator i = myHaltingVehicleDurations.begin(); i != myHaltingVehicleDurations.end(); ++i) {
+    for (std::map<const SUMOVehicle*, SUMOTime> ::iterator i = myHaltingVehicleDurations.begin(); i != myHaltingVehicleDurations.end(); ++i) {
         haltingDurationSum += (*i).second;
         maxHaltingDuration = MAX2(maxHaltingDuration, (*i).second);
         haltingNo++;
@@ -361,7 +398,7 @@ MSE2Collector::writeXMLOutput(OutputDevice& dev, SUMOTime startTime, SUMOTime st
         intervalMaxHaltingDuration = MAX2(intervalMaxHaltingDuration, (*i));
         intervalHaltingNo++;
     }
-    for (std::map<SUMOVehicle*, SUMOTime> ::iterator i = myIntervalHaltingVehicleDurations.begin(); i != myIntervalHaltingVehicleDurations.end(); ++i) {
+    for (std::map<const SUMOVehicle*, SUMOTime> ::iterator i = myIntervalHaltingVehicleDurations.begin(); i != myIntervalHaltingVehicleDurations.end(); ++i) {
         intervalHaltingDurationSum += (*i).second;
         intervalMaxHaltingDuration = MAX2(intervalMaxHaltingDuration, (*i).second);
         intervalHaltingNo++;
@@ -394,7 +431,7 @@ MSE2Collector::writeXMLOutput(OutputDevice& dev, SUMOTime startTime, SUMOTime st
 
 void
 MSE2Collector::writeXMLDetectorProlog(OutputDevice& dev) const {
-    dev.writeXMLHeader("detector", "xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xsi:noNamespaceSchemaLocation=\"http://sumo-sim.org/xsd/det_e2_file.xsd\"");
+    dev.writeXMLHeader("detector", "xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xsi:noNamespaceSchemaLocation=\"http://sumo.dlr.de/xsd/det_e2_file.xsd\"");
 }
 
 
@@ -403,10 +440,83 @@ MSE2Collector::getCurrentVehicleNumber() const {
     return (unsigned) myKnownVehicles.size();
 }
 
+int
+MSE2Collector::getEstimatedCurrentVehicleNumber(SUMOReal speedThreshold) const {
+
+    SUMOReal distance = 0;
+    SUMOReal thresholdSpeed = myLane->getSpeedLimit() / speedThreshold;
+
+    int count = 0;
+    for (std::list<SUMOVehicle*>::const_iterator it = myKnownVehicles.begin();
+            it != myKnownVehicles.end(); it++) {
+        MSVehicle* veh = static_cast<MSVehicle*>(*it);
+        SUMOReal acceleration = veh->getAcceleration();
+        if (distance == 0) {
+            distance = veh->getPositionOnLane();
+        }
+        if (veh->getPositionOnLane() < distance) {
+            distance = veh->getPositionOnLane();
+        }
+        SUMOReal carLength = veh->getVehicleType().getLengthWithGap();
+        SUMOReal vel = veh->getSpeed();
+        SUMOReal realDistance = myLane->getLength() - distance; // the closer vehicle get to the light the greater is the distance
+        if (vel <= thresholdSpeed || acceleration > 0) { //TODO speed less than half of the maximum speed for the lane NEED TUNING
+            count = (int)(realDistance / carLength) + 1;
+        }
+    }
+
+    return count;
+}
+
+SUMOReal
+MSE2Collector::getEstimateQueueLength() const {
+
+    if (myKnownVehicles.empty()) {
+        return -1;
+    }
+
+    SUMOReal distance = 0;
+    SUMOReal realDistance = 0;
+    bool flowing =  true;
+    for (std::list<SUMOVehicle*>::const_iterator it = myKnownVehicles.begin();
+            it != myKnownVehicles.end(); it++) {
+        MSVehicle* veh = static_cast<MSVehicle*>(*it);
+        if (distance == 0) {
+            distance = veh->getPositionOnLane();
+        }
+        if (veh->getPositionOnLane() < distance) {
+            distance = veh->getPositionOnLane();
+        }
+        SUMOReal carLength = veh->getVehicleType().getLengthWithGap();
+        SUMOReal vel = veh->getSpeed();
+        //	SUMOReal distanceTemp = myLane->getLength() - distance;
+        if (vel <= 0.5) {
+            realDistance = distance - carLength;
+            flowing = false;
+        }
+        DBG(
+            std::ostringstream str;
+            str << time2string(MSNet::getInstance()->getCurrentTimeStep())
+            << " MSE2Collector::getEstimateQueueLength::"
+            << " lane " << myLane->getID()
+            << " vehicle " << veh->getID()
+            << " positionOnLane " << veh->getPositionOnLane()
+            << " vel " << veh->getSpeed()
+            << " realDistance " << realDistance;
+            WRITE_MESSAGE(str.str());
+        )
+    }
+    if (flowing) {
+        return 0;
+    } else {
+        return myLane->getLength() - realDistance;
+    }
+}
+
 
 SUMOReal
 MSE2Collector::getCurrentOccupancy() const {
-    return myCurrentOccupancy * (SUMOReal) 100.;
+    return myCurrentOccupancy;
 }
 
 
@@ -470,7 +580,7 @@ MSE2Collector::by_vehicle_position_sorter::operator()(const SUMOVehicle* v1, con
     return v1->getPositionOnLane() > v2->getPositionOnLane();
 }
 
-SUMOReal
+int
 MSE2Collector::getCurrentHaltingNumber() const {
     return myCurrentHaltingsNumber;
 }
@@ -487,5 +597,9 @@ MSE2Collector::getCurrentVehicleIDs() const {
     return ret;
 }
 
+const std::list<SUMOVehicle*>& MSE2Collector::getCurrentVehicles() const {
+    return myKnownVehicles;
+}
+
 /****************************************************************************/
 
