@@ -7,11 +7,11 @@
 /// @author  Robbin Blokpoel
 /// @author  Jakob Erdmann
 /// @date    Mon Feb 03 2014 14:13 CET
-/// @version $Id: MSE2Collector.h 16005 2014-03-24 12:46:02Z cschmidt87 $
+/// @version $Id: MSE2Collector.h 19623 2015-12-16 09:30:37Z behrisch $
 ///
 // An areal (along a single lane) detector
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo-sim.org/
+// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
 // Copyright (C) 2014 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
@@ -38,11 +38,15 @@
 #include <vector>
 #include <list>
 #include <microsim/MSLane.h>
+#include <microsim/MSNet.h>
 #include <microsim/MSMoveReminder.h>
 #include <microsim/output/MSDetectorFileOutput.h>
 #include <utils/common/UtilExceptions.h>
-#include <utils/common/SUMOVehicle.h>
+#include <utils/vehicle/SUMOVehicle.h>
 
+//Debug
+//#define SWARM_DEBUG
+#include <utils/common/SwarmDebug.h>
 
 // ===========================================================================
 // class declarations
@@ -226,6 +230,11 @@ public:
      */
     void reset();
 
+    /** @brief Returns an estimate of the number of vehicles currently on the detector */
+    int getEstimatedCurrentVehicleNumber(SUMOReal speedThreshold) const;
+
+    /** @brief Returns an estimate of the lenght of the queue of vehicles currently stopped on the detector */
+    SUMOReal getEstimateQueueLength() const;
 
     /// @name Methods returning current values
     /// @{
@@ -266,15 +275,35 @@ public:
     *
     * @return The mean number of haltings within the area
     */
-    SUMOReal getCurrentHaltingNumber() const;
+    int getCurrentHaltingNumber() const;
 
     /** @brief Returns the IDs of the vehicles within the area
      *
      * @return The IDs of the vehicles that have passed the entry, but not yet an exit point
      */
     std::vector<std::string> getCurrentVehicleIDs() const;
+    /** @brief Returns the vehicles within the area
+     *
+     * @return The vehicles that have passed the entry, but not yet an exit point
+     */
+    const std::list<SUMOVehicle*>& getCurrentVehicles() const;
     /// @}
 
+    /** \brief Returns the number of vehicles passed over the sensor.
+     *
+     * @return number of cars passed over the sensor
+     */
+    unsigned int getPassedVeh() {
+        return myPassedVeh;
+    }
+
+    /** \brief Subtract the number of vehicles indicated from passed from the sensor count.
+     *
+     * @param[in] passed - int that indicates the number of vehicles to subtract
+     */
+    void subtractPassedVeh(int passed) {
+        myPassedVeh -= passed;
+    }
 
 protected:
     /** @brief Internal representation of a jam
@@ -426,7 +455,9 @@ private:
     /// @brief The number of started halts in the last step
     unsigned myCurrentStartedHalts;
     /// @brief The number of halted vehicles [#]
-    SUMOReal myCurrentHaltingsNumber;
+    int myCurrentHaltingsNumber;
+    /// @brief The number of vehicles passed on the sensor
+    unsigned int myPassedVeh;
     /// @}
 
 
