@@ -3,12 +3,12 @@
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    Tue, 17.11.2009
-/// @version $Id: MSMeanData.h 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: MSMeanData.h 20482 2016-04-18 20:49:42Z behrisch $
 ///
 // Data collector for edges/lanes
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -48,6 +48,7 @@ class MSEdge;
 class MSLane;
 class SUMOVehicle;
 
+typedef std::vector<MSEdge*> MSEdgeVector;
 
 // ===========================================================================
 // class definitions
@@ -279,6 +280,11 @@ public:
             TrackerEntry(MeanDataValues* const values)
                 : myNumVehicleEntered(0), myNumVehicleLeft(0), myValues(values) {}
 
+            /** @brief Constructor */
+            virtual ~TrackerEntry() {
+                delete myValues;
+            }
+
             /// @brief The number of vehicles which entered in the current interval
             int myNumVehicleEntered;
 
@@ -447,7 +453,7 @@ private:
     const SUMOTime myDumpBegin, myDumpEnd;
 
     /// @brief The corresponding first edges
-    std::vector<MSEdge*> myEdges;
+    MSEdgeVector myEdges;
 
     /// @brief Whether empty lanes/edges shall be written
     const bool myPrintDefaults;
