@@ -4,12 +4,12 @@
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    Oct 2003
-/// @version $Id: MS_E2_ZS_CollectorOverLanes.cpp 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: MS_E2_ZS_CollectorOverLanes.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // A detector which joins E2Collectors over consecutive lanes (backward)
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2003-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2003-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -170,10 +170,10 @@ std::vector<MSLane*>
 MS_E2_ZS_CollectorOverLanes::getLanePredeccessorLanes(MSLane* l) {
     std::string eid = l->getEdge().getID();
     // get predecessing edges
-    const std::vector<MSEdge*>& predEdges = l->getEdge().getIncomingEdges();
+    const MSEdgeVector& predEdges = l->getEdge().getIncomingEdges();
     std::vector<MSLane*> ret;
     // find predecessing lanes
-    std::vector<MSEdge*>::const_iterator i = predEdges.begin();
+    MSEdgeVector::const_iterator i = predEdges.begin();
     for (; i != predEdges.end(); ++i) {
         MSEdge* e = *i;
         assert(e != 0);
