@@ -1,13 +1,14 @@
 /****************************************************************************/
-/// @file    MSPModel_Striping.h
+/// @file    MSPModel_Striping.cpp
 /// @author  Jakob Erdmann
+/// @author  Michael Behrisch
 /// @date    Mon, 13 Jan 2014
-/// @version $Id: MSPModel_Striping.cpp 18756 2015-08-31 19:16:33Z behrisch $
+/// @version $Id: MSPModel_Striping.cpp 20482 2016-04-18 20:49:42Z behrisch $
 ///
 // The pedestrian following model (prototype)
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2014-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2014-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -30,11 +31,14 @@
 #include <math.h>
 #include <algorithm>
 #include <utils/common/RandHelper.h>
+#include <utils/geom/GeomHelper.h>
 #include <utils/options/OptionsCont.h>
 #include <microsim/MSNet.h>
 #include <microsim/MSEdge.h>
 #include <microsim/MSLane.h>
 #include <microsim/MSJunction.h>
+#include <microsim/MSGlobals.h>
+#include <microsim/MSPersonControl.h>
 #include "MSPModel_Striping.h"
 
 
@@ -42,8 +46,8 @@
 // DEBUGGING HELPERS
 // ===========================================================================
 //
-#define DEBUG1 "disabled"
-#define DEBUG2 "disabled"
+#define DEBUG1 ""
+#define DEBUG2 ""
 #define DEBUGCOND(PEDID) (PEDID == DEBUG1 || PEDID == DEBUG2)
 //#define LOG_ALL 1
 
@@ -51,8 +55,8 @@ void MSPModel_Striping::DEBUG_PRINT(const Obstacles& obs) {
     for (int i = 0; i < (int)obs.size(); ++i) {
         std::cout
                 << "(" << obs[i].description
-                << " x=" << obs[i].x
-                << " s=" << obs[i].speed
+                << " x=(" << obs[i].xBack << "," << obs[i].xFwd
+                << ") s=" << obs[i].speed
                 << ")   ";
     }
     std::cout << "\n";
@@ -62,11 +66,11 @@ void MSPModel_Striping::DEBUG_PRINT(const Obstacles& obs) {
 // named (internal) constants
 // ===========================================================================
 
+// distances are comparable with lower values being "more important"
+#define DIST_FAR_AWAY 10000
+#define DIST_BEHIND 1000
+#define DIST_OVERLAP -1
 
-// need to be able to subtract without underflow
-#define BLOCKED_STRIPE -10000
-#define FAR_AWAY 10000
-#define ONCOMING_CONFLICT -1000
 
 // ===========================================================================
 // static members
@@ -82,13 +86,17 @@ SUMOReal MSPModel_Striping::dawdling;
 SUMOTime MSPModel_Striping::jamTime;
 const SUMOReal MSPModel_Striping::LOOKAHEAD_SAMEDIR(4.0); // seconds
 const SUMOReal MSPModel_Striping::LOOKAHEAD_ONCOMING(10.0); // seconds
-const SUMOReal MSPModel_Striping::LATERAL_PENALTY(-1);
+const SUMOReal MSPModel_Striping::LATERAL_PENALTY(-1.); // meters
+const SUMOReal MSPModel_Striping::OBSTRUCTED_PENALTY(-300000.); // meters
+const SUMOReal MSPModel_Striping::INAPPROPRIATE_PENALTY(-20000.); // meters
+const SUMOReal MSPModel_Striping::ONCOMING_CONFLICT_PENALTY(-1000.); // meters
+const SUMOReal MSPModel_Striping::OBSTRUCTION_THRESHOLD(MSPModel_Striping::OBSTRUCTED_PENALTY * 0.5); // despite obstruction, additional utility may have been added
 const SUMOReal MSPModel_Striping::SQUEEZE(0.7);
-const SUMOReal MSPModel_Striping::BLOCKER_LOOKAHEAD(10.0);
+const SUMOReal MSPModel_Striping::BLOCKER_LOOKAHEAD(10.0); // meters
 const SUMOReal MSPModel_Striping::RESERVE_FOR_ONCOMING_FACTOR(0.0);
 const SUMOReal MSPModel_Striping::MAX_WAIT_TOLERANCE(120.); // seconds
 const SUMOReal MSPModel_Striping::LATERAL_SPEED_FACTOR(0.4);
-const SUMOReal MSPModel_Striping::MIN_STARTUP_SPEED(0.3);
+const SUMOReal MSPModel_Striping::MIN_STARTUP_DIST(0.4); // meters
 
 
 // ===========================================================================
@@ -118,7 +126,7 @@ MSPModel_Striping::~MSPModel_Striping() {
 PedestrianState*
 MSPModel_Striping::add(MSPerson* person, MSPerson::MSPersonStage_Walking* stage, SUMOTime) {
     assert(person->getCurrentStageType() == MSTransportable::MOVING_WITHOUT_VEHICLE);
-    const MSLane* lane = getSidewalk(person->getEdge());
+    const MSLane* lane = getSidewalk<MSEdge, MSLane>(person->getEdge());
     PState* ped = new PState(person, stage, lane);
     myActiveLanes[lane].push_back(ped);
     myNumActivePedestrians++;
@@ -168,6 +176,11 @@ MSPModel_Striping::getPedestrians(const MSLane* lane) {
 
 void
 MSPModel_Striping::cleanupHelper() {
+    for (ActiveLanes::iterator it_lane = myActiveLanes.begin(); it_lane != myActiveLanes.end(); ++it_lane) {
+        for (Pedestrians::iterator it_p = it_lane->second.begin(); it_p != it_lane->second.end(); ++it_p) {
+            delete *it_p;
+        }
+    }
     myActiveLanes.clear();
     myNumActivePedestrians = 0;
     myWalkingAreaPaths.clear(); // need to recompute when lane pointers change
@@ -198,20 +211,20 @@ MSPModel_Striping::initWalkingAreaPaths(const MSNet*) {
     if (myWalkingAreaPaths.size() > 0) {
         return;
     }
-    for (size_t i = 0; i < MSEdge::dictSize(); ++i) {
-        const MSEdge* edge = MSEdge::dictionary(i);
+    for (MSEdgeVector::const_iterator i = MSEdge::getAllEdges().begin(); i != MSEdge::getAllEdges().end(); ++i) {
+        const MSEdge* edge = *i;
         if (edge->isWalkingArea()) {
-            const MSLane* walkingArea = getSidewalk(edge);
+            const MSLane* walkingArea = getSidewalk<MSEdge, MSLane>(edge);
             // build all possible paths across this walkingArea
             // gather all incident lanes
-            std::vector<MSLane*> lanes;
+            std::vector<const MSLane*> lanes;
             const MSEdgeVector& incoming = edge->getIncomingEdges();
             for (int j = 0; j < (int)incoming.size(); ++j) {
-                lanes.push_back(getSidewalk(incoming[j]));
+                lanes.push_back(getSidewalk<MSEdge, MSLane>(incoming[j]));
             }
             const MSEdgeVector& outgoing = edge->getSuccessors();
             for (int j = 0; j < (int)outgoing.size(); ++j) {
-                lanes.push_back(getSidewalk(outgoing[j]));
+                lanes.push_back(getSidewalk<MSEdge, MSLane>(outgoing[j]));
             }
             // build all combinations
             for (int j = 0; j < (int)lanes.size(); ++j) {
@@ -262,7 +275,7 @@ MSPModel_Striping::getNextLane(const PState& ped, const MSLane* currentLane, con
     const MSEdge* currentEdge = &currentLane->getEdge();
     const MSJunction* junction = ped.myDir == FORWARD ? currentEdge->getToJunction() : currentEdge->getFromJunction();
     const MSEdge* nextRouteEdge = ped.myStage->getNextRouteEdge();
-    const MSLane* nextRouteLane = getSidewalk(nextRouteEdge);
+    const MSLane* nextRouteLane = getSidewalk<MSEdge, MSLane>(nextRouteEdge);
     // result values
     const MSLane* nextLane = nextRouteLane;
     MSLink* link = 0;
@@ -306,7 +319,7 @@ MSPModel_Striping::getNextLane(const PState& ped, const MSLane* currentLane, con
             }
             if (crossingRoute.size() > 1) {
                 const MSEdge* nextEdge = crossingRoute[1];
-                nextLane = getSidewalk(crossingRoute[1]);
+                nextLane = getSidewalk<MSEdge, MSLane>(crossingRoute[1]);
                 assert((nextEdge->getFromJunction() == junction || nextEdge->getToJunction() == junction));
                 assert(nextLane != prevLane);
                 nextDir = connectedDirection(currentLane, nextLane);
@@ -330,6 +343,8 @@ MSPModel_Striping::getNextLane(const PState& ped, const MSLane* currentLane, con
                               << "' to '" << (nextRouteEdge == 0 ? "NULL" : nextRouteEdge->getID())
                               << "\n";
                 }
+                WRITE_WARNING("Pedestrian '" + ped.myPerson->getID() + "' could not find route across junction '" + junction->getID() + "', time=" +
+                              time2string(MSNet::getInstance()->getCurrentTimeStep()) + ".");
                 // error indicated by nextDir == UNDEFINED_DIRECTION
             }
         } else if (currentEdge == nextRouteEdge) {
@@ -421,32 +436,37 @@ MSPModel_Striping::getNextWalkingArea(const MSLane* currentLane, const int dir,
 }
 
 
-MSPModel_Striping::Obstacles
-MSPModel_Striping::mergeObstacles(const Obstacles& obs1, const Obstacles& obs2, int dir) {
-    Obstacles result(obs1.begin(), obs1.end());
-    for (int i = 0; i < (int)obs1.size(); ++i) {
-        if ((obs2[i].x - obs1[i].x) * dir < 0) {
-            result[i] = obs2[i];
-        }
-    }
-    return result;
-}
-
-
 MSPModel_Striping::Obstacles
 MSPModel_Striping::getNeighboringObstacles(const Pedestrians& pedestrians, int egoIndex, int stripes) {
     const PState& ego = *pedestrians[egoIndex];
-    const SUMOReal egoBack = ego.myRelX - ego.myDir * ego.getLength();
-    int index = egoIndex + 1;
     Obstacles obs(stripes, Obstacle(ego.myDir));
-    while (index < (int)pedestrians.size() && ego.myDir * (pedestrians[index]->myRelX - egoBack) > 0) {
+    std::vector<bool> haveBlocker(stripes, false);
+    for (int index = egoIndex + 1; index < (int)pedestrians.size(); index++) {
         const PState& p = *pedestrians[index];
+        if DEBUGCOND(ego.myPerson->getID()) {
+            std::cout << SIMTIME << " ped=" << ego.myPerson->getID() << "  checking neighbor " << p.myPerson->getID();
+        }
         if (!p.myWaitingToEnter) {
-            Obstacle o(p, ego.myDir);
-            obs[p.stripe()] = o;
-            obs[p.otherStripe()] = o;
+            const Obstacle o(p);
+            if DEBUGCOND(ego.myPerson->getID()) {
+                std::cout << " dist=" << ego.distanceTo(o) << std::endl;
+            }
+            if (ego.distanceTo(o) == DIST_BEHIND) {
+                break;
+            }
+            if (ego.distanceTo(o) == DIST_OVERLAP) {
+                obs[p.stripe()] = o;
+                obs[p.otherStripe()] = o;
+                haveBlocker[p.stripe()] = true;
+                haveBlocker[p.otherStripe()] = true;
+            }
+            if (!haveBlocker[p.stripe()]) {
+                obs[p.stripe()] = o;
+            }
+            if (!haveBlocker[p.otherStripe()]) {
+                obs[p.otherStripe()] = o;
+            }
         }
-        index++;
     }
     if DEBUGCOND(ego.myPerson->getID()) {
         std::cout << SIMTIME << " ped=" << ego.myPerson->getID() << "  neighObs=";
@@ -471,9 +491,13 @@ MSPModel_Striping::getNextLaneObstacles(NextLanesObstacles& nextLanesObs, const
 
         // figure out the which pedestrians are ahead on the next lane
         const int nextStripes = numStripes(nextLane);
-        const int offset = (stripes - nextStripes) / 2;
         Obstacles obs(stripes, Obstacle(nextDir));
         if (nextStripes < stripes) {
+            int offset = (stripes - nextStripes) / 2;
+            if (currentDir == nextDir) {
+                // for odd stripe number differences there is a bigger offset at the beginning
+                offset += (stripes - nextStripes) % 2;
+            }
             // some stripes do not continue
             for (int ii = 0; ii < stripes; ++ii) {
                 if (ii < offset || ii >= nextStripes + offset) {
@@ -483,8 +507,6 @@ MSPModel_Striping::getNextLaneObstacles(NextLanesObstacles& nextLanesObs, const
         }
         Pedestrians& pedestrians = getPedestrians(nextLane);
         if (nextLane->getEdge().isWalkingArea()) {
-            const SUMOReal minY = stripeWidth * - 0.5 + NUMERICAL_EPS;
-            const SUMOReal maxY = stripeWidth * (stripes - 0.5) - NUMERICAL_EPS;
             // complex transformation into the coordinate system of the current lane
             // (pedestrians on next lane may walk at arbitrary angles relative to the current lane)
             const SUMOReal lateral_offset = (lane->getWidth() - stripeWidth) * 0.5;
@@ -494,10 +516,8 @@ MSPModel_Striping::getNextLaneObstacles(NextLanesObstacles& nextLanesObs, const
                 PState& p = *pedestrians[ii];
                 Position relPos =  lane->getShape().transformToVectorCoordinates(p.getPosition(*p.myStage, -1), true);
                 const SUMOReal newY = relPos.y() + lateral_offset;
-                if (newY >= minY && newY <= maxY) {
-                    addCloserObstacle(obs, relPos.x(), p.stripe(newY), p.myPerson->getID(), stripes, currentDir);
-                    addCloserObstacle(obs, relPos.x(), p.otherStripe(newY), p.myPerson->getID(), stripes, currentDir);
-                }
+                addCloserObstacle(obs, relPos.x(), p.stripe(newY), stripes, p.myPerson->getID(), p.myPerson->getVehicleType().getWidth(), currentDir);
+                addCloserObstacle(obs, relPos.x(), p.otherStripe(newY), stripes, p.myPerson->getID(), p.myPerson->getVehicleType().getWidth(), currentDir);
             }
         } else {
             // simple transformation into the coordinate system of the current lane
@@ -509,18 +529,34 @@ MSPModel_Striping::getNextLaneObstacles(NextLanesObstacles& nextLanesObs, const
                 if (p.myWaitingToEnter) {
                     continue;
                 }
-                addMappedObstacle(obs, p, p.stripe(), currentDir, nextDir, offset, stripes, nextStripes);
-                addMappedObstacle(obs, p, p.otherStripe(), currentDir, nextDir, offset, stripes, nextStripes);
+                SUMOReal newY = p.myRelY;
+                if (nextDir != currentDir) {
+                    newY = nextLane->getWidth() - stripeWidth - newY;
+                }
+                newY += 0.5 * (lane->getWidth() - nextLane->getWidth());
+                const int stripe = p.stripe(newY);
+                if (stripe >= 0 && stripe < stripes) {
+                    obs[stripe] = Obstacle(p);
+                }
+                const int otherStripe = p.otherStripe(newY);
+                if (otherStripe >= 0 && otherStripe < stripes) {
+                    obs[otherStripe] = Obstacle(p);
+                }
             }
             for (int ii = 0; ii < stripes; ++ii) {
                 Obstacle& o = obs[ii];
                 if (nextDir == BACKWARD) {
-                    o.x = nextLength - o.x;
+                    const SUMOReal xfwd = nextLength - o.xBack;
+                    o.xBack = nextLength - o.xFwd;
+                    o.xFwd = xfwd;
                 }
                 if (currentDir == FORWARD) {
-                    o.x += currentLength;
+                    o.xFwd += currentLength;
+                    o.xBack += currentLength;
                 } else {
-                    o.x *= -1;
+                    const SUMOReal xfwd = -o.xBack;
+                    o.xBack = -o.xFwd;
+                    o.xFwd = xfwd;
                 }
             }
         }
@@ -531,20 +567,11 @@ MSPModel_Striping::getNextLaneObstacles(NextLanesObstacles& nextLanesObs, const
 
 
 void
-MSPModel_Striping::addMappedObstacle(Obstacles& obs, const PState& p, int stripe, int currentDir, int nextDir, int offset, int stripes, int nextStripes) {
-    const int mappedStripe = (nextDir == currentDir
-                              ? stripe + offset
-                              : nextStripes - 1 - stripe + offset);
-    if (mappedStripe >= 0 && mappedStripe < stripes) {
-        obs[mappedStripe] = Obstacle(p, nextDir);
-    }
-}
-
-
-void
-MSPModel_Striping::addCloserObstacle(Obstacles& obs, SUMOReal x, int stripe, const std::string& id, int stripes, int dir) {
-    if (stripe < stripes && obs[stripe].x * dir > x * dir) {
-        obs[stripe] = Obstacle(x, 0, id);
+MSPModel_Striping::addCloserObstacle(Obstacles& obs, SUMOReal x, int stripe, int numStripes, const std::string& id, SUMOReal width, int dir) {
+    if (stripe >= 0 && stripe < numStripes) {
+        if ((dir == FORWARD && x - width / 2. < obs[stripe].xBack) || (dir == BACKWARD && x + width / 2. > obs[stripe].xFwd)) {
+            obs[stripe] = Obstacle(x, 0, id, width);
+        }
     }
 }
 
@@ -613,8 +640,7 @@ MSPModel_Striping::moveInDirection(SUMOTime currentTime, std::set<MSPerson*>& ch
                 for (Pedestrians::iterator it_p = toDelete.begin(); it_p != toDelete.end(); ++it_p) {
                     delete *it_p;
                 }
-            };
-
+            }
         } else {
             moveInDirectionOnLane(pedestrians, lane, currentTime, changedLane, dir);
         }
@@ -656,7 +682,7 @@ MSPModel_Striping::moveInDirectionOnLane(Pedestrians& pedestrians, const MSLane*
         Obstacles currentObs = obs;
         if (p.myDir != dir || changedLane.count(p.myPerson) != 0) {
             if (!p.myWaitingToEnter) {
-                Obstacle o(p, dir);
+                Obstacle o(p);
                 obs[p.stripe()] = o;
                 obs[p.otherStripe()] = o;
             }
@@ -664,6 +690,7 @@ MSPModel_Striping::moveInDirectionOnLane(Pedestrians& pedestrians, const MSLane*
         }
         if DEBUGCOND(p.myPerson->getID()) {
             std::cout << SIMTIME << " ped=" << p.myPerson->getID() << "  currentObs=";
+            gDebugFlag1 = true;
             DEBUG_PRINT(currentObs);
         }
         const MSLane* nextLane = p.myNLI.lane;
@@ -682,41 +709,62 @@ MSPModel_Striping::moveInDirectionOnLane(Pedestrians& pedestrians, const MSLane*
                 std::cout << SIMTIME << " ped=" << p.myPerson->getID() << "  nextObs=";
                 DEBUG_PRINT(nextObs);
             }
-            currentObs = mergeObstacles(currentObs, nextObs, dir);
+            p.mergeObstacles(currentObs, nextObs);
         }
         if DEBUGCOND(p.myPerson->getID()) {
             std::cout << SIMTIME << " ped=" << p.myPerson->getID() << "  obsWithNext=";
             DEBUG_PRINT(currentObs);
         }
-        currentObs = mergeObstacles(currentObs, getNeighboringObstacles(pedestrians, ii, stripes), dir);
+        p.mergeObstacles(currentObs, getNeighboringObstacles(pedestrians, ii, stripes));
         if DEBUGCOND(p.myPerson->getID()) {
             std::cout << SIMTIME << " ped=" << p.myPerson->getID() << "  obsWithNeigh=";
             DEBUG_PRINT(currentObs);
         }
         // check link state
         if (link != 0
-                && dist < speed  // only check close before junction
+                // only check close before junction, @todo we should take deceleration into account here
+                && dist - p.getMinGap() < LOOKAHEAD_SAMEDIR * speed
                 && (!link->opened(currentTime, speed, speed, p.getLength(), p.getImpatience(currentTime), speed, 0)
-                    // XXX check for presence of vehicles blocking the path
+                    // @todo check for presence of vehicles blocking the path
                    )) {
             // prevent movement passed a closed link
-            Obstacles closedLink(stripes, Obstacle(p.myRelX + dir * dist - POSITION_EPS, 0, "closedLink"));
-            currentObs = mergeObstacles(currentObs, closedLink, dir);
+            Obstacles closedLink(stripes, Obstacle(p.myRelX + dir * (dist + NUMERICAL_EPS), 0, "closedLink"));
+            p.mergeObstacles(currentObs, closedLink);
             if DEBUGCOND(p.myPerson->getID()) {
                 std::cout << SIMTIME << " ped=" << p.myPerson->getID() << "  obsWitTLS=";
                 DEBUG_PRINT(currentObs);
             }
             // consider rerouting over another crossing
             if (p.myWalkingAreaPath != 0) {
-                // XXX actually another path would be needed starting at the current position
+                // @todo actually another path would be needed starting at the current position
                 p.myNLI = getNextLane(p, p.myLane, p.myWalkingAreaPath->from);
             }
         }
+        if (&lane->getEdge() == &p.myStage->getDestination() && p.myStage->getDestinationStop() != 0) {
+            Obstacles arrival(stripes, Obstacle(p.myStage->getArrivalPos() + dir * p.getMinGap(), 0, "arrival"));
+            p.mergeObstacles(currentObs, arrival);
+        }
         p.walk(currentObs, currentTime);
-        if (!p.myWaitingToEnter) {
-            Obstacle o(p, dir);
+        gDebugFlag1 = false;
+        if (!p.myWaitingToEnter && !p.myAmJammed) {
+            Obstacle o(p);
             obs[p.stripe()] = o;
             obs[p.otherStripe()] = o;
+            if (MSGlobals::gCheck4Accidents && p.myWalkingAreaPath == 0 && !p.myAmJammed) {
+                for (int coll = 0; coll < ii; ++coll) {
+                    PState& c = *pedestrians[coll];
+                    if (!c.myWaitingToEnter && c.myWalkingAreaPath == 0 && !c.myAmJammed) {
+                        if (c.stripe() == p.stripe() || p.stripe() == c.otherStripe() || p.otherStripe() == c.stripe() || p.otherStripe() == c.otherStripe()) {
+                            Obstacle cObs(c);
+                            // we check only for real collisions, no min gap violations
+                            if (p.distanceTo(cObs, false) == DIST_OVERLAP) {
+                                WRITE_WARNING("Collision of person '" + p.myPerson->getID() + "' and person '" + c.myPerson->getID()
+                                              + "', lane='" + lane->getID() + "', time=" + time2string(currentTime) + ".");
+                            }
+                        }
+                    }
+                }
+            }
         }
         //std::cout << SIMTIME << p.myPerson->getID() << " lane=" << lane->getID() << " x=" << p.myRelX << "\n";
     }
@@ -727,22 +775,19 @@ MSPModel_Striping::moveInDirectionOnLane(Pedestrians& pedestrians, const MSLane*
 // MSPModel_Striping::Obstacle method definitions
 // ===========================================================================
 MSPModel_Striping::Obstacle::Obstacle(int dir) :
-    x(dir* FAR_AWAY),  // far away when seen in dir
+    xFwd(dir * DIST_FAR_AWAY),  // far away when seen in dir
+    xBack(dir * DIST_FAR_AWAY),  // far away when seen in dir
     speed(0),
-    description("")
-{}
+    description("") {
+}
 
 
-MSPModel_Striping::Obstacle::Obstacle(const PState& ped, int dir) :
+MSPModel_Striping::Obstacle::Obstacle(const PState& ped) :
     description(ped.myPerson->getID()) {
     assert(!ped.myWaitingToEnter);
-    if (dir == ped.myDir) {
-        speed = ped.mySpeed;
-        x = ped.myRelX - dir * ped.getLength();
-    } else {
-        speed = -ped.mySpeed;
-        x = ped.myRelX;
-    }
+    xFwd = ped.getMaxX();
+    xBack = ped.getMinX();
+    speed = ped.myDir * ped.mySpeed;
 }
 
 
@@ -805,23 +850,47 @@ MSPModel_Striping::PState::PState(MSPerson* person, MSPerson::MSPersonStage_Walk
 }
 
 
+SUMOReal
+MSPModel_Striping::PState::getMinX(const bool includeMinGap) const {
+    // @todo speed should have an influence here because faster persons need more space
+    if (myDir == FORWARD) {
+        return myRelX - getLength();
+    }
+    return myRelX - (includeMinGap ? getMinGap() : 0.);
+}
+
+
+SUMOReal
+MSPModel_Striping::PState::getMaxX(const bool includeMinGap) const {
+    // @todo speed should have an influence here because faster persons need more space
+    if (myDir == FORWARD) {
+        return myRelX + (includeMinGap ? getMinGap() : 0.);
+    }
+    return myRelX + getLength();
+}
+
+
 SUMOReal
 MSPModel_Striping::PState::getLength() const {
     return myPerson->getVehicleType().getLength();
 }
 
 
+SUMOReal
+MSPModel_Striping::PState::getMinGap() const {
+    return myPerson->getVehicleType().getMinGap();
+}
+
+
 int
 MSPModel_Striping::PState::stripe(SUMOReal relY) const {
-    const int max = numStripes(myLane) - 1;
-    return MIN2(MAX2(0, (int)floor(relY / stripeWidth + 0.5)), max);
+    return (int)floor(relY / stripeWidth + 0.5);
 }
 
 
 int
 MSPModel_Striping::PState::otherStripe(SUMOReal relY) const {
-    const int max = numStripes(myLane) - 1;
-    const int s = stripe();
+    const int s = stripe(relY);
     const SUMOReal offset = relY - s * stripeWidth;
     const SUMOReal threshold = MAX2(NUMERICAL_EPS, stripeWidth - SQUEEZE * myPerson->getVehicleType().getWidth());
     int result;
@@ -832,28 +901,28 @@ MSPModel_Striping::PState::otherStripe(SUMOReal relY) const {
     } else {
         result = s;
     }
-    std::cout.setf(std::ios::fixed , std::ios::floatfield);
-    std::cout << std::setprecision(5);
+    //std::cout.setf(std::ios::fixed , std::ios::floatfield);
+    //std::cout << std::setprecision(5);
     //if DEBUGCOND(myPerson->getID()) std::cout << "  otherStripe " << myPerson->getID() << " offset=" << offset << " threshold=" << threshold << " rawResult=" << result << "\n";
-    return MIN2(MAX2(0, result), max);
+    return result;
 }
 
 int
 MSPModel_Striping::PState::stripe() const {
-    return stripe(myRelY);
+    return MIN2(MAX2(0, stripe(myRelY)), numStripes(myLane) - 1);
 }
 
 
 int
 MSPModel_Striping::PState::otherStripe() const {
-    return otherStripe(myRelY);
+    return MIN2(MAX2(0, otherStripe(myRelY)), numStripes(myLane) - 1);
 }
 
 
 SUMOReal
 MSPModel_Striping::PState::distToLaneEnd() const {
     if (myStage->getNextRouteEdge() == 0) {
-        return myDir * (myStage->getArrivalPos() - myRelX);
+        return myDir * (myStage->getArrivalPos() - myRelX) - POSITION_EPS;
     } else {
         const SUMOReal length = myWalkingAreaPath == 0 ? myLane->getLength() : myWalkingAreaPath->length;
         return myDir == FORWARD ? length - myRelX : myRelX;
@@ -887,6 +956,9 @@ MSPModel_Striping::PState::moveToNextLane(SUMOTime currentTime) {
                       << " newDir=" << myDir
                       << "\n";
         }
+        if (myLane == 0) {
+            myRelX = myStage->getArrivalPos();
+        }
         myStage->moveToNextEdge(myPerson, currentTime, normalLane ? 0 : &myLane->getEdge());
         if (myLane != 0) {
             assert(myDir != UNDEFINED_DIRECTION);
@@ -929,7 +1001,7 @@ MSPModel_Striping::PState::moveToNextLane(SUMOTime currentTime) {
             }
             // adjust to change in direction
             if (myDir != oldDir) {
-                myRelY = (numStripes(oldLane) - 1) * stripeWidth - myRelY;
+                myRelY = oldLane->getWidth() - stripeWidth - myRelY;
             }
             // adjust to differences in sidewalk width
             myRelY += 0.5 * (myLane->getWidth() - oldLane->getWidth());
@@ -950,7 +1022,6 @@ MSPModel_Striping::PState::walk(const Obstacles& obs, SUMOTime currentTime) {
     // ultimate goal is to choose the prefered stripe (chosen)
     const int current = stripe();
     const int other = otherStripe();
-    int chosen = current;
     // compute utility for all stripes
     std::vector<SUMOReal> utility(stripes);
     // penalize lateral movement (may increase jamming)
@@ -960,18 +1031,22 @@ MSPModel_Striping::PState::walk(const Obstacles& obs, SUMOTime currentTime) {
     // compute distances
     std::vector<SUMOReal> distance(stripes);
     for (int i = 0; i < stripes; ++i) {
-        distance[i] = myDir * (obs[i].x - myRelX);
+        distance[i] = distanceTo(obs[i]);
     }
     // forbid stripes which are blocked and also all stripes behind them
     for (int i = 0; i < stripes; ++i) {
-        if (distance[i] <= 0) {
+        if (distanceTo(obs[i]) == DIST_OVERLAP) {
+            if (i == current && !myWaitingToEnter) {
+                utility[i] += OBSTRUCTED_PENALTY;
+            }
             if (i < current) {
                 for (int j = 0; j <= i; ++j) {
-                    utility[j] = 2 * BLOCKED_STRIPE;
+                    utility[j] += OBSTRUCTED_PENALTY;
                 }
-            } else {
+            }
+            if (i > current) {
                 for (int j = i; j < stripes; ++j) {
-                    utility[j] = 2 * BLOCKED_STRIPE;
+                    utility[j] += OBSTRUCTED_PENALTY;
                 }
             }
         }
@@ -982,11 +1057,11 @@ MSPModel_Striping::PState::walk(const Obstacles& obs, SUMOTime currentTime) {
     const int reserved = (int)floor(stripes * RESERVE_FOR_ONCOMING_FACTOR);
     if (myDir == FORWARD) {
         for (int i = 0; i < reserved; ++i) {
-            utility[i] = BLOCKED_STRIPE;
+            utility[i] += INAPPROPRIATE_PENALTY;
         }
     } else {
         for (int i = sMax; i > sMax - reserved; --i) {
-            utility[i] = BLOCKED_STRIPE;
+            utility[i] += INAPPROPRIATE_PENALTY;
         }
     }
     // adapt utility based on obstacles
@@ -1000,8 +1075,9 @@ MSPModel_Striping::PState::walk(const Obstacles& obs, SUMOTime currentTime) {
             }
         }
         // compute expected distance achievable by staying on this stripe for a time horizon
-        const SUMOReal lookAhead = obs[i].speed >= 0 ? LOOKAHEAD_SAMEDIR : LOOKAHEAD_ONCOMING;
-        const SUMOReal expectedDist = MIN2(vMax * LOOKAHEAD_SAMEDIR, distance[i] + obs[i].speed * lookAhead);
+        const SUMOReal walkDist = MAX2((SUMOReal)0, distance[i]); // disregard special distance flags
+        const SUMOReal lookAhead = obs[i].speed * myDir >= 0 ? LOOKAHEAD_SAMEDIR : LOOKAHEAD_ONCOMING;
+        const SUMOReal expectedDist = MIN2(vMax * LOOKAHEAD_SAMEDIR, walkDist + obs[i].speed * myDir * lookAhead);
         if (DEBUGCOND(myPerson->getID())) {
             std::cout << " util=" << utility[i] << " exp=" << expectedDist << "\n";
         }
@@ -1009,25 +1085,26 @@ MSPModel_Striping::PState::walk(const Obstacles& obs, SUMOTime currentTime) {
             utility[i] += expectedDist;
         } else {
             // let only the distance count
-            utility[i] += ONCOMING_CONFLICT + distance[i];
+            utility[i] += ONCOMING_CONFLICT_PENALTY + distance[i];
         }
     }
     // bonus to remain on the rightmost lane (in walking direction) if there
     // are oncoming
     if (((myDir == FORWARD && current == sMax)
             || (myDir == BACKWARD && current == 0))
-            && obs[current].speed < 0) {
-        utility[current] -= ONCOMING_CONFLICT;
+            && obs[current].speed * myDir < 0) {
+        utility[current] -= ONCOMING_CONFLICT_PENALTY;
     }
     // bonus to leave the leftmost lane (in walking direction) if there
     // are oncoming
     if (((myDir == BACKWARD && current == sMax)
             || (myDir == FORWARD && current == 0))
-            && obs[current].speed < 0) {
-        utility[current] += ONCOMING_CONFLICT;
+            && obs[current].speed * myDir < 0) {
+        utility[current] += ONCOMING_CONFLICT_PENALTY;
     }
 
     // select best stripe
+    int chosen = current;
     for (int i = 0; i < stripes; ++i) {
         if (utility[chosen] < utility[i]) {
             chosen = i;
@@ -1037,28 +1114,36 @@ MSPModel_Striping::PState::walk(const Obstacles& obs, SUMOTime currentTime) {
     const int next = (chosen == current ? current : (chosen < current ? current - 1 : current + 1));
     const SUMOReal xDist = MIN3(distance[current], distance[other], distance[next]);
     // XXX preferred gap differs between approaching a standing obstacle or a moving obstacle
-    const SUMOReal preferredGap = myPerson->getVehicleType().getMinGap() + xDist * 0.5;
-    SUMOReal xSpeed = MIN2(vMax, MAX2((SUMOReal)0, xDist - preferredGap));
+    const SUMOReal preferredGap = NUMERICAL_EPS;
+    SUMOReal xSpeed = MIN2(vMax, MAX2((SUMOReal)0, DIST2SPEED(xDist - preferredGap)));
+    if (xSpeed < NUMERICAL_EPS) {
+        xSpeed = 0.;
+    }
     if (DEBUGCOND(myPerson->getID())) {
         std::cout << " xSpeedPotential=" << xSpeed << "\n";
     }
     // avoid tiny steps
     // XXX pressure from behind?
-    if (mySpeed == 0 && xSpeed < MIN_STARTUP_SPEED * vMax) {
+    if (mySpeed == 0 && xDist < MIN_STARTUP_DIST) {
         xSpeed = 0;
     }
     if (xSpeed == 0) {
         if (myWaitingTime > jamTime || myAmJammed) {
             // squeeze slowly through the crowd ignoring others
-            myAmJammed = true;
+            if (!myAmJammed) {
+                MSNet::getInstance()->getPersonControl().registerJammed();
+                WRITE_WARNING("Person '" + myPerson->getID()
+                              + "' is jammed on edge '" + myStage->getEdge()->getID()
+                              + "', time=" + time2string(MSNet::getInstance()->getCurrentTimeStep()) + ".");
+                myAmJammed = true;
+            }
             xSpeed = vMax / 4;
-        } else {
-            myAmJammed = false;
         }
+    } else {
+        myAmJammed = false;
     }
     // dawdling
     const SUMOReal dawdle = MIN2(xSpeed, RandHelper::rand() * vMax * dawdling);
-    xSpeed -= dawdle;
 
     // XXX ensure that diagonal speed <= vMax
     // avoid deadlocks on narrow sidewalks
@@ -1073,11 +1158,15 @@ MSPModel_Striping::PState::walk(const Obstacles& obs, SUMOTime currentTime) {
     //}
     const SUMOReal maxYSpeed = MAX2(vMax * LATERAL_SPEED_FACTOR, vMax - xSpeed);
     SUMOReal ySpeed = 0;
-    const SUMOReal yDist = (chosen * stripeWidth) - myRelY;
-    if (fabs(yDist) > NUMERICAL_EPS) {
-        ySpeed = (yDist > 0 ?
-                  MIN2(maxYSpeed, DIST2SPEED(yDist)) :
-                  MAX2(-maxYSpeed, DIST2SPEED(yDist)));
+    SUMOReal yDist = 0;
+    if (utility[next] > OBSTRUCTION_THRESHOLD && utility[chosen] > OBSTRUCTION_THRESHOLD) {
+        // don't move laterally if the stripes are blocked
+        yDist = (chosen * stripeWidth) - myRelY;
+        if (fabs(yDist) > NUMERICAL_EPS) {
+            ySpeed = (yDist > 0 ?
+                      MIN2(maxYSpeed, DIST2SPEED(yDist)) :
+                      MAX2(-maxYSpeed, DIST2SPEED(yDist)));
+        }
     }
     // DEBUG
     if DEBUGCOND(myPerson->getID()) {
@@ -1100,15 +1189,15 @@ MSPModel_Striping::PState::walk(const Obstacles& obs, SUMOTime currentTime) {
                   << " vMax=" << myStage->getMaxSpeed()
                   << " wTime=" << myStage->getWaitingTime(currentTime)
                   << " jammed=" << myAmJammed
-                  << "\n distance=" << toString(distance)
-                  << " utility=" << toString(utility)
+                  << "\n   distance=" << toString(distance)
+                  << "\n   utility=" << toString(utility)
                   << "\n";
         DEBUG_PRINT(obs);
     }
     myRelX += SPEED2DIST(xSpeed * myDir);
     myRelY += SPEED2DIST(ySpeed);
     mySpeed = xSpeed;
-    if (xSpeed > 0) {
+    if (xSpeed >= SUMO_const_haltingSpeed) {
         myWaitingToEnter = false;
         myWaitingTime = 0;
     } else {
@@ -1153,9 +1242,9 @@ MSPModel_Striping::PState::getAngle(const MSPerson::MSPersonStage_Walking&, SUMO
         return 0;
     }
     const PositionVector& shp = myWalkingAreaPath == 0 ? myLane->getShape() : myWalkingAreaPath->shape;
-    SUMOReal angle = -shp.rotationDegreeAtOffset(myRelX) + (myDir == MSPModel::BACKWARD ? 180 : 0);
-    if (angle > 180) {
-        angle -= 360;
+    SUMOReal angle = shp.rotationAtOffset(myRelX) + (myDir == MSPModel::BACKWARD ? M_PI : 0);
+    if (angle > M_PI) {
+        angle -= 2 * M_PI;
     }
     return angle;
 }
@@ -1179,6 +1268,36 @@ MSPModel_Striping::PState::getNextEdge(const MSPerson::MSPersonStage_Walking&) c
 }
 
 
+SUMOReal
+MSPModel_Striping::PState::distanceTo(const Obstacle& obs, const bool includeMinGap) const {
+    // check for overlap
+    const SUMOReal maxX = getMaxX(includeMinGap);
+    const SUMOReal minX = getMinX(includeMinGap);
+    if ((obs.xFwd >= maxX && obs.xBack <= maxX) || (obs.xFwd <= maxX && obs.xFwd >= minX)) {
+        return DIST_OVERLAP;
+    }
+    if (myDir == FORWARD) {
+        return obs.xFwd < minX ? DIST_BEHIND : obs.xBack - maxX;
+    } else {
+        return obs.xBack > maxX ? DIST_BEHIND : minX - obs.xFwd;
+    }
+}
+
+
+void
+MSPModel_Striping::PState::mergeObstacles(Obstacles& into, const Obstacles& obs2) {
+    for (int i = 0; i < (int)into.size(); ++i) {
+        if (gDebugFlag1) {
+            std::cout << "     i=" << i << " intoDist=" << distanceTo(into[i]) << " obs2Dist=" << distanceTo(obs2[i]) << "\n";
+        }
+        if (distanceTo(obs2[i]) < distanceTo(into[i])) {
+            into[i] = obs2[i];
+        }
+    }
+}
+
+
+
 // ===========================================================================
 // MSPModel_Striping::MovePedestrians method definitions
 // ===========================================================================
