@@ -5,12 +5,12 @@
 /// @author  Sascha Krieg
 /// @author  Michael Behrisch
 /// @date    Mon, 9 Jul 2001
-/// @version $Id: MSPerson.h 18756 2015-08-31 19:16:33Z behrisch $
+/// @version $Id: MSPerson.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // The class for modelling person-movements
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -80,13 +80,13 @@ public:
 
     public:
         /// constructor
-        MSPersonStage_Walking(const ConstMSEdgeVector& route, MSStoppingPlace* toBS, SUMOTime walkingTime, SUMOReal speed, SUMOReal departPos, SUMOReal arrivalPos);
+        MSPersonStage_Walking(const ConstMSEdgeVector& route, MSStoppingPlace* toStop, SUMOTime walkingTime, SUMOReal speed, SUMOReal departPos, SUMOReal arrivalPos);
 
         /// destructor
         ~MSPersonStage_Walking();
 
         /// proceeds to the next step
-        virtual void proceed(MSNet* net, MSTransportable* person, SUMOTime now, MSEdge* previousEdge, const SUMOReal at);
+        virtual void proceed(MSNet* net, MSTransportable* person, SUMOTime now, Stage* previous);
 
         /// Returns the current edge
         const MSEdge* getEdge() const;
@@ -182,12 +182,7 @@ public:
         /// @brief The current internal edge this person is on or 0
         MSEdge* myCurrentInternalEdge;
 
-        /// @brief A vector of computed times an edge is reached
-        //std::vector<SUMOTime> myArrivalTimes;
-
         SUMOReal myDepartPos;
-        SUMOReal myArrivalPos;
-        MSStoppingPlace* myDestinationBusStop;
         SUMOReal mySpeed;
 
         /// @brief state that is to be manipulated by MSPModel
@@ -224,14 +219,14 @@ public:
     class MSPersonStage_Driving : public MSTransportable::Stage {
     public:
         /// constructor
-        MSPersonStage_Driving(const MSEdge& destination, MSStoppingPlace* toBS,
-                              const std::vector<std::string>& lines);
+        MSPersonStage_Driving(const MSEdge& destination, MSStoppingPlace* toStop,
+                              const SUMOReal arrivalPos, const std::vector<std::string>& lines);
 
         /// destructor
         ~MSPersonStage_Driving();
 
         /// proceeds to the next step
-        virtual void proceed(MSNet* net, MSTransportable* person, SUMOTime now, MSEdge* previousEdge, const SUMOReal at);
+        virtual void proceed(MSNet* net, MSTransportable* person, SUMOTime now, Stage* previous);
 
         /// Returns the current edge
         const MSEdge* getEdge() const;
@@ -251,6 +246,11 @@ public:
         /// @brief Whether the person waits for a vehicle
         bool isWaiting4Vehicle() const;
 
+        /// @brief The vehicle the person is riding or 0
+        SUMOVehicle* getVehicle() const {
+            return myVehicle;
+        }
+
         /// @brief time spent waiting for a ride
         SUMOTime getWaitingTime(SUMOTime now) const;
 
@@ -293,11 +293,11 @@ public:
         /// @brief The taken vehicle
         SUMOVehicle* myVehicle;
 
-        MSStoppingPlace* myDestinationBusStop;
         SUMOReal myWaitingPos;
         /// @brief The time since which this person is waiting for a ride
         SUMOTime myWaitingSince;
         const MSEdge* myWaitingEdge;
+        Position myStopWaitPos;
 
     private:
         /// @brief Invalidated copy constructor.
@@ -340,7 +340,7 @@ public:
         }
 
         /// proceeds to the next step
-        virtual void proceed(MSNet* net, MSTransportable* person, SUMOTime now, MSEdge* previousEdge, const SUMOReal at);
+        virtual void proceed(MSNet* net, MSTransportable* person, SUMOTime now, Stage* previous);
 
         /** @brief Called on writing tripinfo output
          *
@@ -381,9 +381,6 @@ public:
         /// @brief The type of activity
         std::string myActType;
 
-        SUMOReal myStartPos;
-
-
     private:
         /// @brief Invalidated copy constructor.
         MSPersonStage_Waiting(const MSPersonStage_Waiting&);
@@ -414,6 +411,9 @@ public:
     /// @brief return the list of internal edges if this person is walking and the pedestrian model allows it
     const std::string& getNextEdge() const;
 
+    /// @brief returns the next edge ptr if this person is walking and the pedestrian model allows it
+    const MSEdge* getNextEdgePtr() const;
+
 private:
     /// @brief Invalidated copy constructor.
     MSPerson(const MSPerson&);
