@@ -4,12 +4,12 @@
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    Sept 2002
-/// @version $Id: MSActuatedTrafficLightLogic.h 18095 2015-03-17 09:39:00Z behrisch $
+/// @version $Id: MSActuatedTrafficLightLogic.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // An actuated (adaptive) traffic light logic
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2002-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2002-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -58,7 +58,7 @@ class NLDetectorBuilder;
 class MSActuatedTrafficLightLogic : public MSSimpleTrafficLightLogic {
 public:
     /// @brief Definition of a map from lanes to induct loops lying on them
-    typedef std::map<MSLane*, MSInductLoop*> InductLoopMap;
+    typedef std::map<MSLane*, MSDetectorFileOutput*> InductLoopMap;
 
 public:
     /** @brief Constructor
@@ -74,7 +74,8 @@ public:
                                 const std::string& id, const std::string& programID,
                                 const MSSimpleTrafficLightLogic::Phases& phases,
                                 unsigned int step, SUMOTime delay,
-                                const std::map<std::string, std::string>& parameter);
+                                const std::map<std::string, std::string>& parameter,
+                                const std::string& basePath);
 
 
     /** @brief Initialises the tls with information about incoming lanes
@@ -133,6 +134,15 @@ protected:
     /// Whether the detectors shall be shown in the GUI
     bool myShowDetectors;
 
+    /// The output file for generated detectors
+    std::string myFile;
+
+    /// The frequency for aggregating detector output
+    SUMOTime myFreq;
+
+    /// Whether detector output separates by vType
+    bool mySplitByType;
+
 };
 
 
