@@ -4,12 +4,12 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    08.05.2007
-/// @version $Id: MSOffTrafficLightLogic.h 18095 2015-03-17 09:39:00Z behrisch $
+/// @version $Id: MSOffTrafficLightLogic.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // A traffic lights logic which represents a tls in an off-mode
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -35,12 +35,9 @@
 #include <utility>
 #include <vector>
 #include <bitset>
-#include <microsim/MSEventControl.h>
-#include <microsim/MSNet.h>
 #include <utils/common/StdDefs.h>
-#include "MSTrafficLightLogic.h"
-#include "MSPhaseDefinition.h"
 #include "MSTLLogicControl.h"
+#include "MSTrafficLightLogic.h"
 
 
 // ===========================================================================
@@ -105,7 +102,7 @@ public:
      * @return The number of this tls program's phases
      * @see MSTrafficLightLogic::getPhaseNumber
      */
-    unsigned int getPhaseNumber() const;
+    int getPhaseNumber() const;
 
 
     /** @brief Returns the phases of this tls program
@@ -120,7 +117,7 @@ public:
      * @return The definition of the phase at the given position
      * @see MSTrafficLightLogic::getPhase
      */
-    const MSPhaseDefinition& getPhase(unsigned int givenstep) const;
+    const MSPhaseDefinition& getPhase(int givenstep) const;
 
     /** @brief Returns the type of the logic as a string
      * @return The type of the logic
@@ -139,7 +136,7 @@ public:
      * @return The index of the current phase within the tls
      * @see MSTrafficLightLogic::getCurrentPhaseIndex
      */
-    unsigned int getCurrentPhaseIndex() const;
+    int getCurrentPhaseIndex() const;
 
 
     /** @brief Returns the definition of the current phase
