@@ -3,12 +3,12 @@
 /// @author  Melanie Weber
 /// @author  Andreas Kendziorra
 /// @date    Jan 2015
-/// @version $Id: MSRailSignal.cpp 19412 2015-11-24 13:04:53Z namdre $
+/// @version $Id: MSRailSignal.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // A rail signal logic
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -232,7 +232,7 @@ MSRailSignal::updateCurrentPhase() {
 
 
 // ------------ Static Information Retrieval
-unsigned int
+int
 MSRailSignal::getPhaseNumber() const {
     return 0;
 }
@@ -243,12 +243,12 @@ MSRailSignal::getPhases() const {
 }
 
 const MSPhaseDefinition&
-MSRailSignal::getPhase(unsigned int) const {
+MSRailSignal::getPhase(int) const {
     return myCurrentPhase;
 }
 
 // ------------ Dynamic Information Retrieval
-unsigned int
+int
 MSRailSignal::getCurrentPhaseIndex() const {
     return 0;
 }
