@@ -6,12 +6,12 @@
 /// @author  Michael Behrisch
 /// @author  Friedemann Wesner
 /// @date    Sept 2002
-/// @version $Id: MSSimpleTrafficLightLogic.h 19173 2015-10-27 08:47:27Z behrisch $
+/// @version $Id: MSSimpleTrafficLightLogic.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // A fixed traffic light logic
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -78,8 +78,6 @@ public:
     /// @brief Destructor
     ~MSSimpleTrafficLightLogic();
 
-
-
     /// @name Switching and setting current rows
     /// @{
 
@@ -87,7 +85,7 @@ public:
      * @return The time of the next switch
      * @see MSTrafficLightLogic::trySwitch
      */
-    SUMOTime trySwitch();
+    virtual SUMOTime trySwitch();
     /// @}
 
 
@@ -99,7 +97,7 @@ public:
      * @return The number of this tls program's phases
      * @see MSTrafficLightLogic::getPhaseNumber
      */
-    unsigned int getPhaseNumber() const;
+    int getPhaseNumber() const;
 
 
     /** @brief Returns the phases of this tls program
@@ -121,7 +119,7 @@ public:
      * @return The definition of the phase at the given position
      * @see MSTrafficLightLogic::getPhase
      */
-    const MSPhaseDefinition& getPhase(unsigned int givenstep) const;
+    const MSPhaseDefinition& getPhase(int givenstep) const;
 
     /** @brief Returns the type of the logic as a string
      * @return The type of the logic
@@ -140,7 +138,7 @@ public:
      * @return The index of the current phase within the tls
      * @see MSTrafficLightLogic::getCurrentPhaseIndex
      */
-    unsigned int getCurrentPhaseIndex() const;
+    int getCurrentPhaseIndex() const;
 
 
     /** @brief Returns the definition of the current phase
@@ -195,7 +193,7 @@ public:
 
     /** @brief Replaces the phases and set the phase index
      */
-    void setPhases(const Phases& phases, unsigned int index);
+    void setPhases(const Phases& phases, int index);
     /// @}
 
 
