@@ -6,12 +6,12 @@
 /// @author  Michael Behrisch
 /// @author  Friedemann Wesner
 /// @date    Sept 2002
-/// @version $Id: MSTrafficLightLogic.h 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: MSTrafficLightLogic.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // The parent class for traffic light logics
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -117,7 +117,7 @@ public:
      * @param[in] lane The lane this link starts at
      * @param[in] pos The link's index (signal group) within this program
      */
-    void addLink(MSLink* link, MSLane* lane, unsigned int pos);
+    virtual void addLink(MSLink* link, MSLane* lane, int pos);
 
 
     /** @brief Applies information about controlled links and lanes from the given logic
@@ -148,10 +148,9 @@ public:
     /// @{
 
     /** @brief Switches to the next phase
-     * @param[in] isActive Whether this program is the currently used one
      * @return The time of the next switch
      */
-    virtual SUMOTime trySwitch(bool isActive) = 0;
+    virtual SUMOTime trySwitch() = 0;
 
 
     /** @brief Applies the current signal states to controlled links
@@ -189,7 +188,11 @@ public:
      * @return The lanes controlled by the signal at the given index
      */
     const LaneVector& getLanesAt(unsigned int i) const {
-        return myLanes[i];
+        if ((size_t)i < myLanes.size()) {
+            return myLanes[i];
+        } else {
+            return myEmptyLaneVector;
+        }
     }
 
 
@@ -220,7 +223,7 @@ public:
     /** @brief Returns the number of phases
      * @return The number of this tls program's phases
      */
-    virtual unsigned int getPhaseNumber() const = 0;
+    virtual int getPhaseNumber() const = 0;
 
 
     /** @brief Returns the phases of this tls program
@@ -233,7 +236,12 @@ public:
      * @param[in] givenstep The index of the phase within the plan
      * @return The definition of the phase at the given position
      */
-    virtual const MSPhaseDefinition& getPhase(unsigned int givenstep) const = 0;
+    virtual const MSPhaseDefinition& getPhase(int givenstep) const = 0;
+
+    /** @brief Returns the type of the logic as a string
+     * @return The type of the logic
+     */
+    virtual const std::string getLogicType() const = 0;
     /// @}
 
 
@@ -244,7 +252,7 @@ public:
     /** @brief Returns the current index within the program
      * @return The index of the current phase within the tls
      */
-    virtual unsigned int getCurrentPhaseIndex() const = 0;
+    virtual int getCurrentPhaseIndex() const = 0;
 
 
     /** @brief Returns the definition of the current phase
@@ -385,14 +393,15 @@ protected:
 
     };
 
+
 protected:
     /// @brief The id of the logic
     std::string myProgramID;
 
-    /// @brief The list of links which do participate in this traffic light
+    /// @brief The list of LinkVectors; each vector contains the links that belong to the same link index
     LinkVectorVector myLinks;
 
-    /// @brief The list of links which do participate in this traffic light
+    /// @brief The list of LaneVectors; each vector contains the incoming lanes that belong to the same link index
     LaneVectorVector myLanes;
 
     /// @brief A list of duration overrides
@@ -407,6 +416,13 @@ protected:
     /// @brief The cycle time (without changes)
     SUMOTime myDefaultCycleTime;
 
+    /// @brief An empty lane vector
+    static const LaneVector myEmptyLaneVector;
+
+private:
+    /// @brief initialize optional meso penalties
+    void initMesoTLSPenalties();
+
 
 private:
     /// @brief invalidated copy constructor
