@@ -3,12 +3,12 @@
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @date    02. March 2012
-/// @version $Id: NBAlgorithms.cpp 18699 2015-08-24 06:54:48Z namdre $
+/// @version $Id: NBAlgorithms.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // Algorithms for network computation
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2012-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2012-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -111,7 +111,7 @@ NBTurningDirectionsComputer::computeTurnDirectionsForNode(NBNode* node, bool war
         if (seen.find((*j).from) != seen.end() || seen.find((*j).to) != seen.end()) {
             // do not regard already set edges
             if ((*j).angle > 360 && warn) {
-                WRITE_WARNING("Ambiguity in turnarounds computation at node '" + node->getID() + "'.");
+                WRITE_WARNING("Ambiguity in turnarounds computation at junction '" + node->getID() + "'.");
                 warn = false;
             }
             continue;
