@@ -3,12 +3,12 @@
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @date    02. March 2012
-/// @version $Id: NBAlgorithms.h 17262 2014-11-04 20:41:44Z behrisch $
+/// @version $Id: NBAlgorithms.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // Algorithms for network computation
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2012-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2012-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -54,14 +54,16 @@ class NBTurningDirectionsComputer {
 public:
     /** @brief Computes turnaround destinations for all edges (if exist)
      * @param[in] nc The container of nodes to loop along
+     * @param[in] warn Whether warnings shall be issued
      */
-    static void computeTurnDirections(NBNodeCont& nc);
+    static void computeTurnDirections(NBNodeCont& nc, bool warn = true);
 
     /** @brief Computes turnaround destinations for all incoming edges of the given nodes (if any)
      * @param[in] node The node for which to compute turnaround destinations
+     * @param[in] warn Whether warnings shall be issued
      * @note: This is needed by NETEDIT
      */
-    static void computeTurnDirectionsForNode(NBNode* node);
+    static void computeTurnDirectionsForNode(NBNode* node, bool warn);
 
 private:
     /** @struct Combination
@@ -107,9 +109,9 @@ class NBNodesEdgesSorter {
 public:
     /** @brief Sorts a node's edges clockwise regarding driving direction
      * @param[in] nc The container of nodes to loop along
-     * @param[in] leftHand Whether the network is left-handed
+     * @param[in] useNodeShape Whether to sort based on the node shape (instead of only the edge angle)
      */
-    static void sortNodesEdges(NBNodeCont& nc, bool leftHand);
+    static void sortNodesEdges(NBNodeCont& nc, bool useNodeShape = false);
 
     /** @class crossing_by_junction_angle_sorter
      * @brief Sorts crossings by minimum clockwise clockwise edge angle. Use the
@@ -117,7 +119,8 @@ public:
      */
     class crossing_by_junction_angle_sorter {
     public:
-        explicit crossing_by_junction_angle_sorter(const EdgeVector& ordering) : myOrdering(ordering) {}
+        explicit crossing_by_junction_angle_sorter(const NBNode* node, const EdgeVector& ordering);
+
         int operator()(const NBNode::Crossing& c1, const NBNode::Crossing& c2) const {
             return (int)(getMinRank(c1.edges) < getMinRank(c2.edges));
         }
@@ -134,7 +137,7 @@ public:
         }
 
     private:
-        const EdgeVector& myOrdering;
+        EdgeVector myOrdering;
 
     private:
         /// @brief invalidated assignment operator
@@ -144,11 +147,10 @@ public:
 private:
     /** @brief Assures correct order for same-angle opposite-direction edges
      * @param[in] n The currently processed node
-     * @param[in] leftHand Whether the network is left-handed
      * @param[in] i1 Pointer to first edge
      * @param[in] i2 Pointer to second edge
      */
-    static void swapWhenReversed(const NBNode* const n, bool leftHand,
+    static void swapWhenReversed(const NBNode* const n,
                                  const std::vector<NBEdge*>::iterator& i1,
                                  const std::vector<NBEdge*>::iterator& i2);
 
