@@ -4,12 +4,12 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: NBConnection.cpp 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: NBConnection.cpp 20482 2016-04-18 20:49:42Z behrisch $
 ///
 // The class holds a description of a connection between two edges
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -55,8 +55,8 @@ NBConnection::NBConnection(NBEdge* from, NBEdge* to) :
     myFrom(from), myTo(to),
     myFromID(from->getID()), myToID(to->getID()),
     myFromLane(-1), myToLane(-1),
-    myTlIndex(InvalidTlIndex)
-{}
+    myTlIndex(InvalidTlIndex) {
+}
 
 
 NBConnection::NBConnection(const std::string& fromID, NBEdge* from,
@@ -64,8 +64,8 @@ NBConnection::NBConnection(const std::string& fromID, NBEdge* from,
     myFrom(from), myTo(to),
     myFromID(fromID), myToID(toID),
     myFromLane(-1), myToLane(-1),
-    myTlIndex(InvalidTlIndex)
-{}
+    myTlIndex(InvalidTlIndex) {
+}
 
 
 NBConnection::NBConnection(NBEdge* from, int fromLane,
@@ -89,8 +89,8 @@ NBConnection::NBConnection(const NBConnection& c) :
     myFrom(c.myFrom), myTo(c.myTo),
     myFromID(c.myFromID), myToID(c.myToID),
     myFromLane(c.myFromLane), myToLane(c.myToLane),
-    myTlIndex(c.myTlIndex)
-{}
+    myTlIndex(c.myTlIndex) {
+}
 
 
 NBEdge*
@@ -109,7 +109,11 @@ bool
 NBConnection::replaceFrom(NBEdge* which, NBEdge* by) {
     if (myFrom == which) {
         myFrom = by;
-        myFromID = myFrom->getID();
+        if (myFrom != 0) {
+            myFromID = myFrom->getID();
+        } else {
+            myFromID = "invalidFrom";
+        }
         return true;
     }
     return false;
@@ -119,9 +123,13 @@ NBConnection::replaceFrom(NBEdge* which, NBEdge* by) {
 bool
 NBConnection::replaceFrom(NBEdge* which, int whichLane,
                           NBEdge* by, int byLane) {
-    if (myFrom == which && (myFromLane == (int) whichLane || myFromLane < 0)) {
+    if (myFrom == which && (myFromLane == whichLane || myFromLane < 0 || whichLane < 0)) {
         myFrom = by;
-        myFromID = myFrom->getID();
+        if (myFrom != 0) {
+            myFromID = myFrom->getID();
+        } else {
+            myFromID = "invalidFrom";
+        }
         myFromLane = byLane;
         return true;
     }
@@ -133,7 +141,11 @@ bool
 NBConnection::replaceTo(NBEdge* which, NBEdge* by) {
     if (myTo == which) {
         myTo = by;
-        myToID = myTo->getID();
+        if (myTo != 0) {
+            myToID = myTo->getID();
+        } else {
+            myToID = "invalidTo";
+        }
         return true;
     }
     return false;
@@ -143,9 +155,13 @@ NBConnection::replaceTo(NBEdge* which, NBEdge* by) {
 bool
 NBConnection::replaceTo(NBEdge* which, int whichLane,
                         NBEdge* by, int byLane) {
-    if (myTo == which && (myToLane == (int) whichLane || myFromLane < 0)) {
+    if (myTo == which && (myToLane == whichLane || myFromLane < 0 || whichLane < 0)) {
         myTo = by;
-        myToID = myTo->getID();
+        if (myTo != 0) {
+            myToID = myTo->getID();
+        } else {
+            myToID = "invalidTo";
+        }
         myToLane = byLane;
         return true;
     }
@@ -238,5 +254,20 @@ NBConnection::shiftLaneIndex(NBEdge* edge, int offset) {
     }
 }
 
+
+std::ostream&
+operator<<(std::ostream& os, const NBConnection& c) {
+    os
+            << "Con(from=" << Named::getIDSecure(c.getFrom())
+            << " fromLane=" << c.getFromLane()
+            << " to=" << Named::getIDSecure(c.getTo())
+            << " toLane=" << c.getToLane()
+            << " tlIndex=" << c.getTLIndex()
+            << ")";
+    return os;
+}
+
+
+
 /****************************************************************************/
 
