@@ -4,12 +4,12 @@
 /// @author  Jakob Erdmann
 /// @author  Sascha Krieg
 /// @date    Sept 2002
-/// @version $Id: NBConnection.h 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: NBConnection.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // The class holds a description of a connection between two edges
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -121,6 +121,9 @@ public:
         return !(*this == c);
     }
 
+    /// Output operator
+    friend std::ostream& operator<<(std::ostream& os, const NBConnection& c);
+
     const static int InvalidTlIndex;
     const static NBConnection InvalidConnection;
 
