@@ -4,12 +4,12 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: NBDistrict.cpp 18639 2015-08-17 13:59:19Z namdre $
+/// @version $Id: NBDistrict.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // A class representing a single district
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -65,8 +65,8 @@ NBDistrict::~NBDistrict() {}
 // -----------  Applying offset
 void
 NBDistrict::reshiftPosition(SUMOReal xoff, SUMOReal yoff) {
-    myPosition.reshiftRotate(xoff, yoff, 0);
-    myShape.reshiftRotate(xoff, yoff, 0);
+    myPosition.add(xoff, yoff, 0);
+    myShape.add(xoff, yoff, 0);
 }
 
 
