@@ -6,12 +6,12 @@
 /// @author  Michael Behrisch
 /// @author  Laura Bieker
 /// @date    Tue, 20 Nov 2001
-/// @version $Id: NBEdge.cpp 19535 2015-12-05 13:47:18Z behrisch $
+/// @version $Id: NBEdge.cpp 20482 2016-04-18 20:49:42Z behrisch $
 ///
 // Methods for the representation of a single edge
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -366,6 +366,15 @@ NBEdge::init(unsigned int noLanes, bool tryIgnoreNodePositions, const std::strin
     // prepare container
     myLength = myFrom->getPosition().distanceTo(myTo->getPosition());
     assert(myGeom.size() >= 2);
+    if (myLanes.size() > noLanes) {
+        // remove connections targeting the removed lanes
+        const EdgeVector& incoming = myFrom->getIncomingEdges();
+        for (EdgeVector::const_iterator i = incoming.begin(); i != incoming.end(); i++) {
+            for (int lane = noLanes; lane < (int)myLanes.size(); ++lane) {
+                (*i)->removeFromConnections(this, -1, lane);
+            }
+        }
+    }
     myLanes.clear();
     for (unsigned int i = 0; i < noLanes; i++) {
         myLanes.push_back(Lane(this, origID));
@@ -690,8 +699,7 @@ NBEdge::addLane2LaneConnection(unsigned int from, NBEdge* dest,
     if (!addEdge2EdgeConnection(dest)) {
         return false;
     }
-    setConnection(from, dest, toLane, type, mayUseSameDestination, mayDefinitelyPass, keepClear, contPos);
-    return true;
+    return setConnection(from, dest, toLane, type, mayUseSameDestination, mayDefinitelyPass, keepClear, contPos);
 }
 
 
@@ -712,7 +720,7 @@ NBEdge::addLane2LaneConnections(unsigned int fromLane,
 }
 
 
-void
+bool
 NBEdge::setConnection(unsigned int lane, NBEdge* destEdge,
                       unsigned int destLane, Lane2LaneInfoType type,
                       bool mayUseSameDestination,
@@ -720,7 +728,7 @@ NBEdge::setConnection(unsigned int lane, NBEdge* destEdge,
                       bool keepClear,
                       SUMOReal contPos) {
     if (myStep == INIT_REJECT_CONNECTIONS) {
-        return;
+        return false;
     }
     // some kind of a misbehaviour which may occure when the junction's outgoing
     //  edge priorities were not properly computed, what may happen due to
@@ -732,18 +740,15 @@ NBEdge::setConnection(unsigned int lane, NBEdge* destEdge,
     //  will be refused
     //
     if (!mayUseSameDestination && hasConnectionTo(destEdge, destLane)) {
-        return;
+        return false;
     }
     if (find_if(myConnections.begin(), myConnections.end(), connections_finder(lane, destEdge, destLane)) != myConnections.end()) {
-        return;
+        return true;
     }
-    if (myLanes.size() <= lane) {
-        WRITE_ERROR("Could not set connection from '" + getLaneIDInsecure(lane) + "' to '" + destEdge->getLaneIDInsecure(destLane) + "'.");
-        return;
-    }
-    if (destEdge->getNumLanes() <= destLane) {
-        WRITE_ERROR("Could not set connection from '" + getLaneIDInsecure(lane) + "' to '" + destEdge->getLaneIDInsecure(destLane) + "'.");
-        return;
+    if (myLanes.size() <= lane || destEdge->getNumLanes() <= destLane) {
+        // problem might be corrigible in post-processing
+        WRITE_WARNING("Could not set connection from '" + getLaneIDInsecure(lane) + "' to '" + destEdge->getLaneIDInsecure(destLane) + "'.");
+        return false;
     }
     for (std::vector<Connection>::iterator i = myConnections.begin(); i != myConnections.end();) {
         if ((*i).toEdge == destEdge && ((*i).fromLane == -1 || (*i).toLane == -1)) {
@@ -772,6 +777,7 @@ NBEdge::setConnection(unsigned int lane, NBEdge* destEdge,
             }
         }
     }
+    return true;
 }
 
 
@@ -1387,7 +1393,7 @@ NBEdge::computeAngle() {
                             (myLaneSpreadFunction == LANESPREAD_RIGHT ?
                              myLanes[getNumLanes() - 1].shape
                              : myLanes[getNumLanes() / 2].shape)
-                                : myGeom);
+                            : myGeom);
 
     // if the junction shape is suspicious we cannot trust the angle to the centroid
     if ((hasFromShape && (myFrom->getShape().distance(shape[0]) > 2 * POSITION_EPS
@@ -1539,9 +1545,9 @@ NBEdge::recheckLanes() {
         //  using the standard algorithm.
         for (unsigned int i = 0; i < myLanes.size(); i++) {
             if (connNumbersPerLane[i] == 0 && !isForbidden(getPermissions((int)i))) {
-                if (i > 0 && connNumbersPerLane[i - 1] > 1) {
+                if (i > 0 && connNumbersPerLane[i - 1] > 1 && getPermissions(i) == getPermissions(i - 1)) {
                     moveConnectionToLeft(i - 1);
-                } else if (i < myLanes.size() - 1 && connNumbersPerLane[i + 1] > 1) {
+                } else if (i < myLanes.size() - 1 && connNumbersPerLane[i + 1] > 1 && getPermissions(i) == getPermissions(i + 1)) {
                     moveConnectionToRight(i + 1);
                 }
             }
@@ -1587,6 +1593,10 @@ NBEdge::recheckLanes() {
                         i = myConnections.erase(i);
                     }
                 }
+            } else if (isRailway(getPermissions(c.fromLane)) && isRailway(c.toEdge->getPermissions(c.toLane))
+                       && isTurningDirectionAt(c.toEdge))  {
+                // do not allow sharp rail turns
+                i = myConnections.erase(i);
             } else {
                 ++i;
             }
@@ -1616,9 +1626,19 @@ NBEdge::divideOnEdges(const EdgeVector* outgoing) {
 
     // build connections for miv lanes
     std::vector<int> availableLanes;
+    for (int i = 0; i < (int)myLanes.size(); ++i) {
+        if ((getPermissions(i) & SVC_PASSENGER) != 0) {
+            availableLanes.push_back(i);
+        }
+    }
+    if (availableLanes.size() > 0) {
+        divideSelectedLanesOnEdges(outgoing, availableLanes, priorities);
+    }
+    // build connections for miscellaneous further modes (more than bike,peds,bus and without passenger)
+    availableLanes.clear();
     for (int i = 0; i < (int)myLanes.size(); ++i) {
         const SVCPermissions perms = getPermissions(i);
-        if ((perms & ~(SVC_PEDESTRIAN | SVC_BICYCLE | SVC_BUS)) == 0 || isForbidden(perms)) {
+        if ((perms & ~(SVC_PEDESTRIAN | SVC_BICYCLE | SVC_BUS)) == 0 || (perms & SVC_PASSENGER) != 0 || isForbidden(perms)) {
             continue;
         }
         availableLanes.push_back(i);
@@ -1816,9 +1836,9 @@ NBEdge::computePrioritySum(const std::vector<unsigned int>& priorities) {
 
 
 void
-NBEdge::appendTurnaround(bool noTLSControlled) {
+NBEdge::appendTurnaround(bool noTLSControlled, bool checkPermissions) {
     // do nothing if no turnaround is known
-    if (myTurnDestination == 0) {
+    if (myTurnDestination == 0 || myTo->getType() == NODETYPE_RAIL_CROSSING) {
         return;
     }
     // do nothing if the destination node is controlled by a tls and no turnarounds
@@ -1826,7 +1846,20 @@ NBEdge::appendTurnaround(bool noTLSControlled) {
     if (noTLSControlled && myTo->isTLControlled()) {
         return;
     }
-    setConnection((unsigned int)(myLanes.size() - 1), myTurnDestination, myTurnDestination->getNumLanes() - 1, L2L_VALIDATED);
+    const int fromLane = (int)myLanes.size() - 1;
+    const int toLane = (int)myTurnDestination->getNumLanes() - 1;
+    if (checkPermissions) {
+        if ((getPermissions(fromLane) & myTurnDestination->getPermissions(toLane)) == 0) {
+            // exclude connection if fromLane and toEdge have no common permissions
+            return;
+        }
+        if ((getPermissions(fromLane) & myTurnDestination->getPermissions(toLane)) == SVC_PEDESTRIAN) {
+            // exclude connection if the only commonly permitted class are pedestrians
+            // these connections are later built in NBNode::buildWalkingAreas
+            return;
+        }
+    }
+    setConnection(fromLane, myTurnDestination, toLane, L2L_VALIDATED);
 }
 
 
@@ -1865,9 +1898,7 @@ NBEdge::moveOutgoingConnectionsFrom(NBEdge* e, unsigned int laneOff) {
         for (std::vector<NBEdge::Connection>::iterator j = elv.begin(); j != elv.end(); j++) {
             NBEdge::Connection el = *j;
             assert(el.tlID == "");
-            bool ok = addLane2LaneConnection(i + laneOff, el.toEdge, el.toLane, L2L_COMPUTED);
-            assert(ok);
-            UNUSED_PARAMETER(ok); // only used for assertion
+            addLane2LaneConnection(i + laneOff, el.toEdge, el.toLane, L2L_COMPUTED);
         }
     }
 }
@@ -1948,7 +1979,7 @@ NBEdge::setControllingTLInformation(const NBConnection& c, const std::string& tl
             (*i).tlLinkNo = tlIndex;
             no++;
         } else {
-            if ((*i).tlID != tlID && static_cast<int>((*i).tlLinkNo) == tlIndex) {
+            if ((*i).tlID != tlID && (*i).tlLinkNo == tlIndex) {
                 WRITE_WARNING("The lane '" + toString<int>((*i).fromLane) + "' on edge '" + getID() + "' already had a traffic light signal.");
                 hadError = true;
             }
@@ -2180,7 +2211,7 @@ NBEdge::incLaneNo(unsigned int by) {
     unsigned int newLaneNo = (unsigned int) myLanes.size() + by;
     while (myLanes.size() < newLaneNo) {
         // recompute shapes on last addition
-        const bool recompute = myLanes.size() == newLaneNo - 1;
+        const bool recompute = (myLanes.size() == newLaneNo - 1) && myStep < LANES2LANES_USER;
         addLane((unsigned int)myLanes.size(), recompute);
     }
 }
@@ -2207,7 +2238,7 @@ NBEdge::decLaneNo(unsigned int by) {
     assert(newLaneNo > 0);
     while (myLanes.size() > newLaneNo) {
         // recompute shapes on last removal
-        const bool recompute = myLanes.size() == newLaneNo + 1;
+        const bool recompute = myLanes.size() == newLaneNo + 1 && myStep < LANES2LANES_USER;
         deleteLane((unsigned int)((int)myLanes.size() - 1), recompute);
     }
 }
@@ -2514,10 +2545,31 @@ NBEdge::shiftPositionAtNode(NBNode* node, NBEdge* other) {
             PositionVector tmp = myGeom;
             // @note this doesn't work well for vissim networks
             //tmp.move2side(MIN2(neededOffset - dist, neededOffset2 - dist2));
-            tmp.move2side(neededOffset - dist);
-            myGeom[i] = tmp[i];
+            try {
+                tmp.move2side(neededOffset - dist);
+                myGeom[i] = tmp[i];
+            } catch (InvalidArgument&) {
+                WRITE_WARNING("Could not avoid overlapping shape at node '" + node->getID() + "' for edge '" + getID() + "'");
+            }
         }
     }
 }
 
+
+SUMOReal
+NBEdge::getFinalLength() const {
+    SUMOReal result = getLoadedLength();
+    if (OptionsCont::getOptions().getBool("no-internal-links") && !hasLoadedLength()) {
+        // use length to junction center even if a modified geometry was given
+        PositionVector geom = cutAtIntersection(myGeom);
+        geom.push_back_noDoublePos(getToNode()->getCenter());
+        geom.push_front_noDoublePos(getFromNode()->getCenter());
+        result = geom.length();
+    }
+    if (result <= 0) {
+        result = POSITION_EPS;
+    }
+    return result;
+}
+
 /****************************************************************************/
