@@ -4,12 +4,12 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Tue, 20 Nov 2001
-/// @version $Id: NBEdge.h 18756 2015-08-31 19:16:33Z behrisch $
+/// @version $Id: NBEdge.h 20482 2016-04-18 20:49:42Z behrisch $
 ///
 // The representation of a single edge during network building
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -42,7 +42,6 @@
 #include <utils/common/VectorHelper.h>
 #include <utils/geom/Bresenham.h>
 #include <utils/geom/PositionVector.h>
-#include <utils/geom/Line.h>
 #include <utils/common/SUMOVehicleClass.h>
 #include <utils/xml/SUMOXMLDefinitions.h>
 #include "NBCont.h"
@@ -121,9 +120,10 @@ public:
      * @brief An (internal) definition of a single lane of an edge
      */
     struct Lane {
-        Lane(NBEdge* e) :
+        Lane(NBEdge* e, const std::string& _origID) :
             speed(e->getSpeed()), permissions(SVCAll), preferred(0),
-            endOffset(e->getEndOffset()), width(e->getLaneWidth()) {}
+            endOffset(e->getEndOffset()), width(e->getLaneWidth()),
+            origID(_origID) {}
         /// @brief The lane's shape
         PositionVector shape;
         /// @brief The speed allowed on this lane
@@ -167,11 +167,13 @@ public:
         /// @brief The id of the traffic light that controls this connection
         std::string tlID;
         /// @brief The index of this connection within the controlling traffic light
-        unsigned int tlLinkNo;
+        int tlLinkNo;
         /// @brief Information about being definitely free to drive (on-ramps)
         bool mayDefinitelyPass;
         /// @brief whether the junction must be kept clear when using this connection
         bool keepClear;
+        /// @brief custom position for internal junction on this connection
+        SUMOReal contPos;
 
 
         std::string origID;
@@ -195,16 +197,14 @@ public:
 
     };
 
-    /// Computes the offset from the edge shape on the current segment
-    static std::pair<SUMOReal, SUMOReal> laneOffset(const Position& from,
-            const Position& to, SUMOReal laneCenterOffset);
-
     /// @brief unspecified lane width
     static const SUMOReal UNSPECIFIED_WIDTH;
     /// @brief unspecified lane offset
     static const SUMOReal UNSPECIFIED_OFFSET;
     /// @brief unspecified lane speed
     static const SUMOReal UNSPECIFIED_SPEED;
+    /// @brief unspecified internal junction position
+    static const SUMOReal UNSPECIFIED_CONTPOS;
 
     /// @brief no length override given
     static const SUMOReal UNSPECIFIED_LOADED_LENGTH;
@@ -255,6 +255,7 @@ public:
      * @param[in] offset Additional offset to the destination node
      * @param[in] geom The edge's geomatry
      * @param[in] streetName The street name (need not be unique)
+     * @param[in] origID The original ID in the source network (need not be unique)
      * @param[in] spread How the lateral offset of the lanes shall be computed
      * @param[in] tryIgnoreNodePositions Does not add node geometries if geom.size()>=2
      * @see init
@@ -266,6 +267,7 @@ public:
            SUMOReal width, SUMOReal offset,
            PositionVector geom,
            const std::string& streetName = "",
+           const std::string& origID = "",
            LaneSpreadFunction spread = LANESPREAD_RIGHT,
            bool tryIgnoreNodePositions = false);
 
@@ -412,6 +414,8 @@ public:
         return myLoadedLength > 0 ? myLoadedLength : myLength;
     }
 
+    /// @brief length that will be assigned to the lanes in the final network
+    SUMOReal getFinalLength() const;
 
     /** @brief Returns whether a length was set explicitly
      * @return Wether the edge's length was specified
@@ -669,7 +673,8 @@ public:
                                 unsigned int toLane, Lane2LaneInfoType type,
                                 bool mayUseSameDestination = false,
                                 bool mayDefinitelyPass = false,
-                                bool keepClear = true);
+                                bool keepClear = true,
+                                SUMOReal contPos = UNSPECIFIED_CONTPOS);
 
 
     /** @brief Builds no connections starting at the given lanes
@@ -705,12 +710,13 @@ public:
      * @param[in] mayDefinitelyPass Whether this connection is definitely undistrubed (special case for on-ramps)
      * @todo Check difference between "setConnection" and "addLane2LaneConnection"
      */
-    void setConnection(unsigned int lane, NBEdge* destEdge,
+    bool setConnection(unsigned int lane, NBEdge* destEdge,
                        unsigned int destLane,
                        Lane2LaneInfoType type,
                        bool mayUseSameDestination = false,
                        bool mayDefinitelyPass = false,
-                       bool keepClear = true);
+                       bool keepClear = true,
+                       SUMOReal contPos = UNSPECIFIED_CONTPOS);
 
 
 
@@ -724,6 +730,12 @@ public:
      */
     std::vector<Connection> getConnectionsFromLane(unsigned int lane) const;
 
+    /** @brief Returns the specified connection
+     * This method goes through "myConnections" and returns the specified one
+     * @see NBEdge::Connection
+     */
+    Connection getConnection(int fromLane, const NBEdge* to, int toLane) const;
+
 
     /** @brief Retrieves info about a connection to a certain lane of a certain edge
      *
@@ -932,7 +944,7 @@ public:
      *  of this edge to the leftmost lane of myTurnDestination).
      * @param[in] noTLSControlled Whether the turnaround shall not be connected if this edge is controlled by a tls
      */
-    void appendTurnaround(bool noTLSControlled);
+    void appendTurnaround(bool noTLSControlled, bool checkPermissions);
 
 
 
@@ -1009,7 +1021,11 @@ public:
 
     void incLaneNo(unsigned int by);
 
-    void decLaneNo(unsigned int by, int dir = 0);
+    void decLaneNo(unsigned int by);
+
+    void deleteLane(unsigned int index, bool recompute = true);
+
+    void addLane(unsigned int index, bool recompute = true);
 
     void markAsInLane2LaneState();
 
@@ -1165,9 +1181,6 @@ private:
     /// Computes the shape for the given lane
     PositionVector computeLaneShape(unsigned int lane, SUMOReal offset) const;
 
-    /// Computes the offset from the edge shape on the current segment
-    //std::pair<SUMOReal, SUMOReal> laneOffset(const Position& from, const Position& to, SUMOReal laneCenterOffset);
-
     void computeLaneShapes();
 
 
@@ -1187,8 +1200,9 @@ private:
      *
      * @param[in] noLanes The number of lanes this edge has
      * @param[in] tryIgnoreNodePositions Does not add node geometries if geom.size()>=2
+     * @param[in] origID The original ID this edge had
      */
-    void init(unsigned int noLanes, bool tryIgnoreNodePositions);
+    void init(unsigned int noLanes, bool tryIgnoreNodePositions, const std::string& origID);
 
 
     /** divides the lanes on the outgoing edges */
