@@ -5,12 +5,12 @@
 /// @author  Sascha Krieg
 /// @author  Michael Behrisch
 /// @date    Tue, 20 Nov 2001
-/// @version $Id: NBEdgeCont.cpp 18756 2015-08-31 19:16:33Z behrisch $
+/// @version $Id: NBEdgeCont.cpp 20482 2016-04-18 20:49:42Z behrisch $
 ///
 // Storage for edges, including some functionality operating on multiple edges
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -35,16 +35,12 @@
 #include <string>
 #include <cassert>
 #include <algorithm>
-#include <iostream>
-#include <fstream>
-#include <iomanip>
 #include <utils/geom/Boundary.h>
 #include <utils/geom/GeomHelper.h>
 #include <utils/geom/GeoConvHelper.h>
 #include <utils/common/MsgHandler.h>
 #include <utils/common/ToString.h>
 #include <utils/common/TplConvert.h>
-#include <utils/common/StringUtils.h>
 #include <utils/options/OptionsCont.h>
 #include "NBNetBuilder.h"
 #include "NBEdgeCont.h"
@@ -72,8 +68,8 @@ NBEdgeCont::NBEdgeCont(NBTypeCont& tc) :
     myEdgesSplit(0),
     myVehicleClasses2Keep(0),
     myVehicleClasses2Remove(0),
-    myNeedGeoTransformedPrunningBoundary(false)
-{}
+    myNeedGeoTransformedPrunningBoundary(false) {
+}
 
 
 NBEdgeCont::~NBEdgeCont() {
@@ -88,34 +84,10 @@ NBEdgeCont::applyOptions(OptionsCont& oc) {
     myRemoveEdgesAfterJoining = oc.exists("keep-edges.postload") && oc.getBool("keep-edges.postload");
     // we possibly have to load the edges to keep/remove
     if (oc.isSet("keep-edges.input-file")) {
-        std::ifstream strm(oc.getString("keep-edges.input-file").c_str());
-        if (!strm.good()) {
-            throw ProcessError("Could not load names of edges too keep from '" + oc.getString("keep-edges.input-file") + "'.");
-        }
-        while (strm.good()) {
-            std::string name;
-            strm >> name;
-            myEdges2Keep.insert(name);
-            // maybe we're loading an edge-selection
-            if (StringUtils::startsWith(name, "edge:")) {
-                myEdges2Keep.insert(name.substr(5));
-            }
-        }
+        NBHelpers::loadEdgesFromFile(oc.getString("keep-edges.input-file"), myEdges2Keep);
     }
     if (oc.isSet("remove-edges.input-file")) {
-        std::ifstream strm(oc.getString("remove-edges.input-file").c_str());
-        if (!strm.good()) {
-            throw ProcessError("Could not load names of edges too remove from '" + oc.getString("remove-edges.input-file") + "'.");
-        }
-        while (strm.good()) {
-            std::string name;
-            strm >> name;
-            myEdges2Remove.insert(name);
-            // maybe we're loading an edge-selection
-            if (StringUtils::startsWith(name, "edge:")) {
-                myEdges2Remove.insert(name.substr(5));
-            }
-        }
+        NBHelpers::loadEdgesFromFile(oc.getString("remove-edges.input-file"), myEdges2Remove);
     }
     if (oc.isSet("keep-edges.explicit")) {
         const std::vector<std::string> edges = oc.getStringVector("keep-edges.explicit");
@@ -486,8 +458,7 @@ NBEdgeCont::splitAt(NBDistrictCont& dc,
     }
 
     if (geoms.second[0] != node->getPosition()) {
-        geoms.second.pop_front();
-        geoms.second.push_front(node->getPosition());
+        geoms.second[0] = node->getPosition();
     }
     // build and insert the edges
     NBEdge* one = new NBEdge(firstEdgeName, edge->myFrom, node, edge, geoms.first, noLanesFirstEdge);
@@ -499,6 +470,15 @@ NBEdgeCont::splitAt(NBDistrictCont& dc,
     // replace information about this edge within the nodes
     edge->myFrom->replaceOutgoing(edge, one, 0);
     edge->myTo->replaceIncoming(edge, two, 0);
+    // patch tls
+    std::set<NBTrafficLightDefinition*> fromTLS = edge->myFrom->getControllingTLS();
+    for (std::set<NBTrafficLightDefinition*>::iterator i = fromTLS.begin(); i != fromTLS.end(); ++i) {
+        (*i)->replaceRemoved(edge, -1, one, -1);
+    }
+    std::set<NBTrafficLightDefinition*> toTLS = edge->myTo->getControllingTLS();
+    for (std::set<NBTrafficLightDefinition*>::iterator i = toTLS.begin(); i != toTLS.end(); ++i) {
+        (*i)->replaceRemoved(edge, -1, two, -1);
+    }
     // the edge is now occuring twice in both nodes...
     //  clean up
     edge->myFrom->removeDoubleEdges();
@@ -636,7 +616,7 @@ NBEdgeCont::recheckLanes() {
 void
 NBEdgeCont::appendTurnarounds(bool noTLSControlled) {
     for (EdgeCont::iterator i = myEdges.begin(); i != myEdges.end(); i++) {
-        (*i).second->appendTurnaround(noTLSControlled);
+        (*i).second->appendTurnaround(noTLSControlled, true);
     }
 }
 
@@ -644,7 +624,7 @@ NBEdgeCont::appendTurnarounds(bool noTLSControlled) {
 void
 NBEdgeCont::appendTurnarounds(const std::set<std::string>& ids, bool noTLSControlled) {
     for (std::set<std::string>::const_iterator it = ids.begin(); it != ids.end(); it++) {
-        myEdges[*it]->appendTurnaround(noTLSControlled);
+        myEdges[*it]->appendTurnaround(noTLSControlled, false);
     }
 }
 
@@ -747,7 +727,7 @@ NBEdgeCont::joinSameNodeConnectingEdges(NBDistrictCont& dc,
     }
     // delete joined edges
     for (i = edges.begin(); i != edges.end(); i++) {
-        erase(dc, *i);
+        extract(dc, *i, true);
     }
 }
 
@@ -774,8 +754,8 @@ NBEdgeCont::recheckLaneSpread() {
 
 // ----- other
 void
-NBEdgeCont::addPostProcessConnection(const std::string& from, int fromLane, const std::string& to, int toLane, bool mayDefinitelyPass, bool keepClear) {
-    myConnections.push_back(PostProcessConnection(from, fromLane, to, toLane, mayDefinitelyPass, keepClear));
+NBEdgeCont::addPostProcessConnection(const std::string& from, int fromLane, const std::string& to, int toLane, bool mayDefinitelyPass, bool keepClear, SUMOReal contPos) {
+    myConnections.push_back(PostProcessConnection(from, fromLane, to, toLane, mayDefinitelyPass, keepClear, contPos));
 }
 
 
@@ -784,10 +764,9 @@ NBEdgeCont::recheckPostProcessConnections() {
     for (std::vector<PostProcessConnection>::const_iterator i = myConnections.begin(); i != myConnections.end(); ++i) {
         NBEdge* from = retrievePossiblySplit((*i).from, true);
         NBEdge* to = retrievePossiblySplit((*i).to, false);
-        if (from != 0 && to != 0) {
-            if (!from->addLane2LaneConnection((*i).fromLane, to, (*i).toLane, NBEdge::L2L_USER, false, (*i).mayDefinitelyPass, (*i).keepClear)) {
-                WRITE_WARNING("Could not insert connection between '" + (*i).from + "' and '" + (*i).to + "' after build.");
-            }
+        if (from == 0 || to == 0 ||
+                !from->addLane2LaneConnection((*i).fromLane, to, (*i).toLane, NBEdge::L2L_USER, true, (*i).mayDefinitelyPass, (*i).keepClear, (*i).contPos)) {
+            WRITE_ERROR("Could not insert connection between '" + (*i).from + "' and '" + (*i).to + "' after build.");
         }
     }
     // during loading we also kept some ambiguous connections in hope they might be valid after processing
@@ -800,8 +779,8 @@ NBEdgeCont::recheckPostProcessConnections() {
         for (std::vector<NBEdge::Connection>::iterator it_con = connections.begin(); it_con != connections.end(); ++it_con) {
             NBEdge::Connection& c = *it_con;
             if (c.toEdge != 0 && c.toEdge->getFromNode() != to) {
-                WRITE_WARNING("Found and removed invalid connection from " + edge->getID() +
-                              " to " + c.toEdge->getID() + " via " + to->getID());
+                WRITE_WARNING("Found and removed invalid connection from edge '" + edge->getID() +
+                              "' to edge '" + c.toEdge->getID() + "' via junction '" + to->getID() + "'.");
                 edge->removeFromConnections(c.toEdge);
             }
         }
@@ -851,7 +830,7 @@ NBEdgeCont::getGeneratedFrom(const std::string& id) const {
 }
 
 
-void
+int
 NBEdgeCont::guessRoundabouts() {
     myGuessedRoundabouts.clear();
     std::set<NBEdge*> loadedRoundaboutEdges;
@@ -936,10 +915,25 @@ NBEdgeCont::guessRoundabouts() {
             }
         } while (doLoop);
         if (doLoop) {
-            // collected edges are marked in markRoundabouts
-            myGuessedRoundabouts.insert(EdgeSet(loopEdges.begin(), loopEdges.end()));
+            // check form factor to avoid elongated shapes (circle: 1, square: ~0.79)
+            if (formFactor(loopEdges) > 0.6) {
+                // collected edges are marked in markRoundabouts
+                myGuessedRoundabouts.insert(EdgeSet(loopEdges.begin(), loopEdges.end()));
+            }
         }
     }
+    return (int)myGuessedRoundabouts.size();
+}
+
+
+SUMOReal
+NBEdgeCont::formFactor(const EdgeVector& loopEdges) {
+    PositionVector points;
+    for (EdgeVector::const_iterator it = loopEdges.begin(); it != loopEdges.end(); ++it) {
+        points.append((*it)->getGeometry());
+    }
+    SUMOReal circumference = points.length2D();
+    return 4 * M_PI * points.area() / (circumference * circumference);
 }
 
 
@@ -994,7 +988,7 @@ NBEdgeCont::generateStreetSigns() {
         // is this a "real" junction?
         // XXX nyi
         //continue
-        const SUMOReal offset = e->getLength() - 3;
+        const SUMOReal offset = MAX2((SUMOReal)0, e->getLength() - 3);
         switch (e->getToNode()->getType()) {
             case NODETYPE_PRIORITY:
                 // yield or major?
@@ -1028,15 +1022,20 @@ NBEdgeCont::generateStreetSigns() {
 int
 NBEdgeCont::guessSidewalks(SUMOReal width, SUMOReal minSpeed, SUMOReal maxSpeed, bool fromPermissions) {
     int sidewalksCreated = 0;
+    const std::vector<std::string> edges = OptionsCont::getOptions().getStringVector("sidewalks.guess.exclude");
+    std::set<std::string> exclude(edges.begin(), edges.end());
     for (EdgeCont::iterator it = myEdges.begin(); it != myEdges.end(); it++) {
         NBEdge* edge = it->second;
-        if ((
-                    // guess.from-permissions
-                    (fromPermissions && (edge->getPermissions() & SVC_PEDESTRIAN) != 0)
-                    // guess from speed
-                    || (!fromPermissions && edge->getSpeed() > minSpeed && edge->getSpeed() <= maxSpeed))
-                // does not yet have a sidewalk
-                && edge->getPermissions(0) != SVC_PEDESTRIAN) {
+        if (// not excluded
+            exclude.count(edge->getID()) == 0
+            // does not yet have a sidewalk
+            && edge->getPermissions(0) != SVC_PEDESTRIAN
+            && (
+                // guess.from-permissions
+                (fromPermissions && (edge->getPermissions() & SVC_PEDESTRIAN) != 0)
+                // guess from speed
+                || (!fromPermissions && edge->getSpeed() > minSpeed && edge->getSpeed() <= maxSpeed)
+            )) {
             edge->addSidewalk(width);
             sidewalksCreated += 1;
         }
