@@ -4,12 +4,12 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Tue, 20 Nov 2001
-/// @version $Id: NBEdgeCont.h 18647 2015-08-18 07:09:50Z namdre $
+/// @version $Id: NBEdgeCont.h 20482 2016-04-18 20:49:42Z behrisch $
 ///
 // Storage for edges, including some functionality operating on multiple edges
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -453,8 +453,9 @@ public:
 
     /** @brief Determines which edges belong to roundabouts and increases their priority
      * @param[out] marked Edges which belong to a roundabout are stored here
+     * @return The number of guessed roundabouts
      */
-    void guessRoundabouts();
+    int guessRoundabouts();
 
 
     /** @brief Returns whether the edge with the id was ignored during parsing
@@ -487,8 +488,9 @@ public:
      * @param[in] toLane The number of the lane the connection ends at
      * @param[in] mayDefinitelyPass Whether the connection may be passed without braking
      * @param[in] keepClear Whether the connection must check to keep the junction clear
+     * @param[in] contPos Custom position for internal junction
      */
-    void addPostProcessConnection(const std::string& from, int fromLane, const std::string& to, int toLane, bool mayDefinitelyPass, bool keepClear);
+    void addPostProcessConnection(const std::string& from, int fromLane, const std::string& to, int toLane, bool mayDefinitelyPass, bool keepClear, SUMOReal contPos);
 
 
     /** @brief Try to set any stored connections
@@ -527,6 +529,9 @@ private:
     /// @brief Returns true if this edge matches one of the removal criteria
     bool ignoreFilterMatch(NBEdge* edge);
 
+    /// @brief compute the form factor for a loop of edges
+    static SUMOReal formFactor(const EdgeVector& loopEdges);
+
 private:
     /// @brief The network builder; used to obtain type information
     NBTypeCont& myTypeCont;
@@ -543,9 +548,9 @@ private:
          * @param[in] toLane The number of the lane the connection ends at
          * @param[in] mayDefinitelyPass Whether the connection may be passed without braking
          */
-        PostProcessConnection(const std::string& from_, int fromLane_, const std::string& to_, int toLane_, bool mayDefinitelyPass_, bool keepClear_)
-            : from(from_), fromLane(fromLane_), to(to_), toLane(toLane_), mayDefinitelyPass(mayDefinitelyPass_), keepClear(keepClear_)
-        { }
+        PostProcessConnection(const std::string& from_, int fromLane_, const std::string& to_, int toLane_, bool mayDefinitelyPass_, bool keepClear_, SUMOReal contPos_) :
+            from(from_), fromLane(fromLane_), to(to_), toLane(toLane_), mayDefinitelyPass(mayDefinitelyPass_), keepClear(keepClear_), contPos(contPos_) {
+        }
         /// @brief The id of the edge the connection starts at
         std::string from;
         /// @brief The number of the lane the connection starts at
@@ -558,6 +563,8 @@ private:
         bool mayDefinitelyPass;
         /// @brief Whether the connection may be passed without braking
         bool keepClear;
+        /// @brief custom position for internal junction on this connection
+        SUMOReal contPos;
     };
 
     /// @brief The list of connections to recheck
