@@ -4,12 +4,12 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    09.05.2011
-/// @version $Id: NBFrame.cpp 18478 2015-06-07 08:02:47Z namdre $
+/// @version $Id: NBFrame.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // Sets and checks options for netbuild
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -79,6 +79,9 @@ NBFrame::fillOptions(bool forNetgen) {
     oc.doRegister("default.junctions.keep-clear", new Option_Bool(true));
     oc.addDescription("default.junctions.keep-clear", "Building Defaults", "Whether junctions should be kept clear by default");
 
+    oc.doRegister("default.junctions.radius", new Option_Float(1.5));
+    oc.addDescription("default.junctions.radius", "Building Defaults", "The default turning radius of intersections");
+
     // register the data processing options
     oc.doRegister("no-internal-links", new Option_Bool(false)); // !!! not described
     oc.addDescription("no-internal-links", "Processing", "Omits internal links");
@@ -107,6 +110,13 @@ NBFrame::fillOptions(bool forNetgen) {
         oc.addSynonyme("geometry.remove", "remove-geometry", true);
         oc.addDescription("geometry.remove", "Processing", "Replace nodes which only define edge geometry by geometry points (joins edges)");
 
+        oc.doRegister("geometry.remove.keep-edges.explicit", new Option_String());
+        oc.addDescription("geometry.remove.keep-edges.explicit", "Processing", "Ensure that the given list of edges is not modified");
+
+        oc.doRegister("geometry.remove.keep-edges.input-file", new Option_FileName());
+        oc.addDescription("geometry.remove.keep-edges.input-file", "Processing",
+                          "Ensure that the edges in FILE are not modified (Each id on a single line. Selection files from SUMO-GUI are also supported)");
+
         oc.doRegister("geometry.max-segment-length", new Option_Float(0));
         oc.addDescription("geometry.max-segment-length", "Processing", "splits geometry to restrict segment length");
 
@@ -149,6 +159,10 @@ NBFrame::fillOptions(bool forNetgen) {
     oc.doRegister("lefthand", new Option_Bool(false));
     oc.addDescription("lefthand", "Processing", "Assumes left-hand traffic on the network");
 
+    oc.doRegister("edges.join", new Option_Bool(false));
+    oc.addDescription("edges.join", "Processing",
+                      "Merges edges which connect the same nodes and are close to each other (recommended for VISSIM import)");
+
     oc.doRegister("junctions.join", new Option_Bool(false));
     oc.addDescription("junctions.join", "Processing",
                       "Joins junctions that are close to each other (recommended for OSM import)");
@@ -171,6 +185,9 @@ NBFrame::fillOptions(bool forNetgen) {
     oc.doRegister("junctions.corner-detail", new Option_Integer(0));
     oc.addDescription("junctions.corner-detail", "Processing", "Generate INT intermediate points to smooth out intersection corners");
 
+    oc.doRegister("junctions.internal-link-detail", new Option_Integer(5));
+    oc.addDescription("junctions.internal-link-detail", "Processing", "Generate INT intermediate points to smooth out lanes within the intersection");
+
     oc.doRegister("check-lane-foes.roundabout", new Option_Bool(true));
     oc.addDescription("check-lane-foes.roundabout", "Processing",
                       "Allow driving onto a multi-lane road if there are foes on other lanes (at roundabouts)");
@@ -195,10 +212,18 @@ NBFrame::fillOptions(bool forNetgen) {
     oc.addDescription("sidewalks.guess.from-permissions", "Processing",
                       "Add sidewalks for edges that allow pedestrians on any of their lanes regardless of speed");
 
+    oc.doRegister("sidewalks.guess.exclude", new Option_String());
+    oc.addDescription("sidewalks.guess.exclude", "Processing",
+                      "Do not guess sidewalks for the given list of edges");
+
     oc.doRegister("crossings.guess", new Option_Bool(false));
     oc.addDescription("crossings.guess", "Processing",
                       "Guess pedestrian crossings based on the presence of sidewalks");
 
+    oc.doRegister("crossings.guess.speed-threshold", new Option_Float(13.89));
+    oc.addDescription("crossings.guess.speed-threshold", "Processing",
+                      "At uncontrolled nodes, do not build crossings across edges with a speed above the threshold");
+
     // tls setting options
     // explicit tls
     oc.doRegister("tls.set", new Option_String());
@@ -231,6 +256,10 @@ NBFrame::fillOptions(bool forNetgen) {
     oc.addDescription("tls.join-dist", "TLS Building",
                       "Determines the maximal distance for joining traffic lights (defaults to 20)");
 
+    oc.doRegister("tls.uncontrolled-within", new Option_Bool(false));
+    oc.addDescription("tls.uncontrolled-within", "TLS Building",
+                      "Do not control edges that lie fully within a joined traffic light. This may cause collisions but allows old traffic light plans to be used");
+
     if (!forNetgen) {
         oc.doRegister("tls.guess-signals", new Option_Bool(false));
         oc.addDescription("tls.guess-signals", "TLS Building", "Interprets tls nodes surrounding an intersection as signal positions for a larger TLS. This is typical pattern for OSM-derived networks");
@@ -241,6 +270,9 @@ NBFrame::fillOptions(bool forNetgen) {
 
 
     // computational
+    oc.doRegister("tls.cycle.time", new Option_Integer(90));
+    oc.addDescription("tls.cycle.time", "TLS Building", "Use INT as cycle duration");
+
     oc.doRegister("tls.green.time", new Option_Integer(31));
     oc.addSynonyme("tls.green.time", "traffic-light-green", true);
     oc.addDescription("tls.green.time", "TLS Building", "Use INT as green phase duration");
@@ -257,6 +289,9 @@ NBFrame::fillOptions(bool forNetgen) {
     oc.addSynonyme("tls.yellow.time", "traffic-light-yellow", true);
     oc.addDescription("tls.yellow.time", "TLS Building", "Set INT as fixed time for yellow phase durations");
 
+    oc.doRegister("tls.left-green.time", new Option_Integer(6));
+    oc.addDescription("tls.left-green.time", "TLS Building", "Use INT as green phase duration for left turns (s)");
+
     // tls-shifts
     oc.doRegister("tls.half-offset", new Option_String());
     oc.addSynonyme("tls.half-offset", "tl-logics.half-offset", true);
@@ -391,6 +426,14 @@ NBFrame::checkOptions() {
         WRITE_ERROR("only one of the options 'no-internal-links' or 'crossings.guess' may be given");
         ok = false;
     }
+    if (!oc.isDefault("tls.green.time") && !oc.isDefault("tls.cycle.time")) {
+        WRITE_ERROR("only one of the options 'tls.green.time' or 'tls.cycle.time' may be given");
+        ok = false;
+    }
+    if (oc.getInt("junctions.internal-link-detail") < 2) {
+        WRITE_ERROR("junctions.internal-link-detail must >= 2");
+        ok = false;
+    }
     return ok;
 }
 
