@@ -5,12 +5,12 @@
 /// @author  Sascha Krieg
 /// @author  Michael Behrisch
 /// @date    Tue, 29.05.2005
-/// @version $Id: NBLoadedTLDef.cpp 18095 2015-03-17 09:39:00Z behrisch $
+/// @version $Id: NBLoadedTLDef.cpp 20482 2016-04-18 20:49:42Z behrisch $
 ///
 // A loaded (complete) traffic light logic
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -85,12 +85,12 @@ NBLoadedTLDef::SignalGroup::sortPhases() {
 
 
 void
-NBLoadedTLDef::SignalGroup::patchTYellow(SUMOTime tyellow, bool forced) {
+NBLoadedTLDef::SignalGroup::patchTYellow(unsigned int tyellow, bool forced) {
     if (myTYellow < 0) {
         // was not set before (was not loaded)
         myTYellow = tyellow;
     } else if (forced && myTYellow < tyellow) {
-        WRITE_WARNING("TYellow of signal group '" + getID() + "' was less than the computed one; patched (was:" + toString<SUMOTime>(myTYellow) + ", is:" + toString<int>(tyellow) + ")");
+        WRITE_WARNING("TYellow of signal group '" + getID() + "' was less than the computed one; patched (was:" + toString(myTYellow) + ", is:" + toString(tyellow) + ")");
         myTYellow = tyellow;
     }
 }
@@ -265,20 +265,23 @@ NBLoadedTLDef::SignalGroup::remap(NBEdge* removed, int removedLane,
 /* -------------------------------------------------------------------------
  * NBLoadedTLDef::Phase-methods
  * ----------------------------------------------------------------------- */
-NBLoadedTLDef::NBLoadedTLDef(const std::string& id,
+NBLoadedTLDef::NBLoadedTLDef(const NBEdgeCont& ec, const std::string& id,
                              const std::vector<NBNode*>& junctions, SUMOTime offset, TrafficLightType type) :
-    NBTrafficLightDefinition(id, junctions, DefaultProgramID, offset, type)
-{}
+    NBTrafficLightDefinition(id, junctions, DefaultProgramID, offset, type),
+    myEdgeCont(&ec) {
+}
 
 
-NBLoadedTLDef::NBLoadedTLDef(const std::string& id, NBNode* junction, SUMOTime offset, TrafficLightType type) :
-    NBTrafficLightDefinition(id, junction, DefaultProgramID, offset, type)
-{}
+NBLoadedTLDef::NBLoadedTLDef(const NBEdgeCont& ec, const std::string& id, NBNode* junction, SUMOTime offset, TrafficLightType type) :
+    NBTrafficLightDefinition(id, junction, DefaultProgramID, offset, type),
+    myEdgeCont(&ec) {
+}
 
 
-NBLoadedTLDef::NBLoadedTLDef(const std::string& id, SUMOTime offset, TrafficLightType type) :
-    NBTrafficLightDefinition(id, DefaultProgramID, offset, type)
-{}
+NBLoadedTLDef::NBLoadedTLDef(const NBEdgeCont& ec, const std::string& id, SUMOTime offset, TrafficLightType type) :
+    NBTrafficLightDefinition(id, DefaultProgramID, offset, type),
+    myEdgeCont(&ec) {
+}
 
 
 NBLoadedTLDef::~NBLoadedTLDef() {
@@ -289,7 +292,7 @@ NBLoadedTLDef::~NBLoadedTLDef() {
 
 
 NBTrafficLightLogic*
-NBLoadedTLDef::myCompute(const NBEdgeCont& ec, unsigned int brakingTime) {
+NBLoadedTLDef::myCompute(unsigned int brakingTimeSeconds) {
     MsgHandler::getWarningInstance()->clear(); // !!!
     NBLoadedTLDef::SignalGroupCont::const_iterator i;
     // compute the switching times
@@ -299,7 +302,7 @@ NBLoadedTLDef::myCompute(const NBEdgeCont& ec, unsigned int brakingTime) {
         // needed later
         group->sortPhases();
         // patch the yellow time for this group
-        group->patchTYellow(brakingTime, OptionsCont::getOptions().getBool("tls.yellow.patch-small"));
+        group->patchTYellow(brakingTimeSeconds, OptionsCont::getOptions().getBool("tls.yellow.patch-small"));
         // copy the now valid times into the container
         //  both the given red and green phases are added and also the
         //  yellow times
@@ -331,44 +334,62 @@ NBLoadedTLDef::myCompute(const NBEdgeCont& ec, unsigned int brakingTime) {
         }
         // no information about yellow times will be generated
         assert((*l) >= 0);
-        logic->addStep(TIME2STEPS(duration), buildPhaseState(ec, (unsigned int)(*l)));
+        logic->addStep(TIME2STEPS(duration), buildPhaseState((unsigned int)(*l)));
     }
     // check whether any warnings were printed
     if (MsgHandler::getWarningInstance()->wasInformed()) {
         WRITE_WARNING("During computation of traffic light '" + getID() + "'.");
     }
     logic->closeBuilding();
+
+    // initialize myNeedsContRelation
+    myNeedsContRelation.clear();
+    const bool controlledWithin = !OptionsCont::getOptions().getBool("tls.uncontrolled-within");
+    const std::vector<NBTrafficLightLogic::PhaseDefinition> phases = logic->getPhases();
+    for (std::vector<NBTrafficLightLogic::PhaseDefinition>::const_iterator it = phases.begin(); it != phases.end(); it++) {
+        const std::string state = (*it).state;
+        for (NBConnectionVector::const_iterator it1 = myControlledLinks.begin(); it1 != myControlledLinks.end(); it1++) {
+            const NBConnection& c1 = *it1;
+            const int i1 = c1.getTLIndex();
+            if (i1 == NBConnection::InvalidTlIndex || state[i1] != 'g' || c1.getFrom() == 0 || c1.getTo() == 0) {
+                continue;
+            }
+            for (NBConnectionVector::const_iterator it2 = myControlledLinks.begin(); it2 != myControlledLinks.end(); it2++) {
+                const NBConnection& c2 = *it2;
+                const int i2 = c2.getTLIndex();
+                if (i2 != NBConnection::InvalidTlIndex
+                        && i2 != i1
+                        && (state[i2] == 'G' || state[i2] == 'g')
+                        && c2.getFrom() != 0 && c2.getTo() != 0) {
+                    const bool rightTurnConflict = NBNode::rightTurnConflict(
+                                                       c1.getFrom(), c1.getTo(), c1.getFromLane(), c2.getFrom(), c2.getTo(), c2.getFromLane());
+                    if (forbids(c2.getFrom(), c2.getTo(), c1.getFrom(), c1.getTo(), true, controlledWithin) || rightTurnConflict) {
+                        myNeedsContRelation.insert(StreamPair(c1.getFrom(), c1.getTo(), c2.getFrom(), c2.getTo()));
+                    }
+                }
+            }
+        }
+    }
+    myNeedsContRelationReady = true;
+
     return logic;
 }
 
 
 void
-NBLoadedTLDef::setTLControllingInformation(const NBEdgeCont& ec) const {
-    // assign the links to the connections
-    unsigned int pos = 0;
-    for (SignalGroupCont::const_iterator m = mySignalGroups.begin(); m != mySignalGroups.end(); m++) {
-        SignalGroup* group = (*m).second;
-        unsigned int linkNo = group->getLinkNo();
-        for (unsigned int j = 0; j < linkNo; j++) {
-            const NBConnection& conn = group->getConnection(j);
-            assert(conn.getFromLane() < 0 || (int) conn.getFrom()->getNumLanes() > conn.getFromLane());
-            NBConnection tst(conn);
-            tst.setTLIndex(pos);
-            if (tst.check(ec)) {
-                NBEdge* edge = conn.getFrom();
-                if (edge->setControllingTLInformation(tst, getID())) {
-                    pos++;
-                }
-            } else {
-                WRITE_WARNING("Could not set signal on connection (signal: " + getID() + ", group: " + group->getID() + ")");
-            }
+NBLoadedTLDef::setTLControllingInformation() const {
+    // assign the tl-indices to the edge connections
+    for (NBConnectionVector::const_iterator it = myControlledLinks.begin(); it != myControlledLinks.end(); it++) {
+        const NBConnection& c = *it;
+        if (c.getTLIndex() != NBConnection::InvalidTlIndex) {
+            c.getFrom()->setControllingTLInformation(c, getID());
         }
     }
 }
 
 
 std::string
-NBLoadedTLDef::buildPhaseState(const NBEdgeCont& ec, unsigned int time) const {
+NBLoadedTLDef::buildPhaseState(unsigned int time) const {
     unsigned int pos = 0;
     std::string state;
     // set the green and yellow information first;
@@ -390,7 +411,7 @@ NBLoadedTLDef::buildPhaseState(const NBEdgeCont& ec, unsigned int time) const {
             const NBConnection& conn = group->getConnection(j);
             NBConnection assConn(conn);
             // assert that the connection really exists
-            if (assConn.check(ec)) {
+            if (assConn.check(*myEdgeCont)) {
                 state = state + c;
                 ++pos;
             }
@@ -404,8 +425,8 @@ NBLoadedTLDef::buildPhaseState(const NBEdgeCont& ec, unsigned int time) const {
         for (unsigned int j = 0; j < linkNo; j++) {
             const NBConnection& conn = group->getConnection(j);
             NBConnection assConn(conn);
-            if (assConn.check(ec)) {
-                if (!mustBrake(ec, assConn, state, pos)) {
+            if (assConn.check(*myEdgeCont)) {
+                if (!mustBrake(assConn, state, pos)) {
                     if (state[pos] == 'g') {
                         state[pos] = 'G';
                     }
@@ -422,8 +443,7 @@ NBLoadedTLDef::buildPhaseState(const NBEdgeCont& ec, unsigned int time) const {
 
 
 bool
-NBLoadedTLDef::mustBrake(const NBEdgeCont& ec,
-                         const NBConnection& possProhibited,
+NBLoadedTLDef::mustBrake(const NBConnection& possProhibited,
                          const std::string& state,
                          unsigned int strmpos) const {
     // check whether the stream has red
@@ -443,7 +463,7 @@ NBLoadedTLDef::mustBrake(const NBEdgeCont& ec,
             const NBConnection& other = group->getConnection(j);
             NBConnection possProhibitor(other);
             // if the connction ist still valid ...
-            if (possProhibitor.check(ec)) {
+            if (possProhibitor.check(*myEdgeCont)) {
                 // ... do nothing if it starts at the same edge
                 if (possProhibited.getFrom() == possProhibitor.getFrom()) {
                     pos++;
@@ -497,6 +517,36 @@ NBLoadedTLDef::collectLinks() {
             }
         }
     }
+
+    // assign tl-indices to myControlledLinks
+    unsigned int pos = 0;
+    for (SignalGroupCont::const_iterator m = mySignalGroups.begin(); m != mySignalGroups.end(); m++) {
+        SignalGroup* group = (*m).second;
+        unsigned int linkNo = group->getLinkNo();
+        for (unsigned int j = 0; j < linkNo; j++) {
+            const NBConnection& conn = group->getConnection(j);
+            assert(conn.getFromLane() < 0 || (int) conn.getFrom()->getNumLanes() > conn.getFromLane());
+            NBConnection tst(conn);
+            tst.setTLIndex(pos);
+            if (tst.check(*myEdgeCont)) {
+                if (tst.getFrom()->mayBeTLSControlled(tst.getFromLane(), tst.getTo(), tst.getToLane())) {
+                    for (NBConnectionVector::iterator it = myControlledLinks.begin(); it != myControlledLinks.end(); it++) {
+                        NBConnection& c = *it;
+                        if (c.getTLIndex() == NBConnection::InvalidTlIndex
+                                && tst.getFrom() == c.getFrom() && tst.getTo() == c.getTo()
+                                && (tst.getFromLane() < 0 || tst.getFromLane() == c.getFromLane())
+                                && (tst.getToLane() < 0 || tst.getToLane() == c.getToLane())) {
+                            c.setTLIndex(pos);
+                        }
+                    }
+                    //std::cout << getID() << " group=" << (*m).first << " tst=" << tst << "\n";
+                    pos++;
+                }
+            } else {
+                WRITE_WARNING("Could not set signal on connection (signal: " + getID() + ", group: " + group->getID() + ")");
+            }
+        }
+    }
 }
 
 
@@ -599,6 +649,13 @@ NBLoadedTLDef::replaceRemoved(NBEdge* removed, int removedLane,
 }
 
 
+void
+NBLoadedTLDef::initNeedsContRelation() const {
+    if (!myNeedsContRelationReady) {
+        throw ProcessError("myNeedsContRelation was not propperly initialized\n");
+    }
+}
+
 
 /****************************************************************************/
 
