@@ -8,12 +8,12 @@
 /// @author  Michael Behrisch
 /// @author  Sascha Krieg
 /// @date    Tue, 20 Nov 2001
-/// @version $Id: NBNodeCont.cpp 18756 2015-08-31 19:16:33Z behrisch $
+/// @version $Id: NBNodeCont.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // Container for nodes during the netbuilding process
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -49,6 +49,7 @@
 #include <utils/xml/SUMOXMLDefinitions.h>
 #include <utils/geom/GeoConvHelper.h>
 #include <utils/iodevices/OutputDevice.h>
+#include "NBHelpers.h"
 #include "NBDistrict.h"
 #include "NBEdgeCont.h"
 #include "NBTrafficLightLogicCont.h"
@@ -186,12 +187,7 @@ NBNodeCont::joinSimilarEdges(NBDistrictCont& dc, NBEdgeCont& ec, NBTrafficLightL
         std::map<NBNode*, EdgeVector> connectionCount;
         const EdgeVector& outgoing = (*i).second->getOutgoingEdges();
         for (EdgeVector::const_iterator j = outgoing.begin(); j != outgoing.end(); j++) {
-            NBEdge* e = (*j);
-            NBNode* connected = e->getToNode();
-            if (connectionCount.find(connected) == connectionCount.end()) {
-                connectionCount[connected] = EdgeVector();
-            }
-            connectionCount[connected].push_back(e);
+            connectionCount[(*j)->getToNode()].push_back(*j);
         }
         // check whether more than a single edge connect another node and join them
         std::map<NBNode*, EdgeVector>::iterator k;
@@ -323,6 +319,18 @@ unsigned int
 NBNodeCont::removeUnwishedNodes(NBDistrictCont& dc, NBEdgeCont& ec,
                                 NBJoinedEdgesMap& je, NBTrafficLightLogicCont& tlc,
                                 bool removeGeometryNodes) {
+    // load edges that shall not be modified
+    std::set<std::string> edges2keep;
+    if (removeGeometryNodes) {
+        const OptionsCont& oc = OptionsCont::getOptions();
+        if (oc.isSet("geometry.remove.keep-edges.input-file")) {
+            NBHelpers::loadEdgesFromFile(oc.getString("geometry.remove.keep-edges.input-file"), edges2keep);
+        }
+        if (oc.isSet("geometry.remove.keep-edges.explicit")) {
+            const std::vector<std::string> edges = oc.getStringVector("geometry.remove.keep-edges.explicit");
+            edges2keep.insert(edges.begin(), edges.end());
+        }
+    }
     unsigned int no = 0;
     std::vector<NBNode*> toRemove;
     for (NodeCont::iterator i = myNodes.begin(); i != myNodes.end(); i++) {
@@ -342,6 +350,13 @@ NBNodeCont::removeUnwishedNodes(NBDistrictCont& dc, NBEdgeCont& ec,
                 // ok, one in, one out or two in, two out
                 //  -> ask the node whether to join
                 remove = current->checkIsRemovable();
+                // check whether any of the edges must be kept
+                for (EdgeVector::const_iterator it_edge = current->getEdges().begin(); it_edge != current->getEdges().end(); ++it_edge) {
+                    if (edges2keep.find((*it_edge)->getID()) != edges2keep.end()) {
+                        remove = false;
+                        break;
+                    }
+                }
                 if (remove) {
                     toJoin = current->getEdgesToJoin();
                 }
@@ -437,9 +452,9 @@ NBNodeCont::addJoinExclusion(const std::vector<std::string>& ids, bool check) {
         // error handling has to take place here since joinExclusions could be
         // loaded from multiple files / command line
         if (myJoined.count(*it) > 0) {
-            WRITE_WARNING("Ignoring join exclusion for node '" + *it +  "' since it already occured in a list of nodes to be joined");
+            WRITE_WARNING("Ignoring join exclusion for junction '" + *it +  "' since it already occured in a list of nodes to be joined");
         } else if (check && retrieve(*it) == 0) {
-            WRITE_WARNING("Ignoring join exclusion for unknown node '" + *it + "'");
+            WRITE_WARNING("Ignoring join exclusion for unknown junction '" + *it + "'");
         } else {
             myJoinExclusions.insert(*it);
         }
@@ -452,10 +467,10 @@ NBNodeCont::addCluster2Join(std::set<std::string> cluster) {
     // error handling has to take place here since joins could be loaded from multiple files
     for (std::set<std::string>::const_iterator it = cluster.begin(); it != cluster.end(); it++) {
         if (myJoinExclusions.count(*it) > 0) {
-            WRITE_WARNING("Ignoring join-cluster because node '" + *it + "' was already excluded from joining");
+            WRITE_WARNING("Ignoring join-cluster because junction '" + *it + "' was already excluded from joining");
             return;
         } else if (myJoined.count(*it) > 0) {
-            WRITE_WARNING("Ignoring join-cluster because node '" + *it + "' already occured in another join-cluster");
+            WRITE_WARNING("Ignoring join-cluster because junction '" + *it + "' already occured in another join-cluster");
             return;
         } else {
             myJoined.insert(*it);
@@ -474,7 +489,7 @@ NBNodeCont::joinLoadedClusters(NBDistrictCont& dc, NBEdgeCont& ec, NBTrafficLigh
         for (std::set<std::string>::iterator it_id = it->begin(); it_id != it->end(); it_id++) {
             NBNode* node = retrieve(*it_id);
             if (node == 0) {
-                WRITE_WARNING("Ignoring unknown node '" + *it_id + "' while joining");
+                WRITE_WARNING("Ignoring unknown junction '" + *it_id + "' while joining");
             } else {
                 cluster.insert(node);
             }
@@ -717,7 +732,7 @@ NBNodeCont::joinNodeClusters(NodeClusters clusters,
                         (*l)->replaceInConnections(e, e->getConnections());
                     }
                 }
-                ec.erase(dc, e);
+                ec.extract(dc, e, true);
                 allEdges.erase(j++); // erase does not invalidate the other iterators
             } else {
                 ++j;
@@ -829,7 +844,7 @@ NBNodeCont::guessTLs(OptionsCont& oc, NBTrafficLightLogicCont& tlc) {
         for (std::vector<std::string>::const_iterator i = notTLControlledNodes.begin(); i != notTLControlledNodes.end(); ++i) {
             NBNode* n = NBNodeCont::retrieve(*i);
             if (n == 0) {
-                throw ProcessError(" The node '" + *i + "' to set as not-controlled is not known.");
+                throw ProcessError(" The junction '" + *i + "' to set as not-controlled is not known.");
             }
             std::set<NBTrafficLightDefinition*> tls = n->getControllingTLS();
             for (std::set<NBTrafficLightDefinition*>::const_iterator j = tls.begin(); j != tls.end(); ++j) {
@@ -869,6 +884,9 @@ NBNodeCont::guessTLs(OptionsCont& oc, NBTrafficLightLogicCont& tlc) {
         // check which nodes should be controlled
         for (std::map<std::string, NBNode*>::const_iterator i = myNodes.begin(); i != myNodes.end(); ++i) {
             NBNode* node = i->second;
+            if (find(ncontrolled.begin(), ncontrolled.end(), node) != ncontrolled.end()) {
+                continue;
+            }
             const EdgeVector& incoming = node->getIncomingEdges();
             const EdgeVector& outgoing = node->getOutgoingEdges();
             if (!node->isTLControlled() && incoming.size() > 1 && !node->geometryLike()) {
@@ -1037,7 +1055,7 @@ NBNodeCont::setAsTLControlled(NBNode* node, NBTrafficLightLogicCont& tlc,
     NBTrafficLightDefinition* tlDef = new NBOwnTLDef(id, node, 0, type);
     if (!tlc.insert(tlDef)) {
         // actually, nothing should fail here
-        WRITE_WARNING("Building a tl-logic for node '" + id + "' twice is not possible.");
+        WRITE_WARNING("Building a tl-logic for junction '" + id + "' twice is not possible.");
         delete tlDef;
         return;
     }
@@ -1099,6 +1117,7 @@ NBNodeCont::printBuiltNodesStatistics() const {
     int numPriorityJunctions = 0;
     int numRightBeforeLeftJunctions = 0;
     int numAllWayStopJunctions = 0;
+    int numZipperJunctions = 0;
     int numRailSignals = 0;
     for (NodeCont::const_iterator i = myNodes.begin(); i != myNodes.end(); i++) {
         switch ((*i).second->getType()) {
@@ -1112,6 +1131,8 @@ NBNodeCont::printBuiltNodesStatistics() const {
             case NODETYPE_PRIORITY:
             case NODETYPE_PRIORITY_STOP:
             case NODETYPE_TRAFFIC_LIGHT:
+            case NODETYPE_TRAFFIC_LIGHT_RIGHT_ON_RED:
+            case NODETYPE_RAIL_CROSSING:
                 ++numPriorityJunctions;
                 break;
             case NODETYPE_RIGHT_BEFORE_LEFT:
@@ -1120,6 +1141,9 @@ NBNodeCont::printBuiltNodesStatistics() const {
             case NODETYPE_ALLWAY_STOP:
                 ++numAllWayStopJunctions;
                 break;
+            case NODETYPE_ZIPPER:
+                ++numZipperJunctions;
+                break;
             case NODETYPE_DISTRICT:
                 ++numRightBeforeLeftJunctions;
                 break;
@@ -1142,6 +1166,9 @@ NBNodeCont::printBuiltNodesStatistics() const {
     if (numAllWayStopJunctions > 0) {
         WRITE_MESSAGE("  All-way stop junctions      : " + toString(numAllWayStopJunctions));
     }
+    if (numZipperJunctions > 0) {
+        WRITE_MESSAGE("  Zipper-merge junctions      : " + toString(numZipperJunctions));
+    }
     if (numRailSignals > 0) {
         WRITE_MESSAGE("  Rail signal junctions      : " + toString(numRailSignals));
     }
