@@ -5,12 +5,12 @@
 /// @author  Sascha Krieg
 /// @author  Michael Behrisch
 /// @date    Tue, 29.05.2005
-/// @version $Id: NBOwnTLDef.cpp 18673 2015-08-20 19:22:23Z behrisch $
+/// @version $Id: NBOwnTLDef.cpp 20482 2016-04-18 20:49:42Z behrisch $
 ///
 // A traffic light logics which must be computed (only nodes/edges are given)
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -48,6 +48,7 @@
 #include <foreign/nvwa/debug_new.h>
 #endif // CHECK_MEMORY_LEAKS
 
+#define MIN_GREEN_TIME 5
 
 // ===========================================================================
 // member method definitions
@@ -56,22 +57,22 @@ NBOwnTLDef::NBOwnTLDef(const std::string& id,
                        const std::vector<NBNode*>& junctions, SUMOTime offset,
                        TrafficLightType type) :
     NBTrafficLightDefinition(id, junctions, DefaultProgramID, offset, type),
-    myHaveSinglePhase(false)
-{}
+    myHaveSinglePhase(false) {
+}
 
 
 NBOwnTLDef::NBOwnTLDef(const std::string& id, NBNode* junction, SUMOTime offset,
                        TrafficLightType type) :
     NBTrafficLightDefinition(id, junction, DefaultProgramID, offset, type),
-    myHaveSinglePhase(false)
-{}
+    myHaveSinglePhase(false) {
+}
 
 
 NBOwnTLDef::NBOwnTLDef(const std::string& id, SUMOTime offset,
                        TrafficLightType type) :
     NBTrafficLightDefinition(id, DefaultProgramID, offset, type),
-    myHaveSinglePhase(false)
-{}
+    myHaveSinglePhase(false) {
+}
 
 
 NBOwnTLDef::~NBOwnTLDef() {}
@@ -114,7 +115,7 @@ NBOwnTLDef::computeUnblockedWeightedStreamNumber(const NBEdge* const e1, const N
                     if (e2->getTurnDestination() == (*e2c).toEdge) {
                         continue;
                     }
-                    if (!foes(e1, (*e1c).toEdge, e2, (*e2c).toEdge)) {
+                    if (!forbids(e1, (*e1c).toEdge, e2, (*e2c).toEdge, true)) {
                         val += getDirectionalWeight(e1->getToNode()->getDirection(e1, (*e1c).toEdge));
                         val += getDirectionalWeight(e2->getToNode()->getDirection(e2, (*e2c).toEdge));
                     }
@@ -182,15 +183,16 @@ NBOwnTLDef::getBestPair(EdgeVector& incoming) {
 }
 
 NBTrafficLightLogic*
-NBOwnTLDef::myCompute(const NBEdgeCont&, unsigned int brakingTimeSeconds) {
+NBOwnTLDef::myCompute(unsigned int brakingTimeSeconds) {
     return computeLogicAndConts(brakingTimeSeconds);
 }
 
 NBTrafficLightLogic*
 NBOwnTLDef::computeLogicAndConts(unsigned int brakingTimeSeconds, bool onlyConts) {
     myNeedsContRelation.clear();
+    myRightOnRedConflicts.clear();
     const SUMOTime brakingTime = TIME2STEPS(brakingTimeSeconds);
-    const SUMOTime leftTurnTime = TIME2STEPS(6); // make configurable
+    const SUMOTime leftTurnTime = TIME2STEPS(OptionsCont::getOptions().getInt("tls.left-green.time"));
     // build complete lists first
     const EdgeVector& incoming = getIncomingEdges();
     EdgeVector fromEdges, toEdges;
@@ -236,9 +238,12 @@ NBOwnTLDef::computeLogicAndConts(unsigned int brakingTimeSeconds, bool onlyConts
     }
 
     NBTrafficLightLogic* logic = new NBTrafficLightLogic(getID(), getProgramID(), noLinksAll, myOffset, myType);
-    EdgeVector toProc = incoming;
-    const SUMOTime greenTime = TIME2STEPS(OptionsCont::getOptions().getInt("tls.green.time"));
+    EdgeVector toProc = getConnectedOuterEdges(incoming);
+    const int greenSeconds = OptionsCont::getOptions().getInt("tls.green.time");
+    const SUMOTime greenTime = TIME2STEPS(greenSeconds);
     // build all phases
+    std::vector<int> greenPhases; // indices of green phases
+    std::vector<bool> hadGreenMajor(noLinksAll, false);
     while (toProc.size() > 0) {
         std::pair<NBEdge*, NBEdge*> chosen;
         if (incoming.size() == 2) {
@@ -257,6 +262,7 @@ NBOwnTLDef::computeLogicAndConts(unsigned int brakingTimeSeconds, bool onlyConts
         }
         unsigned int pos = 0;
         std::string state((size_t) noLinksAll, 'r');
+        //std::cout << " computing " << getID() << " prog=" << getProgramID() << " cho1=" << Named::getIDSecure(chosen.first) << " cho2=" << Named::getIDSecure(chosen.second) << " toProc=" << toString(toProc) << "\n";
         // plain straight movers
         for (unsigned int i1 = 0; i1 < (unsigned int) incoming.size(); ++i1) {
             NBEdge* fromEdge = incoming[i1];
@@ -277,7 +283,9 @@ NBOwnTLDef::computeLogicAndConts(unsigned int brakingTimeSeconds, bool onlyConts
                 }
             }
         }
+        //std::cout << " state after plain straight movers=" << state << "\n";
         // correct behaviour for those that are not in chosen, but may drive, though
+        state = allowFollowersOfChosen(state, fromEdges, toEdges);
         for (unsigned int i1 = 0; i1 < pos; ++i1) {
             if (state[i1] == 'G') {
                 continue;
@@ -293,43 +301,32 @@ NBOwnTLDef::computeLogicAndConts(unsigned int brakingTimeSeconds, bool onlyConts
                 state[i1] = 'G';
             }
         }
+        //std::cout << " state after finding additional 'G's=" << state << "\n";
         // correct behaviour for those that have to wait (mainly left-mover)
         bool haveForbiddenLeftMover = false;
         std::vector<bool> rightTurnConflicts(pos, false);
+        state = correctConflicting(state, fromEdges, toEdges, isTurnaround, fromLanes, hadGreenMajor, haveForbiddenLeftMover, rightTurnConflicts);
         for (unsigned int i1 = 0; i1 < pos; ++i1) {
-            if (state[i1] != 'G') {
-                continue;
-            }
-            for (unsigned int i2 = 0; i2 < pos; ++i2) {
-                if ((state[i2] == 'G' || state[i2] == 'g')) {
-                    if (NBNode::rightTurnConflict(
-                                fromEdges[i1], toEdges[i1], fromLanes[i1], fromEdges[i2], toEdges[i2], fromLanes[i2])) {
-                        rightTurnConflicts[i1] = true;
-                    }
-                    if (forbids(fromEdges[i2], toEdges[i2], fromEdges[i1], toEdges[i1], true) || rightTurnConflicts[i1]) {
-                        state[i1] = 'g';
-                        myNeedsContRelation.insert(StreamPair(fromEdges[i1], toEdges[i1], fromEdges[i2], toEdges[i2]));
-                        if (!isTurnaround[i1]) {
-                            haveForbiddenLeftMover = true;
-                        }
-                    }
-                }
+            if (state[i1] == 'G') {
+                hadGreenMajor[i1] = true;
             }
         }
+        //std::cout << " state after correcting left movers=" << state << "\n";
         const std::string vehicleState = state; // backup state before pedestrian modifications
+        greenPhases.push_back((int)logic->getPhases().size());
         state = addPedestrianPhases(logic, greenTime, state, crossings, fromEdges, toEdges);
         // pedestrians have 'r' from here on
         for (unsigned int i1 = pos; i1 < pos + crossings.size(); ++i1) {
             state[i1] = 'r';
         }
-
+        const bool buildLeftGreenPhase = haveForbiddenLeftMover && !myHaveSinglePhase && leftTurnTime > 0;
         if (brakingTime > 0) {
             // build yellow (straight)
             for (unsigned int i1 = 0; i1 < pos; ++i1) {
                 if (state[i1] != 'G' && state[i1] != 'g') {
                     continue;
                 }
-                if ((vehicleState[i1] >= 'a' && vehicleState[i1] <= 'z') && haveForbiddenLeftMover && !rightTurnConflicts[i1]) {
+                if ((vehicleState[i1] >= 'a' && vehicleState[i1] <= 'z') && buildLeftGreenPhase && !rightTurnConflicts[i1]) {
                     continue;
                 }
                 state[i1] = 'y';
@@ -338,7 +335,7 @@ NBOwnTLDef::computeLogicAndConts(unsigned int brakingTimeSeconds, bool onlyConts
             logic->addStep(brakingTime, state);
         }
 
-        if (haveForbiddenLeftMover && !myHaveSinglePhase) {
+        if (buildLeftGreenPhase) {
             // build left green
             for (unsigned int i1 = 0; i1 < pos; ++i1) {
                 if (state[i1] == 'Y' || state[i1] == 'y') {
@@ -349,6 +346,9 @@ NBOwnTLDef::computeLogicAndConts(unsigned int brakingTimeSeconds, bool onlyConts
                     state[i1] = 'G';
                 }
             }
+            state = allowFollowersOfChosen(state, fromEdges, toEdges);
+            state = correctConflicting(state, fromEdges, toEdges, isTurnaround, fromLanes, hadGreenMajor, haveForbiddenLeftMover, rightTurnConflicts);
+
             // add step
             logic->addStep(leftTurnTime, state);
 
@@ -365,7 +365,44 @@ NBOwnTLDef::computeLogicAndConts(unsigned int brakingTimeSeconds, bool onlyConts
             }
         }
     }
-    const SUMOTime totalDuration = logic->getDuration();
+    // fix pedestrian crossings that did not get the green light yet
+    if (crossings.size() > 0) {
+        addPedestrianScramble(logic, noLinksAll, TIME2STEPS(10), brakingTime, crossings, fromEdges, toEdges);
+    }
+
+    SUMOTime totalDuration = logic->getDuration();
+    if (OptionsCont::getOptions().isDefault("tls.green.time") || !OptionsCont::getOptions().isDefault("tls.cycle.time")) {
+        const SUMOTime cycleTime = TIME2STEPS(OptionsCont::getOptions().getInt("tls.cycle.time"));
+        // adapt to cycle time by changing the duration of the green phases
+        SUMOTime greenPhaseTime = 0;
+        SUMOTime minGreenDuration = SUMOTime_MAX;
+        for (std::vector<int>::const_iterator it = greenPhases.begin(); it != greenPhases.end(); ++it) {
+            const SUMOTime dur = logic->getPhases()[*it].duration;
+            greenPhaseTime += dur;
+            minGreenDuration = MIN2(minGreenDuration, dur);
+        }
+        const int patchSeconds = (int)(STEPS2TIME(cycleTime - totalDuration) / greenPhases.size());
+        const int patchSecondsRest = (int)(STEPS2TIME(cycleTime - totalDuration)) - patchSeconds * (int)greenPhases.size();
+        //std::cout << "cT=" << cycleTime << " td=" << totalDuration << " pS=" << patchSeconds << " pSR=" << patchSecondsRest << "\n";
+        if (STEPS2TIME(minGreenDuration) + patchSeconds < MIN_GREEN_TIME
+                || STEPS2TIME(minGreenDuration) + patchSeconds + patchSecondsRest < MIN_GREEN_TIME
+                || greenPhases.size() == 0) {
+            if (getID() != DummyID) {
+                WRITE_WARNING("The traffic light '" + getID() + "' cannot be adapted to a cycle time of " + time2string(cycleTime) + ".");
+            }
+            // @todo use a multiple of cycleTime ?
+        } else {
+            for (std::vector<int>::const_iterator it = greenPhases.begin(); it != greenPhases.end(); ++it) {
+                logic->setPhaseDuration(*it, logic->getPhases()[*it].duration + TIME2STEPS(patchSeconds));
+            }
+            if (greenPhases.size() > 0) {
+                logic->setPhaseDuration(greenPhases.front(), logic->getPhases()[greenPhases.front()].duration + TIME2STEPS(patchSecondsRest));
+            }
+            totalDuration = logic->getDuration();
+        }
+    }
+
+    myRightOnRedConflictsReady = true;
     // this computation only makes sense for single nodes
     myNeedsContRelationReady = (myControlledNodes.size() == 1);
     if (totalDuration > 0) {
@@ -383,12 +420,11 @@ NBOwnTLDef::computeLogicAndConts(unsigned int brakingTimeSeconds, bool onlyConts
 
 bool
 NBOwnTLDef::hasCrossing(const NBEdge* from, const NBEdge* to, const std::vector<NBNode::Crossing>& crossings) {
-    assert(from != 0);
     assert(to != 0);
     for (std::vector<NBNode::Crossing>::const_iterator it = crossings.begin(); it != crossings.end(); it++) {
         const NBNode::Crossing& cross = *it;
         // only check connections at this crossings node
-        if (from->getToNode() == cross.node) {
+        if (to->getFromNode() == cross.node) {
             for (EdgeVector::const_iterator it_e = cross.edges.begin(); it_e != cross.edges.end(); ++it_e) {
                 const NBEdge* edge = *it_e;
                 if (edge == from || edge == to) {
@@ -498,7 +534,7 @@ NBOwnTLDef::setParticipantsInformation() {
 
 
 void
-NBOwnTLDef::setTLControllingInformation(const NBEdgeCont&) const {
+NBOwnTLDef::setTLControllingInformation() const {
     // set the information about the link's positions within the tl into the
     //  edges the links are starting at, respectively
     for (NBConnectionVector::const_iterator j = myControlledLinks.begin(); j != myControlledLinks.end(); ++j) {
@@ -521,29 +557,142 @@ NBOwnTLDef::replaceRemoved(NBEdge* /*removed*/, int /*removedLane*/,
 
 void
 NBOwnTLDef::initNeedsContRelation() const {
-    if (!myNeedsContRelationReady) {
+    if (!myNeedsContRelationReady && !amInvalid()) {
         assert(myControlledNodes.size() > 0);
-        // there are basically 2 cases for controlling multiple nodes
-        // a) a complex (unjoined) intersection. Here, internal junctions should
-        // not be needed since real nodes are used instead
-        // b) two far-away junctions which shall be coordinated
-        // This is likely to mess up the bestPair computation for each
-        // individual node and thus generate incorrect needsCont data
-        //
-        // Therefore we compute needsCont for individual nodes which doesn't
-        // matter for a) and is better for b)
+        // we use a dummy node just to maintain const-correctness
         myNeedsContRelation.clear();
+        NBOwnTLDef dummy(DummyID, myControlledNodes, 0, TLTYPE_STATIC);
+        dummy.setParticipantsInformation();
+        dummy.computeLogicAndConts(0, true);
+        myNeedsContRelation = dummy.myNeedsContRelation;
         for (std::vector<NBNode*>::const_iterator i = myControlledNodes.begin(); i != myControlledNodes.end(); i++) {
-            NBNode* n = *i;
-            NBOwnTLDef dummy("dummy", n, 0, TLTYPE_STATIC);
-            dummy.setParticipantsInformation();
-            dummy.computeLogicAndConts(0, true);
-            myNeedsContRelation.insert(dummy.myNeedsContRelation.begin(), dummy.myNeedsContRelation.end());
-            n->removeTrafficLight(&dummy);
+            (*i)->removeTrafficLight(&dummy);
         }
         myNeedsContRelationReady = true;
     }
+}
 
+
+EdgeVector
+NBOwnTLDef::getConnectedOuterEdges(const EdgeVector& incoming) {
+    EdgeVector result = incoming;
+    for (EdgeVector::iterator it = result.begin(); it != result.end();) {
+        if ((*it)->getConnections().size() == 0 || (*it)->isInnerEdge()) {
+            it = result.erase(it);
+        } else {
+            ++it;
+        }
+    }
+    return result;
+}
+
+
+std::string
+NBOwnTLDef::allowFollowersOfChosen(std::string state, const EdgeVector& fromEdges, const EdgeVector& toEdges) {
+    bool check = true;
+    while (check) {
+        check = false;
+        for (int i1 = 0; i1 < (int)fromEdges.size(); ++i1) {
+            if (state[i1] == 'G') {
+                continue;
+            }
+            bool followsChosen = false;
+            for (int i2 = 0; i2 < (int)fromEdges.size() && !followsChosen; ++i2) {
+                if (state[i2] == 'G' && fromEdges[i1] == toEdges[i2]) {
+                    followsChosen = true;
+                }
+            }
+            if (followsChosen) {
+                state[i1] = 'G';
+                check = true;
+            }
+        }
+    }
+    return state;
+}
+
+
+std::string
+NBOwnTLDef::correctConflicting(std::string state, const EdgeVector& fromEdges, const EdgeVector& toEdges,
+                               const std::vector<bool>& isTurnaround,
+                               const std::vector<int>& fromLanes,
+                               const std::vector<bool>& hadGreenMajor,
+                               bool& haveForbiddenLeftMover,
+                               std::vector<bool>& rightTurnConflicts) {
+    const bool controlledWithin = !OptionsCont::getOptions().getBool("tls.uncontrolled-within");
+    for (int i1 = 0; i1 < (int)fromEdges.size(); ++i1) {
+        if (state[i1] == 'G') {
+            for (int i2 = 0; i2 < (int)fromEdges.size(); ++i2) {
+                if ((state[i2] == 'G' || state[i2] == 'g')) {
+                    if (NBNode::rightTurnConflict(
+                                fromEdges[i1], toEdges[i1], fromLanes[i1], fromEdges[i2], toEdges[i2], fromLanes[i2])) {
+                        rightTurnConflicts[i1] = true;
+                    }
+                    if (forbids(fromEdges[i2], toEdges[i2], fromEdges[i1], toEdges[i1], true, controlledWithin) || rightTurnConflicts[i1]) {
+                        state[i1] = 'g';
+                        myNeedsContRelation.insert(StreamPair(fromEdges[i1], toEdges[i1], fromEdges[i2], toEdges[i2]));
+                        if (!isTurnaround[i1] && !hadGreenMajor[i1]) {
+                            haveForbiddenLeftMover = true;
+                        }
+                    }
+                }
+            }
+        }
+        if (state[i1] == 'r') {
+            if (fromEdges[i1]->getToNode()->getType() == NODETYPE_TRAFFIC_LIGHT_RIGHT_ON_RED &&
+                    fromEdges[i1]->getToNode()->getDirection(fromEdges[i1], toEdges[i1]) == LINKDIR_RIGHT) {
+                // handle right-on-red conflicts
+                state[i1] = 's';
+                for (int i2 = 0; i2 < (int)fromEdges.size(); ++i2) {
+                    if (state[i2] == 'G' && !isTurnaround[i2] &&
+                            (forbids(fromEdges[i2], toEdges[i2], fromEdges[i1], toEdges[i1], true) || forbids(fromEdges[i1], toEdges[i1], fromEdges[i2], toEdges[i2], true))) {
+                        myRightOnRedConflicts.insert(std::make_pair(i1, i2));
+                    }
+                }
+            }
+        }
+    }
+    return state;
+}
+
+
+void
+NBOwnTLDef::addPedestrianScramble(NBTrafficLightLogic* logic, unsigned int noLinksAll, SUMOTime /* greenTime */, SUMOTime brakingTime,
+                                  const std::vector<NBNode::Crossing>& crossings, const EdgeVector& fromEdges, const EdgeVector& toEdges) {
+    const int vehLinks = noLinksAll - (int)crossings.size();
+    std::vector<bool> foundGreen(crossings.size(), false);
+    const std::vector<NBTrafficLightLogic::PhaseDefinition>& phases = logic->getPhases();
+    for (int i = 0; i < (int)phases.size(); ++i) {
+        const std::string state = phases[i].state;
+        for (int j = 0; j < (int)crossings.size(); ++j) {
+            LinkState ls = (LinkState)state[vehLinks + j];
+            if (ls == LINKSTATE_TL_GREEN_MAJOR || ls == LINKSTATE_TL_GREEN_MINOR) {
+                foundGreen[j] = true;
+            }
+        }
+    }
+    for (int j = 0; j < (int)foundGreen.size(); ++j) {
+        if (!foundGreen[j]) {
+
+            // add a phase where all pedestrians may walk, (preceded by a yellow phase and followed by a clearing phase)
+            if (phases.size() > 0) {
+                bool needYellowPhase = false;
+                std::string state = phases.back().state;
+                for (int i1 = 0; i1 < vehLinks; ++i1) {
+                    if (state[i1] == 'G' || state[i1] == 'g') {
+                        state[i1] = 'y';
+                        needYellowPhase = true;
+                    }
+                }
+                // add yellow step
+                if (needYellowPhase) {
+                    logic->addStep(brakingTime, state);
+                }
+            }
+            addPedestrianPhases(logic, TIME2STEPS(10), std::string(noLinksAll, 'r'), crossings, fromEdges, toEdges);
+            break;
+        }
+    }
 }
 
 /****************************************************************************/
