@@ -4,12 +4,12 @@
 /// @author  Jakob Erdmann
 /// @author  Sascha Krieg
 /// @date    Tue, 29.05.2005
-/// @version $Id: NBOwnTLDef.h 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: NBOwnTLDef.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // A traffic light logics which must be computed (only nodes/edges are given)
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -107,10 +107,9 @@ public:
 
 
     /** @brief Informs edges about being controlled by a tls
-     * @param[in] ec The container of edges
      * @see NBTrafficLightDefinition::setTLControllingInformation
      */
-    void setTLControllingInformation(const NBEdgeCont& ec) const;
+    void setTLControllingInformation() const;
     /// @}
 
 
@@ -120,6 +119,11 @@ public:
         myHaveSinglePhase = true;
     }
 
+    /// @brief add an additional pedestrian phase if there are crossings that did not get green yet
+    static void addPedestrianScramble(NBTrafficLightLogic* logic, unsigned int noLinksAll,
+                                      SUMOTime greenTime, SUMOTime yellowTime,
+                                      const std::vector<NBNode::Crossing>& crossings, const EdgeVector& fromEdges, const EdgeVector& toEdges);
+
     /// @brief add 1 or 2 phases depending on the presence of pedestrian crossings
     static std::string addPedestrianPhases(NBTrafficLightLogic* logic, SUMOTime greenTime,
                                            std::string state, const std::vector<NBNode::Crossing>& crossings, const EdgeVector& fromEdges, const EdgeVector& toEdges);
@@ -128,18 +132,26 @@ public:
     static std::string patchStateForCrossings(const std::string& state,
             const std::vector<NBNode::Crossing>& crossings, const EdgeVector& fromEdges, const EdgeVector& toEdges);
 
+    /** @brief helper function for myCompute
+     * @param[in] brakingTime Duration a vehicle needs for braking in front of the tls
+     * @param[in] onlyConts whether the method is only called to compute myNeedsContRelation
+     * @return The computed logic
+     */
+    NBTrafficLightLogic* computeLogicAndConts(unsigned int brakingTimeSeconds, bool onlyConts = false);
+
+    /* initialize myNeedsContRelation and set myNeedsContRelationReady to true */
+    void initNeedsContRelation() const;
+
 protected:
     /// @name Protected methods from NBTrafficLightDefinition-interface
     /// @{
 
     /** @brief Computes the traffic light logic finally in dependence to the type
-     * @param[in] ec The edge container
      * @param[in] brakingTime Duration a vehicle needs for braking in front of the tls
      * @return The computed logic
      * @see NBTrafficLightDefinition::myCompute
      */
-    NBTrafficLightLogic* myCompute(const NBEdgeCont& ec,
-                                   unsigned int brakingTimeSeconds);
+    NBTrafficLightLogic* myCompute(unsigned int brakingTimeSeconds);
 
 
     /** @brief Collects the nodes participating in this traffic light
@@ -208,6 +220,33 @@ protected:
     std::pair<NBEdge*, NBEdge*> getBestPair(EdgeVector& incoming);
 
 
+    /// @brief compute whether the given connection is crossed by pedestrians
+    static bool hasCrossing(const NBEdge* from, const NBEdge* to, const std::vector<NBNode::Crossing>& crossings);
+
+    /// @brief get edges that have connections
+    static EdgeVector getConnectedOuterEdges(const EdgeVector& incoming);
+
+
+    /// @brief allow connections that follow on of the chosen edges
+    std::string allowFollowersOfChosen(std::string state, const EdgeVector& fromEdges, const EdgeVector& toEdges);
+
+    /** @brief change 'G' to 'g' for conflicting connections
+     * @param[in] state
+     * @param[in] fromEdges
+     * @param[in] toEdges
+     * @param[in] isTurnaround
+     * @param[in] fromLanes
+     * @param[in] hadGreenMajor
+     * @param[out] haveForbiddenLeftMover
+     * @param[out] rightTurnConflicts
+     * @return The corrected state
+     */
+    std::string correctConflicting(std::string state, const EdgeVector& fromEdges, const EdgeVector& toEdges,
+                                   const std::vector<bool>& isTurnaround,
+                                   const std::vector<int>& fromLanes,
+                                   const std::vector<bool>& hadGreenMajor,
+                                   bool& haveForbiddenLeftMover, std::vector<bool>& rightTurnConflicts);
+
     /** @class edge_by_incoming_priority_sorter
      * @brief Sorts edges by their priority within the node they end at
      */
@@ -225,11 +264,11 @@ protected:
         }
     };
 
+
 private:
     /// @brief Whether left-mover should not have an additional phase
     bool myHaveSinglePhase;
 
-
 };
 
 
