@@ -4,12 +4,12 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: NBTrafficLightDefinition.cpp 18155 2015-03-26 09:57:55Z namdre $
+/// @version $Id: NBTrafficLightDefinition.cpp 20482 2016-04-18 20:49:42Z behrisch $
 ///
 // The base class for traffic light logic definitions
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -52,6 +52,7 @@
 // static members
 // ===========================================================================
 const std::string NBTrafficLightDefinition::DefaultProgramID = "0";
+const std::string NBTrafficLightDefinition::DummyID = "dummy";
 
 // ===========================================================================
 // method definitions
@@ -63,7 +64,8 @@ NBTrafficLightDefinition::NBTrafficLightDefinition(const std::string& id,
     myControlledNodes(junctions),
     mySubID(programID), myOffset(offset),
     myType(type),
-    myNeedsContRelationReady(false) {
+    myNeedsContRelationReady(false),
+    myRightOnRedConflictsReady(false) {
     std::vector<NBNode*>::iterator i = myControlledNodes.begin();
     while (i != myControlledNodes.end()) {
         for (std::vector<NBNode*>::iterator j = i + 1; j != myControlledNodes.end();) {
@@ -88,7 +90,8 @@ NBTrafficLightDefinition::NBTrafficLightDefinition(const std::string& id,
     mySubID(programID),
     myOffset(offset),
     myType(type),
-    myNeedsContRelationReady(false) {
+    myNeedsContRelationReady(false),
+    myRightOnRedConflictsReady(false) {
     addNode(junction);
 }
 
@@ -99,15 +102,16 @@ NBTrafficLightDefinition::NBTrafficLightDefinition(const std::string& id, const
     mySubID(programID),
     myOffset(offset),
     myType(type),
-    myNeedsContRelationReady(false)
-{}
+    myNeedsContRelationReady(false),
+    myRightOnRedConflictsReady(false) {
+}
 
 
 NBTrafficLightDefinition::~NBTrafficLightDefinition() {}
 
 
 NBTrafficLightLogic*
-NBTrafficLightDefinition::compute(const NBEdgeCont& ec, OptionsCont& oc) {
+NBTrafficLightDefinition::compute(OptionsCont& oc) {
     // it is not really a traffic light if no incoming edge exists
     if (amInvalid()) {
         // make a copy of myControlledNodes because it will be modified;
@@ -125,7 +129,7 @@ NBTrafficLightDefinition::compute(const NBEdgeCont& ec, OptionsCont& oc) {
     if (oc.isSet("tls.yellow.time")) {
         brakingTime = oc.getInt("tls.yellow.time");
     }
-    NBTrafficLightLogic* ret = myCompute(ec, brakingTime);
+    NBTrafficLightLogic* ret = myCompute(brakingTime);
     ret->addParameter(getMap());
     return ret;
 }
@@ -151,10 +155,31 @@ NBTrafficLightDefinition::setParticipantsInformation() {
     collectLinks();
 }
 
+std::set<NBEdge*>
+NBTrafficLightDefinition::collectReachable(EdgeVector outer, const EdgeVector& within, bool checkControlled) {
+    std::set<NBEdge*> reachable;
+    while (outer.size() > 0) {
+        NBEdge* from = outer.back();
+        outer.pop_back();
+        std::vector<NBEdge::Connection>& cons = from->getConnections();
+        for (std::vector<NBEdge::Connection>::iterator k = cons.begin(); k != cons.end(); k++) {
+            NBEdge* to = (*k).toEdge;
+            if (reachable.count(to) == 0 &&
+                    (find(within.begin(), within.end(), to) != within.end()) &&
+                    (!checkControlled || from->mayBeTLSControlled((*k).fromLane, to, (*k).toLane))) {
+                reachable.insert(to);
+                outer.push_back(to);
+            }
+        }
+    }
+    return reachable;
+}
+
 
 void
 NBTrafficLightDefinition::collectEdges() {
     myIncomingEdges.clear();
+    myEdgesWithin.clear();
     EdgeVector myOutgoing;
     // collect the edges from the participating nodes
     for (std::vector<NBNode*>::iterator i = myControlledNodes.begin(); i != myControlledNodes.end(); i++) {
@@ -163,22 +188,40 @@ NBTrafficLightDefinition::collectEdges() {
         const EdgeVector& outgoing = (*i)->getOutgoingEdges();
         copy(outgoing.begin(), outgoing.end(), back_inserter(myOutgoing));
     }
+    EdgeVector outer;
     // check which of the edges are completely within the junction
-    //  remove these edges from the list of incoming edges
     //  add them to the list of edges lying within the node
-    for (EdgeVector::iterator j = myIncomingEdges.begin(); j != myIncomingEdges.end();) {
+    for (EdgeVector::iterator j = myIncomingEdges.begin(); j != myIncomingEdges.end(); ++j) {
         NBEdge* edge = *j;
         // an edge lies within the logic if it is outgoing as well as incoming
         EdgeVector::iterator k = find(myOutgoing.begin(), myOutgoing.end(), edge);
         if (k != myOutgoing.end()) {
-            if (myControlledInnerEdges.count(edge->getID()) == 0) {
-                myEdgesWithin.push_back(edge);
-                (*j)->setIsInnerEdge();
-                j = myIncomingEdges.erase(j);
-                continue;
+            myEdgesWithin.push_back(edge);
+        } else  {
+            outer.push_back(edge);
+        }
+    }
+    // collect edges that are reachable from the outside via controlled connections
+    std::set<NBEdge*> reachable = collectReachable(outer, myEdgesWithin, true);
+    // collect edges that are reachable from the outside regardless of controllability
+    std::set<NBEdge*> reachable2 = collectReachable(outer, myEdgesWithin, false);
+
+    const bool uncontrolledWithin = OptionsCont::getOptions().getBool("tls.uncontrolled-within");
+    for (EdgeVector::iterator j = myEdgesWithin.begin(); j != myEdgesWithin.end(); ++j) {
+        NBEdge* edge = *j;
+        // edges that are marked as 'inner' will not get their own phase when
+        // computing traffic light logics (unless they cannot be reached from the outside at all)
+        if (reachable.count(edge) == 1) {
+            edge->setIsInnerEdge();
+            // legacy behavior
+            if (uncontrolledWithin && myControlledInnerEdges.count(edge->getID()) == 0) {
+                myIncomingEdges.erase(find(myIncomingEdges.begin(), myIncomingEdges.end(), edge));
             }
         }
-        ++j;
+        if (reachable2.count(edge) == 0 && edge->getFirstNonPedestrianLaneIndex(NBNode::FORWARD, true) >= 0
+                && getID() != DummyID) {
+            WRITE_WARNING("Unreachable edge '" + edge->getID() + "' within tlLogic '" + getID() + "'");
+        }
     }
 }
 
@@ -193,7 +236,8 @@ NBTrafficLightDefinition::mustBrake(const NBEdge* const from, const NBEdge* cons
     if (!node->hasOutgoing(to)) {
         return true; // !!!
     }
-    return node->mustBrake(from, to, -1, true);
+    // @todo recheck relevance of lane indices
+    return node->mustBrake(from, to, -1, -1, true);
 }
 
 
@@ -224,7 +268,8 @@ NBTrafficLightDefinition::forbids(const NBEdge* const possProhibitorFrom,
                                   const NBEdge* const possProhibitorTo,
                                   const NBEdge* const possProhibitedFrom,
                                   const NBEdge* const possProhibitedTo,
-                                  bool regardNonSignalisedLowerPriority) const {
+                                  bool regardNonSignalisedLowerPriority,
+                                  bool sameNodeOnly) const {
     if (possProhibitorFrom == 0 || possProhibitorTo == 0 || possProhibitedFrom == 0 || possProhibitedTo == 0) {
         return false;
     }
@@ -237,7 +282,11 @@ NBTrafficLightDefinition::forbids(const NBEdge* const possProhibitorFrom,
     NBNode* incnode = *incoming;
     NBNode* outnode = *outgoing;
     EdgeVector::const_iterator i;
+
     if (incnode != outnode) {
+        if (sameNodeOnly) {
+            return false;
+        }
         // the links are located at different nodes
         const EdgeVector& ev1 = possProhibitedTo->getConnectedEdges();
         // go through the following edge,
@@ -252,8 +301,11 @@ NBTrafficLightDefinition::forbids(const NBEdge* const possProhibitorFrom,
             if (incnode != outnode2) {
                 continue;
             }
-            bool ret1 = incnode->foes(possProhibitorTo, *i,
-                                      possProhibitedFrom, possProhibitedTo);
+            if (incnode->getDirection(possProhibitedTo, *i) != LINKDIR_STRAIGHT) {
+                continue;
+            }
+            bool ret1 = incnode->foes(possProhibitorFrom, possProhibitorTo,
+                                      possProhibitedTo, *i);
             bool ret2 = incnode->forbids(possProhibitorFrom, possProhibitorTo,
                                          possProhibitedTo, *i,
                                          regardNonSignalisedLowerPriority);
@@ -276,6 +328,9 @@ NBTrafficLightDefinition::forbids(const NBEdge* const possProhibitorFrom,
             if (incnode2 != outnode) {
                 continue;
             }
+            if (incnode2->getDirection(possProhibitorTo, *i) != LINKDIR_STRAIGHT) {
+                continue;
+            }
             bool ret1 = incnode2->foes(possProhibitorTo, *i,
                                        possProhibitedFrom, possProhibitedTo);
             bool ret2 = incnode2->forbids(possProhibitorTo, *i,
@@ -324,8 +379,8 @@ NBTrafficLightDefinition::addNode(NBNode* node) {
     if (std::find(myControlledNodes.begin(), myControlledNodes.end(), node) == myControlledNodes.end()) {
         myControlledNodes.push_back(node);
         std::sort(myControlledNodes.begin(), myControlledNodes.end(), NBNode::nodes_by_id_sorter());
-        node->addTrafficLight(this);
     }
+    node->addTrafficLight(this);
 }
 
 
@@ -360,6 +415,7 @@ NBTrafficLightDefinition::getIncomingEdges() const {
 void
 NBTrafficLightDefinition::collectAllLinks() {
     myControlledLinks.clear();
+    int tlIndex = 0;
     // build the list of links which are controled by the traffic light
     for (EdgeVector::iterator i = myIncomingEdges.begin(); i != myIncomingEdges.end(); i++) {
         NBEdge* incoming = *i;
@@ -372,8 +428,11 @@ NBTrafficLightDefinition::collectAllLinks() {
                     if (el.toEdge != 0 && el.toLane >= (int) el.toEdge->getNumLanes()) {
                         throw ProcessError("Connection '" + incoming->getID() + "_" + toString(j) + "->" + el.toEdge->getID() + "_" + toString(el.toLane) + "' yields in a not existing lane.");
                     }
-                    int tlIndex = (int)myControlledLinks.size();
-                    myControlledLinks.push_back(NBConnection(incoming, el.fromLane, el.toEdge, el.toLane, tlIndex));
+                    if (incoming->getToNode()->getType() != NODETYPE_RAIL_CROSSING || !isRailway(incoming->getPermissions())) {
+                        myControlledLinks.push_back(NBConnection(incoming, el.fromLane, el.toEdge, el.toLane, tlIndex++));
+                    } else {
+                        myControlledLinks.push_back(NBConnection(incoming, el.fromLane, el.toEdge, el.toLane, -1));
+                    }
                 }
             }
         }
@@ -395,7 +454,7 @@ NBTrafficLightDefinition::needsCont(const NBEdge* fromE, const NBEdge* toE, cons
 void
 NBTrafficLightDefinition::initNeedsContRelation() const {
     if (!amInvalid()) {
-        NBOwnTLDef dummy("dummy", myControlledNodes, 0, TLTYPE_STATIC);
+        NBOwnTLDef dummy(DummyID, myControlledNodes, 0, TLTYPE_STATIC);
         dummy.initNeedsContRelation();
         myNeedsContRelation = dummy.myNeedsContRelation;
         for (std::vector<NBNode*>::const_iterator i = myControlledNodes.begin(); i != myControlledNodes.end(); i++) {
@@ -405,5 +464,21 @@ NBTrafficLightDefinition::initNeedsContRelation() const {
     myNeedsContRelationReady = true;
 }
 
+
+bool
+NBTrafficLightDefinition::rightOnRedConflict(int index, int foeIndex) const {
+    if (!myRightOnRedConflictsReady) {
+        NBOwnTLDef dummy(DummyID, myControlledNodes, 0, TLTYPE_STATIC);
+        dummy.setParticipantsInformation();
+        dummy.computeLogicAndConts(0, true);
+        myRightOnRedConflicts = dummy.myRightOnRedConflicts;
+        for (std::vector<NBNode*>::const_iterator i = myControlledNodes.begin(); i != myControlledNodes.end(); i++) {
+            (*i)->removeTrafficLight(&dummy);
+        }
+        myRightOnRedConflictsReady = true;
+    }
+    return std::find(myRightOnRedConflicts.begin(), myRightOnRedConflicts.end(), std::make_pair(index, foeIndex)) != myRightOnRedConflicts.end();
+}
+
 /****************************************************************************/
 
