@@ -5,12 +5,12 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Mar, 2003
-/// @version $Id: netgen_main.cpp 18095 2015-03-17 09:39:00Z behrisch $
+/// @version $Id: netgen_main.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // Main for NETGENERATE
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -205,13 +205,14 @@ main(int argc, char** argv) {
     OptionsCont& oc = OptionsCont::getOptions();
     // give some application descriptions
     oc.setApplicationDescription("Road network generator for the microscopic road traffic simulation SUMO.");
-    oc.setApplicationName("netgenerate", "SUMO netgenerate Version " + getBuildName(VERSION_STRING));
+    oc.setApplicationName("netgenerate", "SUMO netgenerate Version " VERSION_STRING);
     int ret = 0;
     try {
         // initialise the application system (messaging, xml, options)
         XMLSubSys::init();
         fillOptions();
-        OptionsIO::getOptions(true, argc, argv);
+        OptionsIO::setArgs(argc, argv);
+        OptionsIO::getOptions();
         if (oc.processMetaOptions(argc < 2)) {
             SystemFrame::close();
             return 0;
