@@ -4,12 +4,12 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Tue, 20 Nov 2001
-/// @version $Id: NIFrame.cpp 17249 2014-11-04 11:03:05Z namdre $
+/// @version $Id: NIFrame.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // Sets and checks options for netimport
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -38,7 +38,6 @@
 #include <utils/options/OptionsIO.h>
 #include <utils/common/MsgHandler.h>
 #include <utils/common/ToString.h>
-#include "NIFrame.h"
 #include <utils/common/FileHelpers.h>
 #include <utils/common/UtilExceptions.h>
 #include <utils/common/RandHelper.h>
@@ -46,6 +45,7 @@
 #include <netwrite/NWFrame.h>
 #include <utils/common/SystemFrame.h>
 #include "NIImporter_DlrNavteq.h"
+#include "NIFrame.h"
 
 #ifdef CHECK_MEMORY_LEAKS
 #include <foreign/nvwa/debug_new.h>
@@ -62,6 +62,7 @@ NIFrame::fillOptions() {
     oc.doRegister("sumo-net-file", 's', new Option_FileName());
     oc.addSynonyme("sumo-net-file", "sumo-net", true);
     oc.addDescription("sumo-net-file", "Input", "Read SUMO-net from FILE");
+    oc.addXMLDefault("sumo-net-file", "net");
 
     oc.doRegister("node-files", 'n', new Option_FileName());
     oc.addSynonyme("node-files", "xml-node-files", true);
@@ -133,6 +134,9 @@ NIFrame::fillOptions() {
     oc.doRegister("heightmap.shapefiles", new Option_FileName());
     oc.addDescription("heightmap.shapefiles", "Input", "Read heightmap from ArcGIS shapefile");
 
+    oc.doRegister("heightmap.geotiff", new Option_FileName());
+    oc.addDescription("heightmap.geotiff", "Input", "Read heightmap from GeoTIFF");
+
     // register basic processing options
     oc.doRegister("ignore-errors", new Option_Bool(false));
     oc.addSynonyme("ignore-errors", "dismiss-loading-errors", true);
@@ -248,9 +252,6 @@ NIFrame::fillOptions() {
     oc.doRegister("osm.skip-duplicates-check", new Option_Bool(false));
     oc.addDescription("osm.skip-duplicates-check", "Processing", "Skips the check for duplicate nodes and edges");
 
-    oc.doRegister("osm.railway.oneway-default", new Option_Bool(true));
-    oc.addDescription("osm.railway.oneway-default", "Processing", "Imports railway edges as one-way by default");
-
     oc.doRegister("osm.elevation", new Option_Bool(false));
     oc.addDescription("osm.elevation", "Processing", "Imports elevation data");
 
@@ -260,9 +261,6 @@ NIFrame::fillOptions() {
     oc.doRegister("opendrive.ignore-widths", new Option_Bool(false));
     oc.addDescription("opendrive.ignore-widths", "Processing", "Whether lane widths shall be ignored.");
 
-
-
-
     // register some additional options
     oc.doRegister("tls.discard-loaded", new Option_Bool(false));
     oc.addDescription("tls.discard-loaded", "TLS Building", "Does not instatiate traffic lights loaded from other formats than XML");
@@ -304,6 +302,20 @@ NIFrame::checkOptions() {
             oc.set("offset.disable-normalization", "true");
         }
     }
+    if (!oc.isSet("type-files")) {
+        const char* sumoPath = std::getenv("SUMO_HOME");
+        if (sumoPath == 0) {
+            WRITE_WARNING("Environment variable SUMO_HOME is not set, using built in type maps.");
+        } else {
+            const std::string path = sumoPath + std::string("/data/typemap/");
+            if (oc.isSet("osm-files")) {
+                oc.set("type-files", path + "osmNetconvert.typ.xml");
+            }
+            if (oc.isSet("opendrive-files")) {
+                oc.set("type-files", path + "opendriveNetconvert.typ.xml");
+            }
+        }
+    }
     return ok;
 }
 
