@@ -5,12 +5,12 @@
 /// @author  Michael Behrisch
 /// @author  Walter Bamberger
 /// @date    Mon, 14.04.2008
-/// @version $Id: NIImporter_OpenStreetMap.cpp 18611 2015-08-06 14:49:39Z namdre $
+/// @version $Id: NIImporter_OpenStreetMap.cpp 20482 2016-04-18 20:49:42Z behrisch $
 ///
 // Importer for networks stored in OpenStreetMap format
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -258,12 +258,12 @@ NIImporter_OpenStreetMap::insertNodeChecking(long long int id, NBNodeCont& nc, N
         NIOSMNode* n = myOSMNodes.find(id)->second;
         Position pos(n->lon, n->lat, n->ele);
         if (!NBNetBuilder::transformCoordinates(pos, true)) {
-            WRITE_ERROR("Unable to project coordinates for node " + toString(id) + ".");
+            WRITE_ERROR("Unable to project coordinates for junction '" + toString(id) + "'.");
             return 0;
         }
         node = new NBNode(toString(id), pos);
         if (!nc.insert(node)) {
-            WRITE_ERROR("Could not insert node '" + toString(id) + "').");
+            WRITE_ERROR("Could not insert junction '" + toString(id) + "'.");
             delete node;
             return 0;
         }
@@ -295,7 +295,7 @@ NIImporter_OpenStreetMap::insertEdge(Edge* e, int index, NBNode* from, NBNode* t
     // patch the id
     std::string id = toString(e->id);
     if (from == 0 || to == 0) {
-        WRITE_ERROR("Discarding edge " + id + " because the nodes could not be built.");
+        WRITE_ERROR("Discarding edge '" + id + "' because the nodes could not be built.");
         return index;
     }
     if (index >= 0) {
@@ -304,14 +304,18 @@ NIImporter_OpenStreetMap::insertEdge(Edge* e, int index, NBNode* from, NBNode* t
         index = 0;
     }
     if (from == to) {
-        // in the special case of a looped way split again using passed
         assert(passed.size() >= 2);
-        std::vector<long long int> geom(passed);
-        geom.pop_back(); // remove to-node
-        NBNode* intermediate = insertNodeChecking(geom.back(), nc, tlsc);
-        index = insertEdge(e, index, from, intermediate, geom, nb);
-        geom.clear();
-        return insertEdge(e, index, intermediate, to, geom, nb);
+        if (passed.size() == 2) {
+            WRITE_WARNING("Discarding edge '" + id + "' which connects two identical nodes without geometry.");
+            return index;
+        }
+        // in the special case of a looped way split again using passed
+        size_t intermediateIndex = passed.size() / 2;
+        NBNode* intermediate = insertNodeChecking(passed[intermediateIndex], nc, tlsc);
+        std::vector<long long int> part1(passed.begin(), passed.begin() + intermediateIndex);
+        std::vector<long long int> part2(passed.begin() + intermediateIndex + 1, passed.end());
+        index = insertEdge(e, index, from, intermediate, part1, nb);
+        return insertEdge(e, index, intermediate, to, part2, nb);
     }
     const int newIndex = index + 1;
 
@@ -322,7 +326,7 @@ NIImporter_OpenStreetMap::insertEdge(Edge* e, int index, NBNode* from, NBNode* t
         NIOSMNode* n = myOSMNodes.find(*i)->second;
         Position pos(n->lon, n->lat, n->ele);
         if (!NBNetBuilder::transformCoordinates(pos, true)) {
-            WRITE_ERROR("Unable to project coordinates for edge " + id + ".");
+            WRITE_ERROR("Unable to project coordinates for edge '" + id + "'.");
         }
         shape.push_back_noDoublePos(pos);
     }
@@ -345,11 +349,11 @@ NIImporter_OpenStreetMap::insertEdge(Edge* e, int index, NBNode* from, NBNode* t
                         types.push_back(t);
                     }
                 } else if (tok.size() > 1) {
-                    WRITE_WARNING("Discarding unknown compound \"" + t + "\" in type \"" + type + "\" (first occurence for edge \"" + id + "\").");
+                    WRITE_WARNING("Discarding unknown compound '" + t + "' in type '" + type + "' (first occurence for edge '" + id + "').");
                 }
             }
             if (types.size() == 0) {
-                WRITE_WARNING("Discarding unusable type \"" + type + "\" (first occurence for edge \"" + id + "\").");
+                WRITE_WARNING("Discarding unusable type '" + type + "' (first occurence for edge '" + id + "').");
                 myUnusableTypes.insert(type);
                 return newIndex;
             } else {
@@ -387,11 +391,11 @@ NIImporter_OpenStreetMap::insertEdge(Edge* e, int index, NBNode* from, NBNode* t
                         width = MAX2(width, SUMO_const_laneWidth);
                     }
                     if (discard) {
-                        WRITE_WARNING("Discarding compound type \"" + newType + "\" (first occurence for edge \"" + id + "\").");
+                        WRITE_WARNING("Discarding compound type '" + newType + "' (first occurence for edge '" + id + "').");
                         myUnusableTypes.insert(newType);
                         return newIndex;
                     } else {
-                        WRITE_MESSAGE("Adding new type \"" + type + "\" (first occurence for edge \"" + id + "\").");
+                        WRITE_MESSAGE("Adding new type '" + type + "' (first occurence for edge '" + id + "').");
                         tc.insert(newType, numLanes, maxSpeed, prio, permissions, width, defaultIsOneWay, sidewalkWidth, bikelaneWidth);
                         for (std::vector<std::string>::iterator it = types.begin(); it != types.end(); it++) {
                             if (!tc.getShallBeDiscarded(*it)) {
@@ -497,10 +501,12 @@ NIImporter_OpenStreetMap::insertEdge(Edge* e, int index, NBNode* from, NBNode* t
 
     if (ok) {
         LaneSpreadFunction lsf = addBackward ? LANESPREAD_RIGHT : LANESPREAD_CENTER;
+        id = StringUtils::escapeXML(id);
         if (addForward) {
             assert(numLanesForward > 0);
-            NBEdge* nbe = new NBEdge(StringUtils::escapeXML(id), from, to, type, speed, numLanesForward, tc.getPriority(type),
-                                     forwardWidth, NBEdge::UNSPECIFIED_OFFSET, shape, StringUtils::escapeXML(e->streetName), lsf, true);
+            NBEdge* nbe = new NBEdge(id, from, to, type, speed, numLanesForward, tc.getPriority(type),
+                                     forwardWidth, NBEdge::UNSPECIFIED_OFFSET, shape,
+                                     StringUtils::escapeXML(e->streetName), toString(e->id), lsf, true);
             nbe->setPermissions(forwardPermissions);
             if ((e->myBuswayType & WAY_FORWARD) != 0) {
                 nbe->setPermissions(SVC_BUS, 0);
@@ -515,12 +521,12 @@ NIImporter_OpenStreetMap::insertEdge(Edge* e, int index, NBNode* from, NBNode* t
                 delete nbe;
                 throw ProcessError("Could not add edge '" + id + "'.");
             }
-            id = "-" + id;
         }
         if (addBackward) {
             assert(numLanesBackward > 0);
-            NBEdge* nbe = new NBEdge(StringUtils::escapeXML(id), to, from, type, speed, numLanesBackward, tc.getPriority(type),
-                                     backwardWidth, NBEdge::UNSPECIFIED_OFFSET, shape.reverse(), StringUtils::escapeXML(e->streetName), lsf, true);
+            NBEdge* nbe = new NBEdge("-" + id, to, from, type, speed, numLanesBackward, tc.getPriority(type),
+                                     backwardWidth, NBEdge::UNSPECIFIED_OFFSET, shape.reverse(),
+                                     StringUtils::escapeXML(e->streetName), toString(e->id), lsf, true);
             nbe->setPermissions(backwardPermissions);
             if ((e->myBuswayType & WAY_BACKWARD) != 0) {
                 nbe->setPermissions(SVC_BUS, 0);
@@ -533,7 +539,7 @@ NIImporter_OpenStreetMap::insertEdge(Edge* e, int index, NBNode* from, NBNode* t
             }
             if (!ec.insert(nbe)) {
                 delete nbe;
-                throw ProcessError("Could not add edge " + id + "'.");
+                throw ProcessError("Could not add edge '-" + id + "'.");
             }
         }
     }
@@ -554,8 +560,8 @@ NIImporter_OpenStreetMap::NodesHandler::NodesHandler(
     myIsInValidNodeTag(false),
     myHierarchyLevel(0),
     myUniqueNodes(uniqueNodes),
-    myImportElevation(importElevation)
-{ }
+    myImportElevation(importElevation) {
+}
 
 
 NIImporter_OpenStreetMap::NodesHandler::~NodesHandler() {}
