@@ -4,12 +4,12 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Mon, 14.04.2008
-/// @version $Id: NIImporter_SUMO.h 18319 2015-04-29 06:49:01Z namdre $
+/// @version $Id: NIImporter_SUMO.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // Importer for networks stored in SUMO format
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -191,6 +191,8 @@ private:
         bool mayDefinitelyPass;
         /// @brief Whether the junction must be kept clear coming from this connection
         bool keepClear;
+        /// @brief custom position for internal junction on this connection
+        SUMOReal contPos;
     };
 
 
@@ -262,7 +264,7 @@ private:
      * @brief Describes a pedestrian crossing
      */
     struct Crossing {
-        std::string laneID;
+        std::string edgeID;
         std::vector<std::string> crossingEdges;
         SUMOReal width;
         bool priority;
@@ -315,6 +317,15 @@ private:
     /// @brief whether the loaded network contains internal lanes
     bool myHaveSeenInternalEdge;
 
+    /// @brief whether the loaded network was built for lefthand traffic
+    bool myAmLefthand;
+
+    /// @brief the level of corner detail in the loaded network
+    int myCornerDetail;
+
+    /// @brief the level of geometry detail for internal lanes in the loaded network
+    int myLinkDetail;
+
     /// @brief loaded roundabout edges
     std::vector<std::vector<std::string> > myRoundabouts;
 
