@@ -4,12 +4,12 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: NIVissimConnectionCluster.cpp 18095 2015-03-17 09:39:00Z behrisch $
+/// @version $Id: NIVissimConnectionCluster.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // -------------------
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -422,11 +422,7 @@ NIVissimConnectionCluster::isWeakDistrictConnRealisation(NIVissimConnectionClust
     if (oe == 0 || ie == 0) {
         return false;
     }
-    Line l1(oe->getGeometry().front(), oe->getGeometry().back());
-    Line l2(ie->getGeometry().back(), ie->getGeometry().front());
-    SUMOReal a1 = l1.atan2DegreeAngle();
-    SUMOReal a2 = l2.atan2DegreeAngle();
-    return fabs(a1 - a2) < 5;
+    return fabs(GeomHelper::angleDiff(oe->getGeometry().beginEndAngle(), ie->getGeometry().beginEndAngle())) < DEG2RAD(5);
 }
 
 
