@@ -7,12 +7,12 @@
 /// @author  Christian Roessel
 /// @author  Jakob Erdmann
 /// @date    Mon, 15 Apr 2002
-/// @version $Id: NLDetectorBuilder.cpp 18213 2015-04-16 11:55:42Z behrisch $
+/// @version $Id: NLDetectorBuilder.cpp 20482 2016-04-18 20:49:42Z behrisch $
 ///
 // Builds detectors for microsim
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2002-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2002-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -59,11 +59,9 @@
 #include "NLDetectorBuilder.h"
 #include <microsim/output/MSDetectorControl.h>
 
-#ifdef HAVE_INTERNAL
 #include <mesosim/MEInductLoop.h>
 #include <mesosim/MELoop.h>
 #include <mesosim/MESegment.h>
-#endif
 
 #ifdef CHECK_MEMORY_LEAKS
 #include <foreign/nvwa/debug_new.h>
@@ -95,7 +93,9 @@ NLDetectorBuilder::NLDetectorBuilder(MSNet& net)
     : myNet(net), myE3Definition(0) {}
 
 
-NLDetectorBuilder::~NLDetectorBuilder() {}
+NLDetectorBuilder::~NLDetectorBuilder() {
+    delete myE3Definition;
+}
 
 
 void
@@ -105,39 +105,12 @@ NLDetectorBuilder::buildInductLoop(const std::string& id,
     checkSampleInterval(splInterval, SUMO_TAG_E1DETECTOR, id);
     // get and check the lane
     MSLane* clane = getLaneChecking(lane, SUMO_TAG_E1DETECTOR, id);
-    if (!MSGlobals::gUseMesoSim) {
-        // get and check the position
-        pos = getPositionChecking(pos, clane, friendlyPos, id);
-        // build the loop
-        MSDetectorFileOutput* loop = createInductLoop(id, clane, pos, splitByType);
-        // add the file output
-        myNet.getDetectorControl().add(SUMO_TAG_INDUCTION_LOOP, loop, device, splInterval);
-    } else {
-#ifdef HAVE_INTERNAL
-        if (pos < 0) {
-            pos = clane->getLength() + pos;
-        }
-        MESegment* s = MSGlobals::gMesoNet->getSegmentForEdge(clane->getEdge());
-        MESegment* prev = s;
-        SUMOReal cpos = 0;
-        while (cpos + prev->getLength() < pos && s != 0) {
-            prev = s;
-            cpos += s->getLength();
-            s = s->getNextSegment();
-        }
-        SUMOReal rpos = pos - cpos; //-prev->getLength();
-        if (rpos > prev->getLength() || rpos < 0) {
-            if (friendlyPos) {
-                rpos = prev->getLength() - (SUMOReal) 0.1;
-            } else {
-                throw InvalidArgument("The position of detector '" + id + "' lies beyond the lane's '" + lane + "' length.");
-            }
-        }
-        MEInductLoop* loop =
-            createMEInductLoop(id, prev, rpos);
-        myNet.getDetectorControl().add(SUMO_TAG_INDUCTION_LOOP, loop, device, splInterval);
-#endif
-    }
+    // get and check the position
+    pos = getPositionChecking(pos, clane, friendlyPos, id);
+    // build the loop
+    MSDetectorFileOutput* loop = createInductLoop(id, clane, pos, splitByType);
+    // add the file output
+    myNet.getDetectorControl().add(SUMO_TAG_INDUCTION_LOOP, loop, device, splInterval);
 }
 
 
@@ -371,7 +344,7 @@ NLDetectorBuilder::buildRouteProbe(const std::string& id, const std::string& edg
 
 
 // -------------------
-MSDetectorFileOutput*
+MSE2Collector*
 NLDetectorBuilder::buildSingleLaneE2Det(const std::string& id,
                                         DetectorUsage usage,
                                         MSLane* lane, SUMOReal pos, SUMOReal length,
@@ -400,7 +373,10 @@ NLDetectorBuilder::buildMultiLaneE2Det(const std::string& id, DetectorUsage usag
 
 MSDetectorFileOutput*
 NLDetectorBuilder::createInductLoop(const std::string& id,
-                                    MSLane* lane, SUMOReal pos, bool splitByType) {
+                                    MSLane* lane, SUMOReal pos, bool splitByType, bool) {
+    if (MSGlobals::gUseMesoSim) {
+        return new MEInductLoop(id, MSGlobals::gMesoNet->getSegmentForEdge(lane->getEdge(), pos), pos);
+    }
     return new MSInductLoop(id, lane, pos, splitByType);
 }
 
@@ -412,16 +388,7 @@ NLDetectorBuilder::createInstantInductLoop(const std::string& id,
 }
 
 
-#ifdef HAVE_INTERNAL
-MEInductLoop*
-NLDetectorBuilder::createMEInductLoop(const std::string& id,
-                                      MESegment* s, SUMOReal pos) {
-    return new MEInductLoop(id, s, pos);
-}
-#endif
-
-
-MSDetectorFileOutput*
+MSE2Collector*
 NLDetectorBuilder::createSingleLaneE2Detector(const std::string& id,
         DetectorUsage usage, MSLane* lane, SUMOReal pos, SUMOReal length,
         SUMOTime haltingTimeThreshold, SUMOReal haltingSpeedThreshold, SUMOReal jamDistThreshold) {
