@@ -4,12 +4,12 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Mon, 9 Jul 2001
-/// @version $Id: NLEdgeControlBuilder.h 17262 2014-11-04 20:41:44Z behrisch $
+/// @version $Id: NLEdgeControlBuilder.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // Interface for building edges
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -62,9 +62,6 @@ class OutputDevice;
  * @todo Assignment of lanes is not really well. Should be reworked after shapes are given as params.
  */
 class NLEdgeControlBuilder {
-public:
-    /// @brief definition of the used storage for edges
-    typedef std::vector<MSEdge*> EdgeCont;
 
 public:
     /// @brief Constructor
@@ -102,13 +99,14 @@ public:
      * @param[in] shape The shape of the lane
      * @param[in] width The width of the lane
      * @param[in] permissions Encoding of vehicle classes that may drive on this lane
+     * @param[in] index The index of this lane within its parent edge
      * @see SUMOVehicleClass
      * @see MSLane
      * @todo Definitely not a good way
      */
     virtual MSLane* addLane(const std::string& id, SUMOReal maxSpeed,
                             SUMOReal length, const PositionVector& shape,
-                            SUMOReal width, SVCPermissions permissions);
+                            SUMOReal width, SVCPermissions permissions, int index);
 
 
     /** @brief Closes the building of an edge;
@@ -131,6 +129,11 @@ public:
     virtual MSEdge* buildEdge(const std::string& id, const MSEdge::EdgeBasicFunction function,
                               const std::string& streetName, const std::string& edgeType, const int priority);
 
+    /** @brief add the crossingEdges in a crossing edge if present
+     *
+     * @param[in] the vector of crossed edges id
+     */
+    virtual void addCrossingEdges(const std::vector<std::string>&);
 
 protected:
     /// @brief A running number for lane numbering
@@ -140,7 +143,7 @@ protected:
     unsigned int myCurrentNumericalEdgeID;
 
     /// @brief Temporary, internal storage for built edges
-    EdgeCont myEdges;
+    MSEdgeVector myEdges;
 
     /// @brief pointer to the currently chosen edge
     MSEdge* myActiveEdge;
