@@ -5,12 +5,12 @@
 /// @author  Clemens Honomichl
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: NLHandler.h 18467 2015-05-29 03:50:41Z behrisch $
+/// @version $Id: NLHandler.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // The XML-Handler for network loading
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -33,28 +33,19 @@
 #include <config.h>
 #endif
 
-#include <xercesc/sax/HandlerBase.hpp>
-#include <xercesc/sax/SAXException.hpp>
-#include <xercesc/sax/AttributeList.hpp>
-#include <utils/common/SUMOTime.h>
-#include <utils/common/Parameterised.h>
-#include <utils/xml/SUMOXMLDefinitions.h>
+#include <utils/geom/Boundary.h>
 #include <utils/shapes/ShapeHandler.h>
-#include <microsim/MSLink.h>
 #include <microsim/MSRouteHandler.h>
-#include <microsim/traffic_lights/MSSimpleTrafficLightLogic.h>
-#include <microsim/traffic_lights/MSActuatedTrafficLightLogic.h>
-#include <microsim/MSBitSetLogic.h>
-#include "NLBuilder.h"
 #include "NLDiscreteEventBuilder.h"
 
 
-
 // ===========================================================================
 // class declarations
 // ===========================================================================
 class NLContainer;
 class NLDetectorBuilder;
+class NLEdgeControlBuilder;
+class NLJunctionControlBuilder;
 class NLTriggerBuilder;
 class MSTrafficLightLogic;
 
@@ -118,6 +109,14 @@ public:
         return myHaveSeenInternalEdge;
     }
 
+    bool lefthand() const {
+        return myLefthand;
+    }
+
+    SUMOReal networkVersion() const {
+        return myNetworkVersion;
+    }
+
 protected:
     /// @name inherited from GenericSAXHandler
     //@{
@@ -333,6 +332,12 @@ protected:
     /// @brief whether the loaded network contains internal lanes
     bool myHaveSeenInternalEdge;
 
+    /// @brief whether the loaded network was built for left hand traffic
+    bool myLefthand;
+
+    /// @brief the loaded network version
+    SUMOReal myNetworkVersion;
+
     /// @brief whether the location element was already loadee
     bool myNetIsLoaded;
 
