@@ -5,12 +5,12 @@
 /// @author  Sascha Krieg
 /// @author  Michael Behrisch
 /// @date    Mon, 9 Jul 2001
-/// @version $Id: NLJunctionControlBuilder.cpp 18213 2015-04-16 11:55:42Z behrisch $
+/// @version $Id: NLJunctionControlBuilder.cpp 20482 2016-04-18 20:49:42Z behrisch $
 ///
 // Builder of microsim-junctions and tls
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -44,8 +44,19 @@
 #include <microsim/traffic_lights/MSTrafficLightLogic.h>
 #include <microsim/traffic_lights/MSSimpleTrafficLightLogic.h>
 #include <microsim/traffic_lights/MSRailSignal.h>
+#include <microsim/traffic_lights/MSRailCrossing.h>
+#include <microsim/MSEventControl.h>
+#include <microsim/traffic_lights/MSSOTLPolicyBasedTrafficLightLogic.h>
+#include <microsim/traffic_lights/MSSOTLPlatoonPolicy.h>
+#include <microsim/traffic_lights/MSSOTLRequestPolicy.h>
+#include <microsim/traffic_lights/MSSOTLPhasePolicy.h>
+#include <microsim/traffic_lights/MSSOTLMarchingPolicy.h>
+#include <microsim/traffic_lights/MSSwarmTrafficLightLogic.h>
+#include <microsim/traffic_lights/MSDeterministicHiLevelTrafficLightLogic.h>
+#include <microsim/traffic_lights/MSSOTLWaveTrafficLightLogic.h>
 #include <microsim/MSEventControl.h>
 #include <microsim/MSGlobals.h>
+#include <microsim/MSNet.h>
 #include <microsim/traffic_lights/MSOffTrafficLightLogic.h>
 #include <microsim/traffic_lights/MSTLLogicControl.h>
 #include <utils/xml/SUMOXMLDefinitions.h>
@@ -108,7 +119,7 @@ NLJunctionControlBuilder::openJunction(const std::string& id,
 
 
 void
-NLJunctionControlBuilder::closeJunction() {
+NLJunctionControlBuilder::closeJunction(const std::string& basePath) {
     if (myJunctions == 0) {
         throw ProcessError("Information about the number of nodes was missing.");
     }
@@ -122,10 +133,12 @@ NLJunctionControlBuilder::closeJunction() {
             junction = buildNoLogicJunction();
             break;
         case NODETYPE_TRAFFIC_LIGHT:
+        case NODETYPE_TRAFFIC_LIGHT_RIGHT_ON_RED:
         case NODETYPE_RIGHT_BEFORE_LEFT:
         case NODETYPE_PRIORITY:
         case NODETYPE_PRIORITY_STOP:
         case NODETYPE_ALLWAY_STOP:
+        case NODETYPE_ZIPPER:
             junction = buildLogicJunction();
             break;
         case NODETYPE_INTERNAL:
@@ -136,11 +149,12 @@ NLJunctionControlBuilder::closeJunction() {
 #endif
             break;
         case NODETYPE_RAIL_SIGNAL:
+        case NODETYPE_RAIL_CROSSING:
             myOffset = 0;
             myActiveKey = myActiveID;
             myActiveProgram = "0";
             myLogicType = TLTYPE_RAIL;
-            closeTrafficLightLogic();
+            closeTrafficLightLogic(basePath);
             junction = buildLogicJunction();
             break;
         default:
@@ -211,7 +225,7 @@ NLJunctionControlBuilder::getTLLogic(const std::string& id) const {
 
 
 void
-NLJunctionControlBuilder::closeTrafficLightLogic() {
+NLJunctionControlBuilder::closeTrafficLightLogic(const std::string& basePath) {
     if (myActiveProgram == "off") {
         if (myAbsDuration > 0) {
             throw InvalidArgument("The off program for TLS '" + myActiveKey + "' has phases.");
@@ -224,10 +238,18 @@ NLJunctionControlBuilder::closeTrafficLightLogic() {
     }
     SUMOTime firstEventOffset = 0;
     unsigned int step = 0;
+    MSTrafficLightLogic* existing = 0;
     MSSimpleTrafficLightLogic::Phases::const_iterator i = myActivePhases.begin();
     if (myLogicType != TLTYPE_RAIL) {
         if (myAbsDuration == 0) {
-            throw InvalidArgument("TLS program '" + myActiveProgram + "' for TLS '" + myActiveKey + "' has a duration of 0.");
+            existing = getTLLogicControlToUse().get(myActiveKey, myActiveProgram);
+            if (existing == 0) {
+                throw InvalidArgument("TLS program '" + myActiveProgram + "' for TLS '" + myActiveKey + "' has a duration of 0.");
+            } else {
+                // only modify the offset of an existing logic
+                myAbsDuration = existing->getDefaultCycleTime();
+                i = existing->getPhases().begin();
+            }
         }
         // compute the initial step and first switch time of the tls-logic
         // a positive offset delays all phases by x (advance by absDuration - x) while a negative offset advances all phases by x seconds
@@ -244,6 +266,11 @@ NLJunctionControlBuilder::closeTrafficLightLogic() {
             ++i;
         }
         firstEventOffset = (*i)->duration - offset + myNet.getCurrentTimeStep();
+        if (existing != 0) {
+            existing->changeStepAndDuration(getTLLogicControlToUse(),
+                                            myNet.getCurrentTimeStep(), step, (*i)->duration - offset);
+            return;
+        }
     }
 
     if (myActiveProgram == "") {
@@ -252,13 +279,35 @@ NLJunctionControlBuilder::closeTrafficLightLogic() {
     MSTrafficLightLogic* tlLogic = 0;
     // build the tls-logic in dependance to its type
     switch (myLogicType) {
+        case TLTYPE_SWARM_BASED:
+            firstEventOffset = DELTA_T; //this is needed because swarm needs to update the pheromone on the lanes at every step
+            tlLogic = new MSSwarmTrafficLightLogic(getTLLogicControlToUse(), myActiveKey, myActiveProgram, myActivePhases, step, firstEventOffset, myAdditionalParameter);
+            break;
+        case TLTYPE_HILVL_DETERMINISTIC:
+            tlLogic = new MSDeterministicHiLevelTrafficLightLogic(getTLLogicControlToUse(), myActiveKey, myActiveProgram, myActivePhases, step, firstEventOffset, myAdditionalParameter);
+            break;
+        case TLTYPE_SOTL_REQUEST:
+            tlLogic = new MSSOTLPolicyBasedTrafficLightLogic(getTLLogicControlToUse(), myActiveKey, myActiveProgram, myActivePhases, step, firstEventOffset, myAdditionalParameter, new MSSOTLRequestPolicy(myAdditionalParameter));
+            break;
+        case TLTYPE_SOTL_PLATOON:
+            tlLogic = new MSSOTLPolicyBasedTrafficLightLogic(getTLLogicControlToUse(), myActiveKey, myActiveProgram, myActivePhases, step, firstEventOffset, myAdditionalParameter, new MSSOTLPlatoonPolicy(myAdditionalParameter));
+            break;
+        case TLTYPE_SOTL_WAVE:
+            tlLogic = new MSSOTLWaveTrafficLightLogic(getTLLogicControlToUse(), myActiveKey, myActiveProgram, myActivePhases, step, firstEventOffset, myAdditionalParameter);
+            break;
+        case TLTYPE_SOTL_PHASE:
+            tlLogic = new MSSOTLPolicyBasedTrafficLightLogic(getTLLogicControlToUse(), myActiveKey, myActiveProgram, myActivePhases, step, firstEventOffset, myAdditionalParameter, new MSSOTLPhasePolicy(myAdditionalParameter));
+            break;
+        case TLTYPE_SOTL_MARCHING:
+            tlLogic = new MSSOTLPolicyBasedTrafficLightLogic(getTLLogicControlToUse(), myActiveKey, myActiveProgram, myActivePhases, step, firstEventOffset, myAdditionalParameter, new MSSOTLMarchingPolicy(myAdditionalParameter));
+            break;
         case TLTYPE_ACTUATED:
             // @note it is unclear how to apply the given offset in the context
             // of variable-length phases
             tlLogic = new MSActuatedTrafficLightLogic(getTLLogicControlToUse(),
                     myActiveKey, myActiveProgram,
                     myActivePhases, step, (*i)->minDuration + myNet.getCurrentTimeStep(),
-                    myAdditionalParameter);
+                    myAdditionalParameter, basePath);
             break;
         case TLTYPE_STATIC:
             tlLogic =
@@ -268,10 +317,21 @@ NLJunctionControlBuilder::closeTrafficLightLogic() {
                                               myAdditionalParameter);
             break;
         case TLTYPE_RAIL:
-            tlLogic = new MSRailSignal(getTLLogicControlToUse(),
-                                       myActiveKey, myActiveProgram,
-                                       myAdditionalParameter);
+            if (myType == NODETYPE_RAIL_SIGNAL) {
+                tlLogic = new MSRailSignal(getTLLogicControlToUse(),
+                                           myActiveKey, myActiveProgram,
+                                           myAdditionalParameter);
+            } else if (myType == NODETYPE_RAIL_CROSSING) {
+                tlLogic = new MSRailCrossing(getTLLogicControlToUse(),
+                                             myActiveKey, myActiveProgram,
+                                             myAdditionalParameter);
+            } else {
+                throw ProcessError("Invalid node type '" + toString(myType)
+                                   + "' for traffic light type '" + toString(myLogicType) + "'");
+            }
             break;
+        case TLTYPE_INVALID:
+            throw ProcessError("Invalid traffic light type '" + toString(myLogicType) + "'");
     }
     myActivePhases.clear();
     if (tlLogic != 0) {
@@ -311,7 +371,7 @@ NLJunctionControlBuilder::addLogicItem(int request,
         // had an error
         return;
     }
-    if (request > 63) {
+    if (request >= SUMO_MAX_CONNECTIONS) {
         // bad request
         myCurrentHasError = true;
         throw InvalidArgument("Junction logic '" + myActiveKey + "' is larger than allowed; recheck the network.");
@@ -334,9 +394,9 @@ NLJunctionControlBuilder::addLogicItem(int request,
     assert(myActiveLogic.size() == (size_t) request);
     assert(myActiveFoes.size() == (size_t) request);
     // add the read response for the given request index
-    myActiveLogic.push_back(std::bitset<64>(response));
+    myActiveLogic.push_back(std::bitset<SUMO_MAX_CONNECTIONS>(response));
     // add the read junction-internal foes for the given request index
-    myActiveFoes.push_back(std::bitset<64>(foes));
+    myActiveFoes.push_back(std::bitset<SUMO_MAX_CONNECTIONS>(foes));
     // add whether the vehicle may drive a little bit further
     myActiveConts.set(request, cont);
     // increse number of set information
@@ -358,6 +418,23 @@ NLJunctionControlBuilder::initTrafficLightLogic(const std::string& id, const std
 }
 
 
+void
+NLJunctionControlBuilder::addPhase(SUMOTime duration, const std::string& state, SUMOTime minDuration, SUMOTime maxDuration, bool transient_notdecisional, bool commit) throw() {
+    // build and add the phase definition to the list
+    myActivePhases.push_back(new MSPhaseDefinition(duration, minDuration, maxDuration, state, transient_notdecisional, commit));
+    // add phase duration to the absolute duration
+    myAbsDuration += duration;
+}
+
+void
+NLJunctionControlBuilder::addPhase(SUMOTime duration, const std::string& state, SUMOTime minDuration, SUMOTime maxDuration, bool transient_notdecisional, bool commit, MSPhaseDefinition::LaneIdVector& targetLanes) throw() {
+    // build and add the phase definition to the list
+    myActivePhases.push_back(new MSPhaseDefinition(duration, minDuration, maxDuration, state, transient_notdecisional, commit, targetLanes));
+    // add phase duration to the absolute duration
+    myAbsDuration += duration;
+}
+
+
 void
 NLJunctionControlBuilder::addPhase(SUMOTime duration, const std::string& state,
                                    SUMOTime minDuration, SUMOTime maxDuration) {
@@ -393,7 +470,8 @@ NLJunctionControlBuilder::closeJunctionLogic() {
 
 
 MSTLLogicControl*
-NLJunctionControlBuilder::buildTLLogics() const {
+NLJunctionControlBuilder::buildTLLogics() {
+    postLoadInitialization(); // must happen after edgeBuilder is finished
     if (!myLogicControl->closeNetworkReading()) {
         throw ProcessError("Traffic lights could not be built.");
     }
