@@ -4,12 +4,12 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Tue, 20 Nov 2001
-/// @version $Id: NWFrame.cpp 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: NWFrame.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // Sets and checks options for netwrite
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -35,6 +35,7 @@
 #include <utils/options/OptionsCont.h>
 #include <utils/common/MsgHandler.h>
 #include <utils/common/SystemFrame.h>
+#include <utils/common/SysUtils.h>
 #include <utils/iodevices/OutputDevice.h>
 #include <netbuild/NBNetBuilder.h>
 #include "NWFrame.h"
@@ -52,7 +53,7 @@
 // ===========================================================================
 // static members
 // ===========================================================================
-const std::string NWFrame::MAJOR_VERSION = "version=\"0.13\"";
+const std::string NWFrame::MAJOR_VERSION = "version=\"0.25\"";
 
 
 // ===========================================================================
@@ -83,9 +84,6 @@ NWFrame::fillOptions(bool forNetgen) {
     }
 #endif // HAVE_PROJ
 
-    oc.doRegister("map-output", 'M', new Option_FileName());
-    oc.addDescription("map-output", "Output", "Writes joined edges information to FILE");
-
     oc.doRegister("amitran-output", new Option_FileName());
     oc.addDescription("amitran-output", "Output", "The generated net will be written to FILE using Amitran format");
 
@@ -120,7 +118,11 @@ NWFrame::checkOptions() {
             && !oc.isSet("matsim-output")
             && !oc.isSet("opendrive-output")
             && !oc.isSet("dlr-navteq-output")) {
-        oc.set("output-file", "net.net.xml");
+        std::string net = "net.net.xml";
+        if (oc.isSet("configuration-file")) {
+            net = FileHelpers::getConfigurationRelative(oc.getString("configuration-file"), net);
+        }
+        oc.setDefault("output-file", net);
     }
     // some outputs need internal lanes
     if (oc.isSet("opendrive-output") && oc.getBool("no-internal-links")) {
@@ -133,18 +135,15 @@ NWFrame::checkOptions() {
 
 void
 NWFrame::writeNetwork(const OptionsCont& oc, NBNetBuilder& nb) {
+    long before = SysUtils::getCurrentMillis();
+    PROGRESS_BEGIN_MESSAGE("Writing network");
     NWWriter_SUMO::writeNetwork(oc, nb);
     NWWriter_Amitran::writeNetwork(oc, nb);
     NWWriter_MATSim::writeNetwork(oc, nb);
     NWWriter_OpenDrive::writeNetwork(oc, nb);
     NWWriter_DlrNavteq::writeNetwork(oc, nb);
     NWWriter_XML::writeNetwork(oc, nb);
-    // save the mapping information when wished
-    if (oc.isSet("map-output")) {
-        OutputDevice& mdevice = OutputDevice::getDevice(oc.getString("map-output"));
-        mdevice << nb.getJoinedEdgesMap();
-        mdevice.close();
-    }
+    PROGRESS_TIME_MESSAGE(before);
 }
 
 
