@@ -4,12 +4,12 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Tue, 04.05.2011
-/// @version $Id: NWWriter_SUMO.cpp 18739 2015-08-27 06:48:48Z namdre $
+/// @version $Id: NWWriter_SUMO.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // Exporter writing networks using the SUMO format
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -67,7 +67,15 @@ NWWriter_SUMO::writeNetwork(const OptionsCont& oc, NBNetBuilder& nb) {
         return;
     }
     OutputDevice& device = OutputDevice::getDevice(oc.getString("output-file"));
-    device.writeXMLHeader("net", NWFrame::MAJOR_VERSION + " xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xsi:noNamespaceSchemaLocation=\"http://sumo.dlr.de/xsd/net_file.xsd\""); // street names may contain non-ascii chars
+    const std::string lefthand = oc.getBool("lefthand") ? " " + toString(SUMO_ATTR_LEFTHAND) + "=\"true\"" : "";
+    const int cornerDetail = oc.getInt("junctions.corner-detail");
+    const int linkDetail = oc.getInt("junctions.internal-link-detail");
+    const std::string junctionCornerDetail = (cornerDetail > 0
+            ? " " + toString(SUMO_ATTR_CORNERDETAIL) + "=\"" + toString(cornerDetail) + "\"" : "");
+    const std::string junctionLinkDetail = (oc.isDefault("junctions.internal-link-detail") ? "" :
+                                            " " + toString(SUMO_ATTR_LINKDETAIL) + "=\"" + toString(linkDetail) + "\"");
+    device.writeXMLHeader("net", NWFrame::MAJOR_VERSION + lefthand + junctionCornerDetail + junctionLinkDetail +
+                          " xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xsi:noNamespaceSchemaLocation=\"http://sumo.dlr.de/xsd/net_file.xsd\""); // street names may contain non-ascii chars
     device.lf();
     // get involved container
     const NBNodeCont& nc = nb.getNodeCont();
@@ -339,17 +347,7 @@ NWWriter_SUMO::writeEdge(OutputDevice& into, const NBEdge& e, bool noNames, bool
     // write the lanes
     const std::vector<NBEdge::Lane>& lanes = e.getLanes();
 
-    SUMOReal length = e.getLoadedLength();
-    if (OptionsCont::getOptions().getBool("no-internal-links") && !e.hasLoadedLength()) {
-        // use length to junction center even if a modified geometry was given
-        PositionVector geom = e.cutAtIntersection(e.getGeometry());
-        geom.push_back_noDoublePos(e.getToNode()->getCenter());
-        geom.push_front_noDoublePos(e.getFromNode()->getCenter());
-        length = geom.length();
-    }
-    if (length <= 0) {
-        length = POSITION_EPS;
-    }
+    const SUMOReal length = e.getFinalLength();
     for (unsigned int i = 0; i < (unsigned int) lanes.size(); i++) {
         const NBEdge::Lane& l = lanes[i];
         writeLane(into, e.getID(), e.getLaneID(i), l.speed,
@@ -549,6 +547,9 @@ NWWriter_SUMO::writeConnection(OutputDevice& into, const NBEdge& from, const NBE
     if ((from.getToNode()->getKeepClear() == false || c.keepClear == false) && style != TLL) {
         into.writeAttr<bool>(SUMO_ATTR_KEEP_CLEAR, false);
     }
+    if (c.contPos != NBEdge::UNSPECIFIED_CONTPOS && style != TLL) {
+        into.writeAttr(SUMO_ATTR_CONTPOS, c.contPos);
+    }
     if (style != PLAIN) {
         if (includeInternal) {
             into.writeAttr(SUMO_ATTR_VIA, c.getInternalLaneID());
@@ -565,7 +566,7 @@ NWWriter_SUMO::writeConnection(OutputDevice& into, const NBEdge& from, const NBE
             into.writeAttr(SUMO_ATTR_DIR, toString(dir));
             // write the state information
             const LinkState linkState = from.getToNode()->getLinkState(
-                                            &from, c.toEdge, c.fromLane, c.mayDefinitelyPass, c.tlID);
+                                            &from, c.toEdge, c.fromLane, c.toLane, c.mayDefinitelyPass, c.tlID);
             into.writeAttr(SUMO_ATTR_STATE, linkState);
         }
     }
