@@ -4,12 +4,12 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Tue, 04.05.2011
-/// @version $Id: NWWriter_SUMO.h 18096 2015-03-17 09:50:59Z behrisch $
+/// @version $Id: NWWriter_SUMO.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // Exporter writing networks using the SUMO format
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -93,12 +93,6 @@ public:
     /// @brief writes the traffic light logics to the given device
     static void writeTrafficLights(OutputDevice& into, const NBTrafficLightLogicCont& tllCont);
 
-    /// @brief writes allowed disallowed attributes if needed;
-    static void writePermissions(OutputDevice& into, SVCPermissions permissions);
-
-    /// @brief writes allowed disallowed attributes if needed;
-    static void writePreferences(OutputDevice& into, SVCPermissions preferred);
-
     /** @brief Writes roundabouts
      * @param[in] into The device to write the edge into
      * @param[in] roundaboutes The roundabouts to write
