@@ -4,12 +4,12 @@
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    Mon, 05 Dec 2005
-/// @version $Id: PCPolyContainer.h 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: PCPolyContainer.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // A storage for loaded polygons and pois
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2005-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2005-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -138,9 +138,10 @@ public:
 
     /** @brief Saves the stored polygons into the given file
      * @param[in] file The name of the file to write stored objects' definitions into
+     * @param[in] useGeo Whether to write output in geo-coordinates
      * @exception IOError If the file could not be opened
      */
-    void save(const std::string& file);
+    void save(const std::string& file, bool useGeo);
 
 
     /** @brief Retuns a unique id for a given name
