@@ -6,12 +6,12 @@
 /// @author  Michael Behrisch
 /// @author  Melanie Knocke
 /// @date    Mon, 05 Dec 2005
-/// @version $Id: polyconvert_main.cpp 18703 2015-08-24 07:29:42Z namdre $
+/// @version $Id: polyconvert_main.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // Main for POLYCONVERT
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2005-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2005-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -48,8 +48,11 @@
 #include <utils/common/ToString.h>
 #include <utils/iodevices/OutputDevice.h>
 #include <utils/importio/LineReader.h>
+#include <utils/geom/GeoConvHelper.h>
 #include <utils/geom/GeomConvHelper.h>
 #include <utils/geom/Boundary.h>
+#include <utils/xml/SUMOSAXReader.h>
+#include <utils/xml/XMLSubSys.h>
 #include <polyconvert/PCLoaderVisum.h>
 #include <polyconvert/PCLoaderDlrNavteq.h>
 #include <polyconvert/PCLoaderXML.h>
@@ -58,8 +61,7 @@
 #include <polyconvert/PCTypeMap.h>
 #include <polyconvert/PCTypeDefHandler.h>
 #include <polyconvert/PCNetProjectionLoader.h>
-#include <utils/xml/XMLSubSys.h>
-#include <utils/geom/GeoConvHelper.h>
+#include "pc_typemap.h"
 
 #ifdef CHECK_MEMORY_LEAKS
 #include <foreign/nvwa/debug_new.h>
@@ -139,6 +141,9 @@ fillOptions() {
     oc.doRegister("shapefile.add-param", new Option_Bool());
     oc.addDescription("shapefile.add-param", "Input", "Extract all additonal columns as params");
 
+    oc.doRegister("shapefile.fill", new Option_String());
+    oc.addDescription("shapefile.fill", "Input", "[auto|true|false]. Forces the 'fill' status to the given value. Default 'auto' tries to determine it from the data type");
+
     // typemap reading
     oc.doRegister("type-file", new Option_FileName());
     oc.addSynonyme("type-file", "typemap", true);
@@ -200,6 +205,9 @@ fillOptions() {
     oc.doRegister("type", new Option_String("unknown"));
     oc.addDescription("type", "Building Defaults", "Sets STR as default type");
 
+    oc.doRegister("fill", new Option_Bool("false"));
+    oc.addDescription("fill", "Building Defaults", "Fills polygons by default");
+
     oc.doRegister("layer", new Option_Integer(-1));
     oc.addDescription("layer", "Building Defaults", "Sets INT as default layer");
 
@@ -216,7 +224,7 @@ int
 main(int argc, char** argv) {
     OptionsCont& oc = OptionsCont::getOptions();
     oc.setApplicationDescription("Importer of polygons and POIs for the road traffic simulation SUMO.");
-    oc.setApplicationName("polyconvert", "SUMO polyconvert Version " + getBuildName(VERSION_STRING));
+    oc.setApplicationName("polyconvert", "SUMO polyconvert Version " VERSION_STRING);
     int ret = 0;
     try {
         // initialise subsystems
@@ -282,15 +290,44 @@ main(int argc, char** argv) {
         PCPolyContainer toFill(prune, pruningBoundary, oc.getStringVector("remove"));
 
         // read in the type defaults
+        if (!oc.isSet("type-file")) {
+            const char* sumoPath = std::getenv("SUMO_HOME");
+            if (sumoPath == 0) {
+                WRITE_WARNING("Environment variable SUMO_HOME is not set, using built in type maps.");
+            } else {
+                const std::string path = sumoPath + std::string("/data/typemap/");
+                if (oc.isSet("dlr-navteq-poly-files")) {
+                    oc.set("type-file", path + "navteqPolyconvert.typ.xml");
+                }
+                if (oc.isSet("osm-files")) {
+                    oc.set("type-file", path + "osmPolyconvert.typ.xml");
+                }
+                if (oc.isSet("visum-files")) {
+                    oc.set("type-file", path + "visumPolyconvert.typ.xml");
+                }
+            }
+        }
         PCTypeMap tm(oc);
+        PCTypeDefHandler handler(oc, tm);
         if (oc.isSet("type-file")) {
-            PCTypeDefHandler handler(oc, tm);
             if (!XMLSubSys::runParser(handler, oc.getString("type-file"))) {
                 // something failed
                 throw ProcessError();
             }
+        } else {
+            handler.setFileName("built in type map");
+            SUMOSAXReader* reader = XMLSubSys::getSAXReader(handler);
+            if (oc.isSet("dlr-navteq-poly-files")) {
+                reader->parseString(navteqTypemap);
+            }
+            if (oc.isSet("osm-files")) {
+                reader->parseString(osmTypemap);
+            }
+            if (oc.isSet("visum-files")) {
+                reader->parseString(visumTypemap);
+            }
+            delete reader;
         }
-
         // read in the data
         PCLoaderXML::loadIfSet(oc, toFill, tm); // SUMO-XML
         PCLoaderOSM::loadIfSet(oc, toFill, tm); // OSM-XML
