@@ -7,12 +7,12 @@
 /// @author  Melanie Knocke
 /// @author  Yun-Pang Floetteroed
 /// @date    Sept 2002
-/// @version $Id: ROEdge.cpp 18756 2015-08-31 19:16:33Z behrisch $
+/// @version $Id: ROEdge.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // A basic edge for routing applications
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2002-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2002-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -38,13 +38,13 @@
 #include <algorithm>
 #include <cassert>
 #include <iostream>
-#include "ROLane.h"
-#include "ROEdge.h"
-#include "RONet.h"
-#include "ROVehicle.h"
 #include <utils/vehicle/SUMOVTypeParameter.h>
 #include <utils/emissions/PollutantsInterface.h>
 #include <utils/emissions/HelpersHarmonoise.h>
+#include "ROLane.h"
+#include "RONet.h"
+#include "ROVehicle.h"
+#include "ROEdge.h"
 
 #ifdef CHECK_MEMORY_LEAKS
 #include <foreign/nvwa/debug_new.h>
@@ -54,10 +54,7 @@
 // ===========================================================================
 // static member definitions
 // ===========================================================================
-bool ROEdge::myUseBoundariesOnOverrideTT = false;
-bool ROEdge::myUseBoundariesOnOverrideE = false;
 bool ROEdge::myInterpolate = false;
-bool ROEdge::myAmParallel = false;
 bool ROEdge::myHaveTTWarned = false;
 bool ROEdge::myHaveEWarned = false;
 ROEdgeVector ROEdge::myEdges;
@@ -152,6 +149,12 @@ ROEdge::getDistanceTo(const ROEdge* other) const {
 }
 
 
+bool
+ROEdge::hasLoadedTravelTime(SUMOReal time) const {
+    return myUsingTTTimeLine && myTravelTimes.describesTime(time);
+}
+
+
 SUMOReal
 ROEdge::getTravelTime(const ROVehicle* const veh, SUMOReal time) const {
     if (myUsingTTTimeLine) {
@@ -231,7 +234,7 @@ ROEdge::getNumPredecessors() const {
 
 
 void
-ROEdge::buildTimeLines(const std::string& measure) {
+ROEdge::buildTimeLines(const std::string& measure, const bool boundariesOverride) {
     if (myUsingETimeLine) {
         SUMOReal value = myLength / mySpeed;
         const SUMOEmissionClass c = PollutantsInterface::getClassByName("unknown");
@@ -253,10 +256,13 @@ ROEdge::buildTimeLines(const std::string& measure) {
         if (measure == "fuel") {
             value = PollutantsInterface::compute(c, PollutantsInterface::FUEL, mySpeed, 0, 0) * value; // @todo: give correct slope
         }
-        myEfforts.fillGaps(value, myUseBoundariesOnOverrideE);
+        if (measure == "electricity") {
+            value = PollutantsInterface::compute(c, PollutantsInterface::ELEC, mySpeed, 0, 0) * value; // @todo: give correct slope
+        }
+        myEfforts.fillGaps(value, boundariesOverride);
     }
     if (myUsingTTTimeLine) {
-        myTravelTimes.fillGaps(myLength / mySpeed, myUseBoundariesOnOverrideTT);
+        myTravelTimes.fillGaps(myLength / mySpeed, boundariesOverride);
     }
 }
 
@@ -272,10 +278,9 @@ ROEdge::allFollowersProhibit(const ROVehicle* const vehicle) const {
 }
 
 
-ROEdge*
-ROEdge::dictionary(size_t id) {
-    assert(myEdges.size() > id);
-    return myEdges[id];
+const ROEdgeVector&
+ROEdge::getAllEdges() {
+    return myEdges;
 }
 
 
@@ -285,18 +290,11 @@ ROEdge::getSuccessors(SUMOVehicleClass vClass) const {
         return myFollowingEdges;
     }
 #ifdef HAVE_FOX
-    if (myAmParallel) {
-        RONet::getInstance()->lock();
-    }
+    FXMutexLock locker(myLock);
 #endif
     std::map<SUMOVehicleClass, ROEdgeVector>::const_iterator i = myClassesSuccessorMap.find(vClass);
     if (i != myClassesSuccessorMap.end()) {
         // can use cached value
-#ifdef HAVE_FOX
-        if (myAmParallel) {
-            RONet::getInstance()->unlock();
-        }
-#endif
         return i->second;
     } else {
         // this vClass is requested for the first time. rebuild all successors
@@ -321,11 +319,6 @@ ROEdge::getSuccessors(SUMOVehicleClass vClass) const {
         }
         myClassesSuccessorMap[vClass].insert(myClassesSuccessorMap[vClass].begin(),
                                              followers.begin(), followers.end());
-#ifdef HAVE_FOX
-        if (myAmParallel) {
-            RONet::getInstance()->unlock();
-        }
-#endif
         return myClassesSuccessorMap[vClass];
     }
 
