@@ -7,12 +7,12 @@
 /// @author  Melanie Knocke
 /// @author  Yun-Pang Floetteroed
 /// @date    Sept 2002
-/// @version $Id: ROEdge.h 18627 2015-08-13 08:58:59Z namdre $
+/// @version $Id: ROEdge.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // A basic edge for routing applications
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2002-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2002-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -44,6 +44,9 @@
 #include <utils/common/ValueTimeLine.h>
 #include <utils/common/SUMOVehicleClass.h>
 #include <utils/emissions/PollutantsInterface.h>
+#ifdef HAVE_FOX
+#include <fx.h>
+#endif
 #include <utils/vehicle/SUMOVTypeParameter.h>
 #include "RONode.h"
 #include "ROVehicle.h"
@@ -58,6 +61,7 @@ class ROEdge;
 typedef std::vector<ROEdge*> ROEdgeVector;
 typedef std::vector<const ROEdge*> ConstROEdgeVector;
 
+
 // ===========================================================================
 // class definitions
 // ===========================================================================
@@ -166,12 +170,12 @@ public:
      *
      * Should be called after weights / travel times have been loaded.
      *
-     * In the case "weight-attribute" is one of "CO", "CO2", "HC", "NOx", "PMx", or "fuel"
+     * In the case "weight-attribute" is one of "CO", "CO2", "HC", "NOx", "PMx", "fuel", or "electricity"
      *  the proper value (departs/s) is computed and multiplied with the travel time.
      *
      * @param[in] measure The name of the measure to use.
      */
-    void buildTimeLines(const std::string& measure);
+    void buildTimeLines(const std::string& measure, const bool boundariesOverride);
     //@}
 
 
@@ -325,6 +329,14 @@ public:
     SUMOReal getEffort(const ROVehicle* const veh, SUMOReal time) const;
 
 
+    /** @brief Returns whether a travel time for this edge was loaded
+     *
+     * @param[in] time The time for which the travel time shall be returned [s]
+     * @return whether a value was loaded
+     */
+    bool hasLoadedTravelTime(SUMOReal time) const;
+
+
     /** @brief Returns the travel time for this edge
      *
      * @param[in] veh The vehicle for which the travel time on this edge shall be retrieved
@@ -390,23 +402,16 @@ public:
     SUMOReal getDistanceTo(const ROEdge* other) const;
 
 
-    /** @brief Returns the ROEdge at the index */
-    static ROEdge* dictionary(size_t index);
+    /** @brief Returns all ROEdges */
+    static const ROEdgeVector& getAllEdges();
 
     /// @brief Returns the number of edges
     static size_t dictSize() {
         return myEdges.size();
     };
 
-    static void setGlobalOptions(
-        bool useBoundariesOnOverrideTT,
-        bool useBoundariesOnOverrideE,
-        bool interpolate,
-        bool isParallel) {
-        myUseBoundariesOnOverrideTT = useBoundariesOnOverrideTT;
-        myUseBoundariesOnOverrideE = useBoundariesOnOverrideE;
+    static void setGlobalOptions(const bool interpolate) {
         myInterpolate = interpolate;
-        myAmParallel = isParallel;
     }
 
     /// @brief get edge priority (road class)
@@ -463,22 +468,15 @@ protected:
     mutable ValueTimeLine<SUMOReal> myTravelTimes;
     /// @brief Information whether the time line shall be used instead of the length value
     bool myUsingTTTimeLine;
-    /// @brief Whether overriding weight boundaries shall be reported
-    static bool myUseBoundariesOnOverrideTT;
 
     /// @brief Container storing passing time varying over time for the edge
     mutable ValueTimeLine<SUMOReal> myEfforts;
     /// @brief Information whether the time line shall be used instead of the length value
     bool myUsingETimeLine;
-    /// @brief Whether overriding weight boundaries shall be reported
-    static bool myUseBoundariesOnOverrideE;
 
     /// @brief Information whether to interpolate at interval boundaries
     static bool myInterpolate;
 
-    /// @brief Information whether we are routing multi-threaded
-    static bool myAmParallel;
-
     /// @brief Information whether the edge has reported missing weights
     static bool myHaveEWarned;
     /// @brief Information whether the edge has reported missing weights
@@ -508,6 +506,11 @@ protected:
     /// @brief The successors available for a given vClass
     mutable std::map<SUMOVehicleClass, ROEdgeVector> myClassesSuccessorMap;
 
+#ifdef HAVE_FOX
+    /// The mutex used to avoid concurrent updates of myClassesSuccessorMap
+    mutable FXMutex myLock;
+#endif
+
 private:
     /// @brief Invalidated copy constructor
     ROEdge(const ROEdge& src);
