@@ -6,12 +6,12 @@
 /// @author  Michael Behrisch
 /// @author  Christian Roessel
 /// @date    Sept 2002
-/// @version $Id: ROLoader.cpp 18397 2015-05-18 14:04:14Z behrisch $
+/// @version $Id: ROLoader.cpp 20482 2016-04-18 20:49:42Z behrisch $
 ///
 // Loader for networks and route imports
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -112,8 +112,8 @@ ROLoader::ROLoader(OptionsCont& oc, const bool emptyDestinationsAllowed, const b
     myOptions(oc),
     myEmptyDestinationsAllowed(emptyDestinationsAllowed),
     myLogSteps(logSteps),
-    myLoaders(oc.exists("unsorted-input") && oc.getBool("unsorted-input") ? 0 : DELTA_T)
-{}
+    myLoaders(oc.exists("unsorted-input") && oc.getBool("unsorted-input") ? 0 : DELTA_T) {
+}
 
 
 ROLoader::~ROLoader() {
@@ -199,7 +199,7 @@ ROLoader::openRoutes(RONet& net) {
 
 void
 ROLoader::processRoutes(const SUMOTime start, const SUMOTime end, const SUMOTime increment,
-                        RONet& net, SUMOAbstractRouter<ROEdge, ROVehicle>& router) {
+                        RONet& net, const RORouterProvider& provider) {
     const SUMOTime absNo = end - start;
     const bool endGiven = !OptionsCont::getOptions().isDefault("end");
     // skip routes that begin before the simulation's begin
@@ -213,7 +213,7 @@ ROLoader::processRoutes(const SUMOTime start, const SUMOTime end, const SUMOTime
         if (!net.furtherStored() || MsgHandler::getErrorInstance()->wasInformed()) {
             break;
         }
-        lastStep = net.saveAndRemoveRoutesUntil(myOptions, router, time);
+        lastStep = net.saveAndRemoveRoutesUntil(myOptions, provider, time);
         if ((!net.furtherStored() && myLoaders.haveAllLoaded()) || MsgHandler::getErrorInstance()->wasInformed()) {
             break;
         }
@@ -266,7 +266,7 @@ ROLoader::openTypedRoutes(const std::string& optionName,
 
 bool
 ROLoader::loadWeights(RONet& net, const std::string& optionName,
-                      const std::string& measure, bool useLanes) {
+                      const std::string& measure, const bool useLanes, const bool boundariesOverride) {
     // check whether the file exists
     if (!myOptions.isUsableFileList(optionName)) {
         return false;
@@ -280,7 +280,7 @@ ROLoader::loadWeights(RONet& net, const std::string& optionName,
     EdgeFloatTimeLineRetriever_EdgeWeight eRetriever(net);
     if (measure != "traveltime") {
         std::string umeasure = measure;
-        if (measure == "CO" || measure == "CO2" || measure == "HC" || measure == "PMx" || measure == "NOx" || measure == "fuel") {
+        if (measure == "CO" || measure == "CO2" || measure == "HC" || measure == "PMx" || measure == "NOx" || measure == "fuel" || measure == "electricity") {
             umeasure = measure + "_perVeh";
         }
         retrieverDefs.push_back(new SAXWeightsHandler::ToRetrieveDefinition(umeasure, !useLanes, eRetriever));
@@ -301,7 +301,7 @@ ROLoader::loadWeights(RONet& net, const std::string& optionName,
     // build edge-internal time lines
     const std::map<std::string, ROEdge*>& edges = net.getEdgeMap();
     for (std::map<std::string, ROEdge*>::const_iterator i = edges.begin(); i != edges.end(); ++i) {
-        (*i).second->buildTimeLines(measure);
+        (*i).second->buildTimeLines(measure, boundariesOverride);
     }
     return true;
 }
