@@ -5,12 +5,12 @@
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    Sept 2002
-/// @version $Id: ROLoader.h 18397 2015-05-18 14:04:14Z behrisch $
+/// @version $Id: ROLoader.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // Loader for networks and route imports
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2002-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2002-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -34,10 +34,10 @@
 #endif
 
 #include <utils/common/SUMOTime.h>
-#include <utils/vehicle/SUMOAbstractRouter.h>
 #include <utils/common/ValueTimeLine.h>
 #include <utils/xml/SAXWeightsHandler.h>
 #include <utils/xml/SUMORouteLoaderControl.h>
+#include "RORoutable.h"
 
 
 // ===========================================================================
@@ -79,14 +79,14 @@ public:
 
     /// Loads the net weights
     bool loadWeights(RONet& net, const std::string& optionName,
-                     const std::string& measure, bool useLanes);
+                     const std::string& measure, const bool useLanes, const bool boundariesOverride);
 
     /** @brief Builds and opens all route loaders */
     void openRoutes(RONet& net);
 
     /** @brief Loads routes from all previously build route loaders */
     void processRoutes(const SUMOTime start, const SUMOTime end, const SUMOTime increment,
-                       RONet& net, SUMOAbstractRouter<ROEdge, ROVehicle>& router);
+                       RONet& net, const RORouterProvider& provider);
 
 protected:
     /** @brief Opens route handler of the given type
