@@ -4,12 +4,12 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: RONet.cpp 18467 2015-05-29 03:50:41Z behrisch $
+/// @version $Id: RONet.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // The router's network representation
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -31,12 +31,6 @@
 #endif
 
 #include <algorithm>
-#include "ROEdge.h"
-#include "RONode.h"
-#include "RONet.h"
-#include "RORoute.h"
-#include "RORouteDef.h"
-#include "ROVehicle.h"
 #include <utils/vehicle/RouteCostCalculator.h>
 #include <utils/vehicle/SUMOVTypeParameter.h>
 #include <utils/vehicle/SUMOAbstractRouter.h>
@@ -46,6 +40,13 @@
 #include <utils/common/RandHelper.h>
 #include <utils/common/SUMOVehicleClass.h>
 #include <utils/iodevices/OutputDevice.h>
+#include "ROEdge.h"
+#include "RONode.h"
+#include "ROPerson.h"
+#include "RORoute.h"
+#include "RORouteDef.h"
+#include "ROVehicle.h"
+#include "RONet.h"
 
 #ifdef CHECK_MEMORY_LEAKS
 #include <foreign/nvwa/debug_new.h>
@@ -71,7 +72,7 @@ RONet::getInstance(void) {
 
 
 RONet::RONet()
-    : myVehicleTypes(), myDefaultVTypeMayBeDeleted(true),
+    : myVehicleTypes(), myDefaultVTypeMayBeDeleted(true), myDefaultPedTypeMayBeDeleted(true),
       myRoutesOutput(0), myRouteAlternativesOutput(0), myTypesOutput(0),
       myReadRouteNo(0), myDiscardedRouteNo(0), myWrittenRouteNo(0),
       myHavePermissions(false),
@@ -84,6 +85,10 @@ RONet::RONet()
     SUMOVTypeParameter* type = new SUMOVTypeParameter(DEFAULT_VTYPE_ID, SVC_IGNORING);
     type->onlyReferenced = true;
     myVehicleTypes.add(type->id, type);
+    SUMOVTypeParameter* defPedType = new SUMOVTypeParameter(DEFAULT_PEDTYPE_ID, SVC_PEDESTRIAN);
+    defPedType->onlyReferenced = true;
+    defPedType->setParameter |= VTYPEPARS_VEHICLECLASS_SET;
+    myVehicleTypes.add(defPedType->id, defPedType);
     myInstance = this;
 }
 
@@ -93,7 +98,7 @@ RONet::~RONet() {
     myEdges.clear();
     myVehicleTypes.clear();
     myRoutes.clear();
-    myVehicles.clear();
+    myRoutables.clear();
 }
 
 
@@ -223,7 +228,7 @@ RONet::openOutput(const std::string& filename, const std::string altFilename, co
 
 
 void
-RONet::cleanup(SUMOAbstractRouter<ROEdge, ROVehicle>* router) {
+RONet::cleanup() {
     // end writing
     if (myRoutesOutput != 0) {
         myRoutesOutput->close();
@@ -240,10 +245,8 @@ RONet::cleanup(SUMOAbstractRouter<ROEdge, ROVehicle>* router) {
 #ifdef HAVE_FOX
     if (myThreadPool.size() > 0) {
         myThreadPool.clear();
-        return;
     }
 #endif
-    delete router;
 }
 
 
@@ -255,6 +258,9 @@ RONet::getVehicleTypeSecure(const std::string& id) {
     if (id == DEFAULT_VTYPE_ID) {
         myDefaultVTypeMayBeDeleted = false;
     }
+    if (id == DEFAULT_PEDTYPE_ID) {
+        myDefaultPedTypeMayBeDeleted = false;
+    }
     if (type != 0) {
         return type;
     }
@@ -281,6 +287,13 @@ RONet::checkVType(const std::string& id) {
         } else {
             return false;
         }
+    } else if (id == DEFAULT_PEDTYPE_ID) {
+        if (myDefaultPedTypeMayBeDeleted) {
+            myVehicleTypes.remove(id);
+            myDefaultPedTypeMayBeDeleted = false;
+        } else {
+            return false;
+        }
     } else {
         if (myVehicleTypes.get(id) != 0 || myVTypeDistDict.find(id) != myVTypeDistDict.end()) {
             return false;
@@ -315,8 +328,9 @@ RONet::addVTypeDistribution(const std::string& id, RandomDistributor<SUMOVTypePa
 
 bool
 RONet::addVehicle(const std::string& id, ROVehicle* veh) {
-    if (myVehIDs.find(id) == myVehIDs.end() && myVehicles.add(id, veh)) {
+    if (myVehIDs.find(id) == myVehIDs.end()) {
         myVehIDs.insert(id);
+        myRoutables[veh->getDepart()].push_back(veh);
         myReadRouteNo++;
         return true;
     }
@@ -339,56 +353,31 @@ RONet::addFlow(SUMOVehicleParameter* flow, const bool randomize) {
 }
 
 
-void
-RONet::addPerson(const SUMOTime depart, const std::string desc) {
-    myPersons.insert(std::pair<const SUMOTime, const std::string>(depart, desc));
+bool
+RONet::addPerson(ROPerson* person) {
+    if (myPersonIDs.count(person->getID()) == 0) {
+        myPersonIDs.insert(person->getID());
+        myRoutables[person->getDepart()].push_back(person);
+        return true;
+    }
+    WRITE_ERROR("Another person with the id '" + person->getID() + "' exists.");
+    return false;
 }
 
+
 void
 RONet::addContainer(const SUMOTime depart, const std::string desc) {
     myContainers.insert(std::pair<const SUMOTime, const std::string>(depart, desc));
 }
 
 
-bool
-RONet::computeRoute(SUMOAbstractRouter<ROEdge, ROVehicle>& router,
-                    const ROVehicle* const veh, const bool removeLoops,
-                    MsgHandler* errorHandler) {
-    std::string noRouteMsg = "The vehicle '" + veh->getID() + "' has no valid route.";
-    RORouteDef* const routeDef = veh->getRouteDefinition();
-    // check if the route definition is valid
-    if (routeDef == 0) {
-        errorHandler->inform(noRouteMsg);
-        return false;
-    }
-    RORoute* current = routeDef->buildCurrentRoute(router, veh->getDepartureTime(), *veh);
-    if (current == 0 || current->size() == 0) {
-        delete current;
-        errorHandler->inform(noRouteMsg);
-        return false;
-    }
-    // check whether we have to evaluate the route for not containing loops
-    if (removeLoops) {
-        current->recheckForLoops();
-        // check whether the route is still valid
-        if (current->size() == 0) {
-            delete current;
-            errorHandler->inform(noRouteMsg + " (after removing loops)");
-            return false;
-        }
-    }
-    // add built route
-    routeDef->addAlternative(router, veh, current, veh->getDepartureTime());
-    return true;
-}
-
-
 void
-RONet::checkFlows(SUMOTime time) {
+RONet::checkFlows(SUMOTime time, MsgHandler* errorHandler) {
     std::vector<std::string> toRemove;
     for (NamedObjectCont<SUMOVehicleParameter*>::IDMap::const_iterator i = myFlows.getMyMap().begin(); i != myFlows.getMyMap().end(); ++i) {
         SUMOVehicleParameter* pars = i->second;
         if (pars->repetitionProbability > 0) {
+            const SUMOTime origDepart = pars->depart;
             while (pars->depart < time) {
                 if (pars->repetitionEnd <= pars->depart) {
                     toRemove.push_back(i->first);
@@ -399,6 +388,11 @@ RONet::checkFlows(SUMOTime time) {
                     SUMOVehicleParameter* newPars = new SUMOVehicleParameter(*pars);
                     newPars->id = pars->id + "." + toString(pars->repetitionsDone);
                     newPars->depart = pars->depart;
+                    for (std::vector<SUMOVehicleParameter::Stop>::iterator stop = newPars->stops.begin(); stop != newPars->stops.end(); ++stop) {
+                        if (stop->until >= 0) {
+                            stop->until += pars->depart - origDepart;
+                        }
+                    }
                     pars->repetitionsDone++;
                     // try to build the vehicle
                     SUMOVTypeParameter* type = getVehicleTypeSecure(pars->vtypeid);
@@ -408,8 +402,9 @@ RONet::checkFlows(SUMOTime time) {
                         // fix the type id in case we used a distribution
                         newPars->vtypeid = type->id;
                     }
-                    RORouteDef* route = getRouteDef(pars->routeid)->copy("!" + newPars->id);
-                    ROVehicle* veh = new ROVehicle(*newPars, route, type, this);
+                    const SUMOTime stopOffset = pars->routeid[0] == '!' ? pars->depart - origDepart : pars->depart;
+                    RORouteDef* route = getRouteDef(pars->routeid)->copy("!" + newPars->id, stopOffset);
+                    ROVehicle* veh = new ROVehicle(*newPars, route, type, this, errorHandler);
                     addVehicle(newPars->id, veh);
                     delete newPars;
                 }
@@ -430,6 +425,11 @@ RONet::checkFlows(SUMOTime time) {
                 SUMOVehicleParameter* newPars = new SUMOVehicleParameter(*pars);
                 newPars->id = pars->id + "." + toString(pars->repetitionsDone);
                 newPars->depart = depart;
+                for (std::vector<SUMOVehicleParameter::Stop>::iterator stop = newPars->stops.begin(); stop != newPars->stops.end(); ++stop) {
+                    if (stop->until >= 0) {
+                        stop->until += depart - pars->depart;
+                    }
+                }
                 pars->repetitionsDone++;
                 // try to build the vehicle
                 SUMOVTypeParameter* type = getVehicleTypeSecure(pars->vtypeid);
@@ -439,8 +439,9 @@ RONet::checkFlows(SUMOTime time) {
                     // fix the type id in case we used a distribution
                     newPars->vtypeid = type->id;
                 }
-                RORouteDef* route = getRouteDef(pars->routeid)->copy("!" + newPars->id);
-                ROVehicle* veh = new ROVehicle(*newPars, route, type, this);
+                const SUMOTime stopOffset = pars->routeid[0] == '!' ? depart - pars->depart : depart;
+                RORouteDef* route = getRouteDef(pars->routeid)->copy("!" + newPars->id, stopOffset);
+                ROVehicle* veh = new ROVehicle(*newPars, route, type, this, errorHandler);
                 addVehicle(newPars->id, veh);
                 delete newPars;
             }
@@ -456,30 +457,33 @@ RONet::checkFlows(SUMOTime time) {
 
 
 void
-RONet::createBulkRouteRequests(SUMOAbstractRouter<ROEdge, ROVehicle>& router, const SUMOTime time, const bool removeLoops, const std::map<std::string, ROVehicle*>& mmap) {
-    std::map<const ROEdge*, std::vector<ROVehicle*> > bulkVehs;
-    for (std::map<std::string, ROVehicle*>::const_iterator i = mmap.begin(); i != mmap.end(); ++i) {
-        ROVehicle* const vehicle = i->second;
-        if (vehicle->getDepart() < time) {
-            const RORoute* const stub = vehicle->getRouteDefinition()->getFirstRoute();
-            bulkVehs[stub->getFirst()].push_back(vehicle);
-            ROVehicle* const first = bulkVehs[stub->getFirst()].front();
-            if (first->getMaxSpeed() != vehicle->getMaxSpeed()) {
-                WRITE_WARNING("Bulking different maximum speeds ('" + first->getID() + "' and '" + vehicle->getID() + "') may lead to suboptimal routes.");
+RONet::createBulkRouteRequests(const RORouterProvider& provider, const SUMOTime time, const bool removeLoops) {
+    std::map<const unsigned int, std::vector<RORoutable*> > bulkVehs;
+    for (RoutablesMap::const_iterator i = myRoutables.begin(); i != myRoutables.end(); ++i) {
+        if (i->first >= time) {
+            break;
+        }
+        for (std::deque<RORoutable*>::const_iterator r = i->second.begin(); r != i->second.end(); ++r) {
+            RORoutable* const routable = *r;
+            const ROEdge* const depEdge = routable->getDepartEdge();
+            bulkVehs[depEdge->getNumericalID()].push_back(routable);
+            RORoutable* const first = bulkVehs[depEdge->getNumericalID()].front();
+            if (first->getMaxSpeed() != routable->getMaxSpeed()) {
+                WRITE_WARNING("Bulking different maximum speeds ('" + first->getID() + "' and '" + routable->getID() + "') may lead to suboptimal routes.");
             }
-            if (first->getVClass() != vehicle->getVClass()) {
-                WRITE_WARNING("Bulking different vehicle classes ('" + first->getID() + "' and '" + vehicle->getID() + "') may lead to invalid routes.");
+            if (first->getVClass() != routable->getVClass()) {
+                WRITE_WARNING("Bulking different vehicle classes ('" + first->getID() + "' and '" + routable->getID() + "') may lead to invalid routes.");
             }
         }
     }
     int workerIndex = 0;
-    for (std::map<const ROEdge*, std::vector<ROVehicle*> >::const_iterator i = bulkVehs.begin(); i != bulkVehs.end(); ++i) {
+    for (std::map<const unsigned int, std::vector<RORoutable*> >::const_iterator i = bulkVehs.begin(); i != bulkVehs.end(); ++i) {
 #ifdef HAVE_FOX
         if (myThreadPool.size() > 0) {
-            ROVehicle* const first = i->second.front();
+            RORoutable* const first = i->second.front();
             myThreadPool.add(new RoutingTask(first, removeLoops, myErrorHandler), workerIndex);
             myThreadPool.add(new BulkmodeTask(true), workerIndex);
-            for (std::vector<ROVehicle*>::const_iterator j = i->second.begin() + 1; j != i->second.end(); ++j) {
+            for (std::vector<RORoutable*>::const_iterator j = i->second.begin() + 1; j != i->second.end(); ++j) {
                 myThreadPool.add(new RoutingTask(*j, removeLoops, myErrorHandler), workerIndex);
             }
             myThreadPool.add(new BulkmodeTask(false), workerIndex);
@@ -490,54 +494,53 @@ RONet::createBulkRouteRequests(SUMOAbstractRouter<ROEdge, ROVehicle>& router, co
             continue;
         }
 #endif
-        for (std::vector<ROVehicle*>::const_iterator j = i->second.begin(); j != i->second.end(); ++j) {
-            (*j)->setRoutingSuccess(computeRoute(router, *j, removeLoops, myErrorHandler));
-            router.setBulkMode(true);
+        for (std::vector<RORoutable*>::const_iterator j = i->second.begin(); j != i->second.end(); ++j) {
+            (*j)->computeRoute(provider, removeLoops, myErrorHandler);
+            provider.getVehicleRouter().setBulkMode(true);
         }
-        router.setBulkMode(false);
+        provider.getVehicleRouter().setBulkMode(false);
     }
 }
 
 
 SUMOTime
-RONet::saveAndRemoveRoutesUntil(OptionsCont& options, SUMOAbstractRouter<ROEdge, ROVehicle>& router,
+RONet::saveAndRemoveRoutesUntil(OptionsCont& options, const RORouterProvider& provider,
                                 SUMOTime time) {
-    checkFlows(time);
+    MsgHandler* mh = (options.getBool("ignore-errors") ?
+                      MsgHandler::getWarningInstance() : MsgHandler::getErrorInstance());
+    checkFlows(time, mh);
     SUMOTime lastTime = -1;
     const bool removeLoops = options.getBool("remove-loops");
-#ifdef HAVE_FOX
     const int maxNumThreads = options.getInt("routing-threads");
-#endif
-    if (myVehicles.size() != 0) {
-        const std::map<std::string, ROVehicle*>& mmap = myVehicles.getMyMap();
+    if (myRoutables.size() != 0) {
         if (options.getBool("bulk-routing")) {
 #ifdef HAVE_FOX
             while ((int)myThreadPool.size() < maxNumThreads) {
-                new WorkerThread(myThreadPool, myThreadPool.size() == 0 ? &router : router.clone());
+                new WorkerThread(myThreadPool, provider);
             }
 #endif
-            createBulkRouteRequests(router, time, removeLoops, mmap);
+            createBulkRouteRequests(provider, time, removeLoops);
         } else {
-            for (std::map<std::string, ROVehicle*>::const_iterator i = mmap.begin(); i != mmap.end(); ++i) {
-                ROVehicle* const vehicle = i->second;
-                if (vehicle->getDepart() >= time) {
-                    // we cannot go through a sorted list here, because the priority queue in the myVehicles container is not fully sorted
-                    continue;
+            for (RoutablesMap::const_iterator i = myRoutables.begin(); i != myRoutables.end(); ++i) {
+                if (i->first >= time) {
+                    break;
                 }
+                for (std::deque<RORoutable*>::const_iterator r = i->second.begin(); r != i->second.end(); ++r) {
+                    RORoutable* const routable = *r;
 #ifdef HAVE_FOX
-                // add task
-                if (maxNumThreads > 0) {
-                    vehicle->setRoutingSuccess(false);
-                    // add thread if necessary
-                    const int numThreads = (int)myThreadPool.size();
-                    if (numThreads < maxNumThreads && myThreadPool.isFull()) {
-                        new WorkerThread(myThreadPool, numThreads == 0 ? &router : router.clone());
+                    // add task
+                    if (maxNumThreads > 0) {
+                        // add thread if necessary
+                        const int numThreads = (int)myThreadPool.size();
+                        if (numThreads < maxNumThreads && myThreadPool.isFull()) {
+                            new WorkerThread(myThreadPool, provider);
+                        }
+                        myThreadPool.add(new RoutingTask(routable, removeLoops, myErrorHandler));
+                        continue;
                     }
-                    myThreadPool.add(new RoutingTask(vehicle, removeLoops, myErrorHandler));
-                    continue;
-                }
 #endif
-                vehicle->setRoutingSuccess(computeRoute(router, vehicle, removeLoops, myErrorHandler));
+                    routable->computeRoute(provider, removeLoops, myErrorHandler);
+                }
             }
         }
 #ifdef HAVE_FOX
@@ -545,56 +548,48 @@ RONet::saveAndRemoveRoutesUntil(OptionsCont& options, SUMOAbstractRouter<ROEdge,
 #endif
     }
     // write all vehicles (and additional structures)
-    while (myVehicles.size() != 0 || myPersons.size() != 0 || myContainers.size() != 0) {
+    while (myRoutables.size() != 0 || myContainers.size() != 0) {
         // get the next vehicle, person or container
-        const ROVehicle* const veh = myVehicles.getTopVehicle();
-        const SUMOTime vehicleTime = veh == 0 ? SUMOTime_MAX : veh->getDepart();
-        PersonMap::iterator person = myPersons.begin();
-        const SUMOTime personTime = person == myPersons.end() ? SUMOTime_MAX : person->first;
+        RoutablesMap::iterator routables = myRoutables.begin();
+        const SUMOTime routableTime = routables == myRoutables.end() ? SUMOTime_MAX : routables->first;
         ContainerMap::iterator container = myContainers.begin();
         const SUMOTime containerTime = container == myContainers.end() ? SUMOTime_MAX : container->first;
         // check whether it shall not yet be computed
-        if (vehicleTime >= time && personTime >= time && containerTime >= time) {
-            lastTime = MIN3(vehicleTime, personTime, containerTime);
+        if (routableTime >= time && containerTime >= time) {
+            lastTime = MIN2(routableTime, containerTime);
             break;
         }
-        SUMOTime minTime = MIN3(vehicleTime, personTime, containerTime);
-        if (vehicleTime == minTime) {
+        const SUMOTime minTime = MIN2(routableTime, containerTime);
+        if (routableTime == minTime) {
+            const RORoutable* const r = routables->second.front();
             // check whether to print the output
-            if (lastTime != vehicleTime && lastTime != -1) {
+            if (lastTime != routableTime && lastTime != -1) {
                 // report writing progress
-                if (options.getInt("stats-period") >= 0 && ((int) vehicleTime % options.getInt("stats-period")) == 0) {
+                if (options.getInt("stats-period") >= 0 && ((int) routableTime % options.getInt("stats-period")) == 0) {
                     WRITE_MESSAGE("Read: " + toString(myReadRouteNo) + ",  Discarded: " + toString(myDiscardedRouteNo) + ",  Written: " + toString(myWrittenRouteNo));
                 }
             }
-            lastTime = vehicleTime;
+            lastTime = routableTime;
 
             // ok, compute the route (try it)
-            if (veh->getRoutingSuccess()) {
+            if (r->getRoutingSuccess()) {
                 // write the route
-                veh->saveTypeAsXML(*myRoutesOutput, myRouteAlternativesOutput, myTypesOutput);
-                veh->saveAllAsXML(*myRoutesOutput, false, options.getBool("exit-times"));
-                if (myRouteAlternativesOutput != 0) {
-                    veh->saveAllAsXML(*myRouteAlternativesOutput, true, options.getBool("exit-times"));
-                }
+                r->write(*myRoutesOutput, myRouteAlternativesOutput, myTypesOutput, options);
                 myWrittenRouteNo++;
             } else {
                 myDiscardedRouteNo++;
             }
+            const ROVehicle* const veh = dynamic_cast<const ROVehicle*>(r);
             // delete routes and the vehicle
-            if (veh->getRouteDefinition()->getID()[0] == '!') {
+            if (veh != 0 && veh->getRouteDefinition()->getID()[0] == '!') {
                 if (!myRoutes.erase(veh->getRouteDefinition()->getID())) {
                     delete veh->getRouteDefinition();
                 }
             }
-            myVehicles.erase(veh->getID());
-        }
-        if (personTime == minTime) {
-            myRoutesOutput->writePreformattedTag(person->second);
-            if (myRouteAlternativesOutput != 0) {
-                myRouteAlternativesOutput->writePreformattedTag(person->second);
+            routables->second.pop_front();
+            if (routables->second.empty()) {
+                myRoutables.erase(routables);
             }
-            myPersons.erase(person);
         }
         if (containerTime == minTime) {
             myRoutesOutput->writePreformattedTag(container->second);
@@ -610,7 +605,7 @@ RONet::saveAndRemoveRoutesUntil(OptionsCont& options, SUMOAbstractRouter<ROEdge,
 
 bool
 RONet::furtherStored() {
-    return myVehicles.size() > 0 || myFlows.size() > 0 || myPersons.size() > 0 || myContainers.size() > 0;
+    return myRoutables.size() > 0 || myFlows.size() > 0 || myContainers.size() > 0;
 }
 
 
@@ -632,6 +627,33 @@ RONet::getEdgeMap() const {
 }
 
 
+void
+RONet::adaptIntermodalRouter(ROIntermodalRouter& router) {
+    // add access to all public transport stops
+    for (std::map<std::string, SUMOVehicleParameter::Stop*>::const_iterator i = myInstance->myBusStops.begin(); i != myInstance->myBusStops.end(); ++i) {
+        router.addAccess(i->first, myInstance->getEdgeForLaneID(i->second->lane), i->second->endPos);
+        for (std::multimap<std::string, SUMOReal>::const_iterator a = i->second->accessPos.begin(); a != i->second->accessPos.end(); ++a) {
+            router.addAccess(i->first, myInstance->getEdgeForLaneID(a->first), a->second);
+        }
+    }
+    // fill the public transport router with pre-parsed public transport lines
+    for (std::map<std::string, SUMOVehicleParameter*>::const_iterator i = myInstance->myFlows.getMyMap().begin(); i != myInstance->myFlows.getMyMap().end(); ++i) {
+        if (i->second->line != "") {
+            router.addSchedule(*i->second);
+        }
+    }
+    for (RoutablesMap::const_iterator i = myInstance->myRoutables.begin(); i != myInstance->myRoutables.end(); ++i) {
+        for (std::deque<RORoutable*>::const_iterator r = i->second.begin(); r != i->second.end(); ++r) {
+            const ROVehicle* const veh = dynamic_cast<ROVehicle*>(*r);
+            // add single vehicles with line attribute which are not part of a flow
+            if (veh != 0 && veh->getParameter().line != "" && veh->getParameter().repetitionNumber < 0) {
+                router.addSchedule(veh->getParameter());
+            }
+        }
+    }
+}
+
+
 bool
 RONet::hasPermissions() const {
     return myHavePermissions;
@@ -650,7 +672,7 @@ RONet::setPermissionsFound() {
 // ---------------------------------------------------------------------------
 void
 RONet::RoutingTask::run(FXWorkerThread* context) {
-    myVehicle->setRoutingSuccess(RONet::computeRoute(static_cast<WorkerThread*>(context)->getRouter(), myVehicle, myRemoveLoops, myErrorHandler));
+    myRoutable->computeRoute(*static_cast<WorkerThread*>(context), myRemoveLoops, myErrorHandler);
 }
 #endif
 
