@@ -5,12 +5,12 @@
 /// @author  Jakob Erdmann
 /// @author  Yun-Pang Floetteroed
 /// @date    Sept 2002
-/// @version $Id: RONet.h 17262 2014-11-04 20:41:44Z behrisch $
+/// @version $Id: RONet.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // The router's network representation
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2002-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2002-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -33,16 +33,18 @@
 #include <config.h>
 #endif
 
+#include <queue>
 #include <vector>
-#include "ROEdge.h"
-#include "RONode.h"
-#include "ROVehicleCont.h"
-#include "ROVehicle.h"
-#include "RORouteDef.h"
 #include <utils/common/MsgHandler.h>
-#include <utils/vehicle/SUMOVTypeParameter.h>
-#include <utils/vehicle/SUMOAbstractRouter.h>
+#include <utils/common/NamedObjectCont.h>
 #include <utils/common/RandomDistributor.h>
+#include <utils/vehicle/PedestrianRouter.h>
+#include <utils/vehicle/SUMOAbstractRouter.h>
+#include <utils/vehicle/SUMOVehicleParameter.h>
+#include <utils/vehicle/SUMOVTypeParameter.h>
+#include "ROLane.h"
+#include "RORoutable.h"
+#include "RORouteDef.h"
 
 #ifdef HAVE_FOX
 #include <utils/foxtools/FXWorkerThread.h>
@@ -52,8 +54,12 @@
 // ===========================================================================
 // class declarations
 // ===========================================================================
+class ROEdge;
+class ROLane;
 class RONode;
-class RORouteDef;
+class ROPerson;
+class RORoutable;
+class ROVehicle;
 class OptionsCont;
 class OutputDevice;
 
@@ -66,21 +72,42 @@ class OutputDevice;
  * @brief The router's network representation.
  *
  * A router network is responsible for watching loaded edges, nodes,!!!
- *
- * @todo Vehicle ids are not tracked; it may happen that the same id is added twice...
  */
 class RONet {
-    friend class RORouteAggregator;
-
 public:
+
+    typedef std::map<const SUMOTime, std::deque<RORoutable*> > RoutablesMap;
+
     /// @brief Constructor
     RONet();
 
 
+    /** @brief Returns the pointer to the unique instance of RONet (singleton).
+     * @return Pointer to the unique RONet-instance
+     */
+    static RONet* getInstance();
+
+
     /// @brief Destructor
     virtual ~RONet();
 
 
+    /** @brief Adds a restriction for an edge type
+     * @param[in] id The id of the type
+     * @param[in] svc The vehicle class the restriction refers to
+     * @param[in] speed The restricted speed
+     */
+    void addRestriction(const std::string& id, const SUMOVehicleClass svc, const SUMOReal speed);
+
+
+    /** @brief Returns the restrictions for an edge type
+     * If no restrictions are present, 0 is returned.
+     * @param[in] id The id of the type
+     * @return The mapping of vehicle classes to maximum speeds
+     */
+    const std::map<SUMOVehicleClass, SUMOReal>* getRestrictions(const std::string& id) const;
+
+
     /// @name Insertion and retrieval of graph parts
     //@{
 
@@ -96,6 +123,37 @@ public:
     virtual bool addEdge(ROEdge* edge);
 
 
+    /* @brief Adds a district and connecting edges to the network
+     *
+     * If the district is already known (another one with the same id exists),
+     *  an error is generated and given to msg-error-handler. The edges
+     *  are deleted in this case and false is returned.
+     *
+     * @param[in] id The district to add
+     * @return Whether the district was added
+     */
+    bool addDistrict(const std::string id, ROEdge* source, ROEdge* sink);
+
+
+    /* @brief Adds a district and connecting edges to the network
+     *
+     * If the district is already known (another one with the same id exists),
+     *  an error is generated and given to msg-error-handler. The edges
+     *  are deleted in this case and false is returned.
+     *
+     * @param[in] id The district to add
+     * @return Whether the district was added
+     */
+    bool addDistrictEdge(const std::string tazID, const std::string edgeID, const bool isSource);
+
+    /** @brief Retrieves all TAZ (districts) from the network
+     *
+     * @return The map of all districts
+     */
+    const std::map<std::string, std::pair<std::vector<std::string>, std::vector<std::string> > >& getDistricts() const {
+        return myDistricts;
+    }
+
     /** @brief Retrieves an edge from the network
      *
      * This is not very pretty, but necessary, though, as routes run
@@ -109,6 +167,16 @@ public:
     }
 
 
+    /** @brief Retrieves an edge from the network when the lane id is given
+     *
+     * @param[in] laneID The name of the lane to retrieve the edge for
+     * @return The edge of the named lane if known, otherwise 0
+     */
+    ROEdge* getEdgeForLaneID(const std::string& laneID) const {
+        return getEdge(laneID.substr(0, laneID.rfind("_")));
+    }
+
+
     /* @brief Adds a read node to the network
      *
      * If the node is already known (another one with the same id exists),
@@ -142,6 +210,17 @@ public:
     void addBusStop(const std::string& id, SUMOVehicleParameter::Stop* stop);
 
 
+    /* @brief Adds a read container stop to the network
+     *
+     * If the container stop is already known (another one with the same id exists),
+     *  an error is generated and given to msg-error-handler. The stop
+     *  is deleted in this case
+     *
+     * @param[in] node The stop to add
+     */
+    void addContainerStop(const std::string& id, SUMOVehicleParameter::Stop* stop);
+
+
     /** @brief Retrieves a bus stop from the network
      *
      * @param[in] name The name of the stop to retrieve
@@ -154,6 +233,20 @@ public:
         }
         return it->second;
     }
+
+
+    /** @brief Retrieves a container stop from the network
+     *
+     * @param[in] name The name of the stop to retrieve
+     * @return The named stop if known, otherwise 0
+     */
+    const SUMOVehicleParameter::Stop* getContainerStop(const std::string& id) const {
+        std::map<std::string, SUMOVehicleParameter::Stop*>::const_iterator it = myContainerStops.find(id);
+        if (it == myContainerStops.end()) {
+            return 0;
+        }
+        return it->second;
+    }
     //@}
 
 
@@ -200,11 +293,9 @@ public:
 
     /** @brief Retrieves the named vehicle type
      *
-     * If the named vehicle type was not added to the net before, a default
-     *  vehicle type which consists of the id only is generated, added to the net
-     *  and returned.
-     *
-     * Only if the name is "", 0 is returned.
+     * If the name is "" the default type is returned.
+     * If the named vehicle type (or typeDistribution) was not added to the net before
+     * 0 is returned
      *
      * @param[in] id The id of the vehicle type to return
      * @return The named vehicle type
@@ -267,10 +358,17 @@ public:
 
     /* @brief Adds a person to the network
      *
-     * @param[in] depart The departure time of the person
-     * @param[in] desc   The xml description of the person
+     * @param[in] person   The person to add
      */
-    void addPerson(const SUMOTime depart, const std::string desc);
+    bool addPerson(ROPerson* person);
+
+
+    /* @brief Adds a container to the network
+     *
+     * @param[in] depart The departure time of the container
+     * @param[in] desc   The xml description of the container
+     */
+    void addContainer(const SUMOTime depart, const std::string desc);
     // @}
 
 
@@ -284,15 +382,15 @@ public:
      *  the internal container.
      *
      * @param[in] options The options used during this process
-     * @param[in] router The router to use for routes computation
+     * @param[in] provider The router provider for routes computation
      * @param[in] time The time until which route definitions shall be processed
      * @return The last seen departure time>=time
      */
     SUMOTime saveAndRemoveRoutesUntil(OptionsCont& options,
-                                      SUMOAbstractRouter<ROEdge, ROVehicle>& router, SUMOTime time);
+                                      const RORouterProvider& provider, SUMOTime time);
 
 
-    /// Returns the information whether further vehicles are stored
+    /// Returns the information whether further vehicles, persons or containers are stored
     virtual bool furtherStored();
     //@}
 
@@ -313,8 +411,8 @@ public:
     void openOutput(const std::string& filename, const std::string altFilename, const std::string typeFilename);
 
 
-    /** @brief closes the file output for computed routes and deletes routers and associated threads if necessary */
-    void cleanup(SUMOAbstractRouter<ROEdge, ROVehicle>* router);
+    /** @brief closes the file output for computed routes and deletes associated threads if necessary */
+    void cleanup();
 
 
     /// Returns the total number of edges the network contains including internal edges
@@ -325,9 +423,11 @@ public:
 
     const std::map<std::string, ROEdge*>& getEdgeMap() const;
 
-    bool hasRestrictions() const;
+    static void adaptIntermodalRouter(ROIntermodalRouter& router);
 
-    void setRestrictionFound();
+    bool hasPermissions() const;
+
+    void setPermissionsFound();
 
     OutputDevice* getRouteOutput(const bool alternative = false) {
         if (alternative) {
@@ -336,24 +436,51 @@ public:
         return myRoutesOutput;
     }
 
-protected:
-    static bool computeRoute(SUMOAbstractRouter<ROEdge, ROVehicle>& router,
-                             const ROVehicle* const veh, const bool removeLoops,
-                             MsgHandler* errorHandler);
-
-    /// @brief return vehicles for use by RouteAggregator
-    ROVehicleCont& getVehicles() {
-        return myVehicles;
+#ifdef HAVE_FOX
+    FXWorkerThread::Pool& getThreadPool() {
+        return myThreadPool;
     }
 
+    class WorkerThread : public FXWorkerThread, public RORouterProvider {
+    public:
+        WorkerThread(FXWorkerThread::Pool& pool,
+                     const RORouterProvider& original)
+            : FXWorkerThread(pool), RORouterProvider(original) {}
+        virtual ~WorkerThread() {
+            stop();
+        }
+    };
 
-    void checkFlows(SUMOTime time);
+    class BulkmodeTask : public FXWorkerThread::Task {
+    public:
+        BulkmodeTask(const bool value) : myValue(value) {}
+        void run(FXWorkerThread* context) {
+            static_cast<WorkerThread*>(context)->getVehicleRouter().setBulkMode(myValue);
+        }
+    private:
+        const bool myValue;
+    private:
+        /// @brief Invalidated assignment operator.
+        BulkmodeTask& operator=(const BulkmodeTask&);
+    };
+#endif
 
 
-protected:
+private:
+    void checkFlows(SUMOTime time, MsgHandler* errorHandler);
+
+    void createBulkRouteRequests(const RORouterProvider& provider, const SUMOTime time, const bool removeLoops);
+
+private:
+    /// @brief Unique instance of RONet
+    static RONet* myInstance;
+
     /// @brief Known vehicle ids
     std::set<std::string> myVehIDs;
 
+    /// @brief Known person ids
+    std::set<std::string> myPersonIDs;
+
     /// @brief Known nodes
     NamedObjectCont<RONode*> myNodes;
 
@@ -363,6 +490,9 @@ protected:
     /// @brief Known bus stops
     std::map<std::string, SUMOVehicleParameter::Stop*> myBusStops;
 
+    /// @brief Known container stops
+    std::map<std::string, SUMOVehicleParameter::Stop*> myContainerStops;
+
     /// @brief Known vehicle types
     NamedObjectCont<SUMOVTypeParameter*> myVehicleTypes;
 
@@ -371,25 +501,31 @@ protected:
     /// @brief A distribution of vehicle types (probability->vehicle type)
     VTypeDistDictType myVTypeDistDict;
 
-    /// @brief Whether no vehicle type was loaded
+    /// @brief Whether no vehicle type has been loaded yet
     bool myDefaultVTypeMayBeDeleted;
 
+    /// @brief Whether no pedestrian type has been loaded yet
+    bool myDefaultPedTypeMayBeDeleted;
+
     /// @brief Known routes
     NamedObjectCont<RORouteDef*> myRoutes;
 
-    /// @brief Known vehicles
-    ROVehicleCont myVehicles;
+    /// @brief Known routables
+    RoutablesMap myRoutables;
 
     /// @brief Known flows
     NamedObjectCont<SUMOVehicleParameter*> myFlows;
 
-    /// @brief Known persons
-    typedef std::multimap<const SUMOTime, const std::string> PersonMap;
-    PersonMap myPersons;
+    /// @brief Known containers
+    typedef std::multimap<const SUMOTime, const std::string> ContainerMap;
+    ContainerMap myContainers;
 
     /// @brief Departure times for randomized flows
     std::map<std::string, std::vector<SUMOTime> > myDepartures;
 
+    /// @brief traffic assignment zones with sources and sinks
+    std::map<std::string, std::pair<std::vector<std::string>, std::vector<std::string> > > myDistricts;
+
     /// @brief The file to write the computed routes into
     OutputDevice* myRoutesOutput;
 
@@ -409,7 +545,10 @@ protected:
     unsigned int myWrittenRouteNo;
 
     /// @brief Whether the network contains edges which not all vehicles may pass
-    bool myHaveRestrictions;
+    bool myHavePermissions;
+
+    /// @brief The vehicle class specific speed restrictions
+    std::map<std::string, std::map<SUMOVehicleClass, SUMOReal> > myRestrictions;
 
     /// @brief The number of internal edges in the dictionary
     int myNumInternalEdges;
@@ -418,38 +557,25 @@ protected:
     MsgHandler* myErrorHandler;
 
 #ifdef HAVE_FOX
-    FXWorkerThread::Pool myThreadPool;
-
 private:
-    class WorkerThread : public FXWorkerThread {
-    public:
-        WorkerThread(FXWorkerThread::Pool& pool,
-                     SUMOAbstractRouter<ROEdge, ROVehicle>* router)
-            : FXWorkerThread(pool), myRouter(router) {}
-        SUMOAbstractRouter<ROEdge, ROVehicle>& getRouter() const {
-            return *myRouter;
-        }
-        virtual ~WorkerThread() {
-            stop();
-            delete myRouter;
-        }
-    private:
-        SUMOAbstractRouter<ROEdge, ROVehicle>* myRouter;
-    };
-
     class RoutingTask : public FXWorkerThread::Task {
     public:
-        RoutingTask(ROVehicle* v, const bool removeLoops, MsgHandler* errorHandler)
-            : myVehicle(v), myRemoveLoops(removeLoops), myErrorHandler(errorHandler) {}
+        RoutingTask(RORoutable* v, const bool removeLoops, MsgHandler* errorHandler)
+            : myRoutable(v), myRemoveLoops(removeLoops), myErrorHandler(errorHandler) {}
         void run(FXWorkerThread* context);
     private:
-        ROVehicle* const myVehicle;
+        RORoutable* const myRoutable;
         const bool myRemoveLoops;
         MsgHandler* const myErrorHandler;
     private:
         /// @brief Invalidated assignment operator.
         RoutingTask& operator=(const RoutingTask&);
     };
+
+
+private:
+    /// @brief for multi threaded routing
+    FXWorkerThread::Pool myThreadPool;
 #endif
 
 private:
