@@ -5,12 +5,12 @@
 /// @author  Jakob Erdmann
 /// @author  Yun-Pang Floetteroed
 /// @date    Sept 2002
-/// @version $Id: RORoute.h 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: RORoute.h 20482 2016-04-18 20:49:42Z behrisch $
 ///
 // A complete router's route
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2002-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2002-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -47,6 +47,7 @@ class ROEdge;
 class ROVehicle;
 class OutputDevice;
 
+typedef std::vector<const ROEdge*> ConstROEdgeVector;
 
 // ===========================================================================
 // class definitions
@@ -71,10 +72,17 @@ public:
      * @todo Are costs/prob really mandatory?
      */
     RORoute(const std::string& id, SUMOReal costs, SUMOReal prob,
-            const std::vector<const ROEdge*>& route, const RGBColor* const color,
+            const ConstROEdgeVector& route, const RGBColor* const color,
             const std::vector<SUMOVehicleParameter::Stop>& stops);
 
 
+    /** @brief Constructor
+     *
+     * @param[in] id The route's id
+     * @param[in] route The list of edges the route is made of
+     */
+    RORoute(const std::string& id, const ConstROEdgeVector& route);
+
     /** @brief Copy constructor
      *
      * @param[in] src The route to copy
@@ -151,7 +159,7 @@ public:
      *
      * @return The edges this route consists of
      */
-    const std::vector<const ROEdge*>& getEdgeVector() const {
+    const ConstROEdgeVector& getEdgeVector() const {
         return myRoute;
     }
 
@@ -184,6 +192,16 @@ public:
         return myStops;
     }
 
+    /** @brief Adapts the until time of all stops by the given offset
+     */
+    void addStopOffset(const SUMOTime offset) {
+        for (std::vector<SUMOVehicleParameter::Stop>::iterator stop = myStops.begin(); stop != myStops.end(); ++stop) {
+            if (stop->until >= 0) {
+                stop->until += offset;
+            }
+        }
+    }
+
 private:
     /// @brief The costs of the route
     SUMOReal myCosts;
@@ -192,7 +210,7 @@ private:
     SUMOReal myProbability;
 
     /// @brief The edges the route consists of
-    std::vector<const ROEdge*> myRoute;
+    ConstROEdgeVector myRoute;
 
     /// @brief The color of the route
     const RGBColor* myColor;
