@@ -2,12 +2,12 @@
 /// @file    TrajectoriesHandler.h
 /// @author  Michael Behrisch
 /// @date    14.03.2014
-/// @version $Id: TrajectoriesHandler.h 18095 2015-03-17 09:39:00Z behrisch $
+/// @version $Id: TrajectoriesHandler.h 20482 2016-04-18 20:49:42Z behrisch $
 ///
 // An XML-Handler for amitran and netstate trajectories
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2014-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2014-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -53,7 +53,7 @@ public:
      *
      * @param[in] file The file that will be processed
      */
-    TrajectoriesHandler(const bool computeA, const SUMOEmissionClass defaultClass,
+    TrajectoriesHandler(const bool computeA, const bool computeAForward, const SUMOEmissionClass defaultClass,
                         const SUMOReal defaultSlope, std::ostream* stdOut, OutputDevice* xmlOut);
 
 
@@ -61,21 +61,23 @@ public:
     ~TrajectoriesHandler();
 
     const PollutantsInterface::Emissions computeEmissions(const std::string id,
-            const SUMOEmissionClass c, const SUMOReal v,
+            const SUMOEmissionClass c, SUMOReal& v,
             SUMOReal& a, SUMOReal& s);
 
-    void writeEmissions(std::ostream& o, const std::string id,
+    bool writeEmissions(std::ostream& o, const std::string id,
                         const SUMOEmissionClass c,
-                        const SUMOReal t, const SUMOReal v,
-                        SUMOReal a = INVALID_VALUE, SUMOReal s = INVALID_VALUE);
+                        SUMOReal t, SUMOReal& v,
+                        SUMOReal& a, SUMOReal& s);
 
-    void writeXMLEmissions(const std::string id,
+    bool writeXMLEmissions(const std::string id,
                            const SUMOEmissionClass c,
-                           const SUMOTime t, const SUMOReal v,
+                           SUMOTime t, SUMOReal& v,
                            SUMOReal a = INVALID_VALUE, SUMOReal s = INVALID_VALUE);
 
     void writeSums(std::ostream& o, const std::string id);
 
+    void writeNormedSums(std::ostream& o, const std::string id, const SUMOReal factor);
+
 
 protected:
     /// @name inherited from GenericSAXHandler
@@ -97,6 +99,7 @@ protected:
 
 private:
     const bool myComputeA;
+    const bool myComputeAForward;
     const SUMOEmissionClass myDefaultClass;
     const SUMOReal myDefaultSlope;
     std::ostream* myStdOut;
