@@ -5,12 +5,12 @@
 /// @author  Michael Behrisch
 /// @author  Laura Bieker
 /// @date    Sept 2002
-/// @version $Id: TraCIServerAPI_Simulation.cpp 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: TraCIServerAPI_Simulation.cpp 20482 2016-04-18 20:49:42Z behrisch $
 ///
 // APIs for getting/setting edge values via TraCI
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -41,6 +41,7 @@
 #include <microsim/MSEdge.h>
 #include <microsim/MSLane.h>
 #include <microsim/MSVehicle.h>
+#include <microsim/MSStateHandler.h>
 #include "TraCIConstants.h"
 #include "TraCIServerAPI_Simulation.h"
 
@@ -74,7 +75,7 @@ TraCIServerAPI_Simulation::processGet(TraCIServer& server, tcpip::Storage& input
             && variable != VAR_STOP_STARTING_VEHICLES_NUMBER && variable != VAR_STOP_STARTING_VEHICLES_IDS
             && variable != VAR_STOP_ENDING_VEHICLES_NUMBER && variable != VAR_STOP_ENDING_VEHICLES_IDS
        ) {
-        return server.writeErrorStatusCmd(CMD_GET_SIM_VARIABLE, "Get Simulation Variable: unsupported variable specified", outputStorage);
+        return server.writeErrorStatusCmd(CMD_GET_SIM_VARIABLE, "Get Simulation Variable: unsupported variable " + toHex(variable, 2) + " specified", outputStorage);
     }
     // begin response building
     tcpip::Storage tempMsg;
@@ -86,7 +87,7 @@ TraCIServerAPI_Simulation::processGet(TraCIServer& server, tcpip::Storage& input
     switch (variable) {
         case VAR_TIME_STEP:
             tempMsg.writeUnsignedByte(TYPE_INTEGER);
-            tempMsg.writeInt(MSNet::getInstance()->getCurrentTimeStep());
+            tempMsg.writeInt((int)MSNet::getInstance()->getCurrentTimeStep());
             break;
         case VAR_LOADED_VEHICLES_NUMBER:
             writeVehicleStateNumber(server, tempMsg, MSNet::VEHICLE_STATE_BUILT);
@@ -147,7 +148,7 @@ TraCIServerAPI_Simulation::processGet(TraCIServer& server, tcpip::Storage& input
             break;
         case VAR_DELTA_T:
             tempMsg.writeUnsignedByte(TYPE_INTEGER);
-            tempMsg.writeInt(DELTA_T);
+            tempMsg.writeInt((int)DELTA_T);
             break;
         case VAR_NET_BOUNDING_BOX: {
             tempMsg.writeUnsignedByte(TYPE_BOUNDINGBOX);
@@ -190,12 +191,12 @@ TraCIServerAPI_Simulation::processGet(TraCIServer& server, tcpip::Storage& input
             if (!server.readTypeCheckingString(inputStorage, id)) {
                 return server.writeErrorStatusCmd(CMD_GET_SIM_VARIABLE, "Retrieval of persons at busstop requires a string.", outputStorage);
             }
-            MSBusStop* s = MSNet::getInstance()->getBusStop(id);
+            MSStoppingPlace* s = MSNet::getInstance()->getBusStop(id);
             if (s == 0) {
                 return server.writeErrorStatusCmd(CMD_GET_SIM_VARIABLE, "Unknown bus stop '" + id + "'.", outputStorage);
             }
             tempMsg.writeUnsignedByte(TYPE_INTEGER);
-            tempMsg.writeInt(s->getPersonNumber());
+            tempMsg.writeInt(s->getTransportableNumber());
             break;
         }
         default:
@@ -213,8 +214,9 @@ TraCIServerAPI_Simulation::processSet(TraCIServer& server, tcpip::Storage& input
     std::string warning = ""; // additional description for response
     // variable
     int variable = inputStorage.readUnsignedByte();
-    if (variable != CMD_CLEAR_PENDING_VEHICLES) {
-        return server.writeErrorStatusCmd(CMD_SET_SIM_VARIABLE, "Set Simulation Variable: unsupported variable specified", outputStorage);
+    if (variable != CMD_CLEAR_PENDING_VEHICLES
+            && variable != CMD_SAVE_SIMSTATE) {
+        return server.writeErrorStatusCmd(CMD_SET_SIM_VARIABLE, "Set Simulation Variable: unsupported variable " + toHex(variable, 2) + " specified", outputStorage);
     }
     // id
     std::string id = inputStorage.readString();
@@ -229,6 +231,15 @@ TraCIServerAPI_Simulation::processSet(TraCIServer& server, tcpip::Storage& input
             MSNet::getInstance()->getInsertionControl().clearPendingVehicles(route);
         }
         break;
+        case CMD_SAVE_SIMSTATE: {
+            //save current simulation state
+            std::string file;
+            if (!server.readTypeCheckingString(inputStorage, file)) {
+                return server.writeErrorStatusCmd(CMD_SET_SIM_VARIABLE, "A string is needed for saving simulation state.", outputStorage);
+            }
+            MSStateHandler::saveState(file, MSNet::getInstance()->getCurrentTimeStep());
+        }
+        break;
         default:
             break;
     }
@@ -415,7 +426,7 @@ TraCIServerAPI_Simulation::commandDistanceRequest(TraCIServer& server, tcpip::St
             pos1.set(p1x, p1y);
         }
         if (posType == POSITION_3D) {
-            inputStorage.readDouble();		// z value is ignored
+            inputStorage.readDouble();// z value is ignored
         }
         roadPos1 = convertCartesianToRoadMap(pos1);
         break;
@@ -445,7 +456,7 @@ TraCIServerAPI_Simulation::commandDistanceRequest(TraCIServer& server, tcpip::St
             pos2.set(p2x, p2y);
         }
         if (posType == POSITION_3D) {
-            inputStorage.readDouble();		// z value is ignored
+            inputStorage.readDouble();// z value is ignored
         }
         roadPos2 = convertCartesianToRoadMap(pos2);
         break;
@@ -464,7 +475,7 @@ TraCIServerAPI_Simulation::commandDistanceRequest(TraCIServer& server, tcpip::St
             // same edge
             distance = roadPos2.second - roadPos1.second;
         } else {
-            MSEdgeVector newRoute;
+            ConstMSEdgeVector newRoute;
             MSNet::getInstance()->getRouterTT().compute(
                 &roadPos1.first->getEdge(), &roadPos2.first->getEdge(), 0, MSNet::getInstance()->getCurrentTimeStep(), newRoute);
             MSRoute route("", newRoute, false, 0, std::vector<SUMOVehicleParameter::Stop>());
