@@ -8,12 +8,12 @@
 /// @author  Mario Krumnow
 /// @author  Jakob Erdmann
 /// @date    07.05.2009
-/// @version $Id: TraCIServerAPI_Vehicle.cpp 18756 2015-08-31 19:16:33Z behrisch $
+/// @version $Id: TraCIServerAPI_Vehicle.cpp 20482 2016-04-18 20:49:42Z behrisch $
 ///
 // APIs for getting/setting vehicle values via TraCI
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2009-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2009-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -100,7 +100,7 @@ TraCIServerAPI_Vehicle::processGet(TraCIServer& server, tcpip::Storage& inputSto
             && variable != VAR_ROUTE_INDEX
             && variable != VAR_PARAMETER
        ) {
-        return server.writeErrorStatusCmd(CMD_GET_VEHICLE_VARIABLE, "Get Vehicle Variable: unsupported variable specified", outputStorage);
+        return server.writeErrorStatusCmd(CMD_GET_VEHICLE_VARIABLE, "Get Vehicle Variable: unsupported variable " + toHex(variable, 2) + " specified", outputStorage);
     }
     // begin response building
     tcpip::Storage tempMsg;
@@ -157,7 +157,7 @@ TraCIServerAPI_Vehicle::processGet(TraCIServer& server, tcpip::Storage& inputSto
                 break;
             case VAR_ANGLE:
                 tempMsg.writeUnsignedByte(TYPE_DOUBLE);
-                tempMsg.writeDouble(visible ? v->getAngle() : INVALID_DOUBLE_VALUE);
+                tempMsg.writeDouble(visible ? GeomHelper::naviDegree(v->getAngle()) : INVALID_DOUBLE_VALUE);
                 break;
             case VAR_ROAD_ID:
                 tempMsg.writeUnsignedByte(TYPE_STRING);
@@ -391,7 +391,7 @@ TraCIServerAPI_Vehicle::processGet(TraCIServer& server, tcpip::Storage& inputSto
             break;
             case VAR_DISTANCE: {
                 tempMsg.writeUnsignedByte(TYPE_DOUBLE);
-                SUMOReal distance = onRoad ? v->getRoute().getDistanceBetween(0, v->getPositionOnLane(), v->getRoute().getEdges()[0],  &v->getLane()->getEdge()) : INVALID_DOUBLE_VALUE;
+                SUMOReal distance = onRoad ? v->getRoute().getDistanceBetween(v->getDepartPos(), v->getPositionOnLane(), v->getRoute().getEdges()[0],  &v->getLane()->getEdge()) : INVALID_DOUBLE_VALUE;
                 if (distance == std::numeric_limits<SUMOReal>::max()) {
                     distance = INVALID_DOUBLE_VALUE;
                 }
@@ -452,7 +452,7 @@ TraCIServerAPI_Vehicle::processSet(TraCIServer& server, tcpip::Storage& inputSto
             && variable != ADD && variable != ADD_FULL && variable != REMOVE
             && variable != VAR_MOVE_TO_VTD && variable != VAR_PARAMETER/* && variable != VAR_SPEED_TIME_LINE && variable != VAR_LANE_TIME_LINE*/
        ) {
-        return server.writeErrorStatusCmd(CMD_SET_VEHICLE_VARIABLE, "Change Vehicle State: unsupported variable specified", outputStorage);
+        return server.writeErrorStatusCmd(CMD_SET_VEHICLE_VARIABLE, "Change Vehicle State: unsupported variable " + toHex(variable, 2) + " specified", outputStorage);
     }
     // id
     std::string id = inputStorage.readString();
@@ -1253,7 +1253,6 @@ TraCIServerAPI_Vehicle::processSet(TraCIServer& server, tcpip::Storage& inputSto
             // x
             double x = 0;
             double y = 0;
-            double angle = 0;
             double origAngle = 0;
             if (!server.readTypeCheckingDouble(inputStorage, x)) {
                 return server.writeErrorStatusCmd(CMD_SET_VEHICLE_VARIABLE, "The third parameter for setting a VTD vehicle must be the x-position given as a double.", outputStorage);
@@ -1276,7 +1275,7 @@ TraCIServerAPI_Vehicle::processSet(TraCIServer& server, tcpip::Storage& inputSto
                 laneNum = -laneNum;
             }
             Position pos(x, y);
-            angle *= -1.;
+            SUMOReal angle = origAngle;
             if (angle >= 180.) {
                 angle = -360. + angle;
             } else if (angle <= -180.) {
@@ -1304,7 +1303,7 @@ TraCIServerAPI_Vehicle::processSet(TraCIServer& server, tcpip::Storage& inputSto
                 SUMOReal maxRouteDistance = 100;
                 // use the best we have
                 if (found && maxRouteDistance > bestDistance) {
-                    server.setVTDControlled(v, lane, lanePos, routeOffset, edges, MSNet::getInstance()->getCurrentTimeStep());
+                    server.setVTDControlled(v, lane, lanePos, angle, routeOffset, edges, MSNet::getInstance()->getCurrentTimeStep());
                 }
             } else {
                 // case b): vehicle does not follow a pre-fixed route (regard the limiting factor in maxRouteDistance)
@@ -1312,7 +1311,7 @@ TraCIServerAPI_Vehicle::processSet(TraCIServer& server, tcpip::Storage& inputSto
                 SUMOReal maxRouteDistance = 100;
                 // use the best we have
                 if (found && maxRouteDistance > bestDistance) {
-                    server.setVTDControlled(v, lane, lanePos, routeOffset, edges, MSNet::getInstance()->getCurrentTimeStep());
+                    server.setVTDControlled(v, lane, lanePos, angle, routeOffset, edges, MSNet::getInstance()->getCurrentTimeStep());
                 } else {
                     return server.writeErrorStatusCmd(CMD_SET_VEHICLE_VARIABLE, "Could not map vehicle '" + id + "'.", outputStorage);
                 }
@@ -1509,8 +1508,9 @@ TraCIServerAPI_Vehicle::vtdMap(const Position& pos, const std::string& origID, c
     const MSEdge* prevEdge = u.prevEdge;
     if (u.onRoute) {
         const ConstMSEdgeVector& ev = v.getRoute().getEdges();
-        ConstMSEdgeVector::const_iterator prevEdgePos = std::find(ev.begin() + v.getRoutePosition(), ev.end(), prevEdge);
-        routeOffset = (int)std::distance(ev.begin(), prevEdgePos) - v.getRoutePosition();
+        ConstMSEdgeVector::const_iterator prevEdgePos = std::find(ev.begin(), ev.end(), prevEdge);
+        routeOffset = (int)std::distance(ev.begin(), prevEdgePos);
+        //std::cout << SIMTIME << "vtdMap vehicle=" << v.getID() << " currLane=" << v.getLane()->getID() << " routeOffset=" << routeOffset << " edges=" << toString(ev) << " bestLane=" << bestLane->getID() << " prevEdge=" << prevEdge->getID() << "\n";
     } else {
         edges.push_back(u.prevEdge);
         /*
@@ -1538,37 +1538,75 @@ TraCIServerAPI_Vehicle::vtdMap(const Position& pos, const std::string& origID, c
 }
 
 
+bool
+TraCIServerAPI_Vehicle::findCloserLane(const MSEdge* edge, const Position& pos, SUMOReal& bestDistance, MSLane** lane) {
+    if (edge == 0) {
+        return false;
+    }
+    const std::vector<MSLane*>& lanes = edge->getLanes();
+    bool newBest = false;
+    for (std::vector<MSLane*>::const_iterator k = lanes.begin(); k != lanes.end() && bestDistance > POSITION_EPS; ++k) {
+        MSLane* candidateLane = *k;
+        const SUMOReal dist = candidateLane->getShape().distance(pos); // get distance
+#ifdef DEBUG_VTD
+        std::cout << "   b at lane " << candidateLane->getID() << " dist:" << dist << " best:" << bestDistance << std::endl;
+#endif
+        if (dist < bestDistance) {
+            // is the new distance the best one? keep then...
+            bestDistance = dist;
+            *lane = candidateLane;
+            newBest = true;
+        }
+    }
+    return newBest;
+}
+
 bool
 TraCIServerAPI_Vehicle::vtdMap_matchingRoutePosition(const Position& pos, const std::string& origID, MSVehicle& v,
         SUMOReal& bestDistance, MSLane** lane, SUMOReal& lanePos, int& routeOffset, ConstMSEdgeVector& /*edges*/) {
 
-    int lastBestRouteEdge = 0; // index of the best edge found when going down the route's list of edges
-    int lastRouteEdge = 0; // last non-internal edge on the route (seen so far)
-    // get the lanes the vehicle may use
-    const std::vector<MSLane*>& bestLaneConts = v.getBestLanesContinuation(v.getLane());
-    for (std::vector<MSLane*>::const_iterator i = bestLaneConts.begin(); i != bestLaneConts.end() && bestDistance > POSITION_EPS; ++i) { // yes, we quit if the distance is < 0.1m or so
-        if (*i == 0) { // why is that possible???
-            continue;
+    const ConstMSEdgeVector& edges = v.getRoute().getEdges();
+    routeOffset = 0;
+    // routes may be looped which makes routeOffset ambiguous. We first try to
+    // find the closest upcoming edge on the route and then look for closer passed edges
+
+    // look forward along the route
+    const MSEdge* prev = 0;
+    UNUSED_PARAMETER(prev); // silence 'unused variable' warning when built without INTERNAL_LANES
+    for (ConstMSEdgeVector::const_iterator i = v.getCurrentRouteEdge(); i != edges.end(); ++i) {
+#ifdef HAVE_INTERNAL_LANES
+        while (prev != 0) {
+            // check internal edge(s)
+            const MSEdge* internalCand = prev->getInternalFollowingEdge(*i);
+            findCloserLane(internalCand, pos, bestDistance, lane);
+            prev = internalCand;
         }
-        MSEdge& e = (*i)->getEdge();
-        if (i != bestLaneConts.begin() && e.getPurpose() != MSEdge::EDGEFUNCTION_INTERNAL) {
-            ++lastRouteEdge; // increment index, if not internal and not the current one
-        }
-        const std::vector<MSLane*>& lanes = e.getLanes(); // go over the edge's lanes
-        for (std::vector<MSLane*>::const_iterator k = lanes.begin(); k != lanes.end() && bestDistance > POSITION_EPS; ++k) {
-            MSLane* cl = *k;
-            SUMOReal dist = cl->getShape().distance(pos); // get distance
-#ifdef DEBUG_VTD
-            std::cout << "   b at lane " << cl->getID() << " dist:" << dist << " best:" << bestDistance << std::endl;
 #endif
-            if (dist < bestDistance) {
-                // is the new distance the best one? keep then...
-                bestDistance = dist;
-                *lane = cl;
-                lastBestRouteEdge = lastRouteEdge;
-            }
+        if (findCloserLane(*i, pos, bestDistance, lane)) {
+            routeOffset = (int)std::distance(edges.begin(), i);
         }
+        prev = *i;
     }
+    // look backward along the route
+    const MSEdge* next = *v.getCurrentRouteEdge();
+    UNUSED_PARAMETER(next); // silence 'unused variable' warning when built without INTERNAL_LANES
+    for (ConstMSEdgeVector::const_iterator i = v.getCurrentRouteEdge(); i != edges.begin(); --i) {
+        prev = *i;
+#ifdef HAVE_INTERNAL_LANES
+        while (prev != 0) {
+            // check internal edge(s)
+            const MSEdge* internalCand = prev->getInternalFollowingEdge(next);
+            findCloserLane(internalCand, pos, bestDistance, lane);
+            prev = internalCand;
+        }
+#endif
+        if (findCloserLane(*i, pos, bestDistance, lane)) {
+            routeOffset = (int)std::distance(edges.begin(), i);
+        }
+        next = *i;
+    }
+
+    assert(lane != 0);
     // quit if no solution was found, reporting a failure
     if (lane == 0) {
 #ifdef DEBUG_VTD
@@ -1576,6 +1614,8 @@ TraCIServerAPI_Vehicle::vtdMap_matchingRoutePosition(const Position& pos, const
 #endif
         return false;
     }
+
+
     // position may be inaccurate; let's checkt the given index, too
     // a) is enabled for non-internal lanes only, as otherwise the position information may ambiguous
     // b) it's something one has to enable when building the nework - keepin the OSM IDs - is probably not always done
@@ -1590,9 +1630,9 @@ TraCIServerAPI_Vehicle::vtdMap_matchingRoutePosition(const Position& pos, const
     }
     // check position, stuff, we should have the best lane along the route
     lanePos = MAX2(SUMOReal(0), MIN2(SUMOReal((*lane)->getLength() - POSITION_EPS), (*lane)->getShape().nearest_offset_to_point2D(pos, false)));
-    routeOffset = lastBestRouteEdge;
+    //std::cout << SIMTIME << " vtdMap_matchingRoutePosition vehicle=" << v.getID() << " currLane=" << v.getLane()->getID() << " routeOffset=" << routeOffset << " edges=" << toString(edges) << " lane=" << (*lane)->getID() << "\n";
 #ifdef DEBUG_VTD
-    std::cout << "  b ok lane " << (*lane)->getID() << " lanePos:" << lanePos << " best:" << lastBestRouteEdge << std::endl;
+    std::cout << "  b ok lane " << (*lane)->getID() << " lanePos:" << lanePos << std::endl;
 #endif
     return true;
 }
