@@ -7,12 +7,12 @@
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    2008/04/07
-/// @version $Id: TraCITestClient.cpp 18213 2015-04-16 11:55:42Z behrisch $
+/// @version $Id: TraCITestClient.cpp 20482 2016-04-18 20:49:42Z behrisch $
 ///
 /// A test execution class
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2008-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2008-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -45,6 +45,7 @@
 
 #include <traci-server/TraCIConstants.h>
 #include <utils/common/SUMOTime.h>
+#include <utils/common/ToString.h>
 #include "TraCITestClient.h"
 
 #ifdef CHECK_MEMORY_LEAKS
@@ -159,6 +160,9 @@ TraCITestClient::run(std::string fileName, int port, std::string host) {
             std::string objID;
             defFile >> domID >> varID >> objID;
             commandSetValue(domID, varID, objID, defFile);
+        }  else if (lineCommand.compare("testAPI") == 0) {
+            // call all native API methods
+            testAPI();
         } else {
             msg << "Error in definition file: " << lineCommand << " is not a valid command";
             errorMsg(msg);
@@ -667,3 +671,31 @@ TraCITestClient::readAndReportTypeDependent(tcpip::Storage& inMsg, int valueData
 }
 
 
+void
+TraCITestClient::testAPI() {
+    answerLog << "testAPI:\n";
+    answerLog << "  edge:\n";
+    answerLog << "    getIDList: " << joinToString(edge.getIDList(), " ") << "\n";
+    answerLog << "    getIDCount: " << edge.getIDCount() << "\n";
+    answerLog << "  vehicle:\n";
+    answerLog << "    getIDList: " << joinToString(vehicle.getIDList(), " ") << "\n";
+    answerLog << "    getIDCount: " << vehicle.getIDCount() << "\n";
+    answerLog << "  inductionloop:\n";
+    answerLog << "    getIDList: " << joinToString(inductionloop.getIDList(), " ") << "\n";
+    answerLog << "    getVehicleData:\n";
+    std::vector<InductionLoopScope::VehicleData> result = inductionloop.getVehicleData("det1");
+    for (int i = 0; i < (int)result.size(); ++i) {
+        const InductionLoopScope::VehicleData& vd = result[i];
+        answerLog << "      veh=" << vd.id << " length=" << vd.length << " entered=" << vd.entryTime << " left=" << vd.leaveTime << " type=" << vd.typeID << "\n";
+    }
+    answerLog << "  simulation:\n";
+    answerLog << "    getCurrentTime: " << simulation.getCurrentTime() << "\n";
+    answerLog << "  gui:\n";
+    try {
+        answerLog << "    setScheme: \n";
+        gui.setSchema("View #0", "real world");
+        answerLog << "    getScheme: " << gui.getSchema("View #0") << "\n";
+    } catch (tcpip::SocketException&) {
+        answerLog << "    no support for gui commands\n";
+    }
+}
