@@ -3,12 +3,12 @@
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    Mon, 17 Dec 2001
-/// @version $Id: FileHelpers.cpp 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: FileHelpers.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // Functions for an easier usage of files
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -129,6 +129,9 @@ FileHelpers::checkForRelativity(const std::string& filename,
     if (filename == "stderr" || filename == "STDERR") {
         return "stderr";
     }
+    if (filename == "nul" || filename == "NUL") {
+        return "/dev/null";
+    }
     if (!isSocket(filename) && !isAbsolute(filename)) {
         return getConfigurationRelative(basePath, filename);
     }
@@ -136,6 +139,16 @@ FileHelpers::checkForRelativity(const std::string& filename,
 }
 
 
+std::string
+FileHelpers::prependToLastPathComponent(const std::string& prefix, const std::string& path) {
+    size_t sep_index = path.find_last_of("\\/");
+    if (sep_index == std::string::npos) {
+        return prefix + path;
+    } else {
+        return path.substr(0, sep_index + 1) + prefix + path.substr(sep_index + 1);
+    }
+}
+
 // ---------------------------------------------------------------------------
 // binary reading/writing functions
 // ---------------------------------------------------------------------------
