@@ -5,12 +5,12 @@
 /// @author  Michael Behrisch
 /// @author  Laura Bieker
 /// @date    Sept 2002
-/// @version $Id: RGBColor.cpp 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: RGBColor.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // A RGB-color definition
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -56,6 +56,7 @@ const RGBColor RGBColor::BLUE = RGBColor(0, 0, 255, 255);
 const RGBColor RGBColor::YELLOW = RGBColor(255, 255, 0, 255);
 const RGBColor RGBColor::CYAN = RGBColor(0, 255, 255, 255);
 const RGBColor RGBColor::MAGENTA = RGBColor(255, 0, 255, 255);
+const RGBColor RGBColor::ORANGE = RGBColor(255, 128, 0, 255);
 const RGBColor RGBColor::WHITE = RGBColor(255, 255, 255, 255);
 const RGBColor RGBColor::BLACK = RGBColor(0, 0, 0, 255);
 const RGBColor RGBColor::GREY = RGBColor(128, 128, 128, 255);
@@ -111,6 +112,9 @@ operator<<(std::ostream& os, const RGBColor& col) {
     if (col == RGBColor::MAGENTA) {
         return os << "magenta";
     }
+    if (col == RGBColor::ORANGE) {
+        return os << "orange";
+    }
     if (col == RGBColor::WHITE) {
         return os << "white";
     }
@@ -185,6 +189,9 @@ RGBColor::parseColor(std::string coldef) {
     if (coldef == "magenta") {
         return MAGENTA;
     }
+    if (coldef == "orange") {
+        return ORANGE;
+    }
     if (coldef == "white") {
         return WHITE;
     }
