@@ -5,12 +5,12 @@
 /// @author  Michael Behrisch
 /// @author  Felix Brack
 /// @date    Mon, 17 Dec 2001
-/// @version $Id: UtilExceptions.h 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: UtilExceptions.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // Exceptions for used by some utility classes
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -40,20 +40,6 @@
 // ===========================================================================
 // class definitions
 // ===========================================================================
-/**
- * InvalidArgument
- * Thrown when an argument was not proper in the current context
- * A message will be supplied
- */
-class InvalidArgument : public std::runtime_error {
-public:
-    /** constructor */
-    InvalidArgument(const std::string& message)
-        : std::runtime_error(message) {}
-
-};
-
-
 /**
  * ProcessError
  * A general exception type that may be thrown when the process is no
@@ -73,6 +59,20 @@ public:
 };
 
 
+/**
+ * InvalidArgument
+ * Thrown when an argument was not proper in the current context
+ * A message will be supplied
+ */
+class InvalidArgument : public ProcessError {
+public:
+    /** constructor */
+    InvalidArgument(const std::string& message)
+        : ProcessError(message) {}
+
+};
+
+
 /**
  * EmptyData
  * Thrown when data required by a method is missing
