@@ -10,7 +10,7 @@
 // Helper class for PHEM Light, holds CEP data for emission computation
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2013-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2013-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -30,6 +30,7 @@
 #include <config.h>
 #endif
 
+#include <cstdlib>
 #include <fstream>
 #include <sstream>
 #include <string>
@@ -67,10 +68,13 @@ bool
 PHEMCEPHandler::Load(SUMOEmissionClass emissionClass, const std::string& emissionClassIdentifier) {
     // to hold everything.
     std::vector< std::vector<double> > matrixSpeedInertiaTable;
+    std::vector< std::vector<double> > normedDragTable;
     std::vector< std::vector<double> > matrixFC;
     std::vector< std::vector<double> > matrixPollutants;
     std::vector<std::string> headerFC;
     std::vector<std::string> headerPollutants;
+    std::vector<double> idlingValues;
+    std::vector<double> idlingValuesFC;
 
     double vehicleMass;
     double vehicleLoading;
@@ -82,7 +86,11 @@ PHEMCEPHandler::Load(SUMOEmissionClass emissionClass, const std::string& emissio
     double f2;
     double f3;
     double f4;
+    double axleRatio;
     double ratedPower;
+    double engineIdlingSpeed;
+    double engineRatedSpeed;
+    double effectiveWheelDiameter;
     std::string vehicleMassType;
     std::string vehicleFuelType;
     double pNormV0;
@@ -91,27 +99,79 @@ PHEMCEPHandler::Load(SUMOEmissionClass emissionClass, const std::string& emissio
     double pNormP1;
 
     OptionsCont& oc = OptionsCont::getOptions();
-    std::string phemPath = oc.getString("phemlight-path") + "/";
-    if (!ReadVehicleFile(phemPath, emissionClassIdentifier, vehicleMass, vehicleLoading, vehicleMassRot, crosssectionalArea, cwValue, f0, f1, f2, f3, f4, ratedPower, vehicleMassType, vehicleFuelType,
-                         pNormV0, pNormP0, pNormV1, pNormP1, matrixSpeedInertiaTable)) {
+    //std::string phemPath = oc.getString("phemlight-path") + "/";
+    std::vector<std::string> phemPath;
+    phemPath.push_back(oc.getString("phemlight-path") + "/");
+    if (getenv("PHEMLIGHT_PATH") != 0) {
+        phemPath.push_back(std::string(getenv("PHEMLIGHT_PATH")) + "/");
+    }
+    if (getenv("SUMO_HOME") != 0) {
+        phemPath.push_back(std::string(getenv("SUMO_HOME")) + "/data/emissions/PHEMlight/");
+    }
+    if (!ReadVehicleFile(phemPath, emissionClassIdentifier,
+                         vehicleMass,
+                         vehicleLoading,
+                         vehicleMassRot,
+                         crosssectionalArea,
+                         cwValue,
+                         f0,
+                         f1,
+                         f2,
+                         f3,
+                         f4,
+                         axleRatio,
+                         ratedPower,
+                         engineIdlingSpeed,
+                         engineRatedSpeed,
+                         effectiveWheelDiameter,
+                         vehicleMassType,
+                         vehicleFuelType,
+                         pNormV0,
+                         pNormP0,
+                         pNormV1,
+                         pNormP1,
+                         matrixSpeedInertiaTable,
+                         normedDragTable)) {
         return false;
     }
 
-    if (!ReadEmissionData(true, phemPath, emissionClassIdentifier, headerFC, matrixFC)) {
+    if (!ReadEmissionData(true, phemPath, emissionClassIdentifier, headerFC, matrixFC, idlingValuesFC)) {
         return false;
     }
 
-    if (!ReadEmissionData(false, phemPath, emissionClassIdentifier, headerPollutants, matrixPollutants)) {
+    if (!ReadEmissionData(false, phemPath, emissionClassIdentifier, headerPollutants, matrixPollutants, idlingValues)) {
         return false;
     }
 
     _ceps[emissionClass] = new PHEMCEP(vehicleMassType == "HV",
                                        emissionClass, emissionClassIdentifier,
-                                       vehicleMass, vehicleLoading, vehicleMassRot,
-                                       crosssectionalArea, cwValue,
-                                       f0, f1, f2, f3, f4,
-                                       ratedPower, pNormV0, pNormP0, pNormV1, pNormP1,
-                                       vehicleFuelType, matrixFC, headerPollutants, matrixPollutants, matrixSpeedInertiaTable);
+                                       vehicleMass,
+                                       vehicleLoading,
+                                       vehicleMassRot,
+                                       crosssectionalArea,
+                                       cwValue,
+                                       f0,
+                                       f1,
+                                       f2,
+                                       f3,
+                                       f4,
+                                       ratedPower,
+                                       pNormV0,
+                                       pNormP0,
+                                       pNormV1,
+                                       pNormP1,
+                                       axleRatio,
+                                       engineIdlingSpeed,
+                                       engineRatedSpeed,
+                                       effectiveWheelDiameter,
+                                       idlingValuesFC.front(),
+                                       vehicleFuelType,
+                                       matrixFC,
+                                       headerPollutants,
+                                       matrixPollutants,
+                                       matrixSpeedInertiaTable,
+                                       normedDragTable,
+                                       idlingValues);
 
     return true;
 } // end of Load()
@@ -129,13 +189,39 @@ PHEMCEPHandler::GetCep(SUMOEmissionClass emissionClass) {
 
 
 bool
-PHEMCEPHandler::ReadVehicleFile(const std::string& path, const std::string& emissionClass,
-                                double& vehicleMass, double& vehicleLoading, double& vehicleMassRot,
-                                double& crossArea, double& cWValue,
-                                double& f0, double& f1, double& f2, double& f3, double& f4, double& ratedPower, std::string& vehicleMassType, std::string& vehicleFuelType,
-                                double& pNormV0, double& pNormP0, double& pNormV1, double& pNormP1, std::vector< std::vector<double> >& matrixRotFactor) {
-    std::ifstream fileVehicle(std::string(path + emissionClass + ".veh").c_str());
+PHEMCEPHandler::ReadVehicleFile(const std::vector<std::string>& path, const std::string& emissionClass,
+                                double& vehicleMass,
+                                double& vehicleLoading,
+                                double& vehicleMassRot,
+                                double& crossArea,
+                                double& cWValue,
+                                double& f0,
+                                double& f1,
+                                double& f2,
+                                double& f3,
+                                double& f4,
+                                double& axleRatio,
+                                double& ratedPower,
+                                double& engineIdlingSpeed,
+                                double& engineRatedSpeed,
+                                double& effectiveWheelDiameter,
+                                std::string& vehicleMassType,
+                                std::string& vehicleFuelType,
+                                double& pNormV0,
+                                double& pNormP0,
+                                double& pNormV1,
+                                double& pNormP1,
+                                std::vector< std::vector<double> >& matrixSpeedInertiaTable,
+                                std::vector< std::vector<double> >& normedDragTable)
 
+{
+    std::ifstream fileVehicle;
+    for (std::vector<std::string>::const_iterator i = path.begin(); i != path.end(); i++) {
+        fileVehicle.open(((*i) + emissionClass + ".PHEMLight.veh").c_str());
+        if (fileVehicle.good()) {
+            break;
+        }
+    }
     if (!fileVehicle.good()) {
         return false;
     }
@@ -194,6 +280,16 @@ PHEMCEPHandler::ReadVehicleFile(const std::string& path, const std::string& emis
             std::istringstream(cell) >> ratedPower;
         }
 
+        // reading engine rated speed
+        if (dataCount == 11) {
+            std::istringstream(cell) >> engineRatedSpeed;
+        }
+
+        // reading engine idling speed
+        if (dataCount == 12) {
+            std::istringstream(cell) >> engineIdlingSpeed;
+        }
+
         // reading f0
         if (dataCount == 14) {
             std::istringstream(cell) >> f0;
@@ -218,6 +314,15 @@ PHEMCEPHandler::ReadVehicleFile(const std::string& path, const std::string& emis
         if (dataCount == 18) {
             std::istringstream(cell) >> f4;
         }
+        // reading axleRatio
+        if (dataCount == 21) {
+            std::istringstream(cell) >> axleRatio;
+        }
+
+        // reading effective wheel diameter
+        if (dataCount == 22) {
+            std::istringstream(cell) >> effectiveWheelDiameter;
+        }
 
         // reading vehicleMassType
         if (dataCount == 45) {
@@ -250,7 +355,7 @@ PHEMCEPHandler::ReadVehicleFile(const std::string& path, const std::string& emis
         }
     } // end while
 
-    while (std::getline(fileVehicle, line)) {
+    while (std::getline(fileVehicle, line) && line.substr(0, 1) != commentPrefix) {
         std::stringstream  lineStream(line);
         std::string cell;
         std::vector <double> vi;
@@ -260,7 +365,24 @@ PHEMCEPHandler::ReadVehicleFile(const std::string& path, const std::string& emis
             vi.push_back(entry);
 
         } // end while
-        matrixRotFactor.push_back(vi);
+        matrixSpeedInertiaTable.push_back(vi);
+    } // end while
+
+    while (std::getline(fileVehicle, line)) {
+        if (line.substr(0, 1) == commentPrefix) {
+            continue;
+        }
+
+        std::stringstream  lineStream(line);
+        std::string cell;
+        std::vector <double> vi;
+        while (std::getline(lineStream, cell, ',')) {
+            double entry;
+            std::istringstream(cell) >> entry;
+            vi.push_back(entry);
+
+        } // end while
+        normedDragTable.push_back(vi);
     } // end while
 
 
@@ -269,17 +391,21 @@ PHEMCEPHandler::ReadVehicleFile(const std::string& path, const std::string& emis
 } // end of ReadVehicleFile
 
 
-bool
-PHEMCEPHandler::ReadEmissionData(bool readFC, const std::string& path, const std::string& emissionClass,
-                                 std::vector<std::string>& header, std::vector<std::vector<double> >& matrix) {
-
+bool PHEMCEPHandler::ReadEmissionData(bool readFC, const std::vector<std::string>& path, const std::string& emissionClass,
+                                      std::vector<std::string>& header, std::vector<std::vector<double> >& matrix, std::vector<double>& idlingValues) {
 
     std::string pollutantExtension = "";
     if (readFC) {
         pollutantExtension += "_FC";
     }
     // declare file stream
-    std::ifstream fileEmission(std::string(path + emissionClass + pollutantExtension + ".csv").c_str());
+    std::ifstream fileEmission;
+    for (std::vector<std::string>::const_iterator i = path.begin(); i != path.end(); i++) {
+        fileEmission.open(((*i) + emissionClass + pollutantExtension + ".csv").c_str());
+        if (fileEmission.good()) {
+            break;
+        }
+    }
 
     if (!fileEmission.good()) {
         return false;
@@ -303,6 +429,24 @@ PHEMCEPHandler::ReadEmissionData(bool readFC, const std::string& path, const std
     // skip units
     std::getline(fileEmission, line);
 
+    // skip comments
+    std::getline(fileEmission, line);
+
+    // reading idlingValues
+    std::getline(fileEmission, line);
+
+    std::stringstream idlingStream(line);
+    std::string idlingCell;
+
+    //skipping idle comment
+    std::getline(idlingStream, idlingCell, ',');
+
+    while (std::getline(idlingStream, idlingCell, ',')) {
+        double entry;
+        std::istringstream(idlingCell) >> entry;
+        idlingValues.push_back(entry);
+    } // end while
+
     while (std::getline(fileEmission, line)) {
         std::stringstream  lineStream(line);
         std::string cell;
