@@ -3,12 +3,12 @@
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    Mon, 19.08.2013
-/// @version $Id: PollutantsInterface.cpp 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: PollutantsInterface.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // Interface to capsulate different emission models
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2013-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2013-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -101,12 +101,6 @@ PollutantsInterface::isSilent(const SUMOEmissionClass c) {
 }
 
 
-SUMOReal
-PollutantsInterface::getMaxAccel(SUMOEmissionClass c, double v, double a, double slope) {
-    return myHelpers[c >> 16]->getMaxAccel(c, v, a, slope);
-}
-
-
 SUMOEmissionClass
 PollutantsInterface::getClass(const SUMOEmissionClass base, const std::string& vClass,
                               const std::string& fuel, const std::string& eClass, const double weight) {
@@ -148,7 +142,8 @@ PollutantsInterface::Emissions
 PollutantsInterface::computeAll(const SUMOEmissionClass c, const double v, const double a, const double slope) {
     const Helper* const h = myHelpers[c >> 16];
     return Emissions(h->compute(c, CO2, v, a, slope), h->compute(c, CO, v, a, slope), h->compute(c, HC, v, a, slope),
-                     h->compute(c, FUEL, v, a, slope), h->compute(c, NO_X, v, a, slope), h->compute(c, PM_X, v, a, slope));
+                     h->compute(c, FUEL, v, a, slope), h->compute(c, NO_X, v, a, slope), h->compute(c, PM_X, v, a, slope),
+                     h->compute(c, ELEC, v, a, slope));
 }
 
 
