@@ -2,12 +2,12 @@
 /// @file    FXWorkerThread.h
 /// @author  Michael Behrisch
 /// @date    2014-07-13
-/// @version $Id: FXWorkerThread.h 18756 2015-08-31 19:16:33Z behrisch $
+/// @version $Id: FXWorkerThread.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // A thread class together with a pool and a task for parallelized computation
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2004-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2004-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -92,7 +92,7 @@ public:
          *
          * @param[in] numThreads the number of threads to create
          */
-        Pool(int numThreads = 0) : myRunningIndex(0), myNumFinished(0) {
+        Pool(int numThreads = 0) : myPoolMutex(true), myRunningIndex(0), myNumFinished(0) {
             while (numThreads > 0) {
                 new FXWorkerThread(*this);
                 numThreads--;
