@@ -4,12 +4,12 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: PositionVector.h 18639 2015-08-17 13:59:19Z namdre $
+/// @version $Id: PositionVector.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // A list of positions
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -39,7 +39,6 @@
 // ===========================================================================
 // class declarations
 // ===========================================================================
-class Line;
 class Boundary;
 
 
@@ -69,6 +68,20 @@ public:
     PositionVector(const std::vector<Position>& v);
 
 
+    /** @brief Constructor
+     * @param[in] beg The begin iterator for copy
+     * @param[in] end The end iterator to copy
+     */
+    PositionVector(const std::vector<Position>::const_iterator beg, const std::vector<Position>::const_iterator end);
+
+
+    /** @brief Constructor for lines
+     * @param[in] p1 the first position
+     * @param[in] p2 the second position
+     */
+    PositionVector(const Position& p1, const Position& p2);
+
+
     /// @brief Destructor
     ~PositionVector();
 
@@ -85,33 +98,12 @@ public:
     using vp::front;
     using vp::back;
     using vp::reference;
+    using vp::erase;
+    using vp::insert;
 
 
-    /// @name Adding items to the container
-    /// @{
-
-
-    /** @brief Appends all positions from the given vector
-     * @param[in] p The vector from which values shall be appended
-     */
-    void push_back(const PositionVector& p);
-    /// @}
-
-
-    /// Puts the given position at the front of the list
-    void push_front(const Position& p);
-
-    /// Removes and returns the position at the fron of the list
-    Position pop_front();
-
-    void insertAt(int index, const Position& p);
-
-    void replaceAt(int index, const Position& by);
-
-    void eraseAt(int i);
-
     /** @brief Returns the information whether the position vector describes a polygon lying around the given point
-        The optional offset is added to the polygon's bounderies */
+        The optional offset is added to the polygon's boundaries */
     bool around(const Position& p, SUMOReal offset = 0) const;
 
     /** @brief Returns the information whether the given polygon overlaps with this
@@ -125,22 +117,20 @@ public:
     bool intersects(const PositionVector& v1) const;
 
     /** Returns the position of the intersection */
-    Position intersectsAtPoint(const Position& p1,
-                               const Position& p2) const; // !!!
-
-    /** Returns any intersection Points with the given line (ignoring z-coordinates) */
-    PositionVector intersectionPoints2D(const Line& line) const;
+    Position intersectionPosition2D(const Position& p1,
+                                    const Position& p2,
+                                    const SUMOReal withinDist = 0.) const;
 
     /** @brief For all intersections between this vector and other,
      * return the 2D-length of the subvector from this vectors start to the intersection */
-    std::vector<SUMOReal> intersectsAtLengths2D(const PositionVector& other) const; // !!!
+    std::vector<SUMOReal> intersectsAtLengths2D(const PositionVector& other) const;
 
     /** @brief For all intersections between this vector and line,
      * return the 2D-length of the subvector from this vectors start to the intersection */
-    std::vector<SUMOReal> intersectsAtLengths2D(const Line& line) const; // !!!
+    std::vector<SUMOReal> intersectsAtLengths2D(const Position& lp1, const Position& lp2) const;
 
     /** Returns the position of the intersection */
-    Position intersectsAtPoint(const PositionVector& v1) const; // !!!
+    Position intersectionPosition2D(const PositionVector& v1) const;
 
     /// @brief ensures that the last position equals the first
     void closePolygon();
@@ -156,6 +146,9 @@ public:
     /// Returns the position at the given length
     Position positionAtOffset2D(SUMOReal pos, SUMOReal lateralOffset = 0) const;
 
+    /// Returns the rotation at the given length
+    SUMOReal rotationAtOffset(SUMOReal pos) const;
+
     /// Returns the rotation at the given length
     SUMOReal rotationDegreeAtOffset(SUMOReal pos) const;
 
@@ -164,11 +157,11 @@ public:
 
     /// Returns the position between the two given point at the specified position */
     static Position positionAtOffset(const Position& p1,
-                                     const Position& p2, SUMOReal pos, SUMOReal lateralOffset);
+                                     const Position& p2, SUMOReal pos, SUMOReal lateralOffset = 0.);
 
     /// Returns the position between the two given point at the specified position */
     static Position positionAtOffset2D(const Position& p1,
-                                       const Position& p2, SUMOReal pos, SUMOReal lateralOffset);
+                                       const Position& p2, SUMOReal pos, SUMOReal lateralOffset = 0.);
 
     /// Returns a boundary enclosing this list of lines
     Boundary getBoxBoundary() const;
@@ -213,12 +206,8 @@ public:
     /// @brief mirror coordinates along the x-axis
     void mirrorX();
 
-    void reshiftRotate(SUMOReal xoff, SUMOReal yoff, SUMOReal rot);
-
     PositionVector convexHull() const;
 
-    int appendWithCrossingPoint(const PositionVector& v);
-
     // @brief append the given vector to this one
     void append(const PositionVector& v, SUMOReal sameThreshold = 2.0);
 
@@ -232,17 +221,17 @@ public:
 
     void sortByIncreasingXY();
 
-    void extrapolate(SUMOReal val);
+    void extrapolate(const SUMOReal val, const bool onlyFirst = false);
+
+    void extrapolate2D(const SUMOReal val, const bool onlyFirst = false);
 
     PositionVector reverse() const;
 
+    static Position sideOffset(const Position& beg, const Position& end, const SUMOReal amount);
+
     void move2side(SUMOReal amount);
 
-    Line lineAt(int pos) const;
-
-    Line getBegLine() const;
-
-    Line getEndLine() const;
+    SUMOReal angleAt2D(int pos) const;
 
 
     // @brief inserts p between the two closest positions and returns the insertion index
@@ -276,9 +265,7 @@ public:
     // !!!
     SUMOReal isLeft(const Position& P0, const Position& P1, const Position& P2) const;
 
-    void pruneFromBeginAt(const Position& p);
-    void pruneFromEndAt(const Position& p);
-
+    /// returns the angle in radians of the line connecting the first and the last position
     SUMOReal beginEndAngle() const;
 
     SUMOReal nearest_offset_to_point2D(const Position& p, bool perpendicular = true) const;
@@ -318,11 +305,18 @@ public:
      */
     void removeDoublePoints(SUMOReal minDist = POSITION_EPS, bool assertLength = false);
 
-    void removeColinearPoints();
-
     /// @brief return whether two positions differ in z-coordinate
     bool hasElevation() const;
 
+private:
+    /** @brief return whether the line segments defined by Line p11,p12
+     * and Line p21,p22 intersect
+     */
+    static bool intersects(const Position& p11, const Position& p12,
+                           const Position& p21, const Position& p22,
+                           const SUMOReal withinDist = 0.,
+                           SUMOReal* x = 0, SUMOReal* y = 0, SUMOReal* mu = 0);
+
 };
 
 
