@@ -2,12 +2,12 @@
 /// @file    GUIEvent.h
 /// @author  Daniel Krajzewicz
 /// @date    Sept 2002
-/// @version $Id: GUIEvent.h 18095 2015-03-17 09:39:00Z behrisch $
+/// @version $Id: GUIEvent.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // Definition of an own event class
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -59,6 +59,10 @@ enum GUIEventType {
         The reason and the time step are stored within the event */
     EVENT_SIMULATION_ENDED,
 
+    /** @brief Send when a screenshot is requested;
+        View and file name are stored within the event */
+    EVENT_SCREENSHOT,
+
     /// End of events list; use this to define new
     EVENT_END
 };
