@@ -5,12 +5,12 @@
 /// @author  Michael Behrisch
 /// @author  Laura Bieker
 /// @date    Sept 2002
-/// @version $Id: GUIGlObject.cpp 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: GUIGlObject.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // Base class for all objects that may be displayed within the openGL-gui
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -59,6 +59,7 @@ StringBijection<GUIGlObjectType>::Entry GUIGlObject::GUIGlObjectTypeNamesInitial
     {"edge",          GLO_EDGE},
     {"lane",          GLO_LANE},
     {"junction",      GLO_JUNCTION},
+    {"crossing",      GLO_CROSSING},
     {"tlLogic",       GLO_TLLOGIC},
     {"detector",      GLO_DETECTOR},
     {"trigger",       GLO_TRIGGER},
@@ -67,6 +68,7 @@ StringBijection<GUIGlObjectType>::Entry GUIGlObject::GUIGlObjectTypeNamesInitial
     {"poi",           GLO_POI},
     {"vehicle",       GLO_VEHICLE},
     {"person",        GLO_PERSON},
+    {"container",     GLO_CONTAINER},
     {"undefined",     GLO_MAX}
 };
 
@@ -162,6 +164,15 @@ GUIGlObject::buildShowParamsPopupEntry(GUIGLObjectPopupMenu* ret, bool addSepara
 }
 
 
+void
+GUIGlObject::buildShowTypeParamsPopupEntry(GUIGLObjectPopupMenu* ret, bool addSeparator) {
+    new FXMenuCommand(ret, "Show Type Parameter", GUIIconSubSys::getIcon(ICON_APP_TABLE), ret, MID_SHOWTYPEPARS);
+    if (addSeparator) {
+        new FXMenuSeparator(ret);
+    }
+}
+
+
 void
 GUIGlObject::buildPositionCopyEntry(GUIGLObjectPopupMenu* ret, bool addSeparator) {
     new FXMenuCommand(ret, "Copy cursor position to clipboard", 0, ret, MID_COPY_CURSOR_POSITION);
