@@ -4,12 +4,12 @@
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    Mon, 08.03.2004
-/// @version $Id: GUITexturesHelper.cpp 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: GUITexturesHelper.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // Global storage for textures; manages and draws them
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2004-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2004-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -122,10 +122,13 @@ GUITexturesHelper::drawTexturedBox(unsigned int which,
 
 
 int
-GUITexturesHelper::getTextureID(const std::string& filename) {
+GUITexturesHelper::getTextureID(const std::string& filename, const bool mirrorX) {
     if (myTextures.count(filename) == 0) {
         try {
             FXImage* i = MFXImageHelper::loadImage(GUIMainWindow::getInstance()->getApp(), filename);
+            if (mirrorX) {
+                i->mirror(false, true);
+            }
             if (MFXImageHelper::scalePower2(i, getMaxTextureSize())) {
                 WRITE_WARNING("Scaling '" + filename + "'.");
             }
