@@ -4,12 +4,12 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: GUIVisualizationSettings.cpp 18756 2015-08-31 19:16:33Z behrisch $
+/// @version $Id: GUIVisualizationSettings.cpp 20439 2016-04-13 11:39:22Z namdre $
 ///
 // Stores the information about how to visualize structures
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -42,9 +42,7 @@
 // ===========================================================================
 // static members
 // ===========================================================================
-#ifdef HAVE_INTERNAL
 bool GUIVisualizationSettings::UseMesoSim = false;
-#endif
 
 // ===========================================================================
 // member method definitions
@@ -69,10 +67,14 @@ GUIVisualizationSettings::GUIVisualizationSettings()
       containerQuality(0),
       containerSize(1),
       containerName(false, 50, RGBColor(0, 153, 204, 255)),
-      drawLinkTLIndex(false), drawLinkJunctionIndex(false),
+      drawLinkTLIndex(false, 50, RGBColor(128, 128, 255, 255)),
+      drawLinkJunctionIndex(false, 50, RGBColor(128, 128, 255, 255)),
       junctionName(false, 50, RGBColor(0, 255, 128, 255)),
       internalJunctionName(false, 50, RGBColor(0, 204, 128, 255)),
-      showLane2Lane(false), drawJunctionShape(true), addMode(0),
+      showLane2Lane(false),
+      drawJunctionShape(true),
+      drawCrossingsAndWalkingareas(true),
+      addMode(0),
       addSize(1),
       addName(false, 50, RGBColor(255, 0, 128, 255)),
       poiSize(0), poiName(false, 50, RGBColor(255, 0, 128, 255)),
@@ -90,6 +92,7 @@ GUIVisualizationSettings::GUIVisualizationSettings()
     scheme.addColor(RGBColor(0, 0, 0, 0), 3, "green verge");
     scheme.addColor(RGBColor(150, 200, 200), 4, "waterway");
     scheme.addColor(RGBColor(92, 92, 92), 5, "no passenger"); // paths, service roads etc
+    scheme.addColor(RGBColor::RED, 6, "closed"); // road closing
     laneColorer.addScheme(scheme);
     scheme = GUIColorScheme("by selection (lane-/streetwise)", RGBColor(128, 128, 128, 255), "unselected", true);
     scheme.addColor(RGBColor(0, 80, 180, 255), 1, "selected");
@@ -221,6 +224,16 @@ GUIVisualizationSettings::GUIVisualizationSettings()
     scheme.addColor(RGBColor::BLUE, (SUMOReal)(1));
     scheme.addColor(RGBColor::MAGENTA, (SUMOReal)(1.25));
     laneColorer.addScheme(scheme);
+    scheme = GUIColorScheme("by routing device assumed speed ", RGBColor::RED);
+    scheme.addColor(RGBColor::YELLOW, (SUMOReal)(30 / 3.6));
+    scheme.addColor(RGBColor::GREEN, (SUMOReal)(55 / 3.6));
+    scheme.addColor(RGBColor::CYAN, (SUMOReal)(80 / 3.6));
+    scheme.addColor(RGBColor::BLUE, (SUMOReal)(120 / 3.6));
+    scheme.addColor(RGBColor::MAGENTA, (SUMOReal)(150 / 3.6));
+    laneColorer.addScheme(scheme);
+    scheme = GUIColorScheme("by electricity consumption", RGBColor::GREEN);
+    scheme.addColor(RGBColor::RED, (SUMOReal)(1 / 7.5 / 5.));
+    laneColorer.addScheme(scheme);
 
 
     /// add vehicle coloring schemes
@@ -245,11 +258,17 @@ GUIVisualizationSettings::GUIVisualizationSettings()
     scheme.addColor(RGBColor::YELLOW, (SUMOReal)200);
     scheme.addColor(RGBColor::RED, (SUMOReal)300);
     vehicleColorer.addScheme(scheme);
+    scheme = GUIColorScheme("by accumulated waiting time", RGBColor::BLUE);
+    scheme.addColor(RGBColor::CYAN, (SUMOReal)25);
+    scheme.addColor(RGBColor::GREEN, (SUMOReal)50);
+    scheme.addColor(RGBColor::YELLOW, (SUMOReal)75);
+    scheme.addColor(RGBColor::RED, (SUMOReal)100);
+    vehicleColorer.addScheme(scheme);
     scheme = GUIColorScheme("by time since lane change", RGBColor(179, 179, 179, 255), "0");
     scheme.addColor(RGBColor(189, 189, 179, 255), -180);
     scheme.addColor(RGBColor(255, 255, 0, 255), -20);
-    scheme.addColor(RGBColor(255,   0, 0, 255), -1);
-    scheme.addColor(RGBColor(0,   0, 255, 255),  1);
+    scheme.addColor(RGBColor(255,   0, 0, 255), -0.001);
+    scheme.addColor(RGBColor(0,   0, 255, 255),  0.001);
     scheme.addColor(RGBColor(0, 255, 255, 255),  20);
     scheme.addColor(RGBColor(179, 189, 189, 255),  180);
     scheme.setAllowsNegativeValues(true);
@@ -310,6 +329,15 @@ GUIVisualizationSettings::GUIVisualizationSettings()
     scheme.addColor(RGBColor(0,   0, 255, 255), 2);
     scheme.setAllowsNegativeValues(true);
     vehicleColorer.addScheme(scheme);
+    scheme = GUIColorScheme("by depart delay", RGBColor::BLUE);
+    scheme.addColor(RGBColor::CYAN, (SUMOReal)30);
+    scheme.addColor(RGBColor::GREEN, (SUMOReal)100);
+    scheme.addColor(RGBColor::YELLOW, (SUMOReal)200);
+    scheme.addColor(RGBColor::RED, (SUMOReal)300);
+    vehicleColorer.addScheme(scheme);
+    scheme = GUIColorScheme("by electricity consumption", RGBColor::GREEN);
+    scheme.addColor(RGBColor::RED, (SUMOReal)5);
+    vehicleColorer.addScheme(scheme);
     vehicleColorer.addScheme(GUIColorScheme("random", RGBColor::YELLOW, "", true));
 
     /// add person coloring schemes
@@ -382,6 +410,9 @@ GUIVisualizationSettings::GUIVisualizationSettings()
     scheme.addColor(RGBColor::MAGENTA, 7, "unregulated");
     scheme.addColor(RGBColor::BLACK, 8, "dead_end");
     scheme.addColor(RGBColor::ORANGE, 9, "rail_signal");
+    scheme.addColor(RGBColor(192, 128, 64), 10, "zipper"); // light brown
+    scheme.addColor(RGBColor(192, 255, 192), 11, "traffic_light_right_on_red"); // light green
+    scheme.addColor(RGBColor(128, 0, 128), 12, "rail_crossing"); // dark purple
     junctionColorer.addScheme(scheme);
 
 
@@ -460,10 +491,12 @@ GUIVisualizationSettings::GUIVisualizationSettings()
         scheme.addColor(2, (SUMOReal)(1));
         scheme.addColor(10, (SUMOReal)(2));
         laneScaler.addScheme(scheme);
+        scheme = GUIScaleScheme("by electricity consumption", 0);
+        scheme.addColor(10, (SUMOReal)(1 / 7.5 / 5.));
+        laneScaler.addScheme(scheme);
     }
 
 
-#ifdef HAVE_INTERNAL
     /// add edge coloring schemes
     edgeColorer.addScheme(GUIColorScheme("uniform (streetwise)", RGBColor::BLACK, "", true));
     scheme = GUIColorScheme("by selection (streetwise)", RGBColor(128, 128, 128, 255), "unselected", true);
@@ -484,15 +517,24 @@ GUIVisualizationSettings::GUIVisualizationSettings()
     scheme.addColor(RGBColor::RED, (SUMOReal)0.95);
     edgeColorer.addScheme(scheme);
     scheme = GUIColorScheme("by current speed (streetwise)", RGBColor::RED);
-    scheme.addColor(RGBColor::BLUE, (SUMOReal)(150.0 / 3.6));
+    scheme.addColor(RGBColor::YELLOW, (SUMOReal)(30 / 3.6));
+    scheme.addColor(RGBColor::GREEN, (SUMOReal)(55 / 3.6));
+    scheme.addColor(RGBColor::CYAN, (SUMOReal)(80 / 3.6));
+    scheme.addColor(RGBColor::BLUE, (SUMOReal)(120 / 3.6));
+    scheme.addColor(RGBColor::MAGENTA, (SUMOReal)(150 / 3.6));
     edgeColorer.addScheme(scheme);
     scheme = GUIColorScheme("by current flow (streetwise)", RGBColor::BLUE);
     scheme.addColor(RGBColor::RED, (SUMOReal)5000);
     edgeColorer.addScheme(scheme);
     scheme = GUIColorScheme("by relative speed (streetwise)", RGBColor::RED);
-    scheme.addColor(RGBColor::BLUE, (SUMOReal)1);
+    scheme.addColor(RGBColor::YELLOW, (SUMOReal)(0.25));
+    scheme.addColor(RGBColor::GREEN, (SUMOReal)(0.5));
+    scheme.addColor(RGBColor::CYAN, (SUMOReal)(0.75));
+    scheme.addColor(RGBColor::BLUE, (SUMOReal)(1));
+    scheme.addColor(RGBColor::MAGENTA, (SUMOReal)(1.25));
     edgeColorer.addScheme(scheme);
 
+
     /// add edge scaling schemes
     {
         edgeScaler.addScheme(GUIScaleScheme("uniform", 1, "", true));
@@ -515,51 +557,42 @@ GUIVisualizationSettings::GUIVisualizationSettings()
         scheme.addColor(20, (SUMOReal)1);
         edgeScaler.addScheme(scheme);
     }
-#endif
 
 }
 
 
 size_t
 GUIVisualizationSettings::getLaneEdgeMode() const {
-#ifdef HAVE_INTERNAL
     if (UseMesoSim) {
         return edgeColorer.getActive();
     }
-#endif
     return laneColorer.getActive();
 }
 
 
 size_t
 GUIVisualizationSettings::getLaneEdgeScaleMode() const {
-#ifdef HAVE_INTERNAL
     if (UseMesoSim) {
         return edgeScaler.getActive();
     }
-#endif
     return laneScaler.getActive();
 }
 
 
 GUIColorScheme&
 GUIVisualizationSettings::getLaneEdgeScheme() {
-#ifdef HAVE_INTERNAL
     if (UseMesoSim) {
         return edgeColorer.getScheme();
     }
-#endif
     return laneColorer.getScheme();
 }
 
 
 GUIScaleScheme&
 GUIVisualizationSettings::getLaneEdgeScaleScheme() {
-#ifdef HAVE_INTERNAL
     if (UseMesoSim) {
         return edgeScaler.getScheme();
     }
-#endif
     return laneScaler.getScheme();
 }
 
@@ -600,10 +633,8 @@ GUIVisualizationSettings::save(OutputDevice& dev) const {
     streetName.print(dev, "streetName");
     laneColorer.save(dev);
     laneScaler.save(dev);
-#ifdef HAVE_INTERNAL
     edgeColorer.save(dev);
     edgeScaler.save(dev);
-#endif
     dev.closeTag();
     // vehicles
     dev.openTag(SUMO_TAG_VIEWSETTINGS_VEHICLES);
@@ -639,8 +670,12 @@ GUIVisualizationSettings::save(OutputDevice& dev) const {
     // junctions
     dev.openTag(SUMO_TAG_VIEWSETTINGS_JUNCTIONS);
     dev.writeAttr("junctionMode", junctionColorer.getActive());
-    dev.writeAttr("drawLinkTLIndex", drawLinkTLIndex);
-    dev.writeAttr("drawLinkJunctionIndex", drawLinkJunctionIndex);
+    dev.lf();
+    dev << "                  ";
+    drawLinkTLIndex.print(dev, "drawLinkTLIndex");
+    dev.lf();
+    dev << "                  ";
+    drawLinkTLIndex.print(dev, "drawLinkJunctionIndex");
     dev.lf();
     dev << "                  ";
     junctionName.print(dev, "junctionName");
@@ -651,6 +686,7 @@ GUIVisualizationSettings::save(OutputDevice& dev) const {
     dev << "                  ";
     dev.writeAttr("showLane2Lane", showLane2Lane);
     dev.writeAttr("drawShape", drawJunctionShape);
+    dev.writeAttr("drawCrossingsAndWalkingareas", drawCrossingsAndWalkingareas);
     junctionColorer.save(dev);
     dev.closeTag();
     // additionals
@@ -701,14 +737,12 @@ GUIVisualizationSettings::operator==(const GUIVisualizationSettings& v2) {
         return false;
     }
 
-#ifdef HAVE_INTERNAL
     if (!(edgeColorer == v2.edgeColorer)) {
         return false;
     }
     if (!(edgeScaler == v2.edgeScaler)) {
         return false;
     }
-#endif
     if (!(laneColorer == v2.laneColorer)) {
         return false;
     }
@@ -814,6 +848,10 @@ GUIVisualizationSettings::operator==(const GUIVisualizationSettings& v2) {
         return false;
     }
 
+    if (drawCrossingsAndWalkingareas != v2.drawCrossingsAndWalkingareas) {
+        return false;
+    }
+
     if (addMode != v2.addMode) {
         return false;
     }
