@@ -5,12 +5,12 @@
 /// @author  Michael Behrisch
 /// @author  Andreas Gaubatz
 /// @date    Mon, 08.03.2004
-/// @version $Id: GUIAppEnum.h 18296 2015-04-24 12:41:38Z namdre $
+/// @version $Id: GUIAppEnum.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // Message and object IDs used by the FOX-version of the gui
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -123,10 +123,12 @@ enum {
     /// @name Edit Menu Item - IDs
     //@{
 
-    /// Open chosen editor - ID
+    /// Open editor for selections
     MID_EDITCHOSEN,
     /// Edit simulation breakpoints
     MID_EDIT_BREAKPOINTS,
+    /// Open in netedit
+    MID_NETEDIT,
     //@}
 
 
@@ -217,12 +219,16 @@ enum {
     MID_COPY_NAME,
     /// Copy typed object name - popup entry
     MID_COPY_TYPED_NAME,
+    /// Copy edge name (for lanes only)
+    MID_COPY_EDGE_NAME,
     /// Copy cursor position - popup entry
     MID_COPY_CURSOR_POSITION,
     /// Copy cursor geo-coordinate position - popup entry
     MID_COPY_CURSOR_GEOPOSITION,
     /// Show object parameter - popup entry
     MID_SHOWPARS,
+    /// Show object type parameter - popup entry
+    MID_SHOWTYPEPARS,
     /// Show TLS phases - popup entry
     MID_SWITCH_OFF,
     MID_SWITCH,
@@ -293,6 +299,8 @@ enum {
     MID_QUITONSIMEND,
     /// Allow textures - Option
     MID_ALLOWTEXTURES,
+    /// Locate links in messages - Option
+    MID_LOCATELINKS,
     //@}
 
 
@@ -349,6 +357,16 @@ enum {
     MID_SIMPLE_VIEW_SAVE_DECALS,
     //@}
 
+    /** help button */
+    MID_HELP,
+
+    /// @name dynamic interaction with the simulation via SUMO-GUI
+    //@{
+    MID_CLOSE_LANE,
+    MID_CLOSE_EDGE,
+    MID_ADD_REROUTER,
+    //@}
+
 
     /// @name NETEDIT Message IDs
     //@{
@@ -437,6 +455,14 @@ enum {
     MID_GNE_SET_EDGE_ENDPOINT,
     /** restore geometry endpoint to node position */
     MID_GNE_RESET_EDGE_ENDPOINT,
+    /** remove inner geometry */
+    MID_GNE_STRAIGHTEN,
+    /** simplify polygon geometry */
+    MID_GNE_SIMPLIFY_SHAPE,
+    /** delete geometry point */
+    MID_GNE_DELETE_GEOMETRY,
+    /** duplicate a lane */
+    MID_GNE_DUPLICATE_LANE,
     /** set object as template*/
     MID_GNE_SET_TEMPLATE,
     /** copy template*/
@@ -452,9 +478,6 @@ enum {
     /** changes the visual scaling of selected items */
     MID_GNE_SELECT_SCALE,
 
-    /** help button */
-    MID_GNE_HELP,
-
     //@}
 
     MID_LAST
