@@ -4,12 +4,12 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: GUIDanielPerspectiveChanger.cpp 18756 2015-08-31 19:16:33Z behrisch $
+/// @version $Id: GUIDanielPerspectiveChanger.cpp 20482 2016-04-18 20:49:42Z behrisch $
 ///
 // A class that allows to steer the visual output in dependence to
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -54,8 +54,8 @@ GUIDanielPerspectiveChanger::GUIDanielPerspectiveChanger(
     myMouseButtonState(MOUSEBTN_NONE),
     myMoveOnClick(false),
     myZoomBase(viewPort.getCenter()),
-    myDragDelay(0)
-{}
+    myDragDelay(0) {
+}
 
 
 GUIDanielPerspectiveChanger::~GUIDanielPerspectiveChanger() {}
@@ -181,9 +181,9 @@ void
 GUIDanielPerspectiveChanger::onMouseWheel(void* data) {
     FXEvent* e = (FXEvent*) data;
     SUMOReal diff = 0.1;
-    if (e->state & CONTROLMASK) {
+    if ((e->state & CONTROLMASK) != 0) {
         diff /= 2;
-    } else if (e->state & SHIFTMASK) {
+    } else if ((e->state & SHIFTMASK) != 0) {
         diff *= 2;
     }
     if (e->code < 0) {
