@@ -3,12 +3,12 @@
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    2005-05-04
-/// @version $Id: GUIDialog_EditViewport.h 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: GUIDialog_EditViewport.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // A dialog to change the viewport
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -97,7 +97,8 @@ public:
     long onCmdSave(FXObject*, FXSelector, void*);
     /// @}
 
-
+    /// write the settings to the given device
+    void writeXML(OutputDevice& dev);
 
     /** @brief Sets the given values into the dialog
      * @param[in] zoom Current view's zoom
