@@ -4,12 +4,12 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: GUIPerspectiveChanger.h 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: GUIPerspectiveChanger.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // A virtual class that allows to steer the visual output in dependence to
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -78,6 +78,8 @@ public:
     virtual bool onRightBtnRelease(void* data);
     virtual void onMouseWheel(void* data);
     virtual void onMouseMove(void* data);
+    virtual long onKeyPress(void* data);
+    virtual long onKeyRelease(void* data);
 
     /// Returns the rotation of the canvas stored in this changer
     virtual SUMOReal getRotation() const = 0;
