@@ -6,12 +6,12 @@
 /// @author  Laura Bieker
 /// @author  Andreas Gaubatz
 /// @date    Sept 2002
-/// @version $Id: GUISUMOAbstractView.cpp 18756 2015-08-31 19:16:33Z behrisch $
+/// @version $Id: GUISUMOAbstractView.cpp 20482 2016-04-18 20:49:42Z behrisch $
 ///
 // The base class for a view
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -114,19 +114,19 @@ GUISUMOAbstractView::GUISUMOAbstractView(FXComposite* p,
         FXGLVisual* glVis, FXGLCanvas* share)
     : FXGLCanvas(p, glVis, share, p, MID_GLCANVAS,
                  LAYOUT_SIDE_TOP | LAYOUT_FILL_X | LAYOUT_FILL_Y, 0, 0, 0, 0),
-    myApp(&app),
-    myParent(parent),
-    myGrid(&((SUMORTree&)grid)),
-    myChanger(0),
-    myMouseHotspotX(app.getDefaultCursor()->getHotX()),
-    myMouseHotspotY(app.getDefaultCursor()->getHotY()),
-    myPopup(0),
-    myUseToolTips(false),
-    myAmInitialised(false),
-    myViewportChooser(0),
-    myWindowCursorPositionX(getWidth() / 2),
-    myWindowCursorPositionY(getHeight() / 2),
-    myVisualizationChanger(0) {
+      myApp(&app),
+      myParent(parent),
+      myGrid(&((SUMORTree&)grid)),
+      myChanger(0),
+      myMouseHotspotX(app.getDefaultCursor()->getHotX()),
+      myMouseHotspotY(app.getDefaultCursor()->getHotY()),
+      myPopup(0),
+      myUseToolTips(false),
+      myAmInitialised(false),
+      myViewportChooser(0),
+      myWindowCursorPositionX(getWidth() / 2),
+      myWindowCursorPositionY(getHeight() / 2),
+      myVisualizationChanger(0) {
     setTarget(this);
     enable();
     flags |= FLAG_ENABLED;
@@ -286,12 +286,10 @@ GUISUMOAbstractView::getObjectAtPosition(Position pos) {
             if (type == GLO_POI || type == GLO_POLYGON) {
                 layer = dynamic_cast<Shape*>(o)->getLayer();
             }
-#ifdef HAVE_INTERNAL
             if (type == GLO_LANE && GUIVisualizationSettings::UseMesoSim) {
                 // do not select lanes in meso mode
                 continue;
             }
-#endif
             // check whether the current object is above a previous one
             if (layer > maxLayer) {
                 idMax = id;
@@ -514,6 +512,7 @@ GUISUMOAbstractView::centerTo(GUIGlID id, bool applyZoom, SUMOReal zoomDist) {
         } else {
             // called during tracking. update is triggered somewhere else
             myChanger->centerTo(o->getCenteringBoundary().getCenter(), zoomDist, applyZoom);
+            updatePositionInformation();
         }
     }
     GUIGlObjectStorage::gIDStorage.unblockObject(id);
@@ -592,7 +591,7 @@ GUISUMOAbstractView::onLeftBtnPress(FXObject*, FXSelector , void* data) {
     setFocus();
     FXEvent* e = (FXEvent*) data;
     // check whether the selection-mode is activated
-    if (e->state & CONTROLMASK) {
+    if ((e->state & CONTROLMASK) != 0) {
         // try to get the object-id if so
         if (makeCurrent()) {
             unsigned int id = getObjectUnderCursor();
@@ -650,6 +649,7 @@ long
 GUISUMOAbstractView::onMouseWheel(FXObject*, FXSelector , void* data) {
     if (!myApp->isGaming()) {
         myChanger->onMouseWheel(data);
+        updatePositionInformation();
     }
     return 1;
 }
@@ -731,8 +731,10 @@ std::string
 GUISUMOAbstractView::makeSnapshot(const std::string& destFile) {
     std::string errorMessage;
     FXString ext = FXPath::extension(destFile.c_str());
-    bool useGL2PS = ext == "ps" || ext == "eps" || ext == "pdf" || ext == "svg" || ext == "tex" || ext == "pgf";
-
+    const bool useGL2PS = ext == "ps" || ext == "eps" || ext == "pdf" || ext == "svg" || ext == "tex" || ext == "pgf";
+#ifdef HAVE_FFMPEG
+    const bool useVideo = destFile == "" || ext == "h264" || ext == "hevc";
+#endif
     for (int i = 0; i < 10 && !makeCurrent(); ++i) {
         FXSingleEventThread::sleep(100);
     }
@@ -858,9 +860,19 @@ GUISUMOAbstractView::makeSnapshot(const std::string& destFile) {
             } while (pa < paa);
         } while (paa < pbb);
         try {
-            if (!MFXImageHelper::saveImage(destFile, getWidth(), getHeight(), buf)) {
-                errorMessage = "Could not save '" + destFile + "'.";
-            }
+#ifdef HAVE_FFMPEG
+            if (useVideo) {
+                try {
+                    saveFrame(destFile, buf);
+                    errorMessage = "video";
+                } catch (std::runtime_error& err) {
+                    errorMessage = err.what();
+                }
+            } else
+#endif
+                if (!MFXImageHelper::saveImage(destFile, getWidth(), getHeight(), buf)) {
+                    errorMessage = "Could not save '" + destFile + "'.";
+                }
         } catch (InvalidArgument& e) {
             errorMessage = "Could not save '" + destFile + "'.\n" + e.what();
         }
@@ -1123,5 +1135,11 @@ GUISUMOAbstractView::getDelay() const {
     return myApp->getDelay();
 }
 
+
+void
+GUISUMOAbstractView::setDelay(SUMOReal delay) {
+    myApp->setDelay(delay);
+}
+
 /****************************************************************************/
 
