@@ -4,12 +4,12 @@
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    Mon, 17 Dec 2001
-/// @version $Id: Option.h 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: Option.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // Classes representing a single program option (with different types)
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -148,16 +148,12 @@ public:
     virtual const IntVector& getIntVector() const;
 
 
-    /** @brief Stores the given value (used for non-bool options)
+    /** @brief Stores the given value
      *
-     * This method is overriden by all option classes but Option_Bool.
+     * This method is overriden by all option classes.
      *  The value is converted into the proper type and stored in "myValue".
      *  Then, "markSet" is called in order to know that a value has been set.
      *
-     * The method defined in Option throws an exception, because all derived
-     *  classes should override it - for Option_Bool, it is tested on a different
-     *  position whether to use this method or bool set(bool).
-     *
      * The method returns whether the value could be set (the return value from
      *  "markSet").
      *
@@ -165,7 +161,7 @@ public:
      *  is thrown.
      *
      * @return Whether the new value could be set
-     * @exception InvalidArgument If being an instance of Option or Option_Bool or if the value could not be converted
+     * @exception InvalidArgument If the value could not be converted
      */
     virtual bool set(const std::string& v) = 0;
 
@@ -222,6 +218,14 @@ public:
     void resetWritable();
 
 
+    /** @brief Resets the option to be on its default value
+     *
+     * An option is on its default after initialisation with a value, but as soon as it gets set,
+     *  it is no longer. This method resets the default-flag.
+     */
+    void resetDefault();
+
+
     /** @brief Returns the description of what this option does
      *
      * The description stored in myDescription is returned.
@@ -630,6 +634,17 @@ public:
      */
     bool isFileName() const;
 
+
+    /** @brief Returns the string-representation of the value
+     *
+     * The value is URL-encoded using StringUtils::urlEncode and returned.
+     *
+     * @see std::string Option::getValueString()
+     * @return The stored value encoded into a string
+     */
+    std::string getValueString() const;
+
+
 };
 
 
