@@ -5,12 +5,12 @@
 /// @author  Jakob Erdmann
 /// @author  Melanie Knocke
 /// @date    Jun 2004
-/// @version $Id: Polygon.h 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: Polygon.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // A 2D- or 3D-polygon
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2004-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2004-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -37,6 +37,8 @@
 #include <utils/common/Parameterised.h>
 #include "Shape.h"
 
+class OutputDevice;
+
 // we need to put this into a namespace to avoid clashing with wingdi.h::Polygon function
 namespace SUMO {
 
@@ -111,6 +113,10 @@ public:
     }
     /// @}
 
+    /* @brief polygon definition to the given device
+     * @param[in] geo  Whether to write the output in geo-coordinates
+     */
+    void writeXML(OutputDevice& out, bool geo = false);
 
 protected:
     /// @brief The positions of the polygon
