@@ -5,12 +5,12 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    30.05.2012
-/// @version $Id: TraCIAPI.cpp 18213 2015-04-16 11:55:42Z behrisch $
+/// @version $Id: TraCIAPI.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // C++ TraCI client API implementation
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2012-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2012-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -28,7 +28,7 @@
 #ifdef _MSC_VER
 #include <windows_config.h>
 #else
-#include <config.h>
+#include "../../config.h"
 #endif
 
 #include "TraCIAPI.h"
@@ -53,7 +53,7 @@ TraCIAPI::TraCIAPI()
     : edge(*this), gui(*this), inductionloop(*this),
       junction(*this), lane(*this), multientryexit(*this), poi(*this),
       polygon(*this), route(*this), simulation(*this), trafficlights(*this),
-      vehicletype(*this),
+      vehicletype(*this), vehicle(*this),
       mySocket(0) {}
 #ifdef _MSC_VER
 #pragma warning(default: 4355)
@@ -273,6 +273,9 @@ TraCIAPI::check_commandGetResult(tcpip::Storage& inMsg, int command, int expecte
         throw tcpip::SocketException("#Error: received response with command id: " + toString(cmdId) + "but expected: " + toString(command + 0x10));
     }
     if (expectedType >= 0) {
+        // not called from the TraCITestClient but from within the TraCIAPI
+        inMsg.readUnsignedByte(); // variableID
+        inMsg.readString(); // objectID
         int valueDataType = inMsg.readUnsignedByte();
         if (valueDataType != expectedType) {
             throw tcpip::SocketException("Expected " + toString(expectedType) + " but got " + toString(valueDataType));
@@ -428,6 +431,13 @@ TraCIAPI::getColor(int cmd, int var, const std::string& id, tcpip::Storage* add)
 }
 
 
+void
+TraCIAPI::simulationStep(SUMOTime time) {
+    send_commandSimulationStep(time);
+    tcpip::Storage inMsg;
+    check_resultState(inMsg, CMD_SIMSTEP2);
+}
+
 
 // ---------------------------------------------------------------------------
 // TraCIAPI::EdgeScope-methods
@@ -533,6 +543,8 @@ TraCIAPI::EdgeScope::adaptTraveltime(const std::string& edgeID, SUMOReal time) c
     tcpip::Storage content;
     content.writeDouble(time);
     myParent.send_commandSetValue(CMD_SET_EDGE_VARIABLE, VAR_EDGE_TRAVELTIME, edgeID, content);
+    tcpip::Storage inMsg;
+    myParent.check_resultState(inMsg, CMD_SET_EDGE_VARIABLE);
 }
 
 void
@@ -540,6 +552,8 @@ TraCIAPI::EdgeScope::setEffort(const std::string& edgeID, SUMOReal effort) const
     tcpip::Storage content;
     content.writeDouble(effort);
     myParent.send_commandSetValue(CMD_SET_EDGE_VARIABLE, VAR_EDGE_EFFORT, edgeID, content);
+    tcpip::Storage inMsg;
+    myParent.check_resultState(inMsg, CMD_SET_EDGE_VARIABLE);
 }
 
 void
@@ -547,6 +561,8 @@ TraCIAPI::EdgeScope::setMaxSpeed(const std::string& edgeID, SUMOReal speed) cons
     tcpip::Storage content;
     content.writeDouble(speed);
     myParent.send_commandSetValue(CMD_SET_EDGE_VARIABLE, VAR_MAXSPEED, edgeID, content);
+    tcpip::Storage inMsg;
+    myParent.check_resultState(inMsg, CMD_SET_EDGE_VARIABLE);
 }
 
 
@@ -586,6 +602,8 @@ TraCIAPI::GUIScope::setZoom(const std::string& viewID, SUMOReal zoom) const {
     tcpip::Storage content;
     content.writeDouble(zoom);
     myParent.send_commandSetValue(CMD_SET_GUI_VARIABLE, VAR_VIEW_ZOOM, viewID, content);
+    tcpip::Storage inMsg;
+    myParent.check_resultState(inMsg, CMD_SET_GUI_VARIABLE);
 }
 
 void
@@ -595,13 +613,18 @@ TraCIAPI::GUIScope::setOffset(const std::string& viewID, SUMOReal x, SUMOReal y)
     content.writeDouble(x);
     content.writeDouble(y);
     myParent.send_commandSetValue(CMD_SET_GUI_VARIABLE, VAR_VIEW_OFFSET, viewID, content);
+    tcpip::Storage inMsg;
+    myParent.check_resultState(inMsg, CMD_SET_GUI_VARIABLE);
 }
 
 void
 TraCIAPI::GUIScope::setSchema(const std::string& viewID, const std::string& schemeName) const {
     tcpip::Storage content;
+    content.writeUnsignedByte(TYPE_STRING);
     content.writeString(schemeName);
     myParent.send_commandSetValue(CMD_SET_GUI_VARIABLE, VAR_VIEW_SCHEMA, viewID, content);
+    tcpip::Storage inMsg;
+    myParent.check_resultState(inMsg, CMD_SET_GUI_VARIABLE);
 }
 
 void
@@ -613,6 +636,8 @@ TraCIAPI::GUIScope::setBoundary(const std::string& viewID, SUMOReal xmin, SUMORe
     content.writeDouble(xmax);
     content.writeDouble(ymax);
     myParent.send_commandSetValue(CMD_SET_GUI_VARIABLE, VAR_VIEW_BOUNDARY, viewID, content);
+    tcpip::Storage inMsg;
+    myParent.check_resultState(inMsg, CMD_SET_GUI_VARIABLE);
 }
 
 void
@@ -620,6 +645,8 @@ TraCIAPI::GUIScope::screenshot(const std::string& viewID, const std::string& fil
     tcpip::Storage content;
     content.writeString(filename);
     myParent.send_commandSetValue(CMD_SET_GUI_VARIABLE, VAR_SCREENSHOT, viewID, content);
+    tcpip::Storage inMsg;
+    myParent.check_resultState(inMsg, CMD_SET_GUI_VARIABLE);
 }
 
 void
@@ -627,6 +654,8 @@ TraCIAPI::GUIScope::trackVehicle(const std::string& viewID, const std::string& v
     tcpip::Storage content;
     content.writeString(vehID);
     myParent.send_commandSetValue(CMD_SET_GUI_VARIABLE, VAR_TRACK_VEHICLE, viewID, content);
+    tcpip::Storage inMsg;
+    myParent.check_resultState(inMsg, CMD_SET_GUI_VARIABLE);
 }
 
 
@@ -680,9 +709,37 @@ TraCIAPI::InductionLoopScope::getTimeSinceDetection(const std::string& loopID) c
     return myParent.getDouble(CMD_GET_INDUCTIONLOOP_VARIABLE, LAST_STEP_TIME_SINCE_DETECTION, loopID);
 }
 
-unsigned int
+std::vector<TraCIAPI::InductionLoopScope::VehicleData>
 TraCIAPI::InductionLoopScope::getVehicleData(const std::string& loopID) const {
-    return myParent.getInt(CMD_GET_INDUCTIONLOOP_VARIABLE, LAST_STEP_VEHICLE_DATA, loopID);
+    tcpip::Storage inMsg;
+    myParent.send_commandGetVariable(CMD_GET_INDUCTIONLOOP_VARIABLE, LAST_STEP_VEHICLE_DATA, loopID);
+    myParent.processGET(inMsg, CMD_GET_INDUCTIONLOOP_VARIABLE, TYPE_COMPOUND);
+    std::vector<VehicleData> result;
+    inMsg.readInt(); // components
+    // number of items
+    inMsg.readUnsignedByte();
+    const int n = inMsg.readInt();
+    for (int i = 0; i < n; ++i) {
+        VehicleData vd;
+
+        inMsg.readUnsignedByte();
+        vd.id = inMsg.readString();
+
+        inMsg.readUnsignedByte();
+        vd.length = inMsg.readDouble();
+
+        inMsg.readUnsignedByte();
+        vd.entryTime = inMsg.readDouble();
+
+        inMsg.readUnsignedByte();
+        vd.leaveTime = inMsg.readDouble();
+
+        inMsg.readUnsignedByte();
+        vd.typeID = inMsg.readString();
+
+        result.push_back(vd);
+    }
+    return result;
 }
 
 
@@ -832,6 +889,8 @@ TraCIAPI::LaneScope::setAllowed(const std::string& laneID, const std::vector<std
         content.writeString(allowedClasses[i]);
     }
     myParent.send_commandSetValue(CMD_SET_LANE_VARIABLE, LANE_ALLOWED, laneID, content);
+    tcpip::Storage inMsg;
+    myParent.check_resultState(inMsg, CMD_SET_LANE_VARIABLE);
 }
 
 void
@@ -843,6 +902,8 @@ TraCIAPI::LaneScope::setDisallowed(const std::string& laneID, const std::vector<
         content.writeString(disallowedClasses[i]);
     }
     myParent.send_commandSetValue(CMD_SET_LANE_VARIABLE, LANE_DISALLOWED, laneID, content);
+    tcpip::Storage inMsg;
+    myParent.check_resultState(inMsg, CMD_SET_LANE_VARIABLE);
 }
 
 void
@@ -851,6 +912,8 @@ TraCIAPI::LaneScope::setMaxSpeed(const std::string& laneID, SUMOReal speed) cons
     content.writeUnsignedByte(TYPE_DOUBLE);
     content.writeDouble(speed);
     myParent.send_commandSetValue(CMD_SET_LANE_VARIABLE, VAR_MAXSPEED, laneID, content);
+    tcpip::Storage inMsg;
+    myParent.check_resultState(inMsg, CMD_SET_LANE_VARIABLE);
 }
 
 void
@@ -859,6 +922,8 @@ TraCIAPI::LaneScope::setLength(const std::string& laneID, SUMOReal length) const
     content.writeUnsignedByte(TYPE_DOUBLE);
     content.writeDouble(length);
     myParent.send_commandSetValue(CMD_SET_LANE_VARIABLE, VAR_LENGTH, laneID, content);
+    tcpip::Storage inMsg;
+    myParent.check_resultState(inMsg, CMD_SET_LANE_VARIABLE);
 }
 
 
@@ -933,6 +998,8 @@ TraCIAPI::POIScope::setType(const std::string& poiID, const std::string& setType
     content.writeUnsignedByte(TYPE_STRING);
     content.writeString(setType);
     myParent.send_commandSetValue(CMD_SET_POI_VARIABLE, VAR_TYPE, poiID, content);
+    tcpip::Storage inMsg;
+    myParent.check_resultState(inMsg, CMD_SET_POI_VARIABLE);
 }
 
 void
@@ -942,6 +1009,8 @@ TraCIAPI::POIScope::setPosition(const std::string& poiID, SUMOReal x, SUMOReal y
     content.writeDouble(x);
     content.writeDouble(y);
     myParent.send_commandSetValue(CMD_SET_POI_VARIABLE, VAR_POSITION, poiID, content);
+    tcpip::Storage inMsg;
+    myParent.check_resultState(inMsg, CMD_SET_POI_VARIABLE);
 }
 
 void
@@ -953,6 +1022,8 @@ TraCIAPI::POIScope::setColor(const std::string& poiID, const TraCIColor& c) cons
     content.writeUnsignedByte(c.b);
     content.writeUnsignedByte(c.a);
     myParent.send_commandSetValue(CMD_SET_POI_VARIABLE, VAR_COLOR, poiID, content);
+    tcpip::Storage inMsg;
+    myParent.check_resultState(inMsg, CMD_SET_POI_VARIABLE);
 }
 
 void
@@ -973,6 +1044,8 @@ TraCIAPI::POIScope::add(const std::string& poiID, SUMOReal x, SUMOReal y, const
     content.writeDouble(x);
     content.writeDouble(y);
     myParent.send_commandSetValue(CMD_SET_POI_VARIABLE, ADD, poiID, content);
+    tcpip::Storage inMsg;
+    myParent.check_resultState(inMsg, CMD_SET_POI_VARIABLE);
 }
 
 void
@@ -981,6 +1054,8 @@ TraCIAPI::POIScope::remove(const std::string& poiID, int layer) const {
     content.writeUnsignedByte(TYPE_INTEGER);
     content.writeInt(layer);
     myParent.send_commandSetValue(CMD_SET_POI_VARIABLE, REMOVE, poiID, content);
+    tcpip::Storage inMsg;
+    myParent.check_resultState(inMsg, CMD_SET_POI_VARIABLE);
 }
 
 
@@ -1015,6 +1090,8 @@ TraCIAPI::PolygonScope::setType(const std::string& polygonID, const std::string&
     content.writeUnsignedByte(TYPE_STRING);
     content.writeString(setType);
     myParent.send_commandSetValue(CMD_SET_POLYGON_VARIABLE, VAR_TYPE, polygonID, content);
+    tcpip::Storage inMsg;
+    myParent.check_resultState(inMsg, CMD_SET_POLYGON_VARIABLE);
 }
 
 void
@@ -1027,6 +1104,8 @@ TraCIAPI::PolygonScope::setShape(const std::string& polygonID, const TraCIAPI::T
         content.writeDouble(shape[i].y);
     }
     myParent.send_commandSetValue(CMD_SET_POLYGON_VARIABLE, VAR_POSITION, polygonID, content);
+    tcpip::Storage inMsg;
+    myParent.check_resultState(inMsg, CMD_SET_POLYGON_VARIABLE);
 }
 
 void
@@ -1038,6 +1117,8 @@ TraCIAPI::PolygonScope::setColor(const std::string& polygonID, const TraCIColor&
     content.writeUnsignedByte(c.b);
     content.writeUnsignedByte(c.a);
     myParent.send_commandSetValue(CMD_SET_POLYGON_VARIABLE, VAR_COLOR, polygonID, content);
+    tcpip::Storage inMsg;
+    myParent.check_resultState(inMsg, CMD_SET_POLYGON_VARIABLE);
 }
 
 void
@@ -1064,6 +1145,8 @@ TraCIAPI::PolygonScope::add(const std::string& polygonID, const TraCIAPI::TraCIP
         content.writeDouble(shape[i].y);
     }
     myParent.send_commandSetValue(CMD_SET_POLYGON_VARIABLE, ADD, polygonID, content);
+    tcpip::Storage inMsg;
+    myParent.check_resultState(inMsg, CMD_SET_POLYGON_VARIABLE);
 }
 
 void
@@ -1072,6 +1155,8 @@ TraCIAPI::PolygonScope::remove(const std::string& polygonID, int layer) const {
     content.writeUnsignedByte(TYPE_INTEGER);
     content.writeInt(layer);
     myParent.send_commandSetValue(CMD_SET_POLYGON_VARIABLE, REMOVE, polygonID, content);
+    tcpip::Storage inMsg;
+    myParent.check_resultState(inMsg, CMD_SET_POLYGON_VARIABLE);
 }
 
 
@@ -1096,6 +1181,8 @@ TraCIAPI::RouteScope::add(const std::string& routeID, const std::vector<std::str
     content.writeUnsignedByte(TYPE_STRINGLIST);
     content.writeStringList(edges);
     myParent.send_commandSetValue(CMD_SET_ROUTE_VARIABLE, VAR_POSITION, routeID, content);
+    tcpip::Storage inMsg;
+    myParent.check_resultState(inMsg, CMD_SET_ROUTE_VARIABLE);
 }
 
 
@@ -1271,6 +1358,8 @@ TraCIAPI::TrafficLightScope::setRedYellowGreenState(const std::string& tlsID, co
     content.writeUnsignedByte(TYPE_STRING);
     content.writeString(state);
     myParent.send_commandSetValue(CMD_SET_TL_VARIABLE, TL_RED_YELLOW_GREEN_STATE, tlsID, content);
+    tcpip::Storage inMsg;
+    myParent.check_resultState(inMsg, CMD_SET_TL_VARIABLE);
 }
 
 void
@@ -1279,6 +1368,8 @@ TraCIAPI::TrafficLightScope::setPhase(const std::string& tlsID, unsigned int ind
     content.writeUnsignedByte(TYPE_INTEGER);
     content.writeInt(index);
     myParent.send_commandSetValue(CMD_SET_TL_VARIABLE, TL_PHASE_INDEX, tlsID, content);
+    tcpip::Storage inMsg;
+    myParent.check_resultState(inMsg, CMD_SET_TL_VARIABLE);
 }
 
 void
@@ -1287,6 +1378,8 @@ TraCIAPI::TrafficLightScope::setProgram(const std::string& tlsID, const std::str
     content.writeUnsignedByte(TYPE_STRING);
     content.writeString(programID);
     myParent.send_commandSetValue(CMD_SET_TL_VARIABLE, TL_PROGRAM, tlsID, content);
+    tcpip::Storage inMsg;
+    myParent.check_resultState(inMsg, CMD_SET_TL_VARIABLE);
 }
 
 void
@@ -1295,6 +1388,8 @@ TraCIAPI::TrafficLightScope::setPhaseDuration(const std::string& tlsID, unsigned
     content.writeUnsignedByte(TYPE_INTEGER);
     content.writeInt(int(1000 * phaseDuration));
     myParent.send_commandSetValue(CMD_SET_TL_VARIABLE, TL_PHASE_DURATION, tlsID, content);
+    tcpip::Storage inMsg;
+    myParent.check_resultState(inMsg, CMD_SET_TL_VARIABLE);
 }
 
 void
@@ -1323,6 +1418,8 @@ TraCIAPI::TrafficLightScope::setCompleteRedYellowGreenDefinition(const std::stri
         content.writeString(logic.phases[i].phase);
     }
     myParent.send_commandSetValue(CMD_SET_TL_VARIABLE, TL_COMPLETE_PROGRAM_RYG, tlsID, content);
+    tcpip::Storage inMsg;
+    myParent.check_resultState(inMsg, CMD_SET_TL_VARIABLE);
 }
 
 
@@ -1415,6 +1512,8 @@ TraCIAPI::VehicleTypeScope::setLength(const std::string& typeID, SUMOReal length
     content.writeUnsignedByte(TYPE_DOUBLE);
     content.writeDouble(length);
     myParent.send_commandSetValue(CMD_SET_VEHICLETYPE_VARIABLE, VAR_LENGTH, typeID, content);
+    tcpip::Storage inMsg;
+    myParent.check_resultState(inMsg, CMD_SET_VEHICLETYPE_VARIABLE);
 }
 
 void
@@ -1423,6 +1522,8 @@ TraCIAPI::VehicleTypeScope::setMaxSpeed(const std::string& typeID, SUMOReal spee
     content.writeUnsignedByte(TYPE_DOUBLE);
     content.writeDouble(speed);
     myParent.send_commandSetValue(CMD_SET_VEHICLETYPE_VARIABLE, VAR_MAXSPEED, typeID, content);
+    tcpip::Storage inMsg;
+    myParent.check_resultState(inMsg, CMD_SET_VEHICLETYPE_VARIABLE);
 }
 
 void
@@ -1431,6 +1532,8 @@ TraCIAPI::VehicleTypeScope::setVehicleClass(const std::string& typeID, const std
     content.writeUnsignedByte(TYPE_STRING);
     content.writeString(clazz);
     myParent.send_commandSetValue(CMD_SET_VEHICLETYPE_VARIABLE, VAR_VEHICLECLASS, typeID, content);
+    tcpip::Storage inMsg;
+    myParent.check_resultState(inMsg, CMD_SET_VEHICLETYPE_VARIABLE);
 }
 
 void
@@ -1439,6 +1542,8 @@ TraCIAPI::VehicleTypeScope::setSpeedFactor(const std::string& typeID, SUMOReal f
     content.writeUnsignedByte(TYPE_DOUBLE);
     content.writeDouble(factor);
     myParent.send_commandSetValue(CMD_SET_VEHICLETYPE_VARIABLE, VAR_SPEED_FACTOR, typeID, content);
+    tcpip::Storage inMsg;
+    myParent.check_resultState(inMsg, CMD_SET_VEHICLETYPE_VARIABLE);
 }
 
 void
@@ -1447,6 +1552,8 @@ TraCIAPI::VehicleTypeScope::setSpeedDeviation(const std::string& typeID, SUMORea
     content.writeUnsignedByte(TYPE_DOUBLE);
     content.writeDouble(deviation);
     myParent.send_commandSetValue(CMD_SET_VEHICLETYPE_VARIABLE, VAR_SPEED_DEVIATION, typeID, content);
+    tcpip::Storage inMsg;
+    myParent.check_resultState(inMsg, CMD_SET_VEHICLETYPE_VARIABLE);
 }
 
 void
@@ -1455,6 +1562,8 @@ TraCIAPI::VehicleTypeScope::setEmissionClass(const std::string& typeID, const st
     content.writeUnsignedByte(TYPE_STRING);
     content.writeString(clazz);
     myParent.send_commandSetValue(CMD_SET_VEHICLETYPE_VARIABLE, VAR_EMISSIONCLASS, typeID, content);
+    tcpip::Storage inMsg;
+    myParent.check_resultState(inMsg, CMD_SET_VEHICLETYPE_VARIABLE);
 }
 
 void
@@ -1463,6 +1572,8 @@ TraCIAPI::VehicleTypeScope::setWidth(const std::string& typeID, SUMOReal width)
     content.writeUnsignedByte(TYPE_DOUBLE);
     content.writeDouble(width);
     myParent.send_commandSetValue(CMD_SET_VEHICLETYPE_VARIABLE, VAR_WIDTH, typeID, content);
+    tcpip::Storage inMsg;
+    myParent.check_resultState(inMsg, CMD_SET_VEHICLETYPE_VARIABLE);
 }
 
 void
@@ -1471,6 +1582,8 @@ TraCIAPI::VehicleTypeScope::setMinGap(const std::string& typeID, SUMOReal minGap
     content.writeUnsignedByte(TYPE_DOUBLE);
     content.writeDouble(minGap);
     myParent.send_commandSetValue(CMD_SET_VEHICLETYPE_VARIABLE, VAR_MINGAP, typeID, content);
+    tcpip::Storage inMsg;
+    myParent.check_resultState(inMsg, CMD_SET_VEHICLETYPE_VARIABLE);
 }
 
 void
@@ -1479,6 +1592,8 @@ TraCIAPI::VehicleTypeScope::setShapeClass(const std::string& typeID, const std::
     content.writeUnsignedByte(TYPE_STRING);
     content.writeString(clazz);
     myParent.send_commandSetValue(CMD_SET_VEHICLETYPE_VARIABLE, VAR_SHAPECLASS, typeID, content);
+    tcpip::Storage inMsg;
+    myParent.check_resultState(inMsg, CMD_SET_VEHICLETYPE_VARIABLE);
 }
 
 void
@@ -1487,6 +1602,8 @@ TraCIAPI::VehicleTypeScope::setAccel(const std::string& typeID, SUMOReal accel)
     content.writeUnsignedByte(TYPE_DOUBLE);
     content.writeDouble(accel);
     myParent.send_commandSetValue(CMD_SET_VEHICLETYPE_VARIABLE, VAR_ACCEL, typeID, content);
+    tcpip::Storage inMsg;
+    myParent.check_resultState(inMsg, CMD_SET_VEHICLETYPE_VARIABLE);
 }
 
 void
@@ -1495,6 +1612,8 @@ TraCIAPI::VehicleTypeScope::setDecel(const std::string& typeID, SUMOReal decel)
     content.writeUnsignedByte(TYPE_DOUBLE);
     content.writeDouble(decel);
     myParent.send_commandSetValue(CMD_SET_VEHICLETYPE_VARIABLE, VAR_DECEL, typeID, content);
+    tcpip::Storage inMsg;
+    myParent.check_resultState(inMsg, CMD_SET_VEHICLETYPE_VARIABLE);
 }
 
 void
@@ -1503,6 +1622,8 @@ TraCIAPI::VehicleTypeScope::setImperfection(const std::string& typeID, SUMOReal
     content.writeUnsignedByte(TYPE_DOUBLE);
     content.writeDouble(imperfection);
     myParent.send_commandSetValue(CMD_SET_VEHICLETYPE_VARIABLE, VAR_IMPERFECTION, typeID, content);
+    tcpip::Storage inMsg;
+    myParent.check_resultState(inMsg, CMD_SET_VEHICLETYPE_VARIABLE);
 }
 
 void
@@ -1511,6 +1632,8 @@ TraCIAPI::VehicleTypeScope::setTau(const std::string& typeID, SUMOReal tau) cons
     content.writeUnsignedByte(TYPE_DOUBLE);
     content.writeDouble(tau);
     myParent.send_commandSetValue(CMD_SET_VEHICLETYPE_VARIABLE, VAR_TAU, typeID, content);
+    tcpip::Storage inMsg;
+    myParent.check_resultState(inMsg, CMD_SET_VEHICLETYPE_VARIABLE);
 }
 
 void
@@ -1522,6 +1645,126 @@ TraCIAPI::VehicleTypeScope::setColor(const std::string& typeID, const TraCIColor
     content.writeUnsignedByte(c.b);
     content.writeUnsignedByte(c.a);
     myParent.send_commandSetValue(CMD_SET_VEHICLETYPE_VARIABLE, VAR_COLOR, typeID, content);
+    tcpip::Storage inMsg;
+    myParent.check_resultState(inMsg, CMD_SET_VEHICLETYPE_VARIABLE);
+}
+
+
+
+
+
+// ---------------------------------------------------------------------------
+// TraCIAPI::VehicleScope-methods
+// ---------------------------------------------------------------------------
+std::vector<std::string>
+TraCIAPI::VehicleScope::getIDList() const {
+    return myParent.getStringVector(CMD_GET_VEHICLE_VARIABLE, ID_LIST, "");
+}
+
+unsigned int
+TraCIAPI::VehicleScope::getIDCount() const {
+    return myParent.getInt(CMD_GET_VEHICLE_VARIABLE, ID_COUNT, "");
+}
+
+SUMOReal
+TraCIAPI::VehicleScope::getSpeed(const std::string& typeID) const {
+    return myParent.getDouble(CMD_GET_VEHICLE_VARIABLE, VAR_SPEED, typeID);
+}
+
+TraCIAPI::TraCIPosition
+TraCIAPI::VehicleScope::getPosition(const std::string& typeID) const {
+    return myParent.getPosition(CMD_GET_VEHICLE_VARIABLE, VAR_POSITION, typeID);
+}
+
+SUMOReal
+TraCIAPI::VehicleScope::getAngle(const std::string& typeID) const {
+    return myParent.getDouble(CMD_GET_VEHICLE_VARIABLE, VAR_ANGLE, typeID);
+}
+
+std::string
+TraCIAPI::VehicleScope::getRoadID(const std::string& typeID) const {
+    return myParent.getString(CMD_GET_VEHICLE_VARIABLE, VAR_ROAD_ID, typeID);
+}
+
+std::string
+TraCIAPI::VehicleScope::getLaneID(const std::string& typeID) const {
+    return myParent.getString(CMD_GET_VEHICLE_VARIABLE, VAR_LANE_ID, typeID);
+}
+
+unsigned int
+TraCIAPI::VehicleScope::getLaneIndex(const std::string& typeID) const {
+    return myParent.getInt(CMD_GET_VEHICLE_VARIABLE, VAR_LANE_INDEX, typeID);
+}
+
+std::string
+TraCIAPI::VehicleScope::getTypeID(const std::string& typeID) const {
+    return myParent.getString(CMD_GET_VEHICLE_VARIABLE, VAR_TYPE, typeID);
+}
+
+std::string
+TraCIAPI::VehicleScope::getRouteID(const std::string& typeID) const {
+    return myParent.getString(CMD_GET_VEHICLE_VARIABLE, VAR_ROAD_ID, typeID);
+}
+
+unsigned int
+TraCIAPI::VehicleScope::getRouteIndex(const std::string& typeID) const {
+    return myParent.getInt(CMD_GET_VEHICLE_VARIABLE, VAR_ROUTE_INDEX, typeID);
+}
+
+std::vector<std::string>
+TraCIAPI::VehicleScope::getEdges(const std::string& typeID) const {
+    return myParent.getStringVector(CMD_GET_VEHICLE_VARIABLE, VAR_EDGES, typeID);
+}
+
+TraCIAPI::TraCIColor
+TraCIAPI::VehicleScope::getColor(const std::string& typeID) const {
+    return myParent.getColor(CMD_GET_VEHICLE_VARIABLE, VAR_COLOR, typeID);
+}
+
+SUMOReal
+TraCIAPI::VehicleScope::getLanePosition(const std::string& typeID) const {
+    return myParent.getDouble(CMD_GET_VEHICLE_VARIABLE, VAR_LANEPOSITION, typeID);
+}
+
+
+
+
+void
+TraCIAPI::VehicleScope::moveTo(const std::string& typeID, const std::string& laneID, SUMOReal position) const {
+    tcpip::Storage content;
+    content.writeUnsignedByte(TYPE_COMPOUND);
+    content.writeInt(2);
+    content.writeUnsignedByte(TYPE_STRING);
+    content.writeString(laneID);
+    content.writeUnsignedByte(TYPE_DOUBLE);
+    content.writeDouble(position);
+    myParent.send_commandSetValue(CMD_SET_VEHICLE_VARIABLE, VAR_MOVE_TO, typeID, content);
+    tcpip::Storage inMsg;
+    myParent.check_resultState(inMsg, CMD_SET_VEHICLE_VARIABLE);
+}
+
+void
+TraCIAPI::VehicleScope::slowDown(const std::string& typeID, SUMOReal speed, int duration) const {
+    tcpip::Storage content;
+    content.writeUnsignedByte(TYPE_COMPOUND);
+    content.writeInt(2);
+    content.writeUnsignedByte(TYPE_DOUBLE);
+    content.writeDouble(speed);
+    content.writeUnsignedByte(TYPE_INTEGER);
+    content.writeInt(duration);
+    myParent.send_commandSetValue(CMD_SET_VEHICLE_VARIABLE, CMD_SLOWDOWN, typeID, content);
+    tcpip::Storage inMsg;
+    myParent.check_resultState(inMsg, CMD_SET_VEHICLE_VARIABLE);
+}
+
+void
+TraCIAPI::VehicleScope::setSpeed(const std::string& typeID, SUMOReal speed) const {
+    tcpip::Storage content;
+    content.writeUnsignedByte(TYPE_DOUBLE);
+    content.writeDouble(speed);
+    myParent.send_commandSetValue(CMD_SET_VEHICLE_VARIABLE, VAR_SPEED, typeID, content);
+    tcpip::Storage inMsg;
+    myParent.check_resultState(inMsg, CMD_SET_VEHICLE_VARIABLE);
 }
 
 
