@@ -4,12 +4,12 @@
 /// @author  Mario Krumnow
 /// @author  Michael Behrisch
 /// @date    30.05.2012
-/// @version $Id: TraCIAPI.h 18213 2015-04-16 11:55:42Z behrisch $
+/// @version $Id: TraCIAPI.h 20482 2016-04-18 20:49:42Z behrisch $
 ///
 // C++ TraCI client API implementation
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2012-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2012-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -148,6 +148,8 @@ public:
     void close();
     /// @}
 
+    /// @brief Advances by one step (or up to the given time)
+    void simulationStep(SUMOTime time = 0);
 
 
     /// @name Atomar getter
@@ -288,6 +290,22 @@ public:
         InductionLoopScope(TraCIAPI& parent) : TraCIScopeWrapper(parent) {}
         virtual ~InductionLoopScope() {}
 
+        struct VehicleData {
+            /* @brief Constructor
+              (mirrors MSInductLoop::VehicleData) */
+            VehicleData() {}
+            /// @brief The id of the vehicle
+            std::string id;
+            /// @brief Length of the vehicle
+            SUMOReal length;
+            /// @brief Entry-time of the vehicle in [s]
+            SUMOReal entryTime;
+            /// @brief Leave-time of the vehicle in [s]
+            SUMOReal leaveTime;
+            /// @brief Type of the vehicle in
+            std::string typeID;
+        };
+
         std::vector<std::string> getIDList() const;
         SUMOReal  getPosition(const std::string& loopID) const;
         std::string getLaneID(const std::string& loopID) const;
@@ -297,7 +315,8 @@ public:
         SUMOReal getLastStepOccupancy(const std::string& loopID) const;
         SUMOReal getLastStepMeanLength(const std::string& loopID) const;
         SUMOReal getTimeSinceDetection(const std::string& loopID) const;
-        unsigned int getVehicleData(const std::string& loopID) const;
+        std::vector<VehicleData> getVehicleData(const std::string& loopID) const;
+
 
     private:
         /// @brief invalidated copy constructor
@@ -641,6 +660,70 @@ public:
 
     };
 
+
+
+
+
+    /** @class VehicleScope
+     * @brief Scope for interaction with vehicles
+     */
+    class VehicleScope : public TraCIScopeWrapper {
+    public:
+        VehicleScope(TraCIAPI& parent) : TraCIScopeWrapper(parent) {}
+        virtual ~VehicleScope() {}
+
+        std::vector<std::string> getIDList() const;
+        unsigned int getIDCount() const;
+        SUMOReal getSpeed(const std::string& typeID) const;
+        TraCIPosition getPosition(const std::string& typeID) const;
+        SUMOReal getAngle(const std::string& typeID) const;
+        std::string getRoadID(const std::string& typeID) const;
+        std::string getLaneID(const std::string& typeID) const;
+        unsigned int getLaneIndex(const std::string& typeID) const;
+        std::string getTypeID(const std::string& typeID) const;
+        std::string getRouteID(const std::string& typeID) const;
+        unsigned int getRouteIndex(const std::string& typeID) const;
+        std::vector<std::string> getEdges(const std::string& typeID) const;
+        TraCIColor getColor(const std::string& typeID) const;
+        SUMOReal getLanePosition(const std::string& typeID) const;
+        unsigned int getSignalStates(const std::string& typeID) const;
+        SUMOReal getCO2Emissions(const std::string& typeID) const;
+        SUMOReal getCOEmissions(const std::string& typeID) const;
+        SUMOReal getHCEmissions(const std::string& typeID) const;
+        SUMOReal getPMxEmissions(const std::string& typeID) const;
+        SUMOReal getNOxEmissions(const std::string& typeID) const;
+        SUMOReal getFuelConsumption(const std::string& typeID) const;
+        SUMOReal getNoiseEmission(const std::string& typeID) const;
+        unsigned int getBestLanes(const std::string& typeID) const;
+        unsigned int getStopState(const std::string& typeID) const;
+        SUMOReal getLength(const std::string& typeID) const;
+        SUMOReal getMaxSpeed(const std::string& typeID) const;
+        SUMOReal getAccel(const std::string& typeID) const;
+        SUMOReal getDecel(const std::string& typeID) const;
+        SUMOReal getTau(const std::string& typeID) const;
+        SUMOReal getImperfection(const std::string& typeID) const;
+        SUMOReal getSpeedFactor(const std::string& typeID) const;
+        SUMOReal getSpeedDeviation(const std::string& typeID) const;
+        std::string getVClass(const std::string& typeID) const;
+        std::string getEmissionClass(const std::string& typeID) const;
+        std::string getShapeClass(const std::string& typeID) const;
+        SUMOReal getMinGap(const std::string& typeID) const;
+        SUMOReal getWidth(const std::string& typeID) const;
+        SUMOReal getWaitingTime(const std::string& typeID) const;
+
+        void moveTo(const std::string& typeID, const std::string& laneID, SUMOReal position) const;
+        void slowDown(const std::string& typeID, SUMOReal speed, int duration) const;
+        void setSpeed(const std::string& typeID, SUMOReal speed) const;
+
+    private:
+        /// @brief invalidated copy constructor
+        VehicleScope(const VehicleScope& src);
+
+        /// @brief invalidated assignment operator
+        VehicleScope& operator=(const VehicleScope& src);
+
+    };
+
 public:
     /// @brief Scope for interaction with edges
     EdgeScope edge;
@@ -666,6 +749,8 @@ public:
     TrafficLightScope trafficlights;
     /// @brief Scope for interaction with vehicle types
     VehicleTypeScope vehicletype;
+    /// @brief Scope for interaction with vehicles
+    VehicleScope vehicle;
 
 
 protected:
