@@ -5,12 +5,12 @@
 /// @author  Sascha Krieg
 /// @author  Michael Behrisch
 /// @date    Mon, 9 Jul 2001
-/// @version $Id: SUMORouteHandler.cpp 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: SUMORouteHandler.cpp 20482 2016-04-18 20:49:42Z behrisch $
 ///
 // Parser for routes during their loading
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -59,11 +59,12 @@ SUMORouteHandler::SUMORouteHandler(const std::string& file) :
     myCurrentVType(0),
     myBeginDefault(string2time(OptionsCont::getOptions().getString("begin"))),
     myEndDefault(string2time(OptionsCont::getOptions().getString("end"))),
-    myFirstDepart(-1) {
+    myFirstDepart(-1), myInsertStopEdgesAt(-1) {
 }
 
 
 SUMORouteHandler::~SUMORouteHandler() {
+    delete myCurrentVType;
 }
 
 
@@ -87,7 +88,8 @@ SUMORouteHandler::checkLastDepart() {
 
 void
 SUMORouteHandler::registerLastDepart() {
-    if (myVehicleParameter->departProcedure == DEPART_GIVEN) {
+    // register only non public transport to parse all public transport lines in advance
+    if (myVehicleParameter->line == "" && myVehicleParameter->departProcedure == DEPART_GIVEN) {
         myLastDepart = myVehicleParameter->depart;
         if (myFirstDepart == -1) {
             myFirstDepart = myLastDepart;
@@ -106,6 +108,10 @@ SUMORouteHandler::myStartElement(int element,
             myVehicleParameter = SUMOVehicleParserHelper::parseVehicleAttributes(attrs);
             break;
         case SUMO_TAG_PERSON:
+            delete myVehicleParameter;
+            myVehicleParameter = SUMOVehicleParserHelper::parseVehicleAttributes(attrs, false, false, true);
+            break;
+        case SUMO_TAG_CONTAINER:
             delete myVehicleParameter;
             myVehicleParameter = SUMOVehicleParserHelper::parseVehicleAttributes(attrs);
             break;
@@ -164,6 +170,11 @@ SUMORouteHandler::myEndElement(int element) {
             delete myVehicleParameter;
             myVehicleParameter = 0;
             break;
+        case SUMO_TAG_CONTAINER:
+            closeContainer();
+            delete myVehicleParameter;
+            myVehicleParameter = 0;
+            break;
         case SUMO_TAG_VEHICLE:
             if (myVehicleParameter->repetitionNumber > 0) {
                 myVehicleParameter->repetitionNumber++; // for backwards compatibility
@@ -190,9 +201,6 @@ SUMORouteHandler::myEndElement(int element) {
             myBeginDefault = string2time(OptionsCont::getOptions().getString("begin"));
             myEndDefault = string2time(OptionsCont::getOptions().getString("end"));
             break;
-        case SUMO_TAG_TRIP:
-            delete myVehicleParameter;
-            myVehicleParameter = 0;
         default:
             break;
     }
@@ -262,22 +270,37 @@ SUMORouteHandler::parseStop(SUMOVehicleParameter::Stop& stop, const SUMOSAXAttri
     if (attrs.hasAttribute(SUMO_ATTR_TRIGGERED)) {
         stop.setParameter |= STOP_TRIGGER_SET;
     }
+    if (attrs.hasAttribute(SUMO_ATTR_CONTAINER_TRIGGERED)) {
+        stop.setParameter |= STOP_CONTAINER_TRIGGER_SET;
+    }
     if (attrs.hasAttribute(SUMO_ATTR_PARKING)) {
         stop.setParameter |= STOP_PARKING_SET;
     }
     if (attrs.hasAttribute(SUMO_ATTR_EXPECTED)) {
         stop.setParameter |= STOP_EXPECTED_SET;
     }
+    if (attrs.hasAttribute(SUMO_ATTR_EXPECTED_CONTAINERS)) {
+        stop.setParameter |= STOP_EXPECTED_CONTAINERS_SET;
+    }
     bool ok = true;
     stop.busstop = attrs.getOpt<std::string>(SUMO_ATTR_BUS_STOP, 0, ok, "");
+    stop.containerstop = attrs.getOpt<std::string>(SUMO_ATTR_CONTAINER_STOP, 0, ok, "");
     if (stop.busstop != "") {
         errorSuffix = " at '" + stop.busstop + "'" + errorSuffix;
+    } else if (stop.containerstop != "") {
+        errorSuffix = " at '" + stop.containerstop + "'" + errorSuffix;
     } else {
-        errorSuffix = " on lane '" + stop.busstop + "'" + errorSuffix;
+        errorSuffix = " on lane '" + stop.lane + "'" + errorSuffix;
     }
     // get the standing duration
     if (!attrs.hasAttribute(SUMO_ATTR_DURATION) && !attrs.hasAttribute(SUMO_ATTR_UNTIL)) {
-        stop.triggered = attrs.getOpt<bool>(SUMO_ATTR_TRIGGERED, 0, ok, true);
+        if (attrs.hasAttribute(SUMO_ATTR_CONTAINER_TRIGGERED)) {
+            stop.containerTriggered = attrs.getOpt<bool>(SUMO_ATTR_CONTAINER_TRIGGERED, 0, ok, true);
+            stop.triggered = attrs.getOpt<bool>(SUMO_ATTR_TRIGGERED, 0, ok, false);
+        } else {
+            stop.triggered = attrs.getOpt<bool>(SUMO_ATTR_TRIGGERED, 0, ok, true);
+            stop.containerTriggered = attrs.getOpt<bool>(SUMO_ATTR_CONTAINER_TRIGGERED, 0, ok, false);
+        }
         stop.duration = -1;
         stop.until = -1;
     } else {
@@ -288,10 +311,11 @@ SUMORouteHandler::parseStop(SUMOVehicleParameter::Stop& stop, const SUMOSAXAttri
             return false;
         }
         stop.triggered = attrs.getOpt<bool>(SUMO_ATTR_TRIGGERED, 0, ok, false);
+        stop.containerTriggered = attrs.getOpt<bool>(SUMO_ATTR_CONTAINER_TRIGGERED, 0, ok, false);
     }
-    stop.parking = attrs.getOpt<bool>(SUMO_ATTR_PARKING, 0, ok, stop.triggered);
+    stop.parking = attrs.getOpt<bool>(SUMO_ATTR_PARKING, 0, ok, stop.triggered || stop.containerTriggered);
     if (!ok) {
-        errorOutput->inform("Invalid bool for 'triggered' or 'parking' for stop" + errorSuffix);
+        errorOutput->inform("Invalid bool for 'triggered', 'containerTriggered' or 'parking' for stop" + errorSuffix);
         return false;
     }
 
@@ -301,6 +325,12 @@ SUMORouteHandler::parseStop(SUMOVehicleParameter::Stop& stop, const SUMOSAXAttri
     SUMOSAXAttributes::parseStringSet(expectedStr, personIDs);
     stop.awaitedPersons = personIDs;
 
+    // expected containers
+    std::string expectedContainersStr = attrs.getOpt<std::string>(SUMO_ATTR_EXPECTED_CONTAINERS, 0, ok, "");
+    std::set<std::string> containerIDs;
+    SUMOSAXAttributes::parseStringSet(expectedContainersStr, containerIDs);
+    stop.awaitedContainers = containerIDs;
+
     const std::string idx = attrs.getOpt<std::string>(SUMO_ATTR_INDEX, 0, ok, "end");
     if (idx == "end") {
         stop.index = STOP_INDEX_END;
