@@ -21,6 +21,7 @@ package de.tudresden.sumo.cmd;
 import de.tudresden.sumo.config.Constants;
 import de.tudresden.sumo.util.SumoCommand;
 import de.tudresden.ws.container.SumoColor;
+import de.tudresden.ws.container.SumoStopFlags;
 import de.tudresden.ws.container.SumoStringList;
 
 /**
@@ -173,7 +174,7 @@ public class Vehicle {
 	 * @param laneID lane id
 	 * @return the distance
 	 */
-	public static SumoCommand getDrivingDistance(String vehID, String edgeID, double pos, int laneID){
+	public static SumoCommand getDrivingDistance(String vehID, String edgeID, double pos, byte laneID){
 		Object[] array = new Object[]{edgeID, pos, laneID};
 		return new SumoCommand(Constants.CMD_GET_VEHICLE_VARIABLE, Constants.DISTANCE_REQUEST, vehID, array, Constants.RESPONSE_GET_VEHICLE_VARIABLE, Constants.TYPE_DOUBLE);
 	}
@@ -402,6 +403,16 @@ public class Vehicle {
 		return new SumoCommand(Constants.CMD_GET_VEHICLE_VARIABLE, Constants.VAR_ROUTE_ID, vehID, Constants.RESPONSE_GET_VEHICLE_VARIABLE, Constants.TYPE_STRING);
 	}
 
+	/**
+	 *  Returns the index of the current edge within the vehicles route or -1 if the vehicle has not yet departed
+	 * @param vehID id of the vehicle
+	 * @return route index
+	 */
+
+	public static SumoCommand getRouteIndex(String vehID){
+		return new SumoCommand(Constants.CMD_GET_VEHICLE_VARIABLE, Constants.VAR_ROUTE_INDEX, vehID, Constants.RESPONSE_GET_VEHICLE_VARIABLE, Constants.TYPE_INTEGER);
+	}
+	
 	/**
 	 * getShapeClass
 	 * @param vehID id of the vehicle
@@ -462,6 +473,17 @@ public class Vehicle {
 		return new SumoCommand(Constants.CMD_GET_VEHICLE_VARIABLE, Constants.VAR_SPEED_WITHOUT_TRACI, vehID, Constants.RESPONSE_GET_VEHICLE_VARIABLE, Constants.TYPE_DOUBLE);
 	}
 
+	/**
+	 *  Returns information in regard to stopping: The returned integer is defined as 1 * stopped + 2 * parking + 4 * personTriggered + 8 * containerTriggered + 16 * isBusStop + 32 * isContainerStop with each of these flags defined as 0 or 1
+	 * @param vehID id of the vehicle
+	 * @return speed in m/s
+	 */
+
+	public static SumoCommand getStopState(String vehID){
+		return new SumoCommand(Constants.CMD_GET_VEHICLE_VARIABLE, Constants.VAR_STOPSTATE, vehID, Constants.RESPONSE_GET_VEHICLE_VARIABLE, Constants.TYPE_UBYTE);
+	}
+	
+	
 	/**
 	 * getTau
 	 * @param vehID id of the vehicle
@@ -514,6 +536,58 @@ public class Vehicle {
 	}
 
 	
+	/**
+	 * isStopped
+	 * @param vehID id of the vehicle
+	 * @return stop
+	 */
+
+	public static SumoCommand isStopped(String vehID){
+		return new SumoCommand(Constants.CMD_GET_VEHICLE_VARIABLE, Constants.VAR_STOPSTATE, vehID, Constants.RESPONSE_GET_VEHICLE_VARIABLE, Constants.TYPE_UBYTE, "isStopped");
+	}
+	
+	/**
+	 * Return whether the vehicle is stopped and waiting for a person or container
+	 * @param vehID id of the vehicle
+	 * @return stop
+	 */
+
+	public static SumoCommand isStoppedTriggered(String vehID){
+		return new SumoCommand(Constants.CMD_GET_VEHICLE_VARIABLE, Constants.VAR_STOPSTATE, vehID, Constants.RESPONSE_GET_VEHICLE_VARIABLE, Constants.TYPE_UBYTE, "isStoppedTriggered");
+	}
+	
+	/**
+	 * Return whether the vehicle is stopped at a container stop
+	 * @param vehID id of the vehicle
+	 * @return stop
+	 */
+
+	public static SumoCommand isAtContainerStop(String vehID){
+		return new SumoCommand(Constants.CMD_GET_VEHICLE_VARIABLE, Constants.VAR_STOPSTATE, vehID, Constants.RESPONSE_GET_VEHICLE_VARIABLE, Constants.TYPE_UBYTE, "isAtContainerStop");
+	}
+	
+	/**
+	 * isStoppedParking
+	 * @param vehID id of the vehicle
+	 * @return stop
+	 */
+
+	public static SumoCommand isStoppedParking(String vehID){
+		return new SumoCommand(Constants.CMD_GET_VEHICLE_VARIABLE, Constants.VAR_STOPSTATE, vehID, Constants.RESPONSE_GET_VEHICLE_VARIABLE, Constants.TYPE_UBYTE, "isStoppedParking");
+	}
+	
+	/**
+	 * isAtBusStop
+	 * @param vehID id of the vehicle
+	 * @return stop
+	 */
+
+	public static SumoCommand isAtBusStop(String vehID){
+		return new SumoCommand(Constants.CMD_GET_VEHICLE_VARIABLE, Constants.VAR_STOPSTATE, vehID, Constants.RESPONSE_GET_VEHICLE_VARIABLE, Constants.TYPE_UBYTE, "isAtBusStop");
+	}
+	
+	
+	
 	/**
 	 * Returns whether the Route is valid.
 	 * @param vehID id of the vehicle
@@ -684,6 +758,7 @@ public class Vehicle {
 		return new SumoCommand(Constants.CMD_SET_VEHICLE_VARIABLE, Constants.VAR_COLOR, vehID, color);
 	}
 
+	
 	/**
 	 * Sets the deceleration of the named vehicle.
 	 * @param vehID id of the vehicle
@@ -863,7 +938,7 @@ public class Vehicle {
 	 * @param sm speed mode
 	 * @return SumoCommand
 	 */
-	public static SumoCommand setspeedMode(String vehID, int sm){
+	public static SumoCommand setSpeedMode(String vehID, int sm){
 		return new SumoCommand(Constants.CMD_SET_VEHICLE_VARIABLE, Constants.VAR_SPEEDSETMODE, vehID, sm);
 	}
 	
@@ -877,19 +952,49 @@ public class Vehicle {
 	 * @param stopType stop type
 	 * @return SumoCommand
 	 */
-	public static SumoCommand setStop(String vehID, String edgeID, double pos, byte laneIndex, int duration, byte stopType){
+	public static SumoCommand setStop(String vehID, String edgeID, double pos, byte laneIndex, int duration, SumoStopFlags sf){
 
-		Object[] array = new Object[]{edgeID, pos, laneIndex, duration, stopType};
+		Object[] array = new Object[]{edgeID, pos, laneIndex, duration, sf};
 		return new SumoCommand(Constants.CMD_SET_VEHICLE_VARIABLE, Constants.CMD_STOP, vehID, array);
 	}
 
+	/**
+	 * Adds or modifies a bus stop with the given parameters. The duration and the until attribute are in milliseconds.
+	 * @param vehID id of the vehicle
+	 * @param stopID
+	 * @param duration
+	 * @param until
+	 * @param flags
+	 * @return SumoCommand
+	 */
+	public static SumoCommand setBusStop(String vehID, String stopID, int duration, int until){
+		SumoStopFlags sf = new SumoStopFlags(false, false, false, true, false);
+    	return setStop(vehID, stopID, 1, (byte) 0, duration, sf);
+	}
+	
 
+	/**
+	 *  Adds or modifies a container stop with the given parameters. The duration and the until attribute are
+	    in milliseconds.
+
+	 * @param vehID id of the vehicle
+	 * @param stopID
+	 * @param duration
+	 * @param until
+	 * @param flags
+	 * @return SumoCommand
+	 */
+	public static SumoCommand setContainerStop(String vehID, String stopID, int duration, int until){
+		SumoStopFlags sf = new SumoStopFlags(false, false, false, false, true);
+		return setStop(vehID, stopID, 1, (byte) 0, duration, sf);
+	}
+	
 	/**
 	 * Continue after a stop
 	 * @param vehID vehicle id
 	 * @return SumoCommand
 	 */
-	public static SumoCommand Resume(String vehID){
+	public static SumoCommand resume(String vehID){
 		Object[] array = new Object[]{vehID};
 		return new SumoCommand(Constants.CMD_SET_VEHICLE_VARIABLE, Constants.CMD_RESUME, vehID, array);
 	}
