@@ -1,5 +1,5 @@
 /*   
-    Copyright (C) 2014 Mario Krumnow, Dresden University of Technology
+    Copyright (C) 2015 Mario Krumnow, Dresden University of Technology
 
     This file is part of TraaS.
 
@@ -31,16 +31,47 @@ public class Constants {
 	public static final int ADD_FULL = 0x85;
 	
 	public static final int CMD_CLEAR_PENDING_VEHICLES = 0x94;
-	   
+	
+	public static final int CMD_SAVE_STATE = 0x95;
+	
 	public static final int VAR_LEADER = 0x68;
 
+	/**sets/retrieves abstract parameter*/
+	public static final int VAR_PARAMETER = 0x7e;
+	
+	/**edge index in current route (get: vehicle)*/
+	public static final int VAR_ROUTE_INDEX = 0x69;
+	
+	/**person riding / container being transported*/
+	public static final int STAGE_DRIVING = 0x01;
+	
+	/**person / container stopping*/
+	public static final int STAGE_WAITING = 0x02;
+	
+	/**person walking / container transhiping*/
+	public static final int STAGE_WALKING = 0x00;
+	
+	/**retrieve information regarding the next edge including crossings and walkingAreas (pedestrians only)*/
+	public static final int VAR_NEXT_EDGE = 0xc1;
 	
 	/** how lane changing is performed (set: vehicle)*/
 	public static final int VAR_LANECHANGE_MODE = 0xb6;
 
-	/** maximum speed regarding max speed on the current lane and speed factor (get: vehicle)*/
+	/**maximum speed regarding max speed on the current lane and speed factor (get: vehicle)*/
 	public static final int VAR_ALLOWED_SPEED = 0xb7;
 
+	/**retrieve information regarding the current person/container stage*/
+	public static final int VAR_STAGE = 0xc0;
+	
+	/**retrieve the number of stages (person, container)*/
+	public static final int VAR_NUM_STAGES = 0xc2;
+	
+	/**last step person list (get: edges)*/
+	public static final int LAST_STEP_PERSON_ID_LIST = 0x1a;
+	
+	
+	
+	
 	/** number of vehicles starting to park (get: simulation)*/
 	public static final int VAR_STOP_STARTING_VEHICLES_NUMBER = 0x68;
 
@@ -169,6 +200,9 @@ public class Constants {
 
 	/**name of the current program (get: traffic lights)*/
 	public static final int TL_CURRENT_PROGRAM = 0x29;
+	
+	/**current state, using external signal names (get: traffic lights)*/
+	public static final int TL_EXTERNAL_STATE = 0x2e;
 
 	/**set simulation variable*/
 	public static final int CMD_SET_SIM_VARIABLE = 0xcb;
@@ -184,13 +218,35 @@ public class Constants {
 	public static final int RESPONSE_SUBSCRIBE_AREAL_DETECTOR_CONTEXT = 0x9d;
 
 	/**command: subscribe areal detector (e2) variable*/
-	public static final int CMD_SUBSCRIBE_AREAL_DETECTOR_VARIABLE = 0x8f;
+	public static final int CMD_SUBSCRIBE_AREAL_DETECTOR_VARIABLE = 0xdd;
 
 	/**response: subscribe areal detector (e2) variable*/
-	public static final int RESPONSE_SUBSCRIBE_AREAL_DETECTOR_VARIABLE = 0x9f;
+	public static final int RESPONSE_SUBSCRIBE_AREAL_DETECTOR_VARIABLE = 0xed;
 	
 	
-	/**route id (get & set: vehicles)*/
+	/**command: get person variable*/
+	public static final int  CMD_GET_PERSON_VARIABLE = 0xae;
+	
+	/**command: set person variable*/
+	public static final int  CMD_SET_PERSON_VARIABLE = 0xce;
+	
+	/**response: get person variable*/
+	public static final int  RESPONSE_GET_PERSON_VARIABLE = 0xbe;
+	
+	/**command: subscribe person context*/
+	public static final int  CMD_SUBSCRIBE_PERSON_CONTEXT = 0x8e;
+	
+	/**command: subscribe person variable*/
+	public static final int  CMD_SUBSCRIBE_PERSON_VARIABLE = 0xde;
+	
+	/**response: subscribe person context*/
+	public static final int  RESPONSE_SUBSCRIBE_PERSON_CONTEXT = 0x9e;
+	
+	/**response: subscribe person variable*/
+	public static final int  RESPONSE_SUBSCRIBE_PERSON_VARIABLE = 0xee;
+	
+	
+	/**route id (get and set: vehicles)*/
 	public static final int VAR_ROUTE_ID = 0x53;
 
 	/**response: subscribe poi context*/
@@ -236,7 +292,7 @@ public class Constants {
 	public static final int VAR_WIDTH = 0x4d;
 
 	/**get area detector (e2) variable*/
-	public static final int CMD_GET_AREAL_DETECTOR_VARIABLE = 0x8e;
+	public static final int CMD_GET_AREAL_DETECTOR_VARIABLE = 0xad;
 
 	
 	/**get multi-entry/multi-exit detector (e3) variable*/
@@ -314,6 +370,7 @@ public class Constants {
 	/**Position in geo-coordinates with altitude*/
 	public static final int POSITION_LON_LAT_ALT = 0x02;
 
+	
 	/**set poi variable*/
 	public static final int CMD_SET_POI_VARIABLE = 0xc7;
 
@@ -341,7 +398,6 @@ public class Constants {
 	/**waiting time*/	
 	public static final int VAR_WAITING_TIME = 0x7a;
 	
-	/**driver reaction time (get: vehicle types)*/
 	public static final int VAR_TAU = 0x48;
 
 	/**set junction variable*/
@@ -529,6 +585,9 @@ public class Constants {
 	/**position (2D) (get: vehicle, poi, set: poi)*/
 	public static final int VAR_POSITION = 0x42;
 
+	/**position (3D) (get: vehicle, poi, set: poi)*/
+	public static final int VAR_POSITION3D = 0x39;
+	
 	/**ids of arrived vehicles (get: simulation)*/
 	public static final int VAR_ARRIVED_VEHICLES_IDS = 0x7a;
 
@@ -592,7 +651,7 @@ public class Constants {
 	/**current CO emission of a node (get: vehicle, lane, edge)*/
 	public static final int VAR_COEMISSION = 0x61;
 
-	/**list of allowed vehicle classes (get&set: lanes)*/
+	/**list of allowed vehicle classes (get and set: lanes)*/
 	public static final int LANE_ALLOWED = 0x34;
 
 	/**subscribe edge context*/
@@ -631,7 +690,7 @@ public class Constants {
 	/**response: subscribe polygon variable*/
 	public static final int RESPONSE_SUBSCRIBE_POLYGON_VARIABLE = 0xe8;
 
-	/**effort information (get&set: vehicle)*/
+	/**effort information (get and set: vehicle)*/
 	public static final int VAR_EDGE_EFFORT = 0x59;
 
 	/**subscribe simulation variable*/
@@ -661,7 +720,7 @@ public class Constants {
 	/**outgoing link definitions (get: lanes)*/
 	public static final int LANE_LINKS = 0x33;
 
-	/**list of not allowed vehicle classes (get&set: lanes)*/
+	/**list of not allowed vehicle classes (get and set: lanes)*/
 	public static final int LANE_DISALLOWED = 0x35;
 
 	/**response: subscribe lane context*/
@@ -746,7 +805,7 @@ public class Constants {
 
 	
 	/**response: get area detector (e3) variable*/
-	public static final int RESPONSE_GET_AREAL_DETECTOR_VARIABLE = 0x9e;
+	public static final int RESPONSE_GET_AREAL_DETECTOR_VARIABLE = 0xbd;
 	
 	
 	/**response: get multi-/entry detector (e3) variable*/
@@ -793,7 +852,7 @@ public class Constants {
 	/**response: subscribe simulation variable*/
 	public static final int RESPONSE_SUBSCRIBE_SIM_VARIABLE = 0xeb;
 
-	/**travel time information (get&set: vehicle)*/
+	/**travel time information (get and set: vehicle)*/
 	public static final int VAR_EDGE_TRAVELTIME = 0x58;
 
 	/**current PMx emission of a node (get: vehicle, lane, edge)*/
