@@ -1,5 +1,5 @@
 /*   
-    Copyright (C) 2014 Mario Krumnow, Dresden University of Technology
+    Copyright (C) 2015 Mario Krumnow, Dresden University of Technology
 
     This file is part of TraaS.
 
@@ -28,6 +28,7 @@ import de.tudresden.sumo.cmd.Junction;
 import de.tudresden.sumo.cmd.ArealDetector;
 import de.tudresden.sumo.cmd.Lane;
 import de.tudresden.sumo.cmd.Multientryexit;
+import de.tudresden.sumo.cmd.Person;
 import de.tudresden.sumo.cmd.Poi;
 import de.tudresden.sumo.cmd.Polygon;
 import de.tudresden.sumo.cmd.Route;
@@ -41,6 +42,7 @@ import de.tudresden.ws.container.SumoGeometry;
 import de.tudresden.ws.container.SumoLinkList;
 import de.tudresden.ws.container.SumoPosition2D;
 import de.tudresden.ws.container.SumoPosition3D;
+import de.tudresden.ws.container.SumoStopFlags;
 import de.tudresden.ws.container.SumoStringList;
 import de.tudresden.ws.container.SumoTLSLogic;
 import de.tudresden.ws.log.Log;
@@ -205,13 +207,13 @@ public class Traci{
 	}
 
 	@WebMethod(action="Vehicle: setStop")
-	public void Vehicle_setStop(@WebParam(name = "vehID") String vehID, @WebParam(name = "edgeID") String edgeID, @WebParam(name = "pos") double pos, @WebParam(name = "laneIndex") byte laneIndex, @WebParam(name = "duration") int duration, @WebParam(name = "stopType") byte stopType){
+	public void Vehicle_setStop(@WebParam(name = "vehID") String vehID, @WebParam(name = "edgeID") String edgeID, @WebParam(name = "pos") double pos, @WebParam(name = "laneIndex") byte laneIndex, @WebParam(name = "duration") int duration, @WebParam(name = "stopType") SumoStopFlags stopType){
 		this.sumo.set_cmd(Vehicle.setStop(vehID, edgeID, pos, laneIndex, duration, stopType));
 	}
 	
 	@WebMethod(action="Vehicle: setResume")
-	public void Vehicle_Resume(@WebParam(name = "vehID") String vehID){
-		this.sumo.set_cmd(Vehicle.Resume(vehID));
+	public void Vehicle_resume(@WebParam(name = "vehID") String vehID){
+		this.sumo.set_cmd(Vehicle.resume(vehID));
 	}
 
 	@WebMethod(action="Vehicle: setTau")
@@ -584,6 +586,86 @@ public class Traci{
 		return this.helper.getDouble(this.sumo.get_cmd(ArealDetector.getLastStepOccupancy(loopID)));
 	}
 	
+	
+	
+	//new
+	
+	
+	@WebMethod(action="Person: getIDList")
+	public SumoStringList Person_getIDList(){
+		return this.helper.getStringList(this.sumo.get_cmd(Person.getIDList()));
+	}
+	
+	@WebMethod(action="Person: getIDCount")
+	public int Person_getIDCount(){
+		return this.helper.getInt(this.sumo.get_cmd(Person.getIDCount()));
+	}
+	
+	@WebMethod(action="Person: getSpeed")
+	public double Person_getSpeed(@WebParam(name = "personID") String personID){
+		return this.helper.getDouble(this.sumo.get_cmd(Person.getSpeed(personID)));
+	}
+	
+	@WebMethod(action="Person: getPosition")
+	public SumoPosition2D Person_getPosition(@WebParam(name = "personID") String personID){
+		return this.helper.getPosition2D(this.sumo.get_cmd(Person.getPosition(personID)));
+	}
+	
+	@WebMethod(action="Person: getPosition3D")
+	public SumoPosition3D Person_getPosition3D(@WebParam(name = "personID") String personID){
+		return this.helper.getPosition3D(this.sumo.get_cmd(Person.getPosition3D(personID)));
+	}
+
+	@WebMethod(action="Person: getAngle")
+	public int Person_getAngle(@WebParam(name = "vehID") String personID){
+		return this.helper.getInt(this.sumo.get_cmd(Person.getAngle(personID)));
+	}
+
+	@WebMethod(action="Person: getRoadID")
+	public String Person_getRoadID(@WebParam(name = "personID") String personID){
+		return this.helper.getString(this.sumo.get_cmd(Person.getRoadID(personID)));
+	}
+
+	@WebMethod(action="Person: getTypeID")
+	public String Person_getTypeID(@WebParam(name = "personID") String personID){
+		return this.helper.getString(this.sumo.get_cmd(Person.getTypeID(personID)));
+	}
+
+	@WebMethod(action="Person: getLanePosition")
+	public double Person_getLanePosition(@WebParam(name = "personID") String personID){
+		return this.helper.getDouble(this.sumo.get_cmd(Person.getLanePosition(personID)));
+	}
+
+	@WebMethod(action="Person: getColor")
+	public SumoColor Person_getColor(@WebParam(name = "personID") String personID){
+		return this.helper.getColor(this.sumo.get_cmd(Person.getColor(personID)));
+	}
+
+	@WebMethod(action="Person: getPersonNumber")
+	public int Person_getPersonNumber(@WebParam(name = "personID") String personID){
+		return this.helper.getInt(this.sumo.get_cmd(Person.getPersonNumber(personID)));
+	}
+	
+	@WebMethod(action="Person: getLength")
+	public double Person_getLength(@WebParam(name = "personID") String personID){
+		return this.helper.getDouble(this.sumo.get_cmd(Person.getLength(personID)));
+	}
+	
+	@WebMethod(action="Person: getWaitingTime")
+	public double Person_getWaitingTime(@WebParam(name = "personID") String personID){
+		return this.helper.getDouble(this.sumo.get_cmd(Person.getWaitingTime(personID)));
+	}
+	
+	@WebMethod(action="Person: getWidth")
+	public double Person_getWidth(@WebParam(name = "personID") String personID){
+		return this.helper.getDouble(this.sumo.get_cmd(Person.getWidth(personID)));
+	}
+	
+	@WebMethod(action="Person: getMinGap")
+	public double Person_getMinGap(@WebParam(name = "personID") String personID){
+		return this.helper.getDouble(this.sumo.get_cmd(Person.getMinGap(personID)));
+	}
+	
 	@WebMethod(action="Vehicle: getAccel")
 	public double Vehicle_getAccel(@WebParam(name = "vehID") String vehID){
 		return this.helper.getDouble(this.sumo.get_cmd(Vehicle.getAccel(vehID)));
@@ -625,7 +707,7 @@ public class Traci{
 	}
 
 	@WebMethod(action="Vehicle: getDrivingDistance")
-	public double Vehicle_getDrivingDistance(@WebParam(name = "vehID") String vehID, @WebParam(name = "edgeID") String edgeID, @WebParam(name = "pos") double pos, @WebParam(name = "laneID") int laneID){
+	public double Vehicle_getDrivingDistance(@WebParam(name = "vehID") String vehID, @WebParam(name = "edgeID") String edgeID, @WebParam(name = "pos") double pos, @WebParam(name = "laneID") byte laneID){
 		return this.helper.getDouble(this.sumo.get_cmd(Vehicle.getDrivingDistance(vehID, edgeID, pos, laneID)));
 	}
 
@@ -943,12 +1025,12 @@ public class Traci{
 	}
 
 	@WebMethod(action="Simulation: getDistance2D")
-	public double Simulation_getDistance2D(@WebParam(name = "x1") double x1, @WebParam(name = "y1") double y1, @WebParam(name = "x2") double x2, @WebParam(name = "y2") double y2, @WebParam(name = "isGeo") String isGeo, @WebParam(name = "isDriving") String isDriving){
+	public double Simulation_getDistance2D(@WebParam(name = "x1") double x1, @WebParam(name = "y1") double y1, @WebParam(name = "x2") double x2, @WebParam(name = "y2") double y2, @WebParam(name = "isGeo") boolean isGeo, @WebParam(name = "isDriving") boolean isDriving){
 		return this.helper.getDouble(this.sumo.get_cmd(Simulation.getDistance2D(x1, y1, x2, y2, isGeo, isDriving)));
 	}
 
 	@WebMethod(action="Simulation: Returns the subscription results for the last time step.")
-	public double Simulation_getDistanceRoad(@WebParam(name = "edgeID1") String edgeID1, @WebParam(name = "pos1") double pos1, @WebParam(name = "edgeID2") String edgeID2, @WebParam(name = "pos2") double pos2, @WebParam(name = "isDriving") String isDriving){
+	public double Simulation_getDistanceRoad(@WebParam(name = "edgeID1") String edgeID1, @WebParam(name = "pos1") double pos1, @WebParam(name = "edgeID2") String edgeID2, @WebParam(name = "pos2") double pos2, @WebParam(name = "isDriving") boolean isDriving){
 		return this.helper.getDouble(this.sumo.get_cmd(Simulation.getDistanceRoad(edgeID1, pos1, edgeID2, pos2, isDriving)));
 	}
 
