@@ -3,11 +3,11 @@
 @author  Jakob Erdmann
 @author  Michael Behrisch
 @date    2013-05-06
-@version $Id: additional.py 17235 2014-11-03 10:53:02Z behrisch $
+@version $Id: additional.py 20433 2016-04-13 08:00:14Z behrisch $
 
 
 SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-Copyright (C) 2008-2014 DLR (http://www.dlr.de/) and contributors
+Copyright (C) 2008-2016 DLR (http://www.dlr.de/) and contributors
 
 This file is part of SUMO.
 SUMO is free software; you can redistribute it and/or modify
@@ -16,12 +16,12 @@ the Free Software Foundation; either version 3 of the License, or
 (at your option) any later version.
 """
 
+
 def write(name, elements):
     fdo = open(name, "w")
-    fdo.write('<additional xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:noNamespaceSchemaLocation="http://sumo.dlr.de/xsd/additional_file.xsd">\n')
+    fdo.write(
+        '<additional xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:noNamespaceSchemaLocation="http://sumo.dlr.de/xsd/additional_file.xsd">\n')
     for e in elements:
-        fdo.write(e.toXML())
-        fdo.write("\n")
+        fdo.write("%s\n" % e.toXML())
     fdo.write("</additional>\n")
     fdo.close()
-    
\ No newline at end of file
