@@ -4,12 +4,12 @@
 @author  Jakob Erdmann
 @author  Michael Behrisch
 @date    2013-02-25
-@version $Id: geomhelper.py 17235 2014-11-03 10:53:02Z behrisch $
+@version $Id: geomhelper.py 20433 2016-04-13 08:00:14Z behrisch $
 
 Some helper functions for geometrical computations.
 
 SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-Copyright (C) 2013-2014 DLR (http://www.dlr.de/) and contributors
+Copyright (C) 2013-2016 DLR (http://www.dlr.de/) and contributors
 
 This file is part of SUMO.
 SUMO is free software; you can redistribute it and/or modify
@@ -17,14 +17,20 @@ it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 3 of the License, or
 (at your option) any later version.
 """
+from __future__ import absolute_import
 import math
 
 INVALID_DISTANCE = -1
 
+
 def distance(p1, p2):
-  dx = p1[0]-p2[0]
-  dy = p1[1]-p2[1]
-  return math.sqrt(dx*dx + dy*dy)
+    dx = p1[0] - p2[0]
+    dy = p1[1] - p2[1]
+    return math.sqrt(dx * dx + dy * dy)
+
+
+def polyLength(polygon):
+    return sum([distance(a, b) for a, b in zip(polygon[:-1], polygon[1:])])
 
 
 def lineOffsetWithMinimumDistanceToPoint(point, line_start, line_end, perpendicular=False):
@@ -48,22 +54,30 @@ def polygonOffsetWithMinimumDistanceToPoint(point, polygon, perpendicular=False)
     """Return the offset from the polygon start where the distance to point is minimal"""
     p = point
     s = polygon
-    o = 0
+    seen = 0
     minDist = 1e400
     minOffset = INVALID_DISTANCE
-    for i in range(0, len(s)-1):
-        q = lineOffsetWithMinimumDistanceToPoint(p, s[i], s[i+1], perpendicular)
-        if q != INVALID_DISTANCE:
-            d = distance(p, positionAtOffset(s[i], s[i+1], q))
-            if d < minDist:
-                minDist = d
-                minOffset = o + q
-            if perpendicular and i > 0:
-                d = distance(p, s[i])
-                if d < minDist:
-                    minDist = d
-                    minOffset = o
-        o += distance(s[i], s[i+1])
+    for i in range(len(s) - 1):
+        pos = lineOffsetWithMinimumDistanceToPoint(
+            p, s[i], s[i + 1], perpendicular)
+        dist = minDist if pos == INVALID_DISTANCE else distance(
+            p, positionAtOffset(s[i], s[i + 1], pos))
+        if dist < minDist:
+            minDist = dist
+            minOffset = pos + seen
+        if perpendicular and i != 0 and pos == INVALID_DISTANCE:
+            # even if perpendicular is set we still need to check the distance
+            # to the inner points
+            cornerDist = distance(p, s[i])
+            if cornerDist < minDist:
+                pos1 = lineOffsetWithMinimumDistanceToPoint(
+                    p, s[i - 1], s[i], False)
+                pos2 = lineOffsetWithMinimumDistanceToPoint(
+                    p, s[i], s[i + 1], False)
+                if pos1 == distance(s[i - 1], s[i]) and pos2 == 0.:
+                    minOffset = seen
+                    minDist = cornerDist
+        seen += distance(s[i], s[i + 1])
     return minOffset
 
 
@@ -71,13 +85,14 @@ def distancePointToLine(point, line_start, line_end, perpendicular=False):
     """Return the minimum distance between point and the line (line_start, line_end)"""
     p1 = line_start
     p2 = line_end
-    offset = lineOffsetWithMinimumDistanceToPoint(point, line_start, line_end, perpendicular)
-    if offset == INVALID_DISTANCE: 
+    offset = lineOffsetWithMinimumDistanceToPoint(
+        point, line_start, line_end, perpendicular)
+    if offset == INVALID_DISTANCE:
         return INVALID_DISTANCE
     if offset == 0:
         return distance(point, p1)
     u = offset / distance(line_start, line_end)
-    intersection = (p1[0] + u*(p2[0]-p1[0]), p1[1] + u*(p2[1]-p1[1]))
+    intersection = (p1[0] + u * (p2[0] - p1[0]), p1[1] + u * (p2[1] - p1[1]))
     return distance(point, intersection)
 
 
@@ -86,9 +101,9 @@ def distancePointToPolygon(point, polygon, perpendicular=False):
     p = point
     s = polygon
     minDist = None
-    for i in range(0, len(s)-1):
-        dist = distancePointToLine(p, s[i], s[i+1], perpendicular)
-        if dist == INVALID_DISTANCE and perpendicular and i != 0: 
+    for i in range(0, len(s) - 1):
+        dist = distancePointToLine(p, s[i], s[i + 1], perpendicular)
+        if dist == INVALID_DISTANCE and perpendicular and i != 0:
             # distance to inner corner
             dist = distance(point, s[i])
         if dist != INVALID_DISTANCE:
@@ -119,28 +134,25 @@ def positionAtShapeOffset(shape, offset):
     return shape[-1]
 
 
-if __name__ == "__main__":
-    # run some tests
-    point = (81365.994719034992, 9326.8304398041219)
-    polygon = [
-            (81639.699999999997, 9196.8400000000001),
-            (81554.910000000003, 9246.7600000000002),
-            (81488.800000000003, 9288.2999999999993),
-            (81376.100000000006, 9358.5799999999999),
-            (81305.089999999997, 9404.4400000000005),
-            (81230.610000000001, 9452.4200000000001),
-            (81154.699999999997, 9502.6000000000004),
-            (81063.419999999998, 9564.5799999999999),
-            (80969.389999999999, 9627.6100000000006),
-            (80882.990000000005, 9686.3899999999994),
-            (80772.160000000003, 9763.4200000000001),
-            (80682.259999999995, 9825.4500000000007),
-            (80617.509999999995, 9868.1499999999996),
-            (80552.660000000003, 9914.1900000000005)]
-    dist = distancePointToPolygon(point, polygon, True)
-    assert(abs(dist - 32.288) < 0.01)
-    print "Test successful"
+def angle2D(p1, p2):
+    theta1 = math.atan2(p1[1], p1[0])
+    theta2 = math.atan2(p2[1], p2[0])
+    dtheta = theta2 - theta1
+    while dtheta > math.pi:
+        dtheta -= 2.0 * math.pi
+    while dtheta < -math.pi:
+        dtheta += 2.0 * math.pi
+    return dtheta
 
-    assert(polygonOffsetWithMinimumDistanceToPoint((-1,-1), [(0, 1), (0,0), (1,0)], True) == INVALID_DISTANCE)
-    assert(polygonOffsetWithMinimumDistanceToPoint((-1,-1), [(0, 1), (0,0), (1,0)], False) == math.sqrt(2))
 
+def isWithin(pos, shape):
+    angle = 0.
+    for i in range(0, len(shape) - 1):
+        p1 = ((shape[i][0] - pos[0]), (shape[i][1] - pos[1]))
+        p2 = ((shape[i + 1][0] - pos[0]), (shape[i + 1][1] - pos[1]))
+        angle = angle + angle2D(p1, p2)
+    i = len(shape) - 1
+    p1 = ((shape[i][0] - pos[0]), (shape[i][1] - pos[1]))
+    p2 = ((shape[0][0] - pos[0]), (shape[0][1] - pos[1]))
+    angle = angle + angle2D(p1, p2)
+    return math.fabs(angle) >= math.pi
