@@ -6,12 +6,12 @@
 @author  Michael Behrisch
 @author  Jakob Erdmann
 @date    2011-11-28
-@version $Id: edge.py 18096 2015-03-17 09:50:59Z behrisch $
+@version $Id: edge.py 20433 2016-04-13 08:00:14Z behrisch $
 
 This file contains a Python-representation of a single edge.
 
 SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-Copyright (C) 2011-2015 DLR (http://www.dlr.de/) and contributors
+Copyright (C) 2011-2016 DLR (http://www.dlr.de/) and contributors
 
 This file is part of SUMO.
 SUMO is free software; you can redistribute it and/or modify
@@ -19,7 +19,7 @@ it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 3 of the License, or
 (at your option) any later version.
 """
-from connection import Connection
+from .connection import Connection
 
 
 class Edge:
@@ -31,8 +31,10 @@ class Edge:
         self._from = fromN
         self._to = toN
         self._priority = prio
-        fromN.addOutgoing(self)
-        toN.addIncoming(self)
+        if fromN:
+            fromN.addOutgoing(self)
+        if toN:
+            toN.addIncoming(self)
         self._lanes = []
         self._speed = None
         self._length = None
